/*****************************************************************************
NAME:			SATRACK

PURPOSE:  Read one-line navspasur data and two-line norad data from a
          single input file and write them to the appropriate update file 


PROGRAM HISTORY:
Version     Date     Author             Description
-------     ----     ------             --------------
 5.0        10/91    E. Cihak		Original Development
 5.1        02/92    T. Baltzer		Complete original development
 5.2        02/92    T. Baltzer		Make mods so it will converge
					to a solution (timeoflight()).
 5.3        01/95    J. Fenno		Added code for NOAA-14.
 5.5	    06/98    D. Lloyd		Added code for NOAA-15
 5.6	    09/00    D. Lloyd		Added code for NOAA-16

PROJECT:       ADAPS

ALGORITHM:     See in-line documentation

ALGORITHM REFERENCES:      none
******************************************************************************/
#include <stdio.h>
#ifdef VMS
#include <file.h>
#else
#include <sys/file.h>
#endif
#include "las.h"
#include "model.h"

#define   LINESIZ     80
#define   TOTSAT      7

main()
{
struct PARBLK     vblock;           /* TAE parameter block */

long   access = 4,	/* File access(0=exist 1=execute 2=write 4=read)*/
       stdlun,          /* Logical unit number */
       n_out,           /* Number of output files */
       navspid,	        /* NAVSPASUR Satellite id */
       odecay,		/* Orbital decay */
       nonfatal = 1,    /* Nonfatal error code */
       one = 1,		/* constant value */
       len;             /* Length of ptr1 string */  

static long numsat[TOTSAT] ={0,0,0,0,0,0,0};/* Satellite numbers to print out */

char   filnam[CMLEN],		 /* Name of input file */
       fileutil_file[CMLEN],     /* Temporary file for fileutil */
       out[1][CMLEN],            /* List of output files */
       line1[EPHEM_SIZ+1],	 /* Line read from input file */
       line2[LINESIZ],           /* Line read from input file */
       navid[TOTSAT],		 /* NAVSPASUR Satellite id */
       norid[TOTSAT],		 /* Norad satellite id */
       ptr1[LINESIZ],            /* String for lines from input file */
       *pointer1,                /* Pointer to line in input file */ 
       *pointer2;                /* Pointer to line in input file */ 

FILE      *fopen();		 /* function to open a file */

FILE   *fin,                     /* File descriptor for input file */
       *print_file;              /* Name of output file */ 

/* Initialize LAS
-----------------*/
c_inlas (&vblock, &stdlun);

/* Get the input file name and call fileutil to set print options
-----------------------------------------------------------------*/
get_trk_par (&vblock, filnam, &print_file, out, &n_out, numsat, fileutil_file); 

/* Make sure the file exists with read permission, then open the
   input file
-----------------------------------------------------------------*/
if ( c_lsstat(filnam,&access) != E_SUCC )
   {
   c_errmsg ("Error locating file","satrack-locf",&nonfatal);
   fclose (print_file);
   c_clean(&vblock,"SATRACK",fileutil_file,&one);
   }

fin = fopen (filnam, "r");
if (fin == NULL)
    {
    c_errmsg ("Error opening input file", "satrack-inopn", &nonfatal);
    fclose (print_file);
    c_clean(&vblock,"SATRACK",fileutil_file,&one);
    }

/* Read input file line by line and establish name of update file.
   The infinite loop is broken when no more input data is available.
-----------------------------------------------------------------*/
for ( ; ; )
    {
    /*
      Read a line from the file
    */
    pointer1 = fgets (ptr1, LINESIZ, fin);
    if (pointer1 == NULL)
	break;

    len = strlen (ptr1);
    if (len > 70)
	{
        strncpy (&line1[0], &ptr1[0], 69);
        line1[69] = '\0';
        }
    else
        strcpy (line1, ptr1);
  
    strncpy (navid, line1, 5);
    navid[5] = '\0';
    strncpy (norid, &line1[2], 6);  
    norid[6] = '\0';

    /*
      Check if the line is a navspasur line
    */
    if ((strcmp(navid,"15427") == 0) || (strcmp(navid,"16969") == 0) ||
        (strcmp(navid,"19531") == 0) || (strcmp(navid,"21263") == 0) ||
	(strcmp(navid,"23455") == 0) || (strcmp(navid,"25338") == 0) ||
	(strcmp(navid,"26536") == 0))
        {
	/*
	  Some times the orbital decay is way off.  It needs to be reset
          to zero when it is.
	*/
	sscanf(&line1[26],"%5d",&odecay);
	if (odecay > 90000)
	    strncpy(&line1[26],"00000",5);

        if (put_navsp(line1,&print_file,numsat) != E_SUCC)
            {
	    fclose (print_file);
    	    c_clean(&vblock,"SATRACK",fileutil_file,&one);
	    }
        }
    else
        {
	/*
	  Not a navspasur line, it may be a norad line
	*/

	if ((strcmp(norid, "15427U") == 0) || (strcmp(norid, "16969U") == 0) ||
	    (strcmp(norid, "19531U") == 0) || (strcmp(norid, "21263U") == 0) ||
	    (strcmp(norid, "23455U") == 0) || (strcmp(norid, "25338U") == 0) ||
	    (strcmp(norid, "26536U") == 0)) 
	   {
	   pointer2 = fgets (line2, LINESIZ, fin);  
	   if (pointer2 == NULL)
	      {
	      c_errmsg ("Error reading input file", "satrack-inread", 
			&nonfatal);
	      fclose (print_file);
    	      c_clean(&vblock,"SATRACK",fileutil_file,&one);
	      }

	   sscanf (norid, "%5d", &navspid);
           if ( gen_rpt(navspid,numsat) == E_SUCC)
	      sep_norad (line1, line2, &print_file ); 
	   }
        }
    }		/* end of infinite loop construct */

/* Close the input file and the output file  
-------------------------------------------*/
fclose (fin);
fclose (print_file);

/* Write a completion message
-----------------------------*/
c_complt ("SATRACK", out, &n_out); 
}
