/*******************************************************************************
NAME:		C_ACQREAD

PURPOSE:	Read a line from the acquired image 

Date   Programmer       Modifications
-----  ---------------  --------------------------------------------------------
02/92  D.Hollaren	Development
07/92  T.Baltzer	Add BSQ capability
09/95  J.Fenno		Removed code that was commented out.  This will be
			tracked by CVS.  The code tracking a shift in the frame
			sync data.
12/96  D.Etrheim	Allocate inbuf which will make it word alligned.
02/00  D.Lloyd		Added code for NOAA KLM data format which comes in a
			partially packed format.  Data is packed but minor
			frame is not.

PROJECT:  ADAPS
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include "worgen.h"
#include "pixman.h"
#include "tae.h"
#include "acqio.h"
#include "archio.h"
#include "avhrr.h"
#include "adapsutil.h"
#include "local_protos.h"

#define MNR_SHP	238

int c_acqread
(
    long *fdesc,                /* File descriptor to the acquire file       */
    short *mnrbuf,              /* Output buffer for HRPT minor frame        */
    short *tipbuf,              /* Output buffer for TIP data                */
    short *imgbuf               /* Output buffer for acquire image data      */
)
{
struct ACQDESC *acq;		/* Pointer to the acquisition input structure*/

unsigned char *inbuf; 		/* Input buffer for acquire*/

static short tmpbuf[MAX_ACQ];	/* Temporary buffer for conversion of image  */
static long  bands[6] = {1,2,3,4,5,0};	/* Select all bands		     */

long	dtype = EWORD,		/* Type of data wanted in output	     */
	fill = TRUE,		/* Fill in any missing bands		     */
	ftype = EBYTE,		/* Type of data to be converted		     */
	nsamps,			/* Number of samples in a band		     */
	nwords,			/* Number of words to be copied		     */
	status;			/* Return status from the read		     */


/* allocate buffer */
inbuf = (unsigned char *) malloc(MAX_ACQ * sizeof(short));

/*
  Set the acquisition structure to the file descriptor address
*/
acq = (struct ACQDESC *) *fdesc;

/*
  Read a full line of image data from each band
*/
status = read_bands(acq,inbuf);
if ((status != E_SUCC) && (status != E_BOF))
    {
    free (inbuf);
    return(status);
    }

/*
  Check for a fully packed data stream
*/
if ((acq->byte_line == FULL_LAC_PACK) || (acq->byte_line == FULL_LAC_BPACK))
    {
    nwords = MAX_ACQ;
    c_unpck32(inbuf,tmpbuf,&nwords,&acq->swap_flg);

    /*
      Copy the HRPT minor frame
    */
    nwords = MNR_SIZ;
    c_pxcopy((unsigned char *)&tmpbuf[acq->mnr_ptr],(unsigned char *)mnrbuf,
		&dtype,&nwords);

    /*
      Save the AVHRR ID from the HRPT minor frame to the TIP buffer
    */
    tipbuf[0] = mnrbuf[6];

    /*
      Copy the TIP information
    */
    nwords = TIP_SIZ;
    c_pxcopy((unsigned char *)&tmpbuf[acq->tip_ptr],(unsigned char *)&tipbuf[1],
		&dtype,&nwords);

    /*
      Copy the image data
    */
    nwords = LAC_SIZ;
    nsamps = LAC_SAMPS;
    if (acq->img_fmt == BIP)
	c_pxcopy((unsigned char *)&tmpbuf[acq->img_ptr],(unsigned char *)imgbuf,
		 &dtype,&nwords);
    else if ((acq->img_fmt == BIL) || (acq->img_fmt == BSQ))
	c_bil2bip(&tmpbuf[acq->img_ptr],imgbuf,&nsamps,bands,&fill);
    }
else if ((acq->byte_line == FULL_GAC_PACK) || 
	 (acq->byte_line == FULL_GAC_BPACK))
    {
    nwords = acq->byte_line / 2;
    c_unpck32(inbuf,tmpbuf,&nwords,&acq->swap_flg);

    /*
      Copy the HRPT minor frame
    */
    nwords = MNR_SIZ;
    c_pxcopy((unsigned char *)&tmpbuf[acq->mnr_ptr],(unsigned char *)mnrbuf,
		&dtype,&nwords);

    /*
      Save the AVHRR ID from the HRPT minor frame to the TIP buffer
    */
    tipbuf[0] = mnrbuf[6];

    /*
      Copy the TIP information
    */
    nwords = TIP_SIZ;
    c_pxcopy((unsigned char *)&tmpbuf[acq->tip_ptr],(unsigned char *)&tipbuf[1],
		&dtype,&nwords);

    /*
      Copy the image data
    */
    nwords = GAC_SIZ;
    nsamps = GAC_SAMPS;
    if (acq->img_fmt == BIP)
	c_pxcopy((unsigned char *)&tmpbuf[acq->img_ptr],(unsigned char *)imgbuf,
		 &dtype,&nwords);
    else if ((acq->img_fmt == BIL) || (acq->img_fmt == BSQ))
	c_bil2bip(&tmpbuf[acq->img_ptr],imgbuf,&nsamps,bands,&fill);
    }
/*
  Check for a partially packed data stream.  Level 1b formats for
  NOAA KLM series satellites store the minor frame as unpacked and
  the data packed.
*/
else if ((acq->byte_line == KLM_LAC_PACK) ||
         (acq->byte_line == KLM_GAC_PACK))
    {
    if ( acq->byte_line == KLM_LAC_PACK )
	nwords = LAC_SIZ;
    else
	nwords = GAC_SIZ;
    c_unpck32(&inbuf[acq->img_ptr],tmpbuf,&nwords,&acq->swap_flg);

    /*
      Copy the HRPT minor frame
    */
    nwords = MNR_SIZ;
    c_pxcopy((unsigned char *)&inbuf[acq->mnr_ptr],(unsigned char *)mnrbuf,
		&dtype,&nwords);

    /*
      Save the AVHRR ID from the HRPT minor frame to the TIP buffer
    */
    tipbuf[0] = mnrbuf[6];

    /*
      Copy the TIP information
    */
    nwords = TIP_SIZ;
    c_pxcopy((unsigned char *)&inbuf[acq->tip_ptr],(unsigned char *)&tipbuf[1],
		&dtype,&nwords);

    /*
      Copy the image data
    */
    if ( acq->byte_line == KLM_LAC_PACK )
	{
	nwords = LAC_SIZ;
	nsamps = LAC_SAMPS;
	}
    else
	{
	nwords = GAC_SIZ;
	nsamps = GAC_SAMPS;
	}
    if (acq->img_fmt == BIP)
	c_pxcopy((unsigned char *)&tmpbuf[0],(unsigned char *)imgbuf,
		 &dtype,&nwords);
    else if ((acq->img_fmt == BIL) || (acq->img_fmt == BSQ))
	c_bil2bip(&tmpbuf[0],imgbuf,&nsamps,bands,&fill);
    }
else
    {
    /*
      Copy the HRPT minor frame
    */
    nwords = MNR_SIZ;
    switch (acq->mnr_siz)
	{
	case MNR_SIZ:
	    {
	    /*
	      Convert the input byte buffer to the HRPT minor frame buffer
	    */
	    c_pxconv(&ftype,&dtype,&inbuf[acq->mnr_ptr],
			(unsigned char *)mnrbuf,&nwords);
	    break;
	    }

	case MNR_PCK:
	    {
	    /*
	      Unpack the input buffer into the HRPT minor frame buffer
	    */
	    c_unpck32(&inbuf[acq->mnr_ptr],mnrbuf,&nwords,&acq->swap_flg);
	    break;
	    }

	case MNR_UNP:
	    {
	    /*
	      Byte swap the data if needed
	    */
#ifdef vms
	    if (!acq->swap_flg)
		c_pxswap(&inbuf[acq->mnr_ptr],&nwords,&dtype);
#else
	    if (acq->swap_flg)
		c_pxswap(&inbuf[acq->mnr_ptr],&nwords,&dtype);
#endif

	    /*
	      Copy the buffer as is to the HRPT minor frame buffer
	    */
	    c_pxcopy(&inbuf[acq->mnr_ptr],(unsigned char *)mnrbuf,
			&dtype,&nwords);
	    break;
	    }

	case MNR_SHP:
	    {
	    sharp_mnr(inbuf,mnrbuf);
	    break;
	    }

	default:
	    break;
	}

    /*
      Save the AVHRR ID from the HRPT minor frame to the TIP buffer
    */
    tipbuf[0] = mnrbuf[6];

    /*
      Copy the TIP information if it exists
    */
    nwords = TIP_SIZ;
    switch (acq->tip_siz)
	{
	case TIP_SIZ:
	    {
	    /*
	      Convert the input byte buffer to the TIP buffer
	    */
	    c_pxconv(&ftype,&dtype,&inbuf[acq->tip_ptr],
			(unsigned char *)&tipbuf[1],&nwords);
	    break;
	    }

	case TIP_PCK:
	    {
	    /*
	      Unpack the input buffer into the TIP buffer
	    */
	    c_unpck32(&inbuf[acq->tip_ptr],&tipbuf[1],&nwords,&acq->swap_flg);
	    break;
	    }

	case TIP_UNP:
	    {
	    /*
	      Byte swap the data if needed
	    */
#ifdef vms
	    if (!acq->swap_flg)
		c_pxswap(&inbuf[acq->tip_ptr],&nwords,&dtype);
#else
	    if (acq->swap_flg)
		c_pxswap(&inbuf[acq->tip_ptr],&nwords,&dtype);
#endif

	    /*
	      Copy the buffer as is to the TIP buffer
	    */
	    c_pxcopy(&inbuf[acq->tip_ptr],(unsigned char *)&tipbuf[1],
			&dtype,&nwords);
	    break;
	    }

	default:
	    break;
	}

    /*
      Copy the image data
    */
    if (acq->img_typ == GAC)
	{
	if (acq->rec_line > 0)
	    nwords = GAC_SIZ * acq->rec_line;
	else
	    nwords = GAC_SIZ;

	nsamps = GAC_SAMPS;

	switch (acq->img_siz)
	    {
	    case GAC_SIZ:
		{
		/*
		  Convert the input byte buffer to the image buffer
		*/
		if (acq->img_fmt == BIP)
		    c_pxconv(&ftype,&dtype,&inbuf[acq->img_ptr],
				(unsigned char *)imgbuf,&nwords);
		else if ((acq->img_fmt == BIL) || (acq->img_fmt == BSQ))
		    c_bil2bip((short *)&inbuf[acq->img_ptr],imgbuf,&nsamps,
				bands,&fill);
		break;
		}

	    case GAC_PCK:
		{
		/*
		  Unpack the input buffer into the image buffer
		*/
		if (acq->img_fmt == BIP)
		    c_unpck32(&inbuf[acq->img_ptr],imgbuf,&nwords,
			      &acq->swap_flg);
		else if ((acq->img_fmt == BIL) || (acq->img_fmt == BSQ))
		    {
		    c_unpck32(&inbuf[acq->img_ptr],tmpbuf,&nwords,
			      &acq->swap_flg);
		    c_bil2bip(&tmpbuf[acq->img_ptr],imgbuf,&nsamps,bands,&fill);
		    }
		break;
		}

	    case GAC_UNP:
		{
		/*
		  Byte swap the data if needed
		*/
#ifdef vms
		if (!acq->swap_flg)
		    c_pxswap(&inbuf[acq->img_ptr],&nwords,&dtype);
#else
		if (acq->swap_flg)
		    c_pxswap(&inbuf[acq->img_ptr],&nwords,&dtype);
#endif

		/*
		  Copy the buffer as is to the image buffer
		*/
		if (acq->img_fmt == BIP)
		    c_pxcopy(&inbuf[acq->img_ptr],(unsigned char *)imgbuf,
				&dtype,&nwords);
		else if ((acq->img_fmt == BIL) || (acq->img_fmt == BSQ))
		    c_bil2bip((short *)&inbuf[acq->img_ptr],imgbuf,&nsamps,
				bands,&fill);
		break;
		}

	    default:
		break;
	    }
	}
    else
	{
	if (acq->rec_line > 0)
	    nwords = LAC_SIZ * acq->rec_line;
	else
	    nwords = LAC_SIZ;

	nsamps = LAC_SAMPS;

	switch (acq->img_siz)
	    {
	    case LAC_SIZ:
		{
		/*
		  Convert the input byte buffer to the image buffer
		*/
		if (acq->img_fmt == BIP)
		    c_pxconv(&ftype,&dtype,&inbuf[acq->img_ptr],
				(unsigned char *)imgbuf,&nwords);
		else if ((acq->img_fmt == BIL) || (acq->img_fmt == BSQ))
		    c_bil2bip((short *)&inbuf[acq->img_ptr],imgbuf,&nsamps,
				bands,&fill);
		break;
		}

	    case LAC_PCK:
		{
		/*
		  Unpack the input buffer into the image buffer
		*/
		if (acq->img_fmt == BIP)
		    c_unpck32(&inbuf[acq->img_ptr],imgbuf,&nwords,
		              &acq->swap_flg);
		else if ((acq->img_fmt == BIL) || (acq->img_fmt == BSQ))
		    {
		    c_unpck32(&inbuf[acq->img_ptr],tmpbuf,&nwords,
			      &acq->swap_flg);
		    c_bil2bip(&tmpbuf[acq->img_ptr],imgbuf,&nsamps,bands,&fill);
		    }
		break;
		}

	    case LAC_UNP:
		{
		/*
		  Byte swap the data if needed
		*/
#ifdef vms
		if (!acq->swap_flg)
		    c_pxswap(&inbuf[acq->img_ptr],&nwords,&dtype);
#else
		if (acq->swap_flg)
		    c_pxswap(&inbuf[acq->img_ptr],&nwords,&dtype);
#endif

		/*
		  Copy the buffer as is to the image buffer
		*/
		if (acq->img_fmt == BIP)
		    c_pxcopy(&inbuf[acq->img_ptr],(unsigned char *)imgbuf,
				&dtype,&nwords);
		else if ((acq->img_fmt == BIL) || (acq->img_fmt == BSQ))
		    c_bil2bip((short *)&inbuf[acq->img_ptr],imgbuf,&nsamps,
				bands,&fill);
		break;
		}

	    default:
		break;
	    }
	}
    }

free (inbuf);
/*
  status is either E_SUCC or E_BOF from read_bands
*/
return(status);
}
/******************************************************************************
NAME:		READ_BANDS

PURPOSE:	Read a line of data from each band of the acquired
		image (whether the data be BIP, BIL, or BSQ).

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-------------------------------------------
  5.0	 T. Baltzer    02/92	Original Development (modified c_acqread)

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
******************************************************************************/
#ifndef SEEK_END    /* Not defined on PowerNode */
#define SEEK_SET  0
#define SEEK_CUR  1
#define SEEK_END  2
#endif


#define NUM_BANDS  5	/* assume 5 bands of data for BSQ format */

int read_bands
(
    struct ACQDESC *acq,        /* Pointer to the acqusition input structure */
    unsigned char inbuf[]       /* Input buffer for acquire                  */
)
{

static unsigned char rdbuf[MAX_ACQ * sizeof(short)];	/* read buffer	     */

long	bytes_per_imgline,	/* bytes of image data per line		     */
	dtype = EWORD,		/* type of data wanted in output	     */
	i,			/* looping variable			     */
	lines_per_band,		/* lines per band in file		     */
	nonfatal = 1,		/* non-fatal error code			     */
	status,			/* return status from the read		     */
	words_per_imgline,	/* words of image data per line		     */
	zero = 0;		/* constant				     */

long long filesize,		/* total number of bytes in the file	     */
          lstatus,		/* Long long return status		     */
          startpos;		/* starting position within the first band   */

/* 
  If the data is BSQ (Band Sequential) skipping around within the file
  will be required.  Once the lines have been properly read and loaded,
  the output buffer will look like a buffer of BIL data.
*/
if (acq->img_fmt == BSQ)
    {
    startpos = lseek(acq->fd,zero,SEEK_CUR);  /* determine current location */
    if (startpos == E_FAIL)
        {
        c_errmsg("Error seeking within image","acqread-seek",&nonfatal);
        return(E_FAIL);
        }

    filesize = lseek(acq->fd,zero,SEEK_END);	/* determine bytes in file  */
    if (filesize < 0)
        {
        c_errmsg("Error determining file size","acqread-fsiz",&nonfatal);
        return(E_FAIL);
        }

    lstatus = lseek(acq->fd,startpos,SEEK_SET);	/* return to proper location */
    if (lstatus == E_FAIL)
        {
        c_errmsg("Error seeking within image","acqread-seek",&nonfatal);
        return(E_FAIL);
        }

    lines_per_band = (long)(filesize / (acq->byte_line * NUM_BANDS));
    bytes_per_imgline = acq->byte_line - acq->img_ptr;

    status = get_line(acq,inbuf);
    if (status != E_SUCC)
        return(status);

    words_per_imgline = bytes_per_imgline / (long)sizeof(short);

    for (i = 0; i < NUM_BANDS - 1; i++)
        {
        lstatus = lseek(acq->fd,(lines_per_band - 1) * acq->byte_line,SEEK_CUR);
        if (lstatus == E_FAIL)
            {
            c_errmsg("Error seeking within image","acqread-seek",&nonfatal);
            return(E_FAIL);
            }

	status = get_line(acq,rdbuf);
        if (status != E_SUCC)
            return(status);

        c_pxcopy(&rdbuf[acq->img_ptr],&inbuf[acq->byte_line + 
		(bytes_per_imgline * i)],&dtype,&words_per_imgline);
        }

    /* return to first band */
    if (acq->flip_flg)
	{
	/* if going backwards, subtract one line from previous starting line*/
	startpos -= acq->byte_line;
 	if (startpos < 0)
            return(E_BOF);
	}
    else
	startpos += acq->byte_line;

    lstatus = lseek(acq->fd,startpos,SEEK_SET);
    if (lstatus == E_FAIL)
        {
        c_errmsg("Error seeking within image","acqread-seek",&nonfatal);
        return(E_FAIL);
        }
    }
else	/* BIL or BIP format */
    {
    /*
      Determine the current position when the data needs to be flipped
    */
    if (acq->flip_flg)
	{
	startpos = lseek(acq->fd,zero,SEEK_CUR);
	if (startpos == E_FAIL)
	    {
	    c_errmsg("Error seeking within image","acqread-seek",&nonfatal);
            return(E_FAIL);
            }
	}

    if (acq->rec_line < 0)
        {
        /*
          A full image line is contained in more than one acquire line.
          Read the acquire image until the full image line is in the buffer.
        */
        for (i = 0; i < abs(acq->rec_line); i++)
    	    {
	    /*
	      Read a line of the acquire file into the buffer
	    */
	    status = get_line(acq,&inbuf[acq->byte_line * i]);
	    if (status != E_SUCC)
	        return(status);
	    }
        }
    else
        {
        /*
          The full image line is contained within one acquire line.
          Read a line of the acquire file into the buffer.
        */
	status = get_line(acq,inbuf);
        if (status != E_SUCC)
            return(status);
        }

    /*
      When flipping the data the file descriptor must be set back a line
    */
    if (acq->flip_flg)
	{
	startpos -= (acq->byte_line * abs(acq->rec_line));
 	if (startpos < 0)
            return(E_BOF);

	lstatus = lseek(acq->fd,startpos,SEEK_SET);
	if (lstatus == E_FAIL)
	    {
	    c_errmsg("Error seeking within image","acqread-seek",&nonfatal);
	    return(E_FAIL);
	    }
	}
    }

return(E_SUCC);
}
/******************************************************************************
NAME:		GET_LINE

PURPOSE:	Read a line from the acquired image 

PROGRAM HISTORY:
VERSION	 AUTHOR	       DATE	REASON
-------	 ------	       -----	-----------------------------
  5.0	 T. Baltzer   06/92	Original Development
  5.1    D.Lloyd      09/99     Modified to properly handle flipping packed
                                image data.  Also fixed a bug where unpacked
                                BIP data was incorrectly flipped.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:  ADAPS

ALGORITHM REFERENCES:
******************************************************************************/

int get_line
(
    struct ACQDESC *acq,        /* Acquisition structure                     */
    unsigned char *acqbuf       /* Output buffer for acquire image data      */
)
{
long	i,j,k,			/* Loop counter				     */
	nonfatal = 1,		/* Non-fatal error code			     */
	shortend,		/* End of short buffer			     */
	imgptr,			/* Start of the image data in buffer         */
	nwords = LAC_WORDS,	/* number of words to unpack		     */
	status;			/* Return status from the read		     */

long	dtype = EWORD;		/* Type of data wanted in output	     */

char	msg[ERRLEN],		/* Error message string			     */
	*charbuf;		/* Temporary buffer before flipping	     */

short	*shortbuf,		/* Temporary buffer of shorts before flipping*/
	*shortacq;		/* Short version of acquire buffer	     */
short	*flipbuf;		/* flipped buffer of shorts		     */


if (!acq->flip_flg)
    {
    /* 
      No flipping is required; read a line of the acquire file 
      straight into the buffer 
    */
    status = read(acq->fd,acqbuf,acq->byte_line);

    if (status == 0)
        return(E_EOF);
    else if (status < 0)
        {
        c_errmsg("Error reading from the acquired file","acqread-read",
	        &nonfatal);
        return(E_FAIL);
        }
    else if (status != acq->byte_line)
        {
        sprintf(msg,"%s %d %s %d %s %s",
	       "The number of bytes read from the acquire file",status,
	       "doesn't match the expected number of bytes",acq->byte_line,
	       "for the specified station",acq->station);
        c_errmsg(msg,"acqread-nbyte",&nonfatal);
        return(E_SMAL);
        }
    }
else
    {
    /* 
      Flipping is required; read a line of the acquire file 
      into the temp buffer first
    */
    switch ( acq->img_siz )
	{
	case LAC_SIZ :	/* 8-bit LAC data	*/
	case GAC_SIZ :	/* 8-bit GAC DATA	*/
	    {
	    charbuf = (char *)malloc(acq->byte_line);

	    status = read(acq->fd,charbuf,acq->byte_line);
	    if (status == 0)
	        {
		free( charbuf );
		return(E_EOF);
		}
	    else if (status < 0)
		{
		free( charbuf );
		c_errmsg("Error reading from the acquired file","acqread-read",
			&nonfatal);
		return(E_FAIL);
		}
	    else if (status != acq->byte_line)
		{
		free( charbuf );
		sprintf(msg,"%s %d %s %d %s %s",
		       "The number of bytes read from the acquire file",status,
		       "doesn't match the expected number of bytes",
		       acq->byte_line, "for the specified station",
		       acq->station);
		c_errmsg(msg,"acqread-nbyte",&nonfatal);
		return(E_SMAL);
		}
	
	    /*
	      Assume minor frame and tip data aren't flipped
	      Straight copy these; flip image data only
	    */
	    for (i = 0; i < acq->img_ptr; i++)
		acqbuf[i] = charbuf[i];
	    if (acq->img_fmt == BSQ)
		{
		for (i = acq->img_ptr,j = 0; i < acq->byte_line; i++,j++)
		    acqbuf[i] = charbuf[acq->byte_line - j - 1];
		}
	    else
		{
		for (j = 0; j < NCHANS; j++)
		    {
		    for (i = 0; i < LAC_SAMPS; i++)
			{
			acqbuf[(j * LAC_SAMPS + i) + acq->img_ptr] = 
			    charbuf[acq->img_ptr + (j+1) * LAC_SAMPS - i - 1];
			}
		    }
		}

	    free( charbuf );
	    break;
	    }
	case LAC_UNP :	/* unpacked LAC data	*/
	case GAC_UNP :	/* unpacked GAC data	*/
	    {
	    shortbuf = (short *)malloc(acq->byte_line);
	    shortacq = (short *)acqbuf;

	    status = read(acq->fd,shortbuf,acq->byte_line);
	    if (status == 0)
		{
		free( shortbuf );
		return(E_EOF);
		}
	    else if (status < 0)
		{
		free( shortbuf );
		c_errmsg("Error reading from the acquired file","acqread-read",
			&nonfatal);
		return(E_FAIL);
		}
	    else if (status != acq->byte_line)
		{
		free( shortbuf );
		sprintf(msg,"%s %d %s %d %s %s",
		       "The number of bytes read from the acquire file",status,
		       "doesn't match the expected number of bytes",
		       acq->byte_line, "for the specified station",
		       acq->station);
		c_errmsg(msg,"acqread-nbyte",&nonfatal);
		return(E_SMAL);
		}
	
	    /*
	      Assume minor frame and tip data aren't flipped
	      Straight copy these; flip image data only
	    */
	    shortend = acq->byte_line / 2;
	    imgptr = acq->img_ptr / 2;
	    if (acq->img_fmt == BSQ)
		{
		for (i = 0; i < imgptr; i++)
		    shortacq[i] = shortbuf[i];

		for (i = imgptr,j = 0; i < shortend; i++,j++)
		    shortacq[i] = shortbuf[(shortend - 1) - j];
		}
	    else if (acq->img_fmt == BIL)
		{
		/*
		  Copy the full data line and then just flip the image data
		*/
		c_pxcopy((unsigned char *)shortbuf,(unsigned char *)shortacq,
			 &dtype,&shortend);

		for (j = 0; j < NCHANS; j++)
		    {
		    for (i = 0; i < LAC_SAMPS; i++)
			{
			shortacq[imgptr + j * LAC_SAMPS + i] = 
			      shortbuf[imgptr + ((j + 1) * LAC_SAMPS - 1) - i];
			}
		    }
		}
	    else	/* BIP data */
		{
		/* straight copy the minor frame */
		for (i = 0; i < imgptr; i++)
		    shortacq[i] = shortbuf[i];

		/* flip the image data */
		shortend = acq->img_siz / 2 + imgptr;
		for ( i = imgptr, j = shortend - NCHANS; i < shortend;
		      i += NCHANS, j -= NCHANS )
		    for ( k = 0; k < NCHANS; k++ )
			shortacq[i+k] = shortbuf[j+k];
		}

	    free( shortbuf );
	    break;
	    }
	case ARCH_GAC :		/* packed GAC data	*/
	    nwords = GAC_WORDS; /* reset nwords for GAC */
	case ARCH_LAC :		/* packed LAC data	*/
	    /* For this type data we have to unpack the image data before
	    flipping the pixels because three 10-bit values are stored in
	    one 32-bit value.  There is no 'easy' way to step through a
	    buffer of packed data.  Also since the calling routine still
	    expects the data to be packed we must pack the image data
	    once flipped -- messy but necessary.
	    -------------------------------------------------------------*/

	    /* Allocate the appropriate buffers
	    -----------------------------------*/
	    charbuf = (char *)malloc( acq->byte_line );
	    if ( !charbuf )
	        {
		c_lasmsg(FALSE,"acqread-alloc","Error allocating memory");
		return(E_FAIL);
		}
	    shortbuf = (short *)malloc( sizeof(short) * acq->byte_line );
	    if ( !shortbuf )
	        {
		free( charbuf );
		c_lasmsg(FALSE,"acqread-alloc","Error allocating memory");
		return(E_FAIL);
		}
	    flipbuf = (short *)malloc( sizeof(short) * acq->byte_line );
	    if ( !flipbuf )
	        {
		free( charbuf );
		free( shortbuf );
		c_lasmsg(FALSE,"acqread-alloc","Error allocating memory");
		return(E_FAIL);
		}

	    /* Read the line of data from the input file */
	    status = read(acq->fd,charbuf,acq->byte_line);
	    if (status == 0)
		{
		free( charbuf );
		free( shortbuf );
		free( flipbuf );
		return(E_EOF);
		}
	    else if (status < 0)
		{
		free( charbuf );
		free( shortbuf );
		free( flipbuf );
		c_errmsg("Error reading from the acquired file","acqread-read",
			&nonfatal);
		return(E_FAIL);
		}
	    else if (status != acq->byte_line)
		{
		free( charbuf );
		free( shortbuf );
		free( flipbuf );
		sprintf(msg,"%s %d %s %d %s %s",
		       "The number of bytes read from the acquire file",status,
		       "doesn't match the expected number of bytes",
		       acq->byte_line, "for the specified station",
		       acq->station);
		c_errmsg(msg,"acqread-nbyte",&nonfatal);
		return(E_SMAL);
		}

	    /* Assume the minor frame is not flipped
	    and copy directly to the output buffer.
	    ----------------------------------------*/
	    for ( i = 0; i < acq->img_ptr; i++ )
	        acqbuf[i] = charbuf[i];

	    /* Now flip the data by first unpacking, flipping
	    and repacking to the output buffer.
	    --------------------------------------------------*/
	    if ( acq->img_fmt == BIP )
		{
		c_unpck32( (unsigned char *)&charbuf[acq->img_ptr], shortbuf, &nwords,
		    &acq->swap_flg);
		for ( i = 0, j = nwords - NCHANS; i < nwords;
		      i += NCHANS, j -= NCHANS )
		    for ( k = 0; k < NCHANS; k++ )
			flipbuf[i+k] = shortbuf[j+k];
		c_pck32( flipbuf, &acqbuf[acq->img_ptr], &nwords );
		}
	    else
	        {
		free( charbuf );
		free( shortbuf );
		free( flipbuf );
		c_lasmsg(FALSE,"acqread-fmt","Can't flip packed BSQ or BIL");
		return(E_FAIL);
		}

	    free( charbuf );
	    free( shortbuf );
	    free( flipbuf );
	    break;

	default :
	    c_errmsg("Invalid image data size","acqread-size",&nonfatal);
	    return(E_FAIL);
	}
    }

return(E_SUCC);
}
