/*******************************************************************************
FUNCTION:	c_arcopnwrt

PURPOSE:	Open an AVHRR archive image for writing.  The specified image
		must not exist.  A status of E_SUCC will be returned upon
		successful completion, E_FAIL otherwise.


Version  Date   Programmer       Modifications
-------  -----  ---------------  -----------------------------------------------
  1.0    04/93  J.Fenno          Development
  1.1    05/93  J.Fenno          Added code for a temporary VMS buffer to avoid
				 byte swapping the packed buffer.
  1.2    06/93  J.Fenno          Modified to allow seeking with images opened
				 for writing.
  1.3    07/96  T.Baltzer        Modified to call c_dkcre() so that it will
				 follow DATAPATH.

PROJECT:	ADAPS
*******************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/file.h>
#include <stdio.h>

#include "las.h"
#include "labserv.h"
#include "tapeio.h"
#include "diskio.h"
#include "adaps.h"
#include "archio.h"
#include "local_protos.h"


int c_arcopnwrt
(
    char *image,            /* IN  - Host file name or scene ID of image      */
    long *dtype,            /* IN  - Data type of the image (GAC,HRPT,LAC)    */
    long *nlines,           /* IN  - Number of lines in the image	      */
    long *arcdesc           /* OUT - Archive descriptor record                */
)
{
long exist    = 0;          /* File test (0=exist 1=execute 2=write 3=read)   */
long flags;                 /* Access flags for calling open()                */
long nonfatal = 1;          /* Nonfatal error flag                            */

long long imagesize;	    /* Size of image				      */

struct ARCDESC *arc;        /* Archive descriptor record                      */


/* Ensure the archive image does not exist
------------------------------------------*/
*arcdesc = NULL;
if (c_lsstat(image, &exist) != E_FAIL)
    {
    c_errmsg("Archive image already exists", "arcopnwrt-exist", &nonfatal);
    return(E_FAIL);
    }


/* Allocate a new archive descriptor record and initialize it
-------------------------------------------------------------*/
arc = (struct ARCDESC *)malloc(sizeof(struct ARCDESC));
if (arc == NULL)
    {
    c_errmsg("Error allocating memory", "arcopnwrt-alloc", &nonfatal);
    return(E_FAIL);
    }
arc->fdesc     = NULL;
arc->access    = IWRITE;
arc->current   = 0;
arc->first     = 1;
arc->last      = 1;
arc->tpunit    = NULL;
arc->words_mnr = MNR_WORDS;
arc->window[SL] = 1;      /* Keep track of the image size */
arc->window[SS] = 1;
arc->window[NL] = 0;
arc->ascending = FALSE;   /* All images are written like a descending pass */
arc->disk      = TRUE;
arc->mnr       = NULL;
arc->samples   = NULL;
arc->satview   = TRUE;    /* All images are processed in satellite view */
arc->savebuf   = FALSE;
arc->saveline  = FALSE;
arc->packed    = NULL;
arc->vmsbuf    = NULL;
if (*dtype == GAC)
    {
    arc->window[NS] = GAC_SAMPS;
    arc->arcbytes   = GAC_BYTES;
    arc->words_img  = GAC_WORDS;
    }
else
    {
    arc->window[NS] = LAC_SAMPS;
    arc->arcbytes   = LAC_BYTES;
    arc->words_img  = LAC_WORDS;
    }


/* Allocate the buffer needed for writing an archive image
----------------------------------------------------------*/
arc->packed = (unsigned char *)malloc(ARCBUFLINES * arc->arcbytes);
if (arc->packed == NULL)
    {
    c_errmsg("Error allocating memory", "arcopnwrt-alloc", &nonfatal);
    free_desc(arc);
    return(E_FAIL);
    }


/* Open the archive image for write access.  Call c_dkcre() so that
   it follows DATAPATH.
---------------------------------------------------------------------*/
imagesize = (long long) *nlines * arc->arcbytes;
if (c_dkcre(NULL,image,imagesize,600) != E_SUCC)
    {
    c_errmsg("Error opening archive image", "arcopnwrt-open", &nonfatal);
    free_desc(arc);
    return(E_FAIL);
    }


flags = O_WRONLY | O_CREAT;
#ifdef vms
if (*dtype == GAC)
    arc->fdesc = open(image, flags, 0, "rfm=fix", "mrs=2868",
	"bls=2868", "mbc=6", "mbf=16", "deq=60");
else
    arc->fdesc = open(image, flags, 0, "rfm=fix", "mrs=13796",
	"bls=13796", "mbc=27", "mbf=16", "deq=270");
#else
arc->fdesc = open(image, flags, 0664);
#endif
if (arc->fdesc < 0)
    {
    c_errmsg("Error opening archive image", "arcopnwrt-open", &nonfatal);
    free_desc(arc);
    return(E_FAIL);
    }


/* Send back the address of the archive descriptor
   record, hiding the structure from the user.
--------------------------------------------------*/
*arcdesc = (long)arc;
return(E_SUCC);
}

