/******************************************************************************
FUNCTION:	GET_NEXT_SCENE

PURPOSE:	Read the master schedule and determine  the next scheduled
		EROS scene to be acquired.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0      03/01    T. Baltzer		Initial development

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <sys/types.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "acquman.h"


#define MAXRECS	600 /* Master schedule size and maximum number of records */

/* Format needed to read the master schedule.  */
#define SCHEDULE_FORMAT	"%s %s %d %d %d %d %d %s %02d %s %s %s %s %s\n"

/* Structure of the master record */
struct	SCHED
	{
	char	scene_id[17];
	char	satid[4];
	long	lstart;
	long	lstop;
	long	rstart;
	long	rstop;
	long	opmode;
	char	stat[2];
	long	tfile;
	char	pflag[2];
	char	dnflag[2];
	char	station[2];
	char	atype[5];
	char	orbit[6];
	};


extern char sceneid[17];	/* 16 character sceneid		*/
extern char dnflag[2];		/* Day / Night flag (D/N)	*/
extern long gstart_time;     /* GMT start time, seconds since 1/1/86 00:00:00 */
extern long gstop_time;	     /* GMT end time, seconds since 1/1/86 00:00:00   */


void get_next_scene
    (
    char ltime_disp[]	/* OUT:Local time in display format	HH:MM AM/PM  */
    )

{

long	clock;			/* Total seconds from 00:00:00, Jan 1, 1971 */
long	datearry[3];		/* date (month, day, year)		*/
long	dtime_secs;		/* Difference (GMT - local) time	*/
long	gtime_secs;		/* GMT time total seconds since		*/
				/* 00:00:00 on 1/1/86 			*/
long	ltime_secs;		/* local time total seconds since	*/
				/* 00:00:00 on 1/1/86 			*/
long	nrecs;			/* Number of records in the master sched*/
long	rec;			/* Record counter			*/
long	timearry[3];		/* time (hour, min, sec)		*/
long	temp, temp_hr,		/* Temporary variables used to determine*/
	temp_min; 		/* the display time			*/

FILE	*mfp;	 		/* File pointer for master schedule file*/

char	ampm[3];		/* AM / PM variable			*/
char	schedfile[CMLEN];	/* name of schedule file		*/

struct SCHED	schrec;	/* structure holding master schedule record	*/
struct tm	*gtime;		/* GMT time structure			*/
struct tm	*ltime;		/* Local time structure			*/


/*
  Null output parameters
------------------------------*/
strcpy(sceneid,"");
gstart_time = 0;
gstop_time = 0;
strcpy(dnflag,"");
strcpy(ltime_disp,"");


/*
  Get the local / GMT time structures
  Translate these times into total seconds from 00:00:00 on 1/1/86.
----------------------------------------------------------------------*/
clock = time(0);
ltime = localtime(&clock);

timearry[0] = ltime->tm_hour;
timearry[1] = ltime->tm_min;
timearry[2] = ltime->tm_sec;
datearry[0] = ltime->tm_mon + 1;   /* make the month 1 relative */
datearry[1] = ltime->tm_mday;
datearry[2] = ltime->tm_year;
makesecs(datearry,timearry,&ltime_secs);  /* local time, total seconds */

gtime = gmtime(&clock);

timearry[0] = gtime->tm_hour;
timearry[1] = gtime->tm_min;
timearry[2] = gtime->tm_sec;
datearry[0] = gtime->tm_mon + 1;   /* make the month 1 relative */
datearry[1] = gtime->tm_mday;
datearry[2] = gtime->tm_year;

makesecs(datearry,timearry,&gtime_secs);  /* GMT time, total seconds */

/*
  Compute the difference between local time and GMT 
----------------------------------------------------------*/
dtime_secs =  gtime_secs - ltime_secs;

/*
  Determine the master schedule file name
----------------------------------------------*/
if (getenv("ADAPSTABLES") != NULL)
    strcpy(schedfile,getenv("ADAPSTABLES"));
else
    {
    write_error( FALSE, "get_next_scene", "tables",
		"Error retrieving value of ADAPSTABLES" );
    return;
    }
strcat(schedfile,"/master.sch");

/*
  Try to open the schedule file
-------------------------------------*/
mfp = fopen(schedfile,"r+");
if (mfp == NULL)
    {
    write_error( FALSE, "get_next_scene", "schedule",
		"Cannot open the master schedule file" );
    return;
    }

/*
  Get the number of entries
-----------------------------------*/
fscanf(mfp,"%d\n",&nrecs);

/*
  Get each record from the master schedule
----------------------------------------------*/
for (rec = 0; rec < nrecs; rec++)
    {
    /*
      Read a record into the structure
    -----------------------------------*/
    fscanf(mfp,SCHEDULE_FORMAT,schrec.scene_id, schrec.satid,
	   &schrec.lstart, &schrec.lstop, &schrec.rstart, &schrec.rstop,
	   &schrec.opmode, schrec.stat,&schrec.tfile, schrec.pflag,
	   schrec.dnflag, schrec.station,schrec.atype,schrec.orbit);

    /* If the GMT stop time from the schedule is greater than the current
       time, we have determined the next acquireable image.  Only consider
       "E" (EROS) scenes.  Load the return parameters.
    ---------------------------------------------------------------------*/
    if ((schrec.station[0] == 'E') && (schrec.lstop >= gtime_secs))
	{
	strcpy(sceneid,schrec.scene_id);
	if (sceneid[0] == 'A')  sceneid[0] = 'a';
	if (sceneid[1] == 'H')  sceneid[1] = 'h';
	gstart_time = schrec.lstart;
	gstop_time = schrec.lstop;
	strcpy(dnflag,schrec.dnflag);

	/* Determine the local time to be displayed by SHOACQU 
        ---------------------------------------------------------*/
	temp = (schrec.lstart - dtime_secs) % 86400;
	temp_hr = temp / 3600;
	temp -= (3600 * temp_hr);
	temp_min = temp / 60;
	if (temp_hr > 11)
	    {
	    strcpy(ampm,"PM");
	    if (temp_hr > 12)
	        temp_hr -= 12;
	    }
	else
	    strcpy(ampm,"AM");
	sprintf(ltime_disp, "%02d:%02d %s", temp_hr, temp_min, ampm);
	break;
	}

    }

}

