#!/bin/csh
#-------------------------------------------------------------------
# The purpose of this shell script is to download NCEP data from the
# NOAA ftp site and, once downloaded, extract the precipitable water
# record and move the resulting transfer file to $ADAPSACQ/ncepin
#
# Usage:  nceptrans.csh HH YYMMDD
#
# where HH is either 00 06 12 or 18, which represents the forecast
# or observation time and YY is year, MM is month and DD is the day.
#-------------------------------------------------------------------

#-----------------------
# Assign constant values
#-----------------------
set FTPSYS = "ftp.ncep.noaa.gov"
set FTPPSWD = $user@edcmail.cr.usgs.gov

#------------------------------------------------------------
# Check for a parameter and exit if incorrect number supplied
#------------------------------------------------------------
if ($#argv != 2) then
    echo "Usage nceptrans.csh HH YYMMDD"
    echo "HH is forecast time: 00 06 12 18"
    echo "YYMMDD year month and day of forecast"
    exit
endif

#---------------------------------
# Set up directories and change to
# working directory.
#---------------------------------
set datetime = `date +%b%d%H%M%S`
set workdir = $ADAPSACQ/ncepin
if ( ! -e $workdir ) then
    mkdir $workdir
endif
cd $workdir

#----------------------------------------
# Set up the production LAS environment,
# if is hasn't already been setup.
#----------------------------------------
if ((! $?EDCSOFT) && (-r /etc/login)) then
    source /etc/login
    eval `$masterlog response comfile las72`
    unset masterlog
    unset response
    unset comfile
endif

#-----------------------------------------------------------
# Set the logfile
# The logfile needs to be in LAS syntax for wrtlog so noglob
# must be set to prevent the []'s from being interpreted
# as a pattern matching construct.
#-----------------------------------------------------------
set noglob
set logfile = `echo "[$ADAPSLOG]ncepin" | tr / . | sed  's/\[\./\[/'`
alias wrtlog $ADAPSBASE/write_log

#---------------------------
# Set up the local variables
#---------------------------
set directory = fnl.$2
set file = gdas1.T$1Z.PGrbF00
set tranfile = fnl_gdas1_pwat$1_$2.trn


#------------------------
# Print a startup message
#------------------------
set key = "daily_ncep_file"
set text = "Starting the NCEPTRANS procedure"
wrtlog $logfile $key nceptrans-start $text


#------------------------------------------------
# Create a temporary script to download the file.
# The temporary script is needed to expand the
# variables.
#------------------------------------------------
echo "(ftp -n $FTPSYS) >& stderror << ENDOFSTDIN" > ftp1file.csh
echo user anonymous $FTPPSWD >> ftp1file.csh
echo cd /pub/data1/fnl/$directory >> ftp1file.csh
echo prompt >> ftp1file.csh
echo bin >> ftp1file.csh
echo get $file >> ftp1file.csh
echo bye >> ftp1file.csh
echo ENDOFSTDIN >> ftp1file.csh
# Run the script
#---------------
csh ftp1file.csh

#--------------------------------------
# Extract the precipitable water record
# and create a transfer file.
#--------------------------------------
if ( ! -e $file ) then
    set text = "No file downloaded"
    wrtlog $logfile $key nceptrans-error $text
else
    set text = "Downloaded $file"
    wrtlog $logfile $key nceptrans-dwnload $text
    set recnum=`$ADAPSCONTRIB/wgrib $file -v | grep PWAT | awk -F: '{print $1}'`
    $ADAPSCONTRIB/wgrib $file -d $recnum -ieee -o $tranfile

    #---------------------------------------------------------------
    # Move the large tranfile to a temporary file.  These temporary
    # files can be used to extract the PWAT records in case of error
    # They are overwritten the next time the script runs with the 
    # same HH parameter as the time when created.  There will be no
    # more than four temporary files.
    #---------------------------------------------------------------
    mv $file prev_$file
endif

#--------------------------------------
# Print a successful completion message
#--------------------------------------
set text = "NCEPTRANS completed successfully"
wrtlog $logfile $key nceptrans-success $text

exit 0
