/*****************************************************************************
FUNCTIONS:	main
		sig_handler

PROJECT:	ADAPS, AVHRR Acquisition System

$Log: main.c,v $
Revision 1.1  2001/04/04 16:09:00  dlloyd
Initial checkin of shoacqu

****************************************************************************/
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include "shoacqu.h"


/* Global variables
-------------------*/
int		imagedisplay;	/* flag indicating we are displaying	*/
Widget		toplevel;	/* main top level widget		*/
Display		*display;	/* pointer to main display		*/


/* Variables local to this file
-------------------------------*/
static int	    sock_fd;	/* socket descriptor			*/


/*******************************************************************************
FUNCTION:	main

PURPOSE:	Shoacqu is a moving window display for the AVHRR Acquisition
		system.  It's purpose is to read and display sub-sampled
		pixels from a file created by acquire.  It is designed to 
		display as the input file is being created.  The display is
		meant to be North up which is determined by the day/night
		flag.

USAGE:		shoacqu [-f filename] [-dn [D|N]] -next next_start_time
		        [-port port_number]

COMMAND LINE
INPUT:		filename specifies the file to display.  This file must be
		    in the same format as that created by acquire.

		D or N specifies either a Day (ascending) file or a Night
		    (descending) file.
		
		next_start_time is the hour, minute and am/pm of the next
		    start time.  The format is 'HH:MM Xm'.  The time will
		    be shown on the display.

		port_number specifies the Unix socket port number of the
		    calling process.  If communication with the parent is
		    desired a port number must be specified.

SOCKET
INPUT:		<filename> <d|D|n|N> [R G B]
		    Where filename is the input file to read from
		    D or N specifies either a Day (ascending) file or a
		        Night (descending) file
		    RGB values are the optional AVHRR channels (1 to 5)
		    	to use for red blue and green.
		<next_start_time>
		    Where next_start_time is the hour, minute and am/pm
		        of the next start time.  The format is 'HH:MM Xm'. 
			The time will be shown on the display.

OUTPUT:		Graphical display of the input image
		Text showing the next acquisition time

RETURNS:	none
*******************************************************************************/
int main
    (
    int argc,			/* IN - count of arguments passed in	*/
    char *argv[]		/* IN - argument list			*/
    )
    {
    char	infile[STRLEN] = {0};/* input file name			*/
    char	next_text[STRLEN] = {0};/* the next scheduled time text	*/
    char	*thebuf = NULL;	/* image buffer				*/

    GC		gc;		/* the graphics context			*/

    int		daynightflag;	/* flag: DAY or NIGHT			*/
    int		port = 0;	/* parent's socket port			*/
    int		rgb_bands[3];	/* AVHRR bands to use for RGB display	*/
    int		status;		/* function return status		*/

    Pixmap	pixmap;		/* pointer to the pixmap		*/

    Widget	drawing_area;	/* widget to hold the image		*/
    Widget	mainform;	/* main window form			*/

    XEvent      event;
    XGCValues	gcv;		/* graphics context values		*/
    XImage	*theimage;	/* The image that is displayed		*/
    XtAppContext context;	/* the main application context		*/
    XVisualInfo	*vinfo;		/* visual information			*/


    /* Startup message
    ------------------*/
    write_log( FALSE, "main", "msg", "Starting shoacqu" );

    /* Set up the user's locale and get a top level shell
    -----------------------------------------------------*/
    XtSetLanguageProc( NULL, NULL, NULL );
    toplevel = XtVaAppInitialize( &context, WnSHOACQU, NULL, 0,
    	    &argc, argv, NULL, /* or fallbacks if needed */
	    XmNwidth,    WINWIDTH+SCROLLBARWIDTH,
	    XmNmaxWidth, WINWIDTH,
	    XmNminWidth, WINWIDTH,
	    XmNheight,   WINHEIGHT,
	    XmNmaxHeight, WINHEIGHT,
	    XmNminHeight, WINHEIGHT,
	    NULL );
    display = XtDisplay( toplevel );

    /* Now that XtVaAppInitialize has removed its arguments
    from argv, parse the remaining arguments.
    -------------------------------------------------------*/
    status = parse_args( argc, argv, infile, next_text, &daynightflag, &port );
    if ( status == FAIL )
        exit(1);

    /* If the port number was specified,
    establish communication with parent.
    ------------------------------------*/
    sock_fd = 0;
    if ( port > 0 )
        if (( sock_fd = open_socket( port )) == FAIL )
	    exit(1);

    /* Create the main display window
    ---------------------------------*/
    mainform = create_main_display( toplevel, next_text );

    /* Get the visual information
    -----------------------------*/
    vinfo = get_visual( display );
    if ( vinfo == NULL )
        exit(1);

    /* Create the image for display.  The actual display is done
    with the pixmap as the background of the drawing_area.  Doing
    this frees shoacqu from doing its own refresh of the window
    as a result of exposure events.  
    ------------------------------------------------------------*/
    drawing_area = getdrawingarea();
    pixmap = XCreatePixmap( display, 
        RootWindowOfScreen( XtScreen( drawing_area )),
	IMGWIDTH, IMGHEIGHT, vinfo->depth );
    XtVaSetValues( drawing_area, XmNbackgroundPixmap, pixmap, NULL );

    /* Create an image buffer.  The image buffer is used in display()
    and strat_display() to build and then write one line to the pixmap.
    ------------------------------------------------------------------*/
    thebuf = (char *)malloc( vinfo->depth * IMGWIDTH );
    if ( !thebuf )
        {
	write_log( TRUE, "main", "alloc", "Unable to allocate memory" );
	exit(1);
	}
    theimage = XCreateImage( display,
        DefaultVisual( display, DefaultScreen( display )),
	vinfo->depth, ZPixmap, 0, thebuf, IMGWIDTH, 1, 8, 0 );

    /* Create the graphics context
    ------------------------------*/
    gcv.foreground = BlackPixelOfScreen( XtScreen( drawing_area ));
    gc = XCreateGC( XtDisplay( drawing_area ),
        RootWindowOfScreen( XtScreen( drawing_area )), GCBackground, &gcv );

    /* If the user specified an input file,
    display it immediately.
    --------------------------------------*/
    if ( infile[0] != '\0' )
        {
	imagedisplay = TRUE;
	start_display( gc, theimage, pixmap, rgb_bands, sock_fd );
	}

    /* Realize the widgets
    ----------------------*/
    XtRealizeWidget( toplevel );

    /* Enter an event loop.  We aren't using XtAppLoop
    because we need to be able to check the socket for
    communication from the parent.
    --------------------------------------------------*/
    imagedisplay = FALSE;
    while( TRUE )
	{
	/* Dispatch any pending events
	------------------------------*/
	if ( XPending( display ) > 0 )
	    {
	    XNextEvent( display, &event );
	    XtDispatchEvent( &event );
	    }

	/* Check the socket and, if the display
	flag is TRUE, start up the display.
	---------------------------------------*/
	socket_select();
	if ( imagedisplay )
	    start_display( gc, theimage, pixmap, rgb_bands, sock_fd );
	}

    return 0;  /* only here to prevent compiler warning */
    }  /* END main */


/*******************************************************************************
FUNCTION:	socket_select

PURPOSE:	Determine if the socket is readable (message pending) and
		change the imagedisplay variable if there is.

INPUT:		none

OUTPUT:		modifies global variable imagedisplay

RETURNS:	none
*******************************************************************************/
void socket_select
    (
    void
    )
    {
    fd_set readset;		/* file descriptor set to select on	*/
    struct timeval delay;	/* amount of time to delay before return*/
    int    maxfdp1;		/* maximum file descriptor plus one	*/

    /* Set delay to zero for poll
    -----------------------------*/
    delay.tv_sec = delay.tv_usec = 0;

    /* Set the read set to the socket
    ---------------------------------*/
    FD_ZERO( &readset );
    FD_SET( sock_fd, &readset );

    maxfdp1 = sock_fd + 1;

    /* If the socket is readable toggle the display flag
    ----------------------------------------------------*/
    select( maxfdp1, &readset, NULL, NULL, &delay );
    if ( FD_ISSET( sock_fd, &readset ))
	{
	if ( imagedisplay )
	    imagedisplay = FALSE;
	else
	    imagedisplay = TRUE;
	}
    }  /* END socket_select */
