/*********************************************************************
NAME: ARC2LAS

PURPOSE: To take the four files generated by ImageGrid in ARC with
         the bil option and to generate a LAS .DDR and .HIS
         file.  If the input grid coverage is 1-bit, then a LAS
         .IMG file will be generated also.
 
PROGRAM HISTORY:

Version  Date   Programmer       Code  Reason
-------  -----  --------------   ----  ------ 
  1.0    08/92  K. Larson         C    Initial development
  1.1    12/92  K. Larson         C    Set pixel size value
  1.2    01/93  K. Larson         C    

   Fix error with pixel size value and allow invalid information to
   be in the *.stx file and continue processing.  The invalid flag
   value will be set for each band that has invalid information in
   the *.stx file

   2.0   02/94  K. Lowell         C    Conversion to LAS6.0
                                       standards
   2.1   09/94  K. Lowell         C    Upgraded to ARC7.0.  Incorporated
                                       changes made by R.White/PSU 
   2.2   9/96   D.Etrheim	       Raised buffers in bit2byte MAXNS

************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "las.h"
#include "tae.h"
#include "util.h"
#include "labserv.h"
#include "vll.h"
#include "arc2las.h"

#define STXLEN  448     /* max length of readin buffer for stx file */

struct PARBLK vblk;     /*TAE parameter block     */

char imagename[CMLEN];      /* Name of the output image. */
char ddrname[CMLEN];        /* Name of the output image .ddr. */
char hisname[CMLEN];        /* Name of the output image .his. */
char gridstx[CMLEN];        /* Name of the .stx file. */
char gridbilw[CMLEN];       /* Name of the .blw file. */
char gridbilw2[CMLEN];      /* Name of the other .blw file. */
char gridhdr[CMLEN];        /* Name of the .hdr file. */
char gridbil[CMLEN];        /* Name of the .bil file. */
char gridprj[CMLEN];        /* Name of the .prj file */
char hosout[CMLEN];         /* Output root name      */
char workspace[CMLEN];
char ingrid[CMLEN];         /* Name of the input ARC grid */

FILE *hisout;             /* Output image .his file pointer. */
FILE *hdrin;              /* .hdr file pointer. */
FILE *bilin;              /* .bil file pointer. */
FILE *stxin;              /* .stx file pointer. */
FILE *bilwin;             /* .blw file pointer. */

void main
(
  void
)
{
char readin[STXLEN];       /* String used for reading in values.         */
char hostin[1][CMLEN];     /* Array for hstout                           */
char text[CMLEN];          /* Text string for hstout                     */
char proc_num[CMLEN];      /* Current number being extraced from 
                              input string.                              */
char *error_ptr;           /* Points to an invalid character when
                              converting a string to a number.           */
char byteorder[5];         /* Contains the byte order of the input bil.  */
char layout[5];            /* Indicates the image format of the Arc
                              output, Assumed to be BFORM, because this
                              will be the name extension that will be
                              looking for.                               */
char systrg[CMLEN];        /* System type                                */
char progname[CMLEN];      /* Program name                               */

long length;
long outcount = 1;         /* Output file count                          */
long index;                /* Used for indexing readin.                  */
long loop,loop2;           /* Loop counters                              */
long status;               /* Status indicator                           */
long key = DDSYS;          /* DDR field key                              */
long lvalue;               /* Long value retrieved from hdr              */
long tmp_long;             /* dummy long to obtain lng value             */
long std_out;
long ep3flag=0;            /* 8.3 naming convention flag                 */
long zero = 0;             /* Constant of zero                           */
long ii;                   /* Counter                                    */

long nbits;                /* # of bits in # from input .bil.            */
long bandbytes;            /* # of bytes per line of input .bil.         */
long totalbytes;           /* # of bytes per band of input .bil.         */
long msgtim;

long validflag;

double curmin;
double curmax;
double minmin;            /* smallest minimum of all bands               */
double maxmax;            /* largest maximum of all bands                */

double b_rotation = 0;    /* Rotation of image to world transformation.  */
double d_rotation = 0;    /* Rotation of image to world transformation.  */
double xdim;              /* Size in the x dimension of the cell.        */
double ydim;              /* Size in the y dimension of the cell.        */
double ulxmap;            /* Upper left cell, center coordinate for x.   */
double ulymap;            /* Upper left cell, center coordinate for y.   */

/* STRUCT THAT CONTAINS ALL THE PARAMETERS IN THE FIRST 2 RECORDS OF THE DDR */
struct DDR  ddrstuff;

/* STRUCT THAT CONTAINS THE PARAMETERS FOR THIRD OR MORE RECORDS IN THE DDR */
struct BDDR bddrstuff;

c_inlas (&vblk, &std_out);

/* GET THE VARIABLES FROM THE TAE VARABLE BLOCK
-------------------------------------------------*/
getpar(&vblk,hosout,&msgtim,&ep3flag,workspace,ingrid);

length = (long)strlen(hosout);
if (strstr(hosout,".img") == NULL)
    strcpy(imagename,hosout);
else
    {
    strncpy(imagename,hosout,(length-4));
    imagename[(length-3)]=0;
    strcpy(hosout,imagename);
    }

c_lsmknm(imagename,"ddr",ddrname);
c_lsmknm(imagename,"his",hisname);
c_lsmknm(imagename,"img",imagename);

if (ep3flag == 1)
    {
    c_lsmknm(hosout,"hdr",gridhdr);
    c_lsmknm(hosout,"stx",gridstx);
    c_lsmknm(hosout,"blw",gridbilw);
    c_lsmknm(hosout,"bilw",gridbilw2);
    c_lsmknm(hosout,"prj",gridprj);
    c_lsmknm(hosout,"bil",gridbil);
    }
else
    {
    c_lsmknm(hosout,"hdr",gridhdr);
    c_lsmknm(hosout,"stx",gridstx);
    c_lsmknm(hosout,"bilw",gridbilw);
    c_lsmknm(hosout,"blw",gridbilw2);
    c_lsmknm(hosout,"proj",gridprj);
    c_lsmknm(hosout,"bil",gridbil);
    }


/* CHECKING IF FILES ALREADY EXIST
-----------------------------------------*/
/* Checking if .ddr already exists. */
if ((c_lsstat(ddrname,&zero)) == E_SUCC)
    c_lasmsg(TRUE, "arc2las-exist", "DDR file already exists");

/* Checking if .his already exists. */
if ((c_lsstat(hisname,&zero)) == E_SUCC)
    {
    fclose(hisout);
    c_lasmsg(TRUE, "arc2las-exist", "HIS file already exists");
    }


/* INITIALIZING DDR STRUCTURE.  SET INVALID FLAGS
------------------------------------------------*/
c_intddr(&ddrstuff);

for (ii = 0; ii < 8; ii++)
    ddrstuff.valid[ii] = INVAL;




/* OPENING THE .HDR */

if ((hdrin = fopen(gridhdr,"r")) == NULL)
    c_lasmsg(TRUE, "arc2las-open", "Error opening .hdr");

/* READING IN ALL OF THE .HDR INFORMATION */
if ((fscanf(hdrin," BYTEORDER %c LAYOUT %s NROWS %d NCOLS %d",
    byteorder, layout, &ddrstuff.nl, &ddrstuff.ns) < 4)
    || (fscanf(hdrin," NBANDS %d NBITS %d BANDROWBYTES %d TOTALROWBYTES %d",
    &ddrstuff.nbands, &nbits, &bandbytes, &totalbytes) <4))
    {
    fclose(hdrin);
    c_lasmsg(TRUE, "arc2las-read", "Error reading .hdr file");
    }
lvalue = ddrstuff.nl;
if (E_SUCC != lng2tae(&lvalue,1,1,"NL",&tmp_long))
    c_lasmsg(TRUE, "arc2las-qout", "Error passing variables to TAE");
lvalue = ddrstuff.ns;
if (E_SUCC != lng2tae(&lvalue,1,1,"NS",&tmp_long))
    c_lasmsg(TRUE, "arc2las-qout", "Error passing variables to TAE");
lvalue = ddrstuff.nbands;
if (E_SUCC != lng2tae(&lvalue,1,1,"NBANDS",&tmp_long))
    c_lasmsg(TRUE, "arc2las-qout", "Error passing variables to TAE");

lvalue = nbits;
if (E_SUCC != lng2tae(&lvalue,1,1,"NBITS",&tmp_long))
    c_lasmsg(TRUE, "arc2las-qout", "Error passing variables to TAE");

fclose(hdrin);


/*  RETRIEVING THE MIN AND MAX FROM STX
-------------------------------------*/
if ((stxin = fopen(gridstx,"r")) == NULL)
    c_lasmsg(TRUE, "arc2las-open", "Error opening .stx");
for (loop = 1; loop <= ddrstuff.nbands; loop++)
    {
    /* SETTING THE READIN'S INDEXING VARIABLE TO 0 */
    index = 0;
    
    /* READING IN THE BAND INFORMATION */
    if (fgets(readin,STXLEN,stxin) == NULL)
        {
        fclose(stxin);
        c_lasmsg(TRUE, "arc2las-read", "Error reading .stx file");
        }
    
    length = (long)strlen(readin);
    error_ptr = NULL;
    
    /* ASSUMING THE READIN PARAMETERS ARE VALID */
    validflag = VALID;
    
    /* GETTING THE BAND NUMBER, MINIMUM AND MAXIMUM VALUES */
    for (loop2 = 0; loop2 < 3 && validflag == VALID; loop2++)
        {
        strip_num(readin,proc_num,&index);
        switch (loop2)
            {
            case 0:
                break;
            case 1: curmin = strtod(proc_num,&error_ptr);
                break;
            case 2: curmax = strtod(proc_num,&error_ptr);
                break;
            default:
                fclose(stxin);
                c_lasmsg(TRUE, "arc2las-proc", "Error processing .stx file");
            }
        
        /* NEXT TO STATEMENT, JUST SETTING THE BAND NUMBER IF AN
        ERROR OCCURS AND SETTING THE BAND FLAG TO INVALID */
        if (((error_ptr != NULL)&&((*error_ptr) != 0)) || (strlen(proc_num)==0))
            validflag = INVAL;

		/* MAKING SURE INDEX IS NOT LARGER THAN THE LENGTH OF READIN */
		if (length < index)
			validflag = INVAL;

		if (nbits == 1)
			{
			curmin=0;
			curmax= 1;
			validflag=2;
			}

        if (loop == 1) /*FIRST BAND*/
            {
            if (loop2==1)minmin = curmin;
            if (loop2==2)maxmax = curmax;
            }
        else
            {
            if (curmin < minmin)
                minmin = curmin;
            if (curmax > maxmax)
                maxmax = curmax;
            }
        }
    }
fclose(stxin);

/* OPENING THE .BLW */
if ((bilwin = fopen(gridbilw,"r")) == NULL)
	if ((bilwin = fopen(gridbilw2,"r")) == NULL)
		c_lasmsg(TRUE, "las2bil-open", "Error opening %s", gridbilw);

/* READING IN ALL OF THE .BLW INFORMATION */
if (fscanf(bilwin,"%lf %lf %lf %lf %lf %lf",&xdim,&d_rotation,&b_rotation,
    &ydim,&ulxmap,&ulymap) < 0)
    {
    fclose(bilwin);
    c_lasmsg(TRUE, "arc2las-read", "Error reading .blw file");
    }

fclose(bilwin);

/* RESETTING YDIM TO A POSITIVE VALUE */
ydim = ydim * -1;


/* SETTING DTYPE */
if (nbits == 1)
    ddrstuff.dtype = EBYTE;
else if (nbits == 8)
{
    if ((maxmax >= 255.5) || (minmin <= -0.5))
        ddrstuff.dtype = EWORD;
	else
    	ddrstuff.dtype = EBYTE;
}
else if (nbits == 16)
{
    if ((maxmax >= 32767.5) || (minmin <= -32768.5))
        ddrstuff.dtype = ELONG;
	else
		ddrstuff.dtype = EWORD;
}
else if (nbits == 32)
    ddrstuff.dtype = ELONG;
else
    c_lasmsg(TRUE, "arc2las-dtype", "Cannot process data");


lvalue = ddrstuff.dtype;

if (E_SUCC != lng2tae(&lvalue,1,1,"DTYPE",&tmp_long))
    c_lasmsg(TRUE, "arc2las-qout", "Error passing variables to TAE");

if (q_out(&vblk) != SUCCESS)
    c_lasmsg(TRUE, "arc2las-qout", "Error passing variables to TAE");


/* PROCESSING THE .BILW TO COMPUTE CORNER COORDINATES.
-----------------------------------------------------*/
coordinates(xdim,ydim,b_rotation,d_rotation,ulxmap,ulymap,&ddrstuff);
strcpy(systrg,"ieee-std");
c_putddf(&ddrstuff,&key,systrg);

/* GET ARC/INFO PROJECTION INFORMATION
--------------------------------------*/
if (proj2ddr(&ddrstuff,ep3flag,workspace,ingrid) != E_SUCC)
    {
    ddrstuff.valid[DDPCV] = INVAL;
    c_lasmsg(FALSE,"arc2las-editddr","Use EDITDDR to enter projection data");
    }

/* PRINTING OUT THE DDR INFORMATION */
c_putddr(ddrname,&ddrstuff);
c_lused(ddrname);


/* PROCESSING THE .STX FILE
-------------------------*/
if ((stxin = fopen(gridstx,"r")) == NULL)
    c_lasmsg(TRUE, "arc2las-open", "Error opening .stx");

/*  INTIALIZING BDDR, ONLY NEEDS TO BE DONE ONCE */
c_intbdr(&bddrstuff);

/* GOING THROUGH A LOOP TO PROCESS ALL BANDS */
for (loop = 1; loop <= ddrstuff.nbands; loop++)
    {
    /* SETTING THE READIN'S INDEXING VARIABLE TO 0 */
    index = 0;
    
    /* READING IN THE BAND INFORMATION */
    if (fgets(readin,STXLEN,stxin) == NULL)
        {
        fclose(stxin);
        c_lasmsg(TRUE, "arc2las-read", "Error reading .stx file");
        }
    
    length = (long)strlen(readin);
    error_ptr = NULL;
    
    /* ASSUMING THE READIN PARAMETERS ARE VALID */
    bddrstuff.valid = VALID;
    
    /* GETTING THE BAND NUMBER, MINIMUM AND MAXIMUM VALUES */
    for (loop2 = 0; loop2 < 3 && bddrstuff.valid == VALID; loop2++)
        {
        strip_num(readin,proc_num,&index);
        
        switch (loop2)
            {
            case 0: bddrstuff.bandno = strtol(proc_num,&error_ptr,10);
                break;
            case 1: bddrstuff.minval = strtod(proc_num,&error_ptr);
                break;
            case 2: bddrstuff.maxval = strtod(proc_num,&error_ptr);
                break;
            default:
                fclose(stxin);
                c_lasmsg(TRUE, "arc2las-proc", "Error processing .stx file");
            }
        
        /* NEXT TO STATEMENT, JUST SETTING THE BAND NUMBER IF AN
        ERROR OCCURS AND SETTING THE BAND FLAG TO INVALID */
        if (((error_ptr != NULL) && ((*error_ptr) != 0)) ||
            (strlen(proc_num) == 0))
            {
            bddrstuff.bandno = loop;
            bddrstuff.valid = INVAL;
            }
        
        /* MAKING SURE INDEX IS NOT LARGER THAN THE LENGTH OF READIN */
        if (length < index)
            {
            bddrstuff.bandno = loop;
            bddrstuff.valid = INVAL;
            }
        if (nbits == 1)
            {
            bddrstuff.minval = 0;
            bddrstuff.maxval= 1;
            bddrstuff.valid = 2;
            }
        }

    /* INVALIDATE MIN AND MAX PIXEL VALUES (DON'T TRUST ORIGINATOR) */
    bddrstuff.valid = INVAL;

    /* WRITING OUT THIS BDDR INFORMATION */
    c_putbdr(ddrname,&bddrstuff);
    }
fclose(stxin);


/* CREATING THE .HIS FILE
----------------------------*/
strcpy(text,"");
status = c_hstout(&vblk,hisname,hostin,&zero,text);
if (status != E_SUCC)
    c_lasmsg(TRUE, "arc2las-open", "Error opening .his file");
fclose(bilin);

/* CREATING THE .IMG FILE
----------------------------*/
if (nbits == 1)
    bit2byte(gridbil, imagename, ddrstuff.nbands, nbits, ddrstuff.ns,
    ddrstuff.nl, msgtim, bandbytes);
else if (nbits == 8  &&  ddrstuff.dtype == EWORD)
    short2long(gridbil, imagename, ddrstuff.nbands, ddrstuff.dtype-1,
    ddrstuff.ns, ddrstuff.nl, msgtim, bandbytes);
else if (nbits == 16  &&  ddrstuff.dtype == ELONG)
    short2long(gridbil, imagename, ddrstuff.nbands, ddrstuff.dtype-1,
    ddrstuff.ns, ddrstuff.nl, msgtim, bandbytes);


strcpy(progname,"ARC2LAS");
c_complt(progname, imagename, &outcount);
}
