/********************************************************************
FUNCTION: short2long

PURPOSE: Convert unsigned 8-bit BIL to signed 16-bit IMG
      or Convert unsigned 16-bit BIL to signed 32-bit IMG

PROGRAM HISTORY:
    07-95   Initial version of short2long for EDC version of ARC2LAS,
            T.Wolbrink, EDC.

*********************************************************************/
#include <unistd.h>
#include <stdio.h>
#include "las.h"
#include "util.h"
#include "arc2las.h"

/* globals */
FILE *imageout;                      /* Output LAS image  */
extern FILE *bilin;                         /* Input BIL image   */ 
unsigned char   *inbytebuf;          /* Input buffers     */
unsigned short  *inwordbuf;
short           *outwordbuf;         /* Output buffers    */
long            *outlongbuf;

int short2long
(
  char *gridbil,          /* Input BIL image                           */
  char *hostout,          /* Output LAS image                          */
  long nbands,            /* Number of bands                           */
  long idtype,            /* UNSIGNED dtype of input BIL image
                             if EBYTE
                               write IMG file as SIGNED EWORD
                             else if EWORD
                               write IMG file as SIGNED LONG           */
  long ns,                /* number of samples                         */
  long nl,                /* number of lines                           */
  long msgtim,            /* message processing time                   */
  long bandbytes          /* Bytes per line (lines might be padded)    */
)
{
  extern long curcat;     /* Current Band                              */
  extern long curent;     /* Current Line                              */
  long cursmp;            /* Current Sample                            */
  long smpsize;           /* Number of bytes per sample                */
  long byteskip;          /* Number of bytes to skip between lines     */
  long msg_flag=ALLCAT;   /* message processing flag                   */

  void fatalerr(long,char *,char *);

  /*  ALLOCATE MEMORY FOR FILE BUFFERS
  --------------------------------------*/
  if (idtype == EBYTE)
    {
    if ((inbytebuf = calloc(ns,sizeof(char))) == NULL)
      c_lasmsg(TRUE,"arc2las-memory1","Not enough memory");
    if ((outwordbuf = calloc(ns,sizeof(short))) == NULL)
      {
      free(inbytebuf);
      c_lasmsg(TRUE,"arc2las-memory2","Not enough memory");
      }
    }
  else if (idtype == EWORD)
    {
    if ((inwordbuf = calloc(ns,sizeof(short))) == NULL)
      c_lasmsg(TRUE,"arc2las-memory3","Not enough memory");
    if ((outlongbuf = calloc(ns,sizeof(long))) == NULL)
      {
      free(inwordbuf);
      c_lasmsg(TRUE,"arc2las-memory2","Not enough memory");
      }
    }
  else
    c_lasmsg(TRUE,"arc2las-idtype","Cant convert this type");

  /* Check if file already exists
  -------------------------------*/
  if (access(hostout, R_OK) == 0)
    fatalerr(idtype,"arc2las-exists","Output image already exists");

  /* Open output image for binary write
  -------------------------------------*/
  if ((imageout = fopen(hostout, "wb")) == NULL)
    fatalerr(idtype,"arc2las-open","Error opening output image");

  /* Open input image for binary read
  -----------------------------------*/
  if ((bilin = fopen(gridbil,"rb")) == NULL)
    fatalerr(idtype,"arc2las-open","Error opening input image");

  /* INITIALIZING VARIABLES
  ---------------------------*/
  if (idtype == EBYTE)
    smpsize = sizeof(char);
  else
    smpsize = sizeof(short);
  if ((byteskip = bandbytes - (smpsize * ns)) < 0)
    fatalerr(idtype,"arc2las-bandbytes","Inadequate bandbyte size");

  c_inittm("ARC2LAS","Processed","line","bands"," ",
           &nl,&nbands,&msgtim,&msg_flag);
  curent = 1;
  curcat = 1;
  c_startm();

  /* READING IN THE BIL TO CREATE AN IMAGE
  -------------------------------------------*/
  for (curcat = 1; curcat <=nbands; curcat++)
    {
    for (curent = 1; curent <= nl; curent++)
      {
      if (idtype == EBYTE)
        {
        if (fread((void *)inbytebuf,sizeof(char),ns,bilin) != ns)
          fatalerr(idtype,"arc2las-read11","Error reading input image");
        for (cursmp = 0; cursmp < ns; cursmp++)
          outwordbuf[cursmp] = inbytebuf[cursmp];
        if (fwrite((void*)outwordbuf,sizeof(short),ns,imageout) != ns)
          fatalerr(idtype,"arc2las-write1","Error writing output image");
        }
      else      /* idtype == EWORD */
        {
        if (fread((void *)inwordbuf,sizeof(short),ns,bilin) != ns)
          fatalerr(idtype,"arc2las-read11","Error reading input image");
        for (cursmp = 0; cursmp < ns; cursmp++)
          outlongbuf[cursmp] = inwordbuf[cursmp];
        if (fwrite((void *)outlongbuf,sizeof(long),ns,imageout) != ns)
          fatalerr(idtype,"arc2las-write1","Error writing output image");
        }
      if ((curent != nl) && (fseek(bilin, byteskip, SEEK_CUR) != NULL))
        fatalerr(idtype,"arc2las-readnl","Error reading input image");
      }
    
    /* If only one band, it's not necessary
       to reposition file pointer
    ---------------------------------------*/
    if ((nbands != 1) && (fseek(bilin,(bandbytes * curcat),SEEK_SET) != NULL))
      fatalerr(idtype,"arc2las-read","Error moving pointer");
    }

  /*  CLEAN THINGS UP
  ---------------------*/
  c_stoptm();

  if (idtype == EBYTE)
    {
    free(inbytebuf);
    free(outwordbuf);
    }
  else
    {
    free(inwordbuf);
    free(outlongbuf);
    }

  fclose(imageout);
  fclose(bilin);

  return(E_SUCC);
}

/*************************
    LOCAL ERROR HANDLER
**************************/
void fatalerr(idtype,msgkey,msgdesc)
  long idtype;
  char *msgkey;
  char *msgdesc;
{
  if (idtype == EBYTE)
    {
    free(inbytebuf);
    free(outwordbuf);
    }
  else
    {
    free(inwordbuf);
    free(outlongbuf);
    }
  fclose(imageout);
  fclose(bilin);
  c_lasmsg(TRUE,msgkey,msgdesc);
}
