/******************************************************************************
FUNCTION:       GETLEAD

PURPOSE:        Read the leader file from tape and write it to disk if
                specified.  Extract needed information.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "las.h"
#include "geompak.h"
#include "labserv.h"
#include "cm.h"
#include "tapeio.h"
#include "ltwgin.h"

#define PI    3.141592653589793238 /* PI */

extern long cdflag;             /* flag for disk image input            */

int getlead
(
   long *unit,                    /* Tape unit number */
   long *wrtdsk,                  /* Ancillary files to read in */
   char hosout[][CMLEN],          /* Output host names */
   long *num_out,                 /* Number of output files */
   long *numlead,                 /* Number of leader records */
   long *leadsize,                /* Size of leader records */
   double fcenll[],               /* Full scene center lat/long degrees */
   double fcenls[],               /* Full scene center line/sample */
   long *path,                    /* Scene path */
   char *direction,               /* Satellite direction */
   char *maprj,                   /* Map projection */
   double pixsiz[],               /* Image pixel size */
   long *satid,                   /* Satellite id */
   char *sensor,                  /* Sateelite sensor */
   double *angle,                 /* Angle of rotation in radians */
   long *datum_code,              /* Datum of the dataset */
   long *utmzone,                 /* UTM zone code */
   char *satnam,                  /* Satellite name */
   char *data_type,               /* Type of data (corrected or uncorrected) */
   char cornlat[][17],            /* Corner latitudes */
   char cornlong[][17],           /* Corner longitudes */
   char hpath[CMLEN],             /* Path to image data on disk */
   long ceosid,                   /* Id number of receiving station */
   double proj_corners[8],        /* Projection corners */
   double ls_corners[8]           /* Line/sample corners */
)
{
   FILE *fp=NULL;          /* file pointer                                 */
   FILE *infp=NULL;        /* LEA file pointer (for CD input only)         */

   char *in_buf;           /* buffer read from data on tape                */
   char tempbuf[17];       /* temp work buffer                             */
   char leafile[CMLEN];    /* name of leader file on disk                  */
   char scenenum[3];       /* string representation of scene number        */
   char datum[13];         /* the datum of the data                        */
   char deg[4];            /* degrees for fcenll                           */
   char min[3];            /* minutes for fcenll                           */
   char filename[10][CMLEN]; /* array of filenames extracted               */
   char lpath[CMLEN];      /* temporary host path for leader file          */

   long i;                 /* loop counter                                 */
   long mode = HANDLE;     /* tape I/O error flag                          */
   long status;            /* status flag                                  */
   long tzone=0;           /* temp double value                            */
   long location;          /* offset into input buffer for extraction      */
   long location2;         /* see above                                    */
   long nfiles;            /* number of filenames returned                 */
   double temp;            /* temporary holding                            */
   double degrees;         /* degrees for fcenll                           */
   double minutes;         /* minutes for fcenll                           */

   /* Create the leader file name
   ------------------------------*/
   if (*wrtdsk)
   {
      c_lsmknm(hosout[0],"lead",hosout[*num_out]);
      fp = fopen(hosout[(*num_out)++],"w");
      if(fp == NULL)
      {
         c_lasmsg(FALSE,"ltwgin-diskread",
                  "Error reading leader file from disk");
         return(E_FAIL);
      }
   }

   /* Allocate space
   -----------------*/
   in_buf = (char *) malloc(*leadsize);
   if (in_buf == NULL)
   {
      c_lasmsg(FALSE, "ltwgin-alloc",
               "Error allocating dynamic memory");
      return(E_FAIL);
   }

   /* If image is to be read from CD, prepare the LEA file for reading
   -------------------------------------------------------------------*/
   if (cdflag)
   {
      strcpy(leafile, hpath);
      if (cdflag == SAFRICAMSS)
         strcat(leafile, "lea_01.001");
      if (cdflag == SPOTBIL)
      {
         if(imgnum(hpath, scenenum) != E_SUCC)
         {
            c_lasmsg(FALSE, "ltwgin-imgnum",
                     "Error retrieving the image number from CD.");
            return(E_FAIL);
         }
         strcat(leafile, "lead_");
         strcat(leafile, scenenum);
         strcat(leafile, ".dat");
      }
      if (cdflag == NLAPS)
      {
         strcpy(lpath,hpath);
         strcat(lpath,"*.l*");
         c_getfiles(lpath,filename,&nfiles,10,1,10);
         if (nfiles > 0)
            strcpy(leafile, filename[0]); 
      }

      infp = fopen(leafile, "r"); /* Standard leader filename for CD */
      if (infp == NULL)
      {
         c_lasmsg(FALSE, "ltwgin-diskread",
                  "Error reading leader file from disk");
         free(in_buf);
         return(E_FAIL);
      }
   }

   /* Read and optionally write to disk.  Also extract needed information
   ---------------------------------------------------------------------*/
   for (i = 0; i < *numlead; i++)
   {
      if (cdflag)
      {
         if (fread(in_buf, 1, *leadsize, infp) != *leadsize) 
         {
            c_lasmsg(FALSE, "ltwgin-diskread",
                     "Error reading leader file from disk");  
            free(in_buf);
            return(E_FAIL);
         }
      }
      else
      {
         status = c_tpread(unit,(unsigned char *)in_buf,leadsize,&mode);
         if (status != *leadsize) 
         {
            c_lasmsg(FALSE, "ltwgin-read",
                     "Error reading leader file from tape");
            free(in_buf);
            return(E_FAIL);
         }
      }

      if (i == 0)
      {
         /* Get the satellite name, number, sensor and data type
         -------------------------------------------------------*/
         strncpy(satnam,&in_buf[48],2);
         satnam[2] = '\0';
         strncpy(tempbuf,&in_buf[50],1);
         tempbuf[1] = '\0';
         *satid = atol(tempbuf);
         if (ceosid == CAN)
            strncpy(sensor,&in_buf[51],3);
         else
            strncpy(sensor,&in_buf[52],3);
         sensor[3] = '\0';
         *data_type = in_buf[55];
      }

      else if (i == 1)
      {
         if (strncmp(satnam,"LS",2) == 0)
         {
            if ((JAPAN) && (strcmp(sensor, "MSS") == 0))
            {
               /* Get the full scene center line/sample
               ----------------------------------------*/
               strncpy(tempbuf, &in_buf[100], 4);
               tempbuf[4] = '\0';
               fcenls[0] = atol(tempbuf);
               strncpy(tempbuf, &in_buf[104], 4);
               tempbuf[4] = '\0';
               fcenls[1] = atol(tempbuf);

               /* Get the WRS path
               -------------------*/
               strncpy(tempbuf, &in_buf[26], 3);
               tempbuf[3] = '\0';
               sscanf(tempbuf, "%ld", path);

               /* Get the orbital direction
               ----------------------------*/
               strncpy(direction, &in_buf[25], 1);
               direction[1] = '\0';

               /* Get the map projection
               -------------------------*/
               strncpy(maprj, &in_buf[180], 3);
               maprj[3] = '\0';
            }
            else
            {
               /* Retrieve the full scene center lat/long 
               ------------------------------------------*/
               /* Japan (corrected) & Thai store lat/long in
               different location */
               if ((JAPAN && (*data_type == 'P')) || (ceosid == THAI))
               {
                  location = 212;
                  location2 = 228;
               }
               else 
               {
                  location = 52;
                  location2 = 68; 
               }
               strncpy(tempbuf,&in_buf[location],16);
               tempbuf[16] = '\0';
               sscanf(tempbuf,"%lf",&fcenll[0]);

               strncpy(tempbuf,&in_buf[location2],16);
               tempbuf[16] = '\0';
               sscanf(tempbuf,"%lf",&fcenll[1]);

               /* Get the full scene center line/sample 
               ----------------------------------------*/
               if ((JAPAN && (*data_type == 'P')) || (ceosid == THAI))
               {
                  location = 244;
                  location2 = 260;
               }
               else
               {
                  location = 84;
                  location2 = 100;
               }
               strncpy(tempbuf,&in_buf[location],16);
               tempbuf[16] = '\0';
               sscanf(tempbuf,"%lf",&fcenls[0]);
               strncpy(tempbuf,&in_buf[location2],16);
               tempbuf[16] = '\0';
               sscanf(tempbuf,"%lf",&fcenls[1]);

               /* Get the WRS path number 
               -------------------------*/
               strncpy(tempbuf,&in_buf[165],3);
               tempbuf[3] = '\0';
               sscanf(tempbuf,"%ld",path);

               /* Get the orbital direction 
               ----------------------------*/
               if ((ceosid == HBK) || (ceosid == CAN))
               {

                  /* South Africa and Canada data stores the orbital direction 
                     as a single character immediately preceding the WRS path 
                     number. */

                  direction [0] = in_buf[164];
                  direction [1] = '\0';
               }
               else
               {
                  strncpy(direction,&in_buf[356],10);
                  direction[9] = '\0';
               }

               /* Get the map projection 
               -------------------------*/
               strncpy(maprj,&in_buf[1556],3);
               maprj[3] = '\0';
               if (ceosid == HBK)

                  /* South Africa UTM data stores the projection and zone in a
                     slightly different form: whereas the 'standard' is to store
                     the projection as "UTM" and the zone in the next record,
                     this data stores them both together. For example, UTM zone
                     39 would be stored as 'UT39'. */
               {
                  if (maprj[0] == 'U')
                  {
                     maprj[2] = 'M';
                     maprj[3] = '\0';
                  }
                  else 
                     maprj[2] = '\0';
               }
               else if (JAPAN)
               {
                  if (strcmp(maprj, "YN ") == 0)
                  {
                     strncpy(maprj, "UTM", 3);
                     maprj[3] = '\0';
                  }
                  else if (strcmp(maprj, "NY ") == 0)
                  {
                     strncpy(maprj, "SOM", 3);
                     maprj[3] = '\0';
                  }
               }
               else if (ceosid == SAUDI)
               {
                  if (maprj[1] == 'Y')
                  {
                     strncpy(maprj, "UTM", 3);
                     maprj[3] = '\0';
                  }
               }
               else if (ceosid == THAI)
               {
                  if (maprj[1] == 'Y')
                  {
                     strncpy(maprj, "UTM", 3);
                     maprj[3] = '\0';
                  }
                  else if ((maprj[1] == 'N') && (maprj[2] == 'Y'))
                  {
                     strncpy(maprj, "UTM", 3);
                     maprj[3] = '\0';
                     printf("Data is geometrically registered by receiving station ...\n");
                  }
               }

               /* If this is HBK data, get the UTM zone
               ---------------------------------------*/
               if (ceosid == HBK)
               {
                  strncpy(tempbuf, &in_buf[1558],2);
                  tempbuf[2] = '\0';
                  sscanf(tempbuf, "%lf", &temp);
                  tzone = (long)temp;
                  *utmzone = (long)tzone;
               }
            }
         } 
         else if (strncmp(satnam,"SP",2) == 0)
         {
            /* Get the four corners 
            -----------------------*/
            strncpy(cornlat[0],&in_buf[148],16);
            cornlat[0][16] = '\0';
            strncpy(cornlong[0],&in_buf[164],16);
            cornlong[0][16] = '\0';

            strncpy(cornlat[1],&in_buf[212],16);
            cornlat[1][16] = '\0';
            strncpy(cornlong[1],&in_buf[228],16);
            cornlong[1][16] = '\0';
          
            strncpy(cornlat[2],&in_buf[276],16);
            cornlat[2][16] = '\0';
            strncpy(cornlong[2],&in_buf[292],16);
            cornlong[2][16] = '\0';

            strncpy(cornlat[3],&in_buf[340],16);
            cornlat[3][16] = '\0';
            strncpy(cornlong[3],&in_buf[356],16);
            cornlong[3][16] = '\0';
         }
      }
      else if (i == 2)
      {

         /* If this is HBK (South Africa) data, get the image projection
            and line/sample corners to be later converted into their
            corresponding file corners.
         ---------------------------------------------------------------*/
         if (ceosid == HBK)
         {
            for (i = 0; i < 8; i++)
            {
               strncpy(tempbuf, &in_buf[(580 + (i * 16))], 16);
               tempbuf[16] = '\0';
               sscanf(tempbuf, "%lf", &proj_corners[i]);
            }
            for (i = 0; i < 4; i++)
            {
               strncpy(tempbuf, &in_buf[(836 + (i * 32) + 16)], 16);
               tempbuf[16] = '\0';
               sscanf(tempbuf, "%lf", &ls_corners[i * 2]);
               strncpy(tempbuf, &in_buf[(836 + (i * 32))], 16);
               tempbuf[16] = '\0';
               sscanf(tempbuf, "%lf", &ls_corners[(i * 2) + 1]);
            }

            /* Since the corners in the leader file are not stored "neatly"
               in the order expected by the DDR, swap the out-of-order ones
               to rectify things.  This is much easier to do than reading
               in all 16 corners one at a time.  
            --------------------------------------------------------------*/
             
            temp = proj_corners[4];
            proj_corners[4] = proj_corners[6]; 
            proj_corners[6] = temp;
            temp = proj_corners[5];
            proj_corners[5] = proj_corners[7];
            proj_corners[7] = temp;
            temp = ls_corners[4];
            ls_corners[4] = ls_corners[6];
            ls_corners[6] = temp;
            temp = ls_corners[5]; 
            ls_corners[5] = ls_corners[7];
            ls_corners[7] = temp;
         }

         /* Retrieve the full scene center lat/lon for Japan MSS 
         -------------------------------------------------------*/
         if (JAPAN && (strcmp(sensor, "MSS") == 0))
         {
            strncpy(deg, &in_buf[23], 2);
            deg[2] = '\0';
            degrees = (double)atol(deg); 
            strncpy(min, &in_buf[26], 2);
            min[2] = '\0';
            minutes = (double)atol(min);
            fcenll[0] = degrees + (minutes/60);

            strncpy(deg, &in_buf[30], 3);
            deg[3] = '\0';
            degrees = (double)atol(deg);
            strncpy(min, &in_buf[34], 2);
            min[2] = '\0';
            minutes = (double)atol(min);
            fcenll[1] = degrees + (minutes/60); 
         }


         /* Retrieve the pixel size and convert from km to meters
         -------------------------------------------------------*/
         if (ceosid == HBK)
         {
            /* South Africa pixel sizes are found in a different location.
               Also, they are already in meters rather than kilometers, so
               there is no need to multiply them by 1000. */

            strncpy(tempbuf, &in_buf[180], 16);
            tempbuf[16] = '\0';
            sscanf(tempbuf, "%lf", &pixsiz[1]);

            strncpy(tempbuf, &in_buf[196], 16);
            tempbuf[16] = '\0';
            sscanf(tempbuf, "%lf", &pixsiz[0]);
         }
         else if ((JAPAN) && (strcmp(sensor,"MSS") == 0))
         {
            /* MSS data from Japan does not contain the pixel size for
               corrected data.  These hard-coded values are "best guess"
               based on the LTWG Production Description Chart entry on
               Japan MSS data. */
               
            if (*data_type == 'A')
            {
               pixsiz[0] = 57.0;
               pixsiz[1] = 59.0;
            }
            else
            {
               pixsiz[0] = 57.0;
               pixsiz[1] = 57.0;
            }
         }
         else
         {
            if (ceosid == CAN)
               strncpy(tempbuf,&in_buf[180],16);
            else
               strncpy(tempbuf,&in_buf[364],16);
            tempbuf[16] = '\0';
            sscanf(tempbuf,"%lf",&pixsiz[1]);
            /* Thailand, Saudi Arabia, Japan, and Canada data is
            already in meters */
            if ((ceosid != THAI) && (ceosid != SAUDI) && (!(JAPAN)) && 
                (ceosid != CAN))
               pixsiz[1] *= 1000;

            if (ceosid == CAN)
               strncpy(tempbuf,&in_buf[196],16);
            else
               strncpy(tempbuf,&in_buf[380],16);
            tempbuf[16] = '\0';
            sscanf(tempbuf,"%lf",&pixsiz[0]);
            if ((ceosid != THAI) && (ceosid != SAUDI) && (!(JAPAN)) && 
                (ceosid != CAN))
               pixsiz[0] *= 1000;
         }

         /* Get the rotation angle and convert to radians
         ------------------------------------------------*/
         if (ceosid == HBK || ceosid == CAN)
         {
            strncpy(tempbuf,&in_buf[132], 16);
            tempbuf[16] = '\0';
            sscanf(tempbuf, "%lf", angle);
         }
         else if (ceosid == THAI)
         {
            strncpy(tempbuf,&in_buf[444], 16);
            tempbuf[16] = '\0';
            sscanf(tempbuf, "%lf", angle);
         }
         else if (JAPAN && (strcmp(sensor, "MSS") == 0))
         {
            strncpy(tempbuf,&in_buf[128], 16);
            tempbuf[16] = '\0';
            sscanf(tempbuf, "%lf", angle);
         }
         else
         {
            strncpy(tempbuf,&in_buf[204],16);
            tempbuf[16] = '\0';
            sscanf(tempbuf,"%lf",angle);
         }

         if (((ceosid == THAI) || (ceosid == SAUDI)) && (*angle > 180))
         {
            *angle = (*angle - 180);
            *angle = *angle * PI/180.0;
         }
         else if (!(JAPAN))
         {
            *angle = *angle * PI/180.0;
         }

         /* Get the UTM zone and datum 
         -----------------------------*/
         if (ceosid != HBK)
         {
            /* This was in previous record for South Africa */
            if ((ceosid == CAN) || (JAPAN && (strcmp(sensor, "MSS") == 0)))
            {
               tzone = (long)(((fcenll[1] + 180.0) / 6.0) + 1.0);
               if (fcenll[0] < 0.0)
                  tzone = -tzone;
               datum[0] = '\0';
            }
            else if ((ceosid == THAI) || (ceosid == SAUDI))
            {
               strncpy(datum,&in_buf[396],13);
               datum[12] = '\0';
               strncpy(tempbuf,&in_buf[410],2);
               tempbuf[2] = '\0';
               sscanf(tempbuf,"%ld",&tzone);
            }
            else
            {
               strncpy(datum,&in_buf[92],13);
               datum[12] = '\0'; 
               strncpy(tempbuf,&in_buf[396],16);
               tempbuf[16] = '\0';
               sscanf(tempbuf,"%ld",&tzone);
            }
            *utmzone = tzone;
            /* Japan data doesn't contain datum information.  According
               to the receiving station, the spheroid used is Bessel(2).
            ------------------------------------------------------------*/ 
            if (JAPAN)
               strcpy(datum,"BESSEL");
            /* Assign the datum code.  Use the default datum of 0 
               for NAD 27 and 8 for NAD 83 until datums are fully 
               implemented.  The datum_code for NAD 27 will be 225.
               The datum_code for NAD 83 will be 219.
             ------------------------------------------------------*/
             if (strncmp(datum,"NAD 27",6) == 0)
                *datum_code = 0;          
             else if (strncmp(datum,"NAD 83",6) == 0)
                *datum_code = 8;
             else
                /* If c_dtmnum is not successful, the programmer can
                   set the datum code using the semimajor and semiminor
                   axes if they are available in the leader file.  This
                   assignment will be done on a case by case basis.
                -------------------------------------------------------*/
                c_dtmnum(datum, datum_code);      
         }
      }

      /* Write to disk
      ----------------*/
      if (*wrtdsk)
      {
         if(fwrite(in_buf,1,*leadsize,fp) != *leadsize)
         {
            c_lasmsg(FALSE,"ltwgin-write","Error writing to leader file");
            free(in_buf);
            return(E_FAIL);
         }
      }
   }

   /* Close the file
   -----------------*/
   if (*wrtdsk)
   {
      if(fclose(fp) != 0)
      {
         c_lasmsg(FALSE,"ltwgin-write","Error closing the leader file");
         free(in_buf);
         return(E_FAIL);
      }
   }

   /* One more read for EOF
   ------------------------*/
   if (!cdflag) 
   {
      status = c_tpread(unit,(unsigned char *)in_buf,leadsize,&mode);
      if (status != TAPE_EOF) 
      {
         c_lasmsg(FALSE, "ltwgin-read",
                  "Error reading leader file from tape");
         free(in_buf);
         return(E_FAIL);
      }
   }

   /* Free allocated space
   ----------------------*/
   free(in_buf);
   if (cdflag)
      fclose (infp);

   return(E_SUCC);
}
