/******************************************************************************
FUNCTION:       GETVDF

PURPOSE:        Read the volume directory file tape and write it to disk if
                specified.  Extract needed information

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "tapeio.h"
#include "ltwgin.h"

extern long cdflag;     /* flag for cd image input                      */
extern char sceneid[SCENELEN];  /* naming convention for NLAPS files    */

int getvdf
(
  long *unit,                     /* Tape unit number */
  long *wrtdsk,                   /* Ancillary files to read in */
  char hosout[][CMLEN],           /* Output host names */
  long *num_out,                  /* Number of output files */
  long *recsize,                  /* Image record length */
  long *numimg_recs,              /* Number of image records on this tape */
  long *num_trailrec,             /* Number of trailer records */
  char *order,                    /* Band order */
  long *numlead,                  /* Number of leader files */
  long *leadsize,                 /* Size of leader records */
  long *trailsiz,                 /* Max size of trailer records */
  char hpath[CMLEN],              /* Host path to image data */
  long *ceosid                    /* Code for receiving ground station */
)
{
   FILE *fp=NULL;          /* file pointer                                 */
   FILE *infp=NULL;        /* input file, for CD images only               */

   char *in_buf;           /* buffer read from data on tape                */
   char tempbuf[20];       /* temp work buffer                             */
   char vdffile[CMLEN];    /* name of vdf file (on disk)                   */
   char scenenum[3];       /* string representation of scene number        */
   char idstring[32];      /* string extracted to identify ground station  */

   long i;                 /* loop counter                                 */
   long mode = HANDLE;     /* tape I/O error flag                          */
   long numbytes = 360;    /* number of bytes in a volume directory record */
   long numrecs;           /* number of records in the file                */
   long status;            /* status flag                                  */
   long japan_mss=FALSE;   /* flag for Japan MSS data                      */
   long rec;               /* record number being processed                */ 

   /* Allocate space
   -----------------*/
   in_buf = (char *) malloc(numbytes);
   if (in_buf == NULL)
   {
      c_lasmsg(FALSE, "ltwgin-alloc", "Error allocating dynamic memory");
      return(E_FAIL);
   }
        
   /* Create name for volume directory file 
   ----------------------------------------*/
   if (*wrtdsk)
   {
      c_lsmknm(hosout[0],"vdf",hosout[*num_out]);
      fp = fopen(hosout[(*num_out)++],"w");
      if(fp == NULL)
      {
         c_lasmsg(FALSE,"ltwgin-diskread",
                  "Error reading volume directory file from disk");
         free(in_buf);
         return(E_FAIL);
      }
   }

   /* If image is to be read from CD, prepare the VDF file for reading
   -------------------------------------------------------------------*/
   if (cdflag)
   {
      strcpy(vdffile, hpath);
      if (cdflag == SAFRICAMSS)
         strcat(vdffile, "vdf_dat.001");
      if (cdflag == SPOTBIL)
      {
         if(imgnum(hpath, scenenum) != E_SUCC)
         {
            c_lasmsg(FALSE,"ltwgin-imgnum",
                     "Error retrieving the image number from CD.");
            free(in_buf);
            return(E_FAIL);
         }
         strcat(vdffile, "vold_");
         strcat(vdffile, scenenum);
         strcat(vdffile, ".dat");
      }
      if (cdflag == NLAPS)
      {
         strncat(vdffile, sceneid, 8); 
         strcat(vdffile, ".iv");
      }

      infp = fopen(vdffile, "r"); /* Standard name for VDF file on CD */
      if (infp == 0)
      {
         c_lasmsg(FALSE, "ltwgin-diskread",
                  "Error reading VDF file from disk");
         free(in_buf);
         return(E_FAIL);
      }
   }
        
   /* Read and optionally write the volume directory file.
      Get the number of records in this file from the first
      records  Extract other needed information. 
   --------------------------------------------------------*/
   numrecs = -1;
   for (i = 0; ; i++)
   {
      if (i == numrecs)
         break;
      if (cdflag)
      {
         if(fread(in_buf, 1, numbytes, infp) != numbytes)
         {
            c_lasmsg(FALSE,"ltwgin-diskread",
                     "Error reading volume directory file from disk");
            free(in_buf);
            return(E_FAIL);
         }
      }
      else
      {
         status = c_tpread(unit,(unsigned char *)in_buf,&numbytes,&mode);
         if (status != numbytes) 
         {
            c_lasmsg(FALSE, "ltwgin-read",
                     "Error reading VDF file from tape");
            free(in_buf);
            return(E_FAIL);
         }
      }

      /* The second record in the vdf for Japan MSS data is the text 
         record, not a file pointer record.  If we have mss data from
         Japan, skip the second record and process records 3, 4, and 5.
      -----------------------------------------------------------------*/ 
      if ((i == 1) && (japan_mss))
         rec = 4;
      else if ((i > 1) && (japan_mss))
         rec = i - 1;
      else
         rec = i;

      switch (rec)
      {
         /* Get the number of records in volume directory file
         ----------------------------------------------------*/
         case 0: strncpy(tempbuf,&in_buf[164],4);
                 tempbuf[4] = '\0';
                 numrecs = atol(tempbuf);

                 /* Identify the ground station this data is coming from
                 -------------------------------------------------------*/
                 strncpy(idstring, &in_buf[128], 32);
                 idstring[31] = '\0';
                 *ceosid = get_ceosid(idstring);
      
                 /* If this is MSS data from Japan, set flag
                 -------------------------------------------*/
                 if ((*ceosid == 3) && (in_buf[57] == 'M'))
                    japan_mss = TRUE; 
                 break;

         /* Get the number of records in leader file and size of records
         ---------------------------------------------------------------*/
         case 1: strncpy(tempbuf,&in_buf[100],8);
                 tempbuf[8] = '\0';
                 *numlead = atol(tempbuf);
                 strncpy(tempbuf,&in_buf[116],8); 
                 /* strncpy(tempbuf,&in_buf[108],8); */
                 tempbuf[8] = '\0';
                 *leadsize = atol(tempbuf);
                 break;

         /* Get the record length and band order  Also get the
            number of image records on this tape for BIL.
         -----------------------------------------------------*/
         case 2: strncpy(tempbuf,&in_buf[116],8);
                 tempbuf[8] = '\0';
                 *recsize = atol(tempbuf);
                 strncpy(order,&in_buf[32],3);
                 order[3] = '\0';
                 strncpy(tempbuf,&in_buf[100],8);
                 tempbuf[8] = '\0';
                 /* Subtract the one image file descriptor record */
                 *numimg_recs = atol(tempbuf) - 1;
                 break;

         /* Get the number and size of the trailer records
         ------------------------------------------------*/
         case 3: strncpy(tempbuf,&in_buf[100],8);
                 tempbuf[8] = '\0';
                 *num_trailrec = atol(tempbuf);
                 strncpy(tempbuf,&in_buf[116],8); 
                 /* strncpy(tempbuf, &in_buf[108], 8); */
                 tempbuf[8] = '\0';
                 *trailsiz = atol(tempbuf);
                 break;

         default: break;
      }
   
      /* Write to disk
      ----------------*/
      if (*wrtdsk)
      {
         if(fwrite(in_buf,1,numbytes,fp) != numbytes)
         {
            c_lasmsg(FALSE,"ltwgin-write",
                     "Error writing to volume directory file");
            free(in_buf);
            return(E_FAIL);
         }
      }
   }

   /* Close the file
   ----------------*/
   if (*wrtdsk)
   {
      if(fclose(fp) != 0)
      {
         c_lasmsg(FALSE,"ltwgin-write",
                  "Error closing the volume directory file");
         free(in_buf);
         return(E_FAIL);
      }
   }

   /* One more read for EOF
   -----------------------*/
   if (!cdflag)
   {
      status = c_tpread(unit,(unsigned char *)in_buf,&numbytes,&mode);
      if (status != TAPE_EOF) 
      {
         c_lasmsg(FALSE, "ltwgin-read",
                  "Error reading VDF file from tape");
         free(in_buf);
         return(E_FAIL);
      }
   }

   /*Free allocated space 
   ----------------------*/
   free(in_buf);
   if (cdflag)
      fclose(infp);

   return(E_SUCC);
}
