/******************************************************************************
FUNCTION:	LTWGIN

PURPOSE:	Read in image files from tapes that are in the LTWG format 
                and optionally read the ancillary data

PROGRAM HISTORY:
Version  Date   Author           Change Request
-------  -----  ---------------  ----------------------------------------------
1.0      11/90  T. Baltzer       Original development
2.0	 03/94	D. Etrheim	 Rewrote to be more generic to allow LTWG tapes
				 from different receiving stations
2.1      09/94  K. Lowell        Modified to allow reading of entire scene from
				 a single 8mm tape
2.2	 05/95  C. Engebretson   Added routines to allow for 1) data to be read
				 directly from CDs and disk files 2) more
				 general types of input data 3) ingesting 
				 non-standard foreign LTWG-format data.
7.0	 07/98  J. Janssen	 Modified to ingest SAUDI data
7.1	 07/98  J. Janssen       Modified to allow users to write all the bands
				 to one image or to write each band to its own
				 image. 
7.2	 08/98  J. Janssen	 Modified to ingest Japan data
7.3	 03/99  D. Hames	 Modified to ingest NLAPS data as produced 
				 by PDS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	Run under TAE

ALGORITHM REFERENCES
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "cm.h"
#include "util.h"
#include "ltwgin.h"

/* This global flag indicates if the image is to be read from a CD, and if it 
   is, which directory structure it uses.
-----------------------------------------------------------------------------*/
long	cdflag;
char	sceneid[SCENELEN];

int main
(
  void
)
{
struct PARBLK vblock;	/* TAE parameter block                        */

long bands[MAX_BAND]; 	/* bands to be processed within image         */
long i;			/* counter				      */
long msgtim;		/* elapsed time between display msg           */
long num_out; 	 	/* number of output files 		      */ 
long one = 1;		/* value of one			              */
long path;		/* satellite path			      */
long unit;             	/* tape unit number                           */
long totbnd = 0;	/* total number of bands to be copied         */
long wrtdsk[5];        	/* ancillary files to read in 		      */
long window[4];		/* window of input image                      */
long zero = 0;		/* value of zero                              */
long leadsize;		/* size of leader records		      */
long fatal = -1;	/* fatal error 				      */
long minmax;		/* min max flag                               */
long no_lead = FALSE;   /* Flag if successive tapes will have leader files */
long numimg_files;      /* number of output image files               */
long numimg_recs;	/* number of image records on current tape    */
long numtapes;		/* number of tapes specified                  */
long num_trailrec;	/* number of trailer records                  */
long numlead;		/* number of leader records		      */
long recsize;		/* image record length                        */
long tapenum = 1;	/* tape number currently mounted              */
long trailsiz;		/* max size of trailer records		      */
long satid;		/* satellite id				      */
long datum;		/* the datum code of the data 	              */	
long utmzone;		/* UTM zone code			      */
long std_out;		/* Standard output			      */

char dpath[CMLEN];	/* path to CD or disk image (TAE)	      */
char hpath[CMLEN];	/* actual machine path to CD or disk image    */
long ceosid;		/* integer ID number for ground station       */
char comment[3][CMLEN];	/* tape mount message to the operator         */
char hosout[12][CMLEN];	/* Output host file name                      */
char taeout[12][CMLEN];	/* Output TAE file names	              */
char order[4];		/* band order on tape                         */
char maprj[5];		/* map projection			      */
char direction[10];	/* satellite direction			      */
char sensor[5];		/* sensor type				      */
char satnam[5];		/* satellite name			      */
char cornlat[4][17];	/* corner latitudes			      */
char cornlong[4][17];	/* corner longitudes			      */
char ttype[CMLEN];	/* input tape type			      */
char data_type; 	/* type of data (corrected or uncorrected)    */

double fcenll[2];       /* Full scene center latitude/longitude degrees */
double fcenls[2];       /* Full scene center line/sample              */
double pixsiz[2];	/* image pixel size			      */
double angle;		/* rotation angle 			      */
double proj_corners[8]; /* Projection corners  			      */
double ls_corners[8];	/* Line/sample corners			      */

/* Initialize LAS
-----------------*/
c_inlas(&vblock, &std_out);

/* Retrieve the input parameters
--------------------------------*/
getpar(&vblock, bands, &msgtim, window, &minmax, hosout, taeout, 
       comment, wrtdsk, &numtapes, &numimg_files, dpath);

num_out = numimg_files;

/* If the user specified neither a tape mount comment nor a disk pathname,
   error out. Note that if BOTH are specified, the tape comment will be 
   ignored and the image is expected to be at INFILE.
--------------------------------------------------------------------------*/
if ((strlen(dpath) == 0) && (strlen(comment[0]) == 0))
    c_lasmsg(TRUE, "ltwgin-comment",
	     "Either COMMENT or INFILE must be specified");

/* If the image to be read is on CD, determine the directory structure used
   by that particular CD, and create a host path from the TAE path 
---------------------------------------------------------------------------*/
if (strlen(dpath) > 0) /* user specified a disk path */
    {
    cdflag = 1;
    if (c_cmhost(dpath, hpath) != E_SUCC) /* Create actual host path */
	c_errmsg("Fatal error encountered","ltwgin-fatal", &fatal);
    strcat(hpath,"/");
    det_ds(hpath, &cdflag, sceneid);
    }
else 	
    {
    /* Open the tape
    ---------------*/
    if (mount_tape(&unit, comment, one, &numtapes,ttype) != E_SUCC)
       c_errmsg("Fatal error encountered","ltwgin-fatal",&fatal);
    }

/* Read the volume directory file and write to disk if specified
---------------------------------------------------------------*/
if (getvdf(&unit, &wrtdsk[VDF], hosout, &num_out, &recsize, &numimg_recs, 
           &num_trailrec, order, &numlead, &leadsize, &trailsiz, hpath,
	   &ceosid) != E_SUCC)
   {
   if (!cdflag)
       close_tape(&unit);
   c_clean(&vblock,"ltwgin",hosout,&num_out);
   }

/* Read the leader file and write to disk if specified
-----------------------------------------------------*/
if (getlead(&unit, &wrtdsk[LEAD], hosout, &num_out, &numlead, &leadsize, fcenll,
            fcenls, &path, direction, maprj, pixsiz, &satid, sensor, &angle, 
            &datum, &utmzone, satnam, &data_type, cornlat, cornlong, hpath, 
            ceosid, proj_corners, ls_corners) != E_SUCC)
   {
   if (!cdflag)
       close_tape(&unit);
   c_clean(&vblock,"ltwgin",hosout,&num_out);
   }

/* Process the image data
------------------------*/
if (strncmp(order,"BIL",3) == 0)
   {
   if (proces_bil(hosout, window, &totbnd, &unit, wrtdsk, num_out,
                  numimg_files, bands, &msgtim, &minmax, comment, &recsize,
                  &numimg_recs, &tapenum, &numtapes, &no_lead, ttype,
                  hpath, sensor, ceosid) != E_SUCC)
      {
      if (!cdflag)
	  close_tape(&unit);
      c_clean(&vblock,"ltwgin",hosout,&num_out);
      }
   }
else
   { 
   if (proces_bsq(hosout,window,&totbnd,&unit,wrtdsk,&num_out,&numimg_files,
                  bands,&msgtim,&minmax,comment,&recsize,&tapenum,&numtapes,
                  &no_lead,ttype,hpath,sensor,ceosid) != E_SUCC)
      {
      if (!cdflag)
	  close_tape(&unit);
      c_clean(&vblock,"ltwgin",hosout,&num_out);
      }
   }

/* Get the trailer and supplemental files if requested
------------------------------------------------------*/
if (wrtdsk[TRAI] || wrtdsk[SUPP])
   if (getenf(hosout, &unit, wrtdsk, &num_out, comment, &num_trailrec, &tapenum,
              &trailsiz, &numtapes, &no_lead, satnam, hpath, ceosid) != E_SUCC)
      {
      if (!cdflag)
          close_tape(&unit);
      c_clean(&vblock,"ltwgin",hosout,&num_out);
      }

/* Dismount the tape
--------------------*/
if (!cdflag)
    close_tape(&unit);

/* Update the DDR file 
---------------------*/
if (update_ddr(hosout, window, &totbnd, fcenll, fcenls, &path, direction, maprj,
               pixsiz, &satid, sensor, data_type, &angle, &datum, &utmzone, 
               satnam, cornlat, cornlong, proj_corners, ls_corners, &numimg_files,
               ceosid) != E_SUCC)
   c_errmsg("Nonfatal error encountered","ltwgin-nonfatal",&one);

/* Output the history information
---------------------------------*/
for (i = 0; i < numimg_files; i++)
   if (c_hstout(&vblock,hosout[i],hosout,&zero," ") != E_SUCC)
      c_errmsg("Nonfatal error encountered","ltwgin-nonfatal",&one);

for (i = 0; i < num_out; i++)
   strcpy(taeout[i], hosout[i]); 

/* Issue the completion message
-------------------------------*/
c_complt("LTWGIN",taeout,&num_out);

return EXIT_SUCCESS;
}
