/******************************************************************************
FUNCTION:       PROCES_BSQ

PURPOSE:        Read image data from tape and write it to disk.  The image
                data is BSQ.  Each image line contains LTWG standard 
                12 bytes, suffix and prefix data, and image data.

ALGORITHM REFERENCES    none
*******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "cm.h"
#include "labserv.h"
#include "pixman.h"
#include "tapeio.h"
#include "ltwgin.h"

extern long cdflag;     /* for reading images from CDs               */

int proces_bsq
(
  char hosout[][CMLEN],        /* Output host names */
  long *window,                /* Window of image to be copied */
  long *totbnd,                /* Total number of bands to be copied */
  long *unit,                  /* Tape unit number */
  long *wrtdsk,                /* Ancillary files to read in */
  long *num_out,               /* Number of output files */
  long *numimg_files,          /* Number of output image files */
  long *bands,                 /* Bands to be processed within image */
  long *msgtim,                /* Time between display messages */
  long *minmax,                /* Min/Max flag */
  char comment[][CMLEN],       /* Mount message to the operator */
  long *recsize,               /* Image record length */
  long *tapenum,               /* Tape number mounted */
  long *numtapes,              /* Number of tapes specified */
  long *no_lead,               /* Flag if successive tapes have leader file */
  char ttype[],                /* Tape type */
  char hpath[CMLEN],           /* Path to image file on disk */
  char *sensor,                /* Satellite sensor */
  long ceosid                  /* Code for ground station */
)
{
   extern long curcat;     /* curent tape band being processed          */
   extern long curent;     /* output line being processed               */
   struct FDESC *fdesc[MAXBND]; /* image file descriptors used for image I/O */

   FILE *fp=NULL;          /* file descriptor                           */
   FILE *infp=NULL;        /* image data file (on CD only)              */

   long  access = IWRITE;  /* write access to output image              */
   long  bandoff[MAX_BAND];/* offset into each band to get to image data */
   long  cb;               /* current tape band being processed  */
   long  i;                /* loop counter                              */
   long  idtype;           /* data type of input image                  */
   long  mode = HANDLE;    /* tape I/O error flag                       */
   long  msgflg;           /* Processing message format flag            */
   long  one = 1;          /* constant one                              */
   long  option[2];        /* output option array                       */
   long  outspc[2];        /* output image file specifications          */
   long  status;           /* return status                             */
   long  temp;             /* temp value                                */
   long  dumlong;          /* dummy value                               */
   long  dum2 = FALSE;     /* dummy value                               */
   long  prefix,suffix;    /* number of prefix and suffix pixles        */
   long  curblk;           /* current block within the blocked data     */
   long  recleng;          /* record length of image descriptor record  */
   long  skip;             /* number of files to skip                   */
   long  dmod;             /* part of curblk that is needed for the window */
   long  ptrband = 1;      /* pointer to which band the tape is currently at */
   long  swreel = FALSE;   /* flag if need to switch reels              */
   long  blkfct;           /* blocking factor                           */
   long  bndcnt = 0;       /* number of bands requested                 */
   long  wbnd = 0;
   long  maxbnd;           /* maximum number of bands on the tape/disk  */
   long  nfiles;           /* Number of filenames returned              */

   static long datasize[4] = {0,1,2,4};
                           /* defines number bytes for each data type */

   unsigned char *in_buf;          /* buffer read from image on tape    */
   unsigned char *in_ptr=NULL;     /* pointer to input buffer window    */
   char tempbuf[10];               /* temp work buffer                  */
   char imgfile[CMLEN];            /* name of image file on disk        */
   char ipath[CMLEN];              /* temporary host path for image     */
   char filename[10][CMLEN];       /* Array of filenames extracted      */

   double outmax[MAX_BAND];        /* max pixel values of output image  */
   double outmin[MAX_BAND];        /* min pixel values of output image  */

   /* Allocate memory for the input buffer
   ---------------------------------------*/
   in_buf = (unsigned char *) malloc(*recsize);
   if (in_buf == NULL)
   {
      c_lasmsg(FALSE, "ltwgin-alloc", "Error allocating dynamic memory");
      return(E_FAIL);
   }

   /* If reading from CD, open the image data file 
   -----------------------------------------------*/
   if (cdflag)
   {
      strcpy(imgfile, hpath);
      if (cdflag == NLAPS)
      {
         strcpy(ipath,hpath);
         strcat(ipath,"*.i*");
         c_getfiles(ipath,filename,&nfiles,10,1,10);
         if (nfiles > 0)
            strcpy(imgfile, filename[0]);
      }
      else
         strcat(imgfile, "img_01.001"); 
      infp = fopen(imgfile, "r");
      if (infp == NULL)
      {
         c_lasmsg(FALSE, "ltwgin-diskread",
                  "Error reading initial image data from disk");
         free(in_buf);
         return(E_FAIL);
      }
   }

   /* Read first line.  This is the file descriptor record in the image file. 
      Get the record length.  This is different from the record size if the 
      image data is blocked.
   -------------------------------------------------------------------------*/
   if (cdflag)
      status = (long)fread(in_buf, 1, *recsize, infp);
   else
      status = c_tpread(unit, in_buf, recsize, &mode);

   recleng = atol((char *)&in_buf[186]);
   if (status != recleng)
   {
      if(!cdflag)
      {
         c_lasmsg(FALSE,"ltwgin-diskread",
                  "Error reading file descriptor record from tape");
      }
      else
      {
         c_lasmsg(FALSE,"ltwgin-diskread",
                  "Error reading file descriptor record from disk");
      }
      free(in_buf);
      return(E_FAIL);
   }

   /* If specified, open the image data logical record file and write this
      record.  This file contains the file descriptor record from the
      image file and the prefix and suffix data from the image line.
   ---------------------------------------------------------------*/
   if (wrtdsk[IDR])
   {
      c_lsmknm(hosout[0],"idr",hosout[*num_out]);
      fp = fopen(hosout[(*num_out)++],"w");
      if(fp == NULL)
      {
         c_lasmsg(FALSE, "ltwgin-diskread",
                  "Error reading IDR file");
         free(in_buf);
         return(E_FAIL);
      }
      if(fwrite(in_buf,1,*recsize,fp) != *recsize)
      {
         c_lasmsg(FALSE,"ltwgin-write","Error writing to IDR file");
         free(in_buf);
         return(E_FAIL);
      }
   }

   /* Set offsets into each line to get to image data. 
      xx bytes of prefix data and xx bytes of suffix data.
      12 bytes are for standard LTWG record
   ------------------------------------------------------*/
   if (JAPAN && (strncmp(sensor,"TM",2) == 0))
   {
      strncpy(tempbuf, (char *)&in_buf[280], 4);
      tempbuf[4] = '\0';
      prefix = atol(tempbuf);
   }
   else
      prefix = atol((char *)&in_buf[276]);

   suffix = atol((char *)&in_buf[288]);
   for (i = 0; i < MAX_BAND; i++)
      bandoff[i] = prefix + 12;

   /* Get data type (bits per pixel)
   ---------------------------------*/
   temp = atol((char *)&in_buf[216]);
   switch (temp)
   {
      case 8:  idtype = EBYTE; break;
      default: c_lasmsg(FALSE, "ltwgin-dtype",
                        "Data type not currently handled");
               free(in_buf);
               return(E_FAIL);
   }

   /* Get blocking factor.  Blocking factor was not correctly specified within
      the file descriptor record.  Calculating the blocking factor instead
      should work in all instances.
   ---------------------------------------------------------------------------*/
   /*blkfct = atol(&in_buf[272]);*/
   blkfct = *recsize/recleng;

   /* Get the image size on tape
   ----------------------------*/
   outspc[0]= atol((char *)&in_buf[236]);
   strncpy(tempbuf,(char *)&in_buf[248],8);
   tempbuf[8] = '\0';
   outspc[1]= atol(tempbuf);

   /* Count number of bands the user wants 
   ---------------------------------------*/
   for (i = 0; i < MAX_BAND; i++)
   {
      if (bands[i])
         bndcnt++;
   }
   *totbnd = bndcnt;
   maxbnd = MAX_BAND;

   /* If we have a South African CD, Canadian MSS or Japan 
      MSS data, there are only four bands on the tape/disk.   
   ---------------------------------------------------------*/
   if ((cdflag == SAFRICAMSS) || (ceosid == CAN) ||
       ((JAPAN) && (strcmp(sensor,"MSS") == 0))) 
   {
      maxbnd = 4;
      if (*totbnd > 4)
      {
         c_lasmsg(FALSE,"ltwgin-bands","Only four bands are available"); 
         *totbnd = 4;
      }
   }

   /* If the user wants 1 image file per band, check that the 
      number of output image files specified is equal to totbnd
   ------------------------------------------------------------*/
   if ((*numimg_files != 1) && (*numimg_files != *totbnd))
   {
      c_errmsg("Number of output images must equal number of bands specified",
               "ltwgin-bndcnt",&one);
      free(in_buf);
      return(E_FAIL);
   }

   /* Check the window
   -------------------*/
   chk_window(outspc,window);

   /* Open the output image file(s) for random write access
   --------------------------------------------------------*/
   option[0] = 0;
   option[1] = 0;
   if (*numimg_files == 1)
   {
      if (c_eopenr(&fdesc[0],hosout[0],totbnd,&one,&one,&window[NL],&window[NS],
                   &access,&idtype,option,&one) != E_SUCC)
      {
         free(in_buf);
         return(E_FAIL);
      }
   }
   else
   {
      for (i = 0; i < *numimg_files; i++)
      {
         if (c_eopenr(&fdesc[i],hosout[i],&one,&one,&one,&window[NL],
                      &window[NS],&access,&idtype,option,&one) != E_SUCC)
         {
            free(in_buf);
            return(E_FAIL);
         }
      }
   }

   /* If min max values are to be computed, call inmnmx to
      initialize the min max array for the output data type.
   --------------------------------------------------------*/
   if (*minmax)
      c_inmnmx(&idtype,totbnd,outmin,outmax);

   /* Initialize the processing message
   ------------------------------------*/
   msgflg = ONECAT;
   c_inittm("LTWGIN","Processed","line","band","",&window[NL],totbnd,
            msgtim,&msgflg);

   /* Start processing message
   ---------------------------*/
   curent = 1;
   curcat = 1;
   c_startm();

   /* Reset for processing
   -----------------------*/
   curcat = 0;

   /* Process each requested band  
   ------------------------------*/
   for (cb = 1; cb <= maxbnd; cb++)
   {
      /* Assuming bands are on three tapes - 3,3,1
      --------------------------------------------*/
      if (!cdflag)
      {
         if (((cb == 4) || (cb == 7)) && (strcmp(ttype,"8mm") != 0) &&
              (strcmp(ttype,"3490") != 0))
            swreel = TRUE;
         if ((strcmp(ttype,"8mm") == 0) || (strcmp(ttype,"3490") == 0))
            swreel = FALSE; 
         if (ceosid == CAN)
            swreel = FALSE;
      }

      if (bands[cb-1])
      {
         curcat++;
         curent = 1;
       
         /* Skip to correct band or window as needed
         -------------------------------------------*/
         if (cdflag)
         {
            fclose(infp);
            if (cdskip_bsq (window, &cb, &dmod, &blkfct, infp, recleng,
                            recsize, hpath) != E_SUCC)
            {
               c_lasmsg(FALSE, "ltwgin-skip", "Error skipping records");
               free(in_buf);
               return(E_FAIL);
            }
         } 
         else
         {
            if (skip_bsq(window, unit, comment, tapenum, numtapes, &cb,
                         &dmod, &blkfct, &ptrband, &swreel)  != E_SUCC)
            {
               c_lasmsg(FALSE, "ltwgin-skip", "Error skipping records");
               free(in_buf);
               return(E_FAIL);
            }
         } 

         /* Process each line
         --------------------*/
         for (curent=0; curent < window[NL];) 
         {
            if (cdflag)
            {
               status = (long)fread(in_buf, 1, *recsize, infp);
               if (status != *recsize)
               {
                  c_lasmsg(FALSE, "ltwgin-diskread",
                          "Error reading line %d, band %d from disk", curent,
                          cb);
                  free(in_buf);
                  return(E_FAIL);
               }
            }   
            else    /* if this is a tape  */
            {
               status = c_tpread(unit,in_buf,recsize,&mode);
               if ((status == TAPE_EOV) || (status == TAPE_EOT))
               {
                  /* Close current tape, mount next tape, and read volume 
                     directory file.
                  --------------------------------------------------------*/
                  c_stoptm();
                  close_tape(unit);
                  if (mount_tape(unit, comment, ++(*tapenum), numtapes,ttype)
                      != E_SUCC)
                  {
                     free(in_buf);
                     return(E_FAIL);
                  }

                  if (getvdf(unit, &dum2, hosout, &dumlong, recsize, &dumlong, 
                             &dumlong,tempbuf,&dumlong,&dumlong,&dumlong, 
                             hpath,&ceosid) != E_SUCC)
                  {
                     free(in_buf);
                     return(E_FAIL);
                  }

                  /* Read first line.  This is the file descriptor record in
                     the image file. It is a REPEAT of the file descriptor
                     record on the first tape, so ignore it.
                  ----------------------------------------------------------*/
                  status = c_tpread(unit,in_buf,recsize,&mode);
                  if (status != *recsize)
                  {
                     if (cdflag)
                     {
                        c_lasmsg(FALSE, "ltwgin-diskread",
                                 "Error reading imagery file descriptor"
                                 " record");
                        free(in_buf);
                        return(E_FAIL);
                     }
                     else
                     {
                        c_lasmsg(FALSE, "ltwgin-read",
                                 "Error reading imagery file descriptor"
                                 " record");
                        free(in_buf);
                        return(E_FAIL);
                     }
                  }

                  /* Now read the next line of image data
                  ---------------------------------------*/
                  *no_lead = TRUE;
                  status = c_tpread(unit,in_buf,recsize,&mode);
                  c_startm();
               }

               if (status != *recsize)
               {
                  c_lasmsg(FALSE, "ltwgin-read",
                           "Error reading line %d, band %d from tape", curent,
                           cb);
                  free(in_buf);
                  return(E_FAIL);
               }
            } 

            /* Position the input pointer to the start of the image data.
            ------------------------------------------------------------*/
            for(curblk=dmod; (curblk<blkfct) && (curent < window[NL]); curblk++)
            {
               in_ptr = &in_buf[(recleng * curblk * datasize[idtype]) +
                                bandoff[cb-1] + 
                                ((window[SS] - 1) * datasize[idtype])];

               /* Write the data to the output file
               -------------------------------------*/
               curent++;
               if (*numimg_files == 1)
               {
                  if (c_ewrite(&fdesc[0],&curcat,&curent,in_ptr,&one) == E_FAIL)
                  {
                     c_lasmsg(FALSE, "ltwgin-write",
                              "Error writing output image");
                     free(in_buf);
                     return(E_FAIL);
                  }
               }
               else
               {
                  if (c_ewrite(&fdesc[curcat-1],&one,&curent,in_ptr,&one) 
                               == E_FAIL)
                  {
                     c_lasmsg(FALSE, "ltwgin-write",
                              "Error writing output image");
                     free(in_buf);
                     return(E_FAIL);
                  }
               }
            }
            dmod = 0;

            /* Compute the min/max values if required
            -----------------------------------------*/
            if (*minmax)
               status = c_pxmnmx((long *)in_ptr,&window[NS],&outmin[curcat-1],
                                 &outmax[curcat-1], &idtype);

            /* Write the image data logical record to disk
            ---------------------------------------------*/
            if (wrtdsk[IDR] && (cb == 1))
            {
               in_ptr = &in_buf[12];
               if(fwrite(in_ptr,1,prefix,fp) != prefix)
               {
                  c_lasmsg(FALSE,"ltwgin-write","Error writing to IDR file");
                  free(in_buf);
                  return(E_FAIL);
               }
               in_ptr = &in_buf[(*recsize - suffix)];
               if(fwrite(in_ptr,1,suffix,fp) != suffix)
               {
                  c_lasmsg(FALSE,"ltwgin-write","Error writing to IDR file");
                  free(in_buf);
                  return(E_FAIL);
               }
            }
         }  
      }  

      /* Skip to the next image file.  If this is the last tape,
         skip to the end of the file as needed. 
         Assuming bands are on three tapes - 3,3,1
      ----------------------------------------------------------*/
      if (!cdflag)
      {
         if (ceosid == CAN)
            wbnd = 4;
         else
            wbnd = 3;

         if (((cb != wbnd) && (cb != 6) && (strcmp(ttype,"8mm") != 0) &&
              (strcmp(ttype,"3490") != 0)) ||
              ((strcmp(ttype,"3490") == 0) && (cb != maxbnd)) ||
              ((cb != maxbnd) && (strcmp(ttype,"8mm") == 0))) 
         {
            status = c_tpread(unit,in_buf,recsize,&mode);
            if (status != TAPE_EOF)
               skip = 3;
            else
               skip = 2;

            /* If this is the last tape, set skip as needed
            -----------------------------------------------*/
            if ((strcmp(ttype,"8mm") != 0) && (strcmp(ttype,"3490") != 0) &&
                (ceosid != CAN))
            { 
               if ((wrtdsk[TRAI] || wrtdsk[SUPP]) && (*tapenum == *numtapes))
                  skip -= 2;
            }

            status = c_tpfsf(unit,&skip,&mode);

            if (status != TAPE_SUCC)
            {
               free(in_buf);
               return(E_FAIL);
            }
         }
      }

      if (cdflag == NLAPS)
         skip = 1;

      else if ((cb == maxbnd) && (cdflag != NLAPS))
      {
         skip = 1;
         status = c_tpfsf(unit,&skip,&mode); 
      }
       
      /* At next band 
      ---------------*/
      ptrband = cb + 1;
   }

   /* Close idr file
   -----------------*/
   if (wrtdsk[IDR])
   {
      if(fclose(fp) != 0)
      {
         c_lasmsg(FALSE,"ltwgin-write","Error closing the IDR file");
         free(in_buf);
         return(E_FAIL);
      }
   }

   /* Stop the processing message
   -----------------------------*/
   c_stoptm();

   /* Close the image file(s)
   --------------------------*/
   for (i = 0; i < *numimg_files; i++)
   {
      if (c_eclose(&fdesc[i]) != E_SUCC)
      {
         free(in_buf);
         return(E_FAIL);
      }
   }

   /* Update the min max file for the output image
   -----------------------------------------------*/
   if (*minmax)
   {
      if (c_svmnmx(hosout[0],totbnd,outmin,outmax,"YES") != E_SUCC)
         c_errmsg("Nonfatal error encountered","ltwgin-nonfatal",&one);
   }

   /* Free allocated space
   -----------------------*/
   free(in_buf);

   /* Close file opened for CD read
   ---------------------------------*/
   if ( cdflag)
      fclose(infp);

   return(E_SUCC);
}
