/*****************************************************************************
FUNCTION:       out_frame

PURPOSE:        Calculates a set of coefficients mapping from output space
                image coordinates to output space projection coordinates

*****************************************************************************/
#include <math.h>
#include "resample.h"


int out_frame
(
    double *pixsize,      /* I: Projection distance in x & y                */
    long *window,         /* O: Output window (SL, SS, NL, NS)              */
    double *corners,      /* O: Output window corner coordinates            */
    struct PROJ *outproj, /* I: Output projection parameters                */
    struct PROJ *inproj,  /* I: Input projection parameters                 */
    long frame_mode,      /* I: Type of framing                             */
    double *ul,           /* I: Upper left corner coordinates               */
    double *lr,           /* I: Lower right corner coordinates              */
    long *ls,             /* I: Line/sample coordinates of ul               */
    long nl,              /* I: Number of lines in output space             */
    long ns,              /* I: Number of samples in output space           */
    long units,           /* I: Units code of coordinates entered           */
    double pxmax,         /* I: Maximum X projection coordinate             */
    double pxmin,         /* I: Minimum X projection coordinate             */
    double pymax,         /* I: Maximum Y projection coordinate             */
    double pymin          /* I: Minimum Y projection coordinate             */
)
{
    double inx, iny;        /* Temporary storage of projection min/max        */
    double temp[8];         /* Temporary lat/long values                      */
    double temp_mod;        /* Results from fmod operation                    */

    int flg1 = TRUE,        /* Flags that are set if IN_BREAK                 */
        flg2 = TRUE,
        flg3 = TRUE,
        flg4 = TRUE;

    long status;            /* function status                                */
    long zero = 0;          /* Constant of zero                               */


    /* If geographic corner coordinates were entered, determine the
       lat/long for each corner and find the min box.  Proceed as if
       frame is to be determined from a rectangular geographic space.
       c_transinit is called in getpar prior to calling out_frame.
       -------------------------------------------------------------*/
    if (frame_mode == INPUT_XY)
    {
        /* UL */
        inx = pxmin;
        iny = pymax;
        if ((status = c_trans(&inproj->proj_code, &inproj->units, &zero, &units,
                &inx, &iny, &(temp[1]), &(temp[0]))) != E_SUCC)
        {
            if (status == E_FAIL)
                return (E_FAIL);
            flg1 = FALSE;             /* status = IN_BREAK */
        }

        /* LR */
        inx = pxmax;
        iny = pymin;
        if ((status = c_trans(&inproj->proj_code, &inproj->units, &zero, &units,
                &inx, &iny, &(temp[3]), &(temp[2]))) != E_SUCC)
        {
            if (status == E_FAIL)
                return (E_FAIL);
            flg2 = FALSE;             /* status = IN_BREAK */
        }

        /* LL */
        inx = pxmin;
        iny = pymin;
        if ((status = c_trans(&inproj->proj_code, &inproj->units, &zero, &units,
                &inx, &iny, &(temp[5]), &(temp[4]))) != E_SUCC)
        {
            if (status == E_FAIL)
                return (E_FAIL);
            flg3 = FALSE;             /* status = IN_BREAK */
        }

        /* UR */
        inx = pxmax;
        iny = pymax;
        if ((status = c_trans(&inproj->proj_code, &inproj->units, &zero, &units,
                &inx, &iny, &(temp[7]), &(temp[6]))) != E_SUCC)
        {
            if (status == E_FAIL)
                return (E_FAIL);
            flg4 = FALSE;             /* status = IN_BREAK */
        }

        /* Find the min/max corners.  Only use
           the lat/long if c_trans was successfull
           Find UL Y
           ---------------------------------------*/
        if (flg1 && flg2)
            ul[0] = GMAX(temp[0],temp[2]);
        else if (flg1)
            ul[0] = temp[0];
        else if (flg2)
            ul[0] = temp[2];

        if (flg3)
            ul[0] = GMAX(ul[0],temp[4]);
        if (flg4)
            ul[0] = GMAX(ul[0],temp[6]);

        /* Find UR Y
           ------------*/
        if (flg1 && flg2)
            lr[0] = GMIN(temp[0],temp[2]);
        else if (flg1)
            lr[0] = temp[0];
        else if (flg2)
            lr[0] = temp[2];

        if (flg3)
            lr[0] = GMIN(lr[0],temp[4]);
        if (flg4)
            lr[0] = GMIN(lr[0],temp[6]);

        /* Find UL X
           ------------*/
        if (flg1 && flg2)
            ul[1] = GMIN(temp[1],temp[3]);
        else if (flg1)
            ul[1] = temp[1];
        else if (flg2)
            ul[1] = temp[3];

        if (flg3)
            ul[1] = GMIN(ul[1],temp[5]);
        if (flg4)
            ul[1] = GMIN(ul[1],temp[7]);

        /* Find LR X
           ------------*/
        if (flg1 && flg2)
            lr[1] = GMAX(temp[1],temp[3]);
        else if (flg1)
            lr[1] = temp[1];
        else if (flg2)
            lr[1] = temp[3];

        if (flg3)
            lr[1] = GMAX(lr[1],temp[5]);
        if (flg4)
            lr[1] = GMAX(lr[1],temp[7]);

        /* When remapping an image in Goodes (either North America or
           Eurasia), the UL and UR could both be in the break area.  This
           will check a point in the middle in order to find a max y.  If
           this fails, the user must break the image into small sections and
           resample individually.
           -------------------------------------------------------------------*/

        if (!flg1 && !flg4)
        {
            inx = (pxmax + pxmin)/2;
            iny = pymax;

            if ((status = c_trans(&inproj->proj_code,&inproj->units, &zero,
                    &units, &inx, &iny, &(temp[1]), &(temp[0])))
                    != E_SUCC)
            {
                if (status == E_FAIL)
                    return (E_FAIL);

                /* status = IN_BREAK */
                c_lasmsg(FALSE, "resample-break",
                        "Error determining framing coordinates.");
                return (E_FAIL);
            }

            ul[0] = GMAX(ul[0],temp[0]);
        }

        /* The LL and LR coordinate transformations failed.  The same thing
           as above could be done, but I don't have test data.
           ------------------------------------------------------------------*/
        if (!flg2 && !flg3)
        {
            c_lasmsg(FALSE, "resample-break",
                    "Error determining framing coordinates.");
            return (E_FAIL);
        }

        frame_mode = INPUT;
    }

    if (frame_mode == INPUT)
    {
        /* Determine output projection space minimums and maximums.
           -----------------------------------------------------------*/
        if (c_mapedg(&(ul[1]), &(lr[1]), &(ul[0]), &(lr[0]), &units,
                &(outproj->proj_code), &(outproj->zone_code),
                outproj->proj_coef, &(outproj->units), &pxmin, &pxmax,
                &pymin, &pymax, &(outproj->datum_code)) != E_SUCC)
            return(E_FAIL);

        /* The UL corner is used as the center of the first line/sample.
           Round the LR corner to the nearest multiple of the pixel size,
           ensuring that the computed min/max values are included.
           --------------------------------------------------------------*/
        temp_mod = pymax - pymin;
        temp_mod = fmod(temp_mod, pixsize[0]);
        if (temp_mod/pixsize[0] > 1e-3)
            pymin -= pixsize[0] - temp_mod;
        else
            pymin += temp_mod;
        temp_mod = pxmax - pxmin;
        temp_mod = fmod(temp_mod, pixsize[1]);
        if (temp_mod/pixsize[1] > 1e-3)
            pxmax += pixsize[1] - temp_mod;
        else
            pxmax -= temp_mod;

    }
    else if (frame_mode == OUTPUT_XY)
    {
        /* If output frame was given in projection coordinates
           (two points).
           ------------------------------------------------------*/
        pymax = ul[0] + ((ls[0] - 1) * pixsize[0]);
        pxmin = ul[1] - ((ls[1] - 1) * pixsize[1]);
        pymin = lr[0];
        pxmax = lr[1];
    }
    else if (frame_mode == OUTPUT_LS)
    {
        /* If output frame was given in projection coordinates
           (1 pt & nl, ns).
           ------------------------------------------------------*/
        pymax = ul[0] + ((ls[0] - 1) * pixsize[0]);
        pxmin = ul[1] - ((ls[1] - 1) * pixsize[1]);
        pymin = pymax - ((nl - 1) * pixsize[0]);
        pxmax = pxmin + ((ns - 1) * pixsize[1]);
    }

    /* Set the output window.  The addition of the 1.5 consists of 0.5
       to handle roundoff and 1 to handle the "zeroth" element.
       ------------------------------------------------------------------*/
    window[0] = window[1] = 1;
    window[2] = (long)((pymax - pymin) / pixsize[0] + 1.5);
    window[3] = (long)((pxmax - pxmin) / pixsize[1] + 1.5);

    /* Pack the corner projection coordinates in the frame structure.
       -----------------------------------------------------------------*/
    corners[1] = corners[3] = pxmin;
    corners[0] = corners[4] = pymax;
    corners[6] = corners[2] = pymin;
    corners[5] = corners[7] = pxmax;

    return(E_SUCC);
}
