#include <math.h>
#include <dirent.h>
#include <X11/cursorfont.h>
#include "tiepts.h"
#include "dlgsup.h"
#include "convert.h"
#include "manage_cursor.h"
#include "protos.h"

#define NUMCATS 8

/*******************************************************************************
Purpose:
    This routine handles updates to the DLG display based on area.
By T. Rockvam, USGS/EROS Data Center, 2/94

tbeckman:  Modified to work with the new and old file dlg file naming convention
           by checking each file in the dlg directory to see if it is a feature
           we're looking for that covers the correct area.

*******************************************************************************/
void vector_dlgupdate
(
   Widget        w,
   XtPointer client,
   XtPointer call_data
)
{
    double  minlat,
            maxlat,
            minlon,
            maxlon;

    int     startlatind,
            stoplatind,
            startlatsub,
            stoplatsub,
            lat,
            startlonind,
            stoplonind,
            startlonsub,
            stoplonsub,
            lon,
            anything = FALSE,
            i,j,k,l,
            fnum,
            feature_used[NUMCATS];
    int status;
    char    filename[100];

    static int  feature_index[NUMCATS] = 
       {2,      5,      7,      9,      15,     17,     18,     
        /*19,*/     20};
       /* feature numbers for the following file name lists */
    static const char feature_list[NUMCATS][6] = 
       {"hypso","hydro","cover","bound","pipes","roads","rail",
        /*"mtran",*/"manma"};
       /* old dlg file naming convention information */
    static const char extensions[NUMCATS][4] = 
       {".hp",".hy",".sc",".bd",".mt",".rd",".rr",".ms"};
       /* new dlg file naming convention file extensions */
    DIR *directory;      /* directory structure pointer for searching 
                            directories */
    struct dirent *entry;/* directory entry pointer for searching the 
                            directory */
    int feature;         /* feature counter loop index */
    Window  imgwin;      /* Window ID for main image window.           */
    Cursor  img_cursor;  /* Cursor for image window.                   */

    /*
     *  *** initialize the feature_used array
     */
    for (i = 0; i < NUMCATS; i++)
       feature_used[i] = FALSE;
    i = 0;
    while (dlg_attrib[i].mincode != 0)
    {
       for (j = 0; j < NUMCATS; j++)
       {
           if ((feature_index[j] == dlg_attrib[i].mincode/10000) &&
               (dlg_attrib[i].present == TRUE))
           {
              anything = TRUE;
              feature_used[j] = TRUE;
              break;
           }
       }
       i++;
    }

    /*
     *  *** check that bounds are entered
     */
    if ((strcmp(XmTextGetString(vect_max_lat), "") == 0) ||
        (strcmp(XmTextGetString(vect_max_lon), "") == 0) ||
        (strcmp(XmTextGetString(vect_min_lat), "") == 0) ||
        (strcmp(XmTextGetString(vect_min_lon), "") == 0))
    {
        tie_message(Shell_Vector, "Must define Lat / Lon boundaries");
        return;
    }

    /*
     * *** check that there are some features selected
     */
    if (!anything)
    {
       tie_message(Shell_Vector, "No Feature requested");
       return;
    }

    /*
     *  *** free all previously read vectors
     */
    if (tie_vector->numlines > 0)
       vector_free();

    /* check the files in the dlg directory to see if they are files we
       are interested in
       ----------------------------------------------------------------*/
    directory = opendir(tie_parms.cd);
    if (!directory)
    {
        printf ("Error opening directory %s\n",tie_parms.cd);
        return;
    }

    /* Display the watch cursor in both the image window and main panel.  */
    show_wait_cursor();

    /* loop through the directory */
    while ((entry = readdir(directory)) != NULL)
    {
        char *ext_ptr;
        int name_matched;
        
        name_matched = 0;
        
        /* check for new dlg names */
        ext_ptr = strrchr(entry->d_name,'.');
        if (ext_ptr)
        {
            /* check list of file extensions for a file that matches the
               new dlg file naming convention */
            for (feature=0; feature < NUMCATS; feature++)
            {
                /* skip features that aren't used */
                if (!feature_used[feature])
                    continue;
                    
                if (strcmp(ext_ptr,extensions[feature]) == 0)
                {
                    name_matched = 1;
                    break;
                }
            }
        }

        if (!name_matched)
        {
            /* name didn't match new dlg file names, try the old one */
            for (feature=0; feature < NUMCATS; feature++)
            {
                /* skip features that aren't used */
                if (!feature_used[feature])
                    continue;
                    
                if (strstr(entry->d_name,feature_list[feature]) != 0)
                {
                    name_matched = 1;
                    break;
                }
            }
        }

        if (name_matched)
        {
            /* the name matched one of the naming conventions, so 
               try to read it if it hasn't been loaded previously */
            sprintf(filename,"%s/%s",tie_parms.cd,entry->d_name);
            if (!vector_isloaded(filename))
            {
                status = dlg_optread(filename);
                if (status != E_SUCC)
                {
                    /* issue an error if E_FAIL returned */
                    if (status == E_FAIL)
                    {
                        fprintf(stderr,"Error trying to read %s.\n",filename);
                    }
                    
                    continue;
                }
            }
            else
               continue;

            /* convert all points to image proj coords */
            fnum = tie_vector->numfiles-1;
            for (l = tie_vectfile[fnum].startline; 
                 l <= tie_vectfile[fnum].endline; l++)
            {
                if (vect2image(tie_vector->line[l].pts,
                               tie_vector->line[l].npts, fnum) != E_SUCC)
                {
                    /* conversion failed so skip this file */
                    printf ("The previous error was fatal\n");
                    exit(1);
                }
            }
        }
    }

    closedir(directory);

    if (tie_vector->numlines > 0)
    {
       /*
        * *** find the bounds of all the files
        *     and set the attributes
        */
       vector_dlgbounds();
       vector_dlgattribs();

      /*
       *  *** fit the vectors on the screen
       */
       vector_scaletofit(tie_vector->tmatrix);
       XClearArea(tie.display, XtWindow(ref_image_bulletinboard),0,0,0,0,False);
       vector_redrawall();
    }
    else
       tie_message(Shell_Vector, "No lines found");

    /*
        Redisplay the default arrow cursor.
    */
    unshow_wait_cursor();

}
