/******************************************************************************
NAME:                           PROCES 

PURPOSE:    To copy input image(s) into the output image. The input 
            image(s) can be sub-windowed and sub-banded when copied to 
            the output image.  The input image may also be converted to
            BYTE, INTEGER*2, INTEGER*4, or REAL*4 and written to the 
            output image.

PROGRAM HISTORY:
VERSION    PROGRAMMER      DATE         CODE/CONT       REASON
-------    ----------      ----         ---------       ------
  6.0	   T.Wolbrink	   09/95	     EDC	Development
	   D.Etrheim	   08/95             EDC	Change all messages to TRANSLAS5

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   TRANSLAS5 must be run under TAE.

PROJECT:                        LAS

ALGORITHM:
   Open the input images for read access
   Open the output image for write access
   The data is converted upon input to the data type of the output image
   The one line of each band and each image are processed at the same time.
   Save the minmax values if $MINMAX was set to "YES"
   Update the output ddr
   Close the input and output image files
   ALGORITHM REFERENCES:                none
******************************************************************************/

#include "las.h"

FUNCTION proces(vblock,hosin,hosout,comp, nmrimg,window,bands,nbands,totbnd ,
					odtype,msgtim,minmax)

   struct PARBLK *vblock;       /*  TAE parameter block                      */
   char hosin[][CMLEN];         /* host names of input images                */
   char *hosout;                /* host name of output image                 */
   char *comp;                  /* compression flag                          */
   long nmrimg;                 /* number of input images                    */
   long (*window)[4];           /* window specification of input images      */
   long (*bands)[MAXBND+1];     /* bands to be processed for all images      */
   long *nbands;                /* number of bands per input image           */
   long totbnd;                 /* total number of bands to be processed     */
   long odtype;                 /* data type of output image                 */
   long msgtim;                 /* elapsed time between display messages     */
   char *minmax;                /* min max flag                              */

{
int in_fdesc[MAXIMG];           /* image file descriptors used for image I/O */
int out_fdesc[2];               /* image file descriptors used for image I/O */
int in_gdesc;                   /* group descriptor used for image I/O       */
int out_gdesc;                  /* group descriptor used for image I/O       */
long in_offset[MAXIMG*MAXBND];  /* offsets into inbuf for input bands        */
long out_offset[MAXBND];        /* offsets into outbuf for output bands      */
extern long curent;             /* current processing line                   */
extern long curcat;             /* current processing band                   */

static double  minval[MAXBND];  /* min pixel value array                     */
static double  maxval[MAXBND];  /* max pixel value array                     */
double  line_inc;               /* line increment                            */
double  samp_inc;               /* sample increment                          */

long    access;                 /* image I/O access type                     */
long    band;                   /* loop index counting the bands             */
long    in_bufsiz;              /* size of image input buffer                */
long    out_bufsiz;             /* size of image output buffer               */
long    curlin;                 /* current line being processed              */
long    image;                  /* loop index counting input images          */
long    mmflag;
long    msg_flag=ALLCAT;        /* flag for porcessing message               */
long    nl;                     /* number of image lines                     */
long    ns;                     /* number of image samples                   */
long    one = 1;                /* one                                       */
long    size;                   /* number of bytes per pixel                 */
long    sl;                     /* starting image line                       */
long    ss;                     /* starting image sample                     */
long    status;                 /* function return status code               */
long    upddr_flag;             /* flag for update DDR                       */
long    zero = 0;               /* value of zero                             */
long    opt[3];

char    *c_ealloc();            /* function to allocate image input buffer   */
char    *las50_c_ealloc();      /* function to allocate image output buffer  */
char    *inbuf;                 /* image input buffer                        */
char    *outbuf;                /* image output buffer                       */

/* Open input image(s)
----------------------*/
access = IREAD;
nl = window[0][NL];
ns = window[0][NS];

for (band=0,image=0;image<nmrimg;image++)
    {
    sl = window[image][0];
    ss = window[image][1];
    status=c_eopens(&in_fdesc[image],hosin[image],bands[image],&sl,&ss,&nl,&ns,
    &in_offset[band],&access,&odtype,&zero,&zero);
    if (status != E_SUCC)
        c_lasmsg(TRUE,"translas5-fatal","Fatal error encountered");
    band += nbands[image];
    }
in_fdesc[nmrimg]=0;


/* Open output image
--------------------*/
access = IWRITE;     
opt[0] = 0;
if (strcmp(comp,"YES") == 0)
    opt[1] = COMPRES;
else if (strcmp(comp,"NO") == 0)
    opt[1] = NO_COMPRES;
else
    opt[1] = 1;
opt[2] = 0;
status = las50_c_eopens(&out_fdesc[0],hosout,&totbnd,&one,&one,&nl,&ns,
out_offset, &access,&odtype,&zero,opt);
if (status != E_SUCC)
    c_clean(vblock,"translas5",hosout,&one);
out_fdesc[1] = 0;


/* Allocate input buffer
------------------------*/
if ((inbuf = c_ealloc(in_fdesc,&in_bufsiz)) == NULL)
    {
    c_lasmsg(FALSE,"translas5-alloc","Error allocating dynamic memory");
    c_clean(vblock,"translas5",hosout,&one);
    }

/* Allocate output buffer
-------------------------*/
if ((outbuf = las50_c_ealloc(out_fdesc,&out_bufsiz)) == NULL)
    {
    c_lasmsg(FALSE,"translas5-alloc","Error allocating dynamic memory");
    c_clean(vblock,"translas5",hosout,&one);
    }

/* Group input images
---------------------*/
if (c_egroup(in_fdesc,&in_gdesc,inbuf,&in_bufsiz) != E_SUCC)
    c_clean(vblock,"translas5",hosout,&one);

/* Group output images
----------------------*/
if (las50_c_egroup(out_fdesc,&out_gdesc,outbuf,&out_bufsiz) != E_SUCC)
    c_clean(vblock,"translas5",hosout,&one);

/* Setup for data type
----------------------*/
switch(odtype)
    {
    case EBYTE:  size = 1; break;
    case EWORD:  size = 2; break;
    case ELONG:  size = 4; break;
    case EREAL:  size = 4; break;
    }

/* Setup for processing message utility
---------------------------------------*/
c_inittm("TRANSLAS5","Processed","line","bands"," ",&nl,&totbnd,&msgtim,&msg_flag);
curent = 1; 
c_startm();

/* Initialize min/max arrays
mmflag is set to be used within the processing loop -- quicker than strcmp
-----------------------------------------------------------------------------*/
if (strcmp(minmax,"YES") == 0)
    {
    c_inmnmx(&odtype,&totbnd,minval,maxval);
    mmflag = TRUE;
    }
else
    mmflag = FALSE;

/* Main processing loop
-----------------------*/
if (las50_c_estep(&out_gdesc) != E_SUCC)
	c_clean(vblock,"translas5",hosout,&one);

for (curlin = 0;curlin < nl;curlin++)
    {
    if (c_estep(&in_gdesc) != E_SUCC)
        c_clean(vblock,"translas5",hosout,&one);
    
    for (band=0;band<totbnd;band++)
        {
        c_pxcopy(inbuf + (in_offset[band] * size),
        outbuf + (out_offset[band] * size),&odtype,&ns);
        if (mmflag)
            status = c_pxmnmx(inbuf + (*(in_offset+band) * size),&ns,
            &minval[band],&maxval[band],&odtype);
        }

    if (las50_c_estep(&out_gdesc) != E_SUCC)
        c_clean(vblock,"translas5",hosout,&one);
    curent++;
    }

/* Turn off display messages
----------------------------*/
c_stoptm();

/* Update the DDR
-----------------*/
line_inc = (double)one;
samp_inc = (double)one;
upddr_flag = NOCOMB;
status = c_upddr(vblock,hosin[0],hosout,&nmrimg,window,bands,nbands,&line_inc,
&samp_inc,&totbnd,&upddr_flag);
if (status != E_SUCC)
    c_clean(vblock,"translas5",hosout,&one);

/* Create associated min/max file
---------------------------------*/
if (c_svmnmx(hosout,&totbnd,minval,maxval,minmax) != E_SUCC)
    c_lasmsg(FALSE,"translas5-warn","Nonfatal error encountered");

/* Close all image files in the group
-------------------------------------*/
if (c_egclse(&in_gdesc) != E_SUCC)
    c_lasmsg(FALSE,"translas5-warn","Nonfatal error encountered");

if (las50_c_egclse(&out_gdesc) != E_SUCC)
    c_lasmsg(FALSE,"translas5-warn","Nonfatal error encountered");

/* Free buffer
--------------*/
free(inbuf);
free(outbuf);

return;
}
