/******************************************************************************
FUNCTION:       deletefile

PURPOSE:        Deletes the specified file on disk,
                or disk directory.  If verify = 1, the user is prompted
                to confirm or deny the delete.
                E_SUCC is returned if the delete is successful,
                E_FAIL otherwise.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      11/86    C. Greenhagen       New Development
  1.1      12/87    B. Ailts            replace DESC with char. i
                                        strings
                                        change include file specs.
  5.0      04/88    D. Hollaren         changed include spec to las
                                        changed cerrmsg to c_errmsg and
                                        passed status code by address
                                        SUCCESS/FAIL to E_SUCC/E_FAIL
  5.0      09/88    B. Davis            updated dirlist call, use dkdel 
                                        instead of system calls
  5.1      05/90    B. Davis            Only check for the file in the user 
                                        tape catalog file if it was not found 
                                        on disk because catalog manager allows
                                        is to be only one place.  This saves 
                                        time in locking, reading, and unlocking
                                        the tapelist file.  Also, changed the 
                                        call for marking the tapelist file for 
                                        deleting an entry.
  5.2      11/90    D. Etrheim          Changes for offline deletions
  5.3      01/91    D. Etrheim          Standardized error message handling
  6.0      08/00    T. Beckmann         Removed tape catalog support since
                                        it was never fully implemented and
                                        is not used.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
When deleting a directory;
    translate the host directory name to a tae directory,
    make sure the host file is a directory
    optionally verify that the directory is to be deleted.
    Try to delete the directory report any errors encountered.

When deleting a file on disk;
    make sure the host name is not a directory,
    optionally verify the file is to be deleted
    delete the file.

ALGORITHM REFERENCES    none

Note:   This support module breaks the rule of calling errmsg before returning
        E_FAIL.  
*******************************************************************************/

#include <stdio.h>
#include <string.h>

#ifdef unix
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#endif

#include "las.h"
#include "diskio.h"
#include "labserv.h"
#include "cm.h"

int deletefile
( /* delete the file */
    char *hname,            /* host name of file    */
    char *type,             /* "file" or "dir"      */
    long verify,            /* verify? (yes=1)      */
    long report             /* report it? (yes=1)   */
)
{
long
    status,                                     /* file delete status   */
    zero=0;                                     /* for lsstat           */
int termcode;                                   /* termination code     */

long nonfatal=1;                                /* nonfatal error code  */

char
    msgtxt[CMLEN],                              /* message text         */
    name[CMLEN],                                /* tae name             */
    tmpnam[CMLEN];                              /* tmp file name to hide*/

struct stat buf;                                /* usd for lstat info   */

/************************************************************************/

strcpy(tmpnam," ");

status = E_SUCC;

strcpy(msgtxt,hname);                           /* put host name in     */
                                                /* msgtxt temporarily   */
if (strcmp (type,"dir") == 0)
        strcat (msgtxt,"/");                                    /* unix */

strcpy(name, msgtxt);

/***  delete directory  ***/

if (strcmp (type,"dir") == 0)
    {
    if (!isdir(hname))
        {
        sprintf (msgtxt,"Error: %s is not a directory",hname);
        c_errmsg (msgtxt,"deletefile-notdir",&nonfatal);
        return (E_FAIL);
        }

    if (c_lsstat(hname,&zero) == E_SUCC)        /* delete directory     */
        {
#       ifndef NOTAE
        if (verify)
            {
            sprintf (msgtxt,"Directory %s",name);
            t_write (msgtxt,T_STDCC);
            t_write ("Delete? (Y/N)",T_PROMPT);
            t_read  (msgtxt,&termcode);
            if ((msgtxt[0] != 'Y') && (msgtxt[0] != 'y'))
                return (E_FAIL);
            }
#       endif
            if (hname[strlen(hname)-1] == '/')                  /* unix */
                hname[strlen(hname)-1] = 0;                     /* unix */

            if (c_dkdel (hname) != E_SUCC)                      /* unix */
                {                                               /* unix */
                c_errmsg ("Error returned from c_ckdel","deletefile-call",
                           &nonfatal);                          /* unix */
                sprintf (msgtxt,"Error deleting directory from disk: %s",name);
                c_errmsg (msgtxt,"deletefile-dir",&nonfatal);   /* unix */
                }                                               /* unix */
            else                                                /* unix */
                {                                               /* unix */
                sprintf (msgtxt,"Directory deleted: %s",name);  /* unix */
                c_errmsg (msgtxt," ",&nonfatal);                /* unix */
                }                                               /* unix */

        return (status);
        }
    else
        {
        sprintf (msgtxt,"Specified directory not found: %s",name);
        c_errmsg (msgtxt,"deletefile-nodir",&nonfatal);
        return (E_FAIL);
        }
    }                                           /* end delete directory */

/***  delete file  ***/

else if (strcmp (type,"file") == 0)             /* delete file          */
    {
    if (isdir(hname))
        {
        sprintf (msgtxt,"Error: %s is a directory",hname);
        c_errmsg (msgtxt,"deletefile-isadir",&nonfatal);
        return (E_FAIL);
        }

    if ((c_lsstat (hname,&zero) == E_SUCC) || (lstat(hname,&buf) == 0))
        {                                       /* delete file from disk*/
#       ifndef NOTAE
        if (verify && report)
            {
            sprintf (msgtxt,"File %s",name);
            t_write (msgtxt,T_STDCC);
            t_write ("Delete? (Y/N)",T_PROMPT);
            t_read  (msgtxt,&termcode);
            if ((msgtxt[0] != 'Y') && (msgtxt[0] != 'y'))
                {
                return (E_FAIL);
                }
            }
#       endif

        if (c_dkdel(hname) != E_SUCC)
            status = E_FAIL;

         if (status != E_SUCC)
            {
            c_errmsg("Error returned from c_dkdel","deletefile-call",&nonfatal);
            sprintf (msgtxt,"Error deleting file from disk: %s",name);
            c_errmsg (msgtxt,"deletefile-disk",&nonfatal);
            }

        if ((status == E_SUCC) && report)
            {
            sprintf (msgtxt,"File deleted from disk: %s",name);
            c_errmsg (msgtxt," ",&nonfatal);
            }
        }                                   /* end delete file from disk*/
    else
        {
        status = E_FAIL;
        sprintf (msgtxt,"Specified file not found: %s",name);
        c_errmsg (msgtxt,"deletefile-nofile",&nonfatal);
        }
    }                                           /* end delete file      */

return (status);
}
