/*******************************************************************************
NAME			       C_GETBDR

PURPOSE	     Retrieve specified band dependent record of a DDR file

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
K. Zanter	      Feb. 1988 	original development
B. Ailts	      Apr. 1988		Replaced newlas.h with las.h
				  	Changed the calling sequence -- used
					  only one structure
					Replaced ls___ status to E_SUCC
					Replaced cerrmsg with c_errmsg

D. Akkerman	      Sept. 1989	Added code to convert the minval and
					maxval (double) portions of the BDR
					structure to the floating point format
					of the host system.  This occurs if the
					DDR is read from a "remote" (NFS)
					system or if the DDR was created from
					the "remote" (NFS) system and resides
					on the current host system.
B. Ailts	      Dec. 1990		Updated error messages

PROJECT      LAS

ALGORITHM 
   Create the associated DDR file name from the input host image name.
   Retrieve the host system name.
   Open the DDR file for read access using label services.  
   Read the integer portion of the DDR (record 1).
   Retrieve the system name from the DDR.
   If the host system name and the DDR specified system name are not the same:
	Set the conversion flag to TRUE.
   Set up for and read the specified record the corresponds to the input band #.
   If the conversion flag is set:
	Retrieve the system code (integer) for the host name.
	Retrieve the system code (integer) for the DDR specified system.
	Convert the double precision elements of the BDR record to the floating
	 point format of the host system.
   Close the DDR file.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "pixman.h"
#include "sysdef.h"
#include "util.h"

int c_getbdr
(
   char *hname,        /* I:  input host image name                       */
   struct BDDR *bddr,  /* O:  output band depenent record                 */
   long *band          /* I:  specified band number                       */
)

{
struct DDR ddr;		       /* working DDR structure			      */

long	access;		       /* file access type			      */
long	action;		       /* file close action flag		      */
long	clen;		       /* length of character part of record	      */
long	ctemp;		       /* temporary storage for clen		      */
long	dlen;		       /* length of data part of record	      	      */
long	dtemp;		       /* temporary storage for dlen		      */
long	fd;		       /* file descriptor for opened file	      */
long    one = 1;	       /* non-fatal error code for c_errmsg	      */
long    status;
long	size = DBSIZE;	       /* number of doubles in the BDR		      */
long	flag = SYS_SET;	       /* flag that sets the doubles in the BDR to the
				  maximum size of the host machine if overflow
				  or underflow is detected in the conversion
				  process				      */
long	dtyp = EDOUBLE;        /* data type of the buffer to be converted     */
long	convflg = FALSE;       /* working conversion flag		      */
long	under, over;	       /* dummy underflow and overflow flags	      */
long	incode;		       /* host system code			      */
long	syscode;	       /* DDR system code			      */
long	sysflg = SETN;	       /* system conversion flag (string to int)      */

unsigned char *dbuf;	       /* pointer to area where data is stuffed       */

char dtype[3];	       	       /* type of data portion of record	      */
char hostddr[CMLEN];           /* host name of DDR file			      */
char key[16];	       	       /* record key				      */
char tempchar[DDTSLN];	       /* character portion of the record	      */
char *tempptr;
char insys[DDSYLN];	       /* host system name			      */

c_lsmknm(hname,"ddr",hostddr);	        /* make associated DDR file name */

if (c_getsys(insys) != E_SUCC)		/* retrieve the host system name */
   {
   c_errmsg("Error returned from getsys","getbdr-call",&one);
   return(E_FAIL);
   }

access = 0;				/* open DDR file for read */
if (c_lsopen(&fd,hostddr,&access) != E_SUCC)
   {
   c_errmsg("Error returned from lsopen","getbdr-call",&one);
   return(E_FAIL);
   }

clen = DDSTCT * DDSYLN;	       /* set up and read record 1 of the DDR */
ctemp = clen - 1;
dlen = dtemp = DISIZE * 4;
dbuf = (unsigned char *) &ddr;
strcpy(key,"DDRINT");
status = c_lsread(&fd,key,&clen,&dlen,ddr.system,dbuf,dtype);
if (status != E_SUCC)
   {
   c_errmsg("Error returned from lsread","getbdr-call",&one);
   return(E_FAIL);
   }
if ((clen != ctemp) || (dlen != dtemp))
   {
   c_errmsg("Error reading record of associated DDR file","getbdr-read",&one);
   return(E_FAIL);
   }

if (strcmp(insys,ddr.system) != 0)	/* if the DDR system and the host    */
   convflg = TRUE;			/* system are not the same then set  */
					/* the conversion flag		     */

sprintf(key,"BAND%d",*band);   /* set up for and read in specified band record*/
clen = DDTSLN;
ctemp = clen - 1;
dlen = dtemp = DBSIZE * 8;
dbuf = (unsigned char *) &(bddr->minval);
status = c_lsread(&fd,key,&clen,&dlen,tempchar,dbuf,dtype);
if (status != E_SUCC)
   {
   c_errmsg("Error returned from lsread","getbdr-call",&one);
   return(E_FAIL);
   }

if ( (clen != ctemp) || (dlen != dtemp) )
   {
   c_errmsg("Error reading record of associated DDR file","getbdr-read",&one);
   return(E_FAIL);
   }

if (convflg)				/* if the conversion flag is set */
   {
   if (c_sysset(insys,&incode,&sysflg) != E_SUCC) /* retrieve the host system */
      {
      c_errmsg("Error returned from sysset","getbdr-call",&one);
      return(E_FAIL);
      }

   if (c_sysset(ddr.system,&syscode,&sysflg) != E_SUCC) /* retrieve the DDR   */
      {
      c_errmsg("Error returned from sysset","getbdr-call",&one);
      return(E_FAIL);
      }

   if (c_pxsys(&syscode,&incode,dbuf,&dtyp,&size,&under,&over,
	       &flag) != E_SUCC)		  /* perform the conversion   */
      {
      c_errmsg("Error returned from pxsys","getbdr-call",&one);
      return(E_FAIL);
      }
   }

sscanf(tempchar,"%4d%2d",&(bddr->bandno),&(bddr->valid));
tempptr = tempchar + DDBNLN + DDVLLN;
strcpy(bddr->source,tempptr);
tempptr += DDSRLN;
strcpy(bddr->instrument,tempptr);
tempptr += DDINLN;
strcpy(bddr->direction,tempptr);
tempptr += DDDRLN;
strcpy(bddr->date,tempptr);
tempptr += DDCDLN;
strcpy(bddr->time,tempptr);

action = 0;				 /* close associated DDR file */
if (c_lsclos(&fd,hostddr,&action) != E_SUCC)
   {
   c_errmsg("Error returned from lsclos","getbdr-call",&one);
   return(E_FAIL);
   }

return(E_SUCC);
}
