/*******************************************************************************
FUNCTION:	c_getddr

PURPOSE:	Retrieve records 1 and 2 of a DDR file

Programmer       Date   Modification
---------------  -----  --------------------------------------------------------
K.Zanter	 01/88  Development
B.Ailts		 04/88  Replace newlas.h with las.h.  Changed the calling
			sequence -- only one structure.  Replaced the ls___
			status checks with the constant E_SUCC.  Replaced
			cerrmsg with c_errmsg.
D.Akkerman	 09/89  Added code to convert the double portions of the second
			record to the floating point format of the host system.
			This occurs if the DDR is read from a "remote" (NFS)
			system or if the DDR was created from a "remote" (NFS)
			system and resides on the current host system.
T.Baltzer	 08/90  Added code to convert the long portions of the data to
			the long format of the current system.
B.Ailts		 12/90  Updated error messages.
D.Etrheim	 11/91  Fix error message.
D.Etrheim	 03/93  Make proj_units lower case.
J.Fenno		 05/95  Modifications to remove all invalid memory accesses.

PROJECT       LAS

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "pixman.h"
#include "sysdef.h"
#include "util.h"


int c_getddr
(
   char *hname,     /* I:   Host image file name                          */
   struct DDR *ddr  /* O:  DDR structure                                  */
)

{
char cbuffer[DDSTCT*DDSYLN];	/* Character buffer for reading from the DDR  */
char datatype[5];		/* Data type returned from c_lsread()         */
char hostddr[CMLEN];		/* Host file name of the DDR                  */
char systype[DDSYLN];		/* Current system type ($DATASYS)             */
char key[16];			/* DDR key */

double dbuffer[DDSIZE];		/* Double buffer for reading from the DDR     */

long access;			/* File access flag                           */
long clen;			/* Length of the character buffer             */
long ddrcode;			/* System code coming from the DDR being read */
long dlen;			/* Length of the data buffer (long or double) */
long dtype;			/* Data type of the buffer sent to supports   */
long fd;			/* File descriptor                            */
long i;				/* Loop control                               */
long lbuffer[DISIZE];		/* Long buffer for reading from the DDR       */
long nonfatal = 1;		/* Nonfatal error flag                        */
long over;			/* Overflow flag for c_pxsys()                */
long status;			/* Function return status                     */
long syscode;			/* Integer code for current type of system    */
long tmp;			/* Temporary value                            */
long under;			/* Underflow flag for c_pxsys()               */

short convert;			/* Conversion flag                            */



/*
  Get the type of system we are currently running on ($DATASYS)
*/
status = c_getsys(systype);
if (status != E_SUCC)
    {
    c_errmsg("Error returned from getsys", "getddr-call", &nonfatal);
    return(E_FAIL);
    }


/*
  Create the host name of the DDR file
*/
c_lsmknm(hname, "ddr", hostddr);


/*
  Does the DDR exist?
*/
access = 0;
status = c_lsstat(hostddr, &access);
if (status != E_SUCC)
    {
    c_errmsg("DDR file does not exist on disk", "getddr-exist", &nonfatal);
    return(E_FAIL);
    }


/*
  Open the DDR file for reading
*/
access = 0;
status = c_lsopen(&fd, hostddr, &access);
if (status != E_SUCC)
    {
    c_errmsg("Error returned from lsopen", "getddr-call", &nonfatal);
    return(E_FAIL);
    }


/*
  Read the first record of the DDR.  This is the "integer" record (ie. DDRINT).
*/
clen = sizeof(cbuffer);
dlen = sizeof(lbuffer);
strcpy(key,"DDRINT");
status = c_lsread(&fd, key, &clen, &dlen, cbuffer,
    (unsigned char *)lbuffer, datatype);
if (status != E_SUCC)
    {
    c_errmsg("Error returned from lsread", "getddr-call", &nonfatal);
    return(E_FAIL);
    }


/*
  Ensure the buffer sizes are what we expect
*/
if ((clen != (sizeof(cbuffer) - 1)) || (dlen != sizeof(lbuffer)))
    {
    c_errmsg("Error reading record 1 of associated DDR file",
	"getddr-read", &nonfatal);
    return(E_FAIL);
    }


/*
  Extract the DDR's character fields from the character buffer
*/
strncpy(ddr->system, &cbuffer[DDSYLN*0], DDSYLN);
ddr->system[DDSYLN-1] = '\0';

strncpy(ddr->proj_units, &cbuffer[DDSYLN*1], DDSYLN);
ddr->proj_units[DDSYLN-1] = '\0';

strncpy(ddr->last_used_date, &cbuffer[DDSYLN*2], DDSYLN);
ddr->last_used_date[DDSYLN-1] = '\0';

strncpy(ddr->last_used_time, &cbuffer[DDSYLN*3], DDSYLN);
ddr->last_used_time[DDSYLN-1] = '\0';


/*
  Determine if the current system type is the same as where the DDR was created
*/
if (strcmp(systype, ddr->system))
    convert = TRUE;
else
    convert = FALSE;



/*
  If necessary, convert the integer portion of the first record.
*/
if (convert)
    {
    /*
      Convert the current system type string to an integer constant
    */
    tmp = SETN;
    status = c_sysset(systype, &syscode, &tmp);
    if (status != E_SUCC)
	{
	c_errmsg("Error returned from sysset", "getddr-call", &nonfatal);
	return(E_FAIL);
	}
    

    /*
      Convert the DDR's system type string to an integer constant
    */
    tmp = SETN;
    status = c_sysset(ddr->system, &ddrcode, &tmp);
    if (status != E_SUCC)
	{
	c_errmsg("Error returned from sysset", "getddr-call", &nonfatal);
	return(E_FAIL);
	}


    /*
      Convert the integer portion of the first record
    */
    dtype = ELONG;
    dlen = DISIZE;
    status = c_pxsys(&ddrcode, &syscode, (unsigned char *)lbuffer, &dtype, 
                     &dlen, &under, &over, &tmp);
    if (status != E_SUCC)
	{
	c_errmsg("Error returned from pxsys", "getddr-call", &nonfatal);
	return(E_FAIL);
	}
    

    /*
      Update the DDR because it is now converted to the current system
    */
    strcpy(ddr->system, systype);
    }


/*
  Extract the DDR's integer fields from the integer buffer
*/
tmp = 0;
ddr->nl            = lbuffer[tmp++];
ddr->ns            = lbuffer[tmp++];
ddr->nbands        = lbuffer[tmp++];
ddr->dtype         = lbuffer[tmp++];
ddr->master_line   = lbuffer[tmp++];
ddr->master_sample = lbuffer[tmp++];
for (i = 0; i < DDNVAL; i++)
    ddr->valid[i]  = lbuffer[tmp++];
ddr->proj_code     = lbuffer[tmp++];
ddr->zone_code     = lbuffer[tmp++];
ddr->datum_code    = lbuffer[tmp++];
ddr->spare         = lbuffer[tmp++];




/*
  Read the second record of the DDR.  This
  is the "double" record (ie. "DDRDUB").
*/
clen = 0;
dlen = sizeof(dbuffer);
strcpy(key,"DDRDUB");
status = c_lsread(&fd, key, &clen, &dlen, cbuffer,
    (unsigned char *)dbuffer, datatype);
if (status != E_SUCC)
    {
    c_errmsg("Error returned from lsread", "getddr-call", &nonfatal);
    return(E_FAIL);
    }


/*
  Ensure the buffer sizes are what we expect
*/
if ((clen != 0) || (dlen != sizeof(dbuffer)))
    {
    c_errmsg("Error reading record 2 of associated DDR file",
	"getddr-read", &nonfatal);
    return(E_FAIL);
    }


/*
  If necessary, convert the double portion of the second record.
*/
if (convert)
    {
    dtype = EDOUBLE;
    dlen = DDSIZE;
    status = c_pxsys(&ddrcode, &syscode, (unsigned char *)dbuffer, &dtype, 
                     &dlen, &under, &over, &tmp);
    if (status != E_SUCC)
	{
	c_errmsg("Error returned from pxsys", "getddr-call", &nonfatal);
	return(E_FAIL);
	}
    }


/*
  Extract the DDR's double fields from the double buffer
*/
tmp = 0;
for (i = 0; i < 15; i++)
    ddr->proj_coef[i] = dbuffer[tmp++];
for (i = 0; i < 2; i++)
    ddr->upleft[i] = dbuffer[tmp++];
for (i = 0; i < 2; i++)
    ddr->loleft[i] = dbuffer[tmp++];
for (i = 0; i < 2; i++)
    ddr->upright[i] = dbuffer[tmp++];
for (i = 0; i < 2; i++)
    ddr->loright[i] = dbuffer[tmp++];
ddr->pdist_y = dbuffer[tmp++];
ddr->pdist_x = dbuffer[tmp++];
ddr->line_inc = dbuffer[tmp++];
ddr->sample_inc = dbuffer[tmp++];



/*
  Convert the projection units to lowercase
*/
clen = (long)strlen(ddr->proj_units);
c_up2low(ddr->proj_units, &clen);


/*
  Close the DDR file
*/
access = 0;
if (c_lsclos(&fd, hostddr, &access) != E_SUCC)   
    {
    c_errmsg("Error returned from lsclos", "getddr-call", &nonfatal);
    return(E_FAIL);
    }

return(E_SUCC);
}
