/******************************************************************************

NAME:   MINMAX

FUNCTION:
	MINMAX is an application callable function which is called to retreive
	or calculate the minimum and maximum pixel values of an image.  

PROGRAM HISTORY:
	Version		Date	Author		Request
	-------		----	-----		-------
	  1.0		5/86	B.Ailts	   	initial development
          2.0		9/86	K. Gacke	use host file names rather than
						tae file names
	  3.0		7/88	B.Ailts		LAS 5.0 standards
						Added nimg and nbands as 
						  arguments
						Changed sl ss nl ss to window
	  4.0		3/89	B.Ailts		Now does not return a fatal 
						error when the ddr could not
						be updated because of protection

    	  5.0		8/90	D. Akkerman	Added code to automatically 
						convert the image DDR to the 
						current system format if the 
						DDR is from an alien system.
						This solves the problem of 
						networked (NFS) DDRs that 
						reside on systems with 
						different real data formats.
    	  5.1		12/90	B. Ailts        Updated error messages
	  5.2		10/92	T. Mittan	Malloced one more element to 
						cl_bands to correctly work on 
						MAXBND images.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
		Run under TAE

PROJECT:   LAS

ALGORITHM DESCRIPTION:

Check the DDR system fields and convert the DDRs to the local host
 format if necessary.
The space needed for the cl_bands array is dynamically allocated
The space needed for the mmbands array is dynamically allocated
For each input image
   The current image size is retreived
   The min and max arrays are initialized
   If the window specified are the full image
      CKMNMX is called to read the ddr file 
      If valid min and max values of the specified bands are not in the DDR file
         CLMNMX is called to calculate them for each band that is needed
         UPMNMX is called to update the min/max values in the DDR file
   Else
      CLMNMX is called to calculate them for each band that is needed
The memory for the cl_bands array is freed
The memory for the mmbands array is freed
These values are then passed back to the calling program.

ALGORITHM REFERENCES:  none

*****************************************************************************/
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "sysdef.h"
#include "util.h"

int c_minmax
(
   char    hname[][CMLEN],      /* I:  string descriptor of host name     */
   long    *nimg,               /* I:  total number of images             */
   long    window[][4],         /* I:  starting line of image             */
   long    bands[][MAXBND + 1], /* I:  array of band numbers needed       */
   long    *nbands,             /* I:  number of bands for each image     */
   double  *minval,             /* O:  minimum value array                */
   double  *maxval              /* O:  maximum value array                */
)

{
struct  DDR ddr;	/*  ddr structure				*/
long	band=0;		/*  total number of bands needed		*/
long    *cl_bands;	/*  bands that min/max has to be calculated     */
long    curbnd;		/*  current band recalculated min/max bands	*/
long	i;		/*  counter--current image			*/
long    j;		/*  counter--currnet band		  	*/
long    k;		/*  counter--pseudo band			*/
long    *mmbands;	/*  bands of the min/max arrays to be used	*/
long    nonfatal = 1;	/*  nonfatal error code				*/
long	status;		/*  status flag					*/
long    valid[MAXBND];  /*  valid flag of min/max array			*/


/*  Check the DDR system fields and convert the DDRs to the local host
    format if necessary.
------------------------*/

for (i = 0; i < *nimg; i++)
 {
  status = ddrconv(hname[i]);
  if (status != E_SUCC)
    return(status);
 }

/*  Allocate memory for cl_bands
--------------------------------*/
cl_bands = (long *)calloc(MAXBND + 1,4);
if (cl_bands == 0)
   {
   c_errmsg("Error allocating dynamic memory","minmax-alloc",&nonfatal);
   return(E_FAIL);
   }

/*  Allocate memory for mmbands
-------------------------------*/
mmbands = (long *)calloc(MAXBND,4);
if (mmbands == 0)
   {
   c_errmsg("Error allocating dynamic memory","minmax-alloc",&nonfatal);
   return(E_FAIL);
   }

for (i = 0; i < *nimg; i++)
   {
   /* Retreive the full image size
   -------------------------------*/
   status = c_getddr(hname[i],&ddr);
   if (status != E_SUCC)
      {
      c_errmsg("Error returned from getddr","minmax-call",&nonfatal);
      return(E_FAIL);
      }

   /*  Initialize the min/max arrays for dtype of input image
   ----------------------------------------------------------*/
   status = c_inmnmx(&ddr.dtype,&nbands[i],&minval[band],&maxval[band]);
   if (status != E_SUCC)
      {
      c_errmsg("Error returned from inmnmx","minmax-call",&nonfatal);
      return(E_FAIL);
      }

   if ((window[i][0] == 1) && (window[i][1] == 1) &&
       (window[i][2] == ddr.nl) && (window[i][3] == ddr.ns))
      {
      /*  Try to retreive valid min/max values from the DDR file
      ----------------------------------------------------------*/
      status = ckmnmx(hname[i],bands[i],nbands[i],&minval[band],&maxval[band],
		      &valid[band]);
      if (status != E_SUCC)
         return(E_FAIL);

      /*  Create a temporary array of bands that the min/max values were not
          valid in the DDR file
      ----------------------------------------------------------------------*/
      for (k = 0, j = band, curbnd = 0; k < nbands[i]; k++, j++)
         if (valid[j] != VALID)
 	    {
            *(cl_bands + curbnd) = bands[i][k];
            *(mmbands + curbnd) = k;
	    curbnd ++;
	    }
      cl_bands[curbnd] = 0;

      /*  Calculate the min/max values for the specified bands
      --------------------------------------------------------*/
      status = clmnmx(hname[i],window[i],cl_bands,mmbands,&minval[band],
		      &maxval[band]);
      if (status != E_SUCC)
         return(E_FAIL);

      /*  Update the min/max and valid fields of the DDR file
      -------------------------------------------------------*/
      status = upmnmx(hname[i],cl_bands,mmbands,&minval[band],&maxval[band]);
      if (status == E_FAIL)
         return(E_FAIL);

      }
   else  /*  The full image was not specified  */
      {
      for (k = 0; k < nbands[i]; k++)
         {
         *(cl_bands + k) = bands[i][k];
	 *(mmbands + k) = k;
         }
      cl_bands[k] = 0;

      /*  Calculate the min/max values of the window specified
      --------------------------------------------------------*/
      status = clmnmx(hname[i],window[i],cl_bands,mmbands,&minval[band],
		      &maxval[band]);
      if (status != E_SUCC)
         return(E_FAIL);
      }
   band += nbands[i];
   }
 
/*  Free the space needed for the temporary array cl_bands
----------------------------------------------------------*/
free(cl_bands);

/*  Free the space needed for the temporary array mmbands
---------------------------------------------------------*/
free(mmbands);

return(E_SUCC);
}






int ddrconv
(
    char *hostname	/* I:  Host name of the image file.               */
)

/******************************************************************************
 *									      *
 *	Purpose:  This routine is designed to check the DDR of the specified  *
 *		  image and insure that it has the same real data format as   *
 *		  the current host system.  If the format is different, then  *
 *		  the real data portions of the DDR are converted to the      *
 *		  format of the current host system.  This allows the MINMAX  *
 *		  support routine to function correctly in those instances    *
 *		  when an "alien" DDR is selected for min/max update (i.e.    *
 *		  NFS).							      *
 *									      *
 *	By D. Akkerman, USGS/EROS Data Center, 8/90.			      *
 *									      *
 *	Algorithm(flow):						      *
 *									      *
 *	Create the associated DDR file name.				      *
 *	Open the DDR for read.						      *
 *	Read the main DDR record.					      *
 *	Close the DDR file.						      *
 *	Retrieve the host system name.					      *
 *	If the DDR system name and the host system name are not the same then *
 *	 convert the DDR to the format of the host system.		      *
 *	 Check the DDR file protection.					      *
 *	 Convert the first two records of the DDR.  Note that this conversion *
 *	 relies on the c_getddr routine to convert the records when they are  *
 *	 read.								      *
 *	 Convert the band records of the DDR.  Note that this conversion also *
 *	 relies on the c_getbdr routine to convert the records when they are  *
 *	 read.								      *
 *	 Now create a new ddr with the converted data fields.		      *
 *	 Free the dynamically allocated memory.				      *
 *	Return.								      *
 *									      *
 ******************************************************************************/
{
struct BDDR *bddr;	/* Pointer for the band data of the DDR.	*/

struct DDR ddr;		/* Main DDR record.				*/

char 	hostddr[CMLEN],	/* Host ddr name.				*/
	dtype[3],	/* Type of data portion of record.		*/
	insys[DDSYLN];	/* Host system name.				*/


unsigned char *dbuf;	/* Pointer to area where data is stuffed.	*/

long 	mode,		/* lsstat mode.					*/
	access,		/* File access type.				*/
	i,j,		/* Working index variables.			*/
	fd,		/* File descriptor for opened file.		*/
	clen,		/* Length of character part of record.		*/
	ctemp,		/* Temporary storage for clen.			*/
	dlen,		/* Length of data part of record.		*/
	dtemp,		/* Temporary storage for dlen.			*/
	status,		/* Return status variable.			*/
	nonfatal = 1;	/* Errmsg status code.				*/

char key[16];

/* Create the associated DDR file name.					*/

#ifdef vms
c_lsmknm(hostname,"ddr;1",hostddr);
#else
c_lsmknm(hostname,"ddr",hostddr);
#endif

/* Open the DDR for read.						*/

access = 0;
if (c_lsopen(&fd,hostddr,&access) != E_SUCC)
   {
   c_errmsg("Error returned from lsopen","minmax-call",&nonfatal);
   return(E_FAIL);
   }

/* Read the main DDR record.						*/

clen = DDSTCT * DDSYLN;
ctemp = clen - 1;
dlen = dtemp = DISIZE * 4;
dbuf = (unsigned char *)&ddr;

strcpy(key,"DDRINT");
if (c_lsread(&fd,key,&clen,&dlen,ddr.system,dbuf,dtype) != E_SUCC)
   {
   c_errmsg("Error returned from lsread","minmax-call",&nonfatal);
   return(E_FAIL);
   }

if ((clen != ctemp) || (dlen != dtemp))
 {
  c_errmsg("Error reading record 1 of associated DDR file","minmax-read",
           &nonfatal);
  return(E_FAIL);
 }

/* Close the DDR file.							*/

if (c_lsclos(&fd,hostddr,&access) != E_SUCC)
   {
   c_errmsg("Error returned from lsclos","minmax-call",&nonfatal);
   return(E_FAIL);
   }

/* Retrieve the host system name.					*/

if (c_getsys(insys) != E_SUCC)
   {
   c_errmsg("Error returned from getsys","minmax-call",&nonfatal);
   return(E_FAIL);
   }

/* If the DDR system name and the host system name are not the same then
   convert the DDR to the format of the host system.			*/

if (strcmp(insys,ddr.system) != 0)
 {
  /* Check the DDR file protection.					*/

  mode = 2;
  status = c_lsstat(hostddr,&mode);
  if (status != E_SUCC)
     return(E_PROT);

  /* Convert the first two records of the DDR.  Note that this conversion
     relies on the c_getddr routine to convert the records when they are
     read.								*/

   status = c_getddr(hostname,&ddr);
   if (status != E_SUCC)
      {
      c_errmsg("Error returned from getddr","minmax-call",&nonfatal);
      return(E_FAIL);
      }

  /* Allocate space to contain the band records.			*/

  bddr = (struct BDDR *)calloc(ddr.nbands,sizeof(struct BDDR));
  if (bddr == NULL)
   {
    c_errmsg("Error allocating dynamic memory","minmax-alloc",&nonfatal);
    return(E_FAIL);
   }

  /* Convert the band records of the DDR.  Note that this conversion also
     relies on the c_getbdr routine to convert the records when they are
     read.								*/

  for (i = 0, j = 1; i < ddr.nbands; i++, j++)
   {
    status = c_getbdr(hostname,&bddr[i],&j);
    if (status != E_SUCC)
      {
      c_errmsg("Error returned from getbdr","minmax-call",&nonfatal);
      return(E_FAIL);
      }
   }

  /* Now create a new ddr with the converted data fields.		*/

  status = c_putddr(hostname,&ddr);
  if (status != E_SUCC)
      {
      c_errmsg("Error returned from putddr","minmax-call",&nonfatal);
      return(E_FAIL);
      }

  for (i = 0; i < ddr.nbands; i++)
   {
    status = c_putbdr(hostname,&bddr[i]);
    if (status != E_SUCC)
       {
       c_errmsg("Error returned from putbdr","minmax-call",&nonfatal);
       return(E_FAIL);
       }
   }

  /* Free the dynamically allocated memory.				*/

  free(bddr);
 }

return(E_SUCC);
}
