/*******************************************************************************
NAME			      C_PUTDDR

PURPOSE	     Output records 1 and 2 of a DDR file

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
K. Zanter	      Jan. 1988 	Original development
B. Ailts	      Apr. 1988		Replaced newlas.h with las.h
					Changed the calling sequence -- only
					  one structure
					Replaced the ls___ status checks with
					  the constat E_SUCC
					Replaced cerrmsg with c_errmsg
D. Akkerman	      Jan. 1990		Installed code to ensure that
					required parameters are specified.
B. Ailts	      Dec. 1990		Updeated error messages

PROJECT       LAS

ALGORITHM 
   Ensure that required parameters were specified.
   Create the associated DDR file name from the input host image name.
   Open the DDR file for write access using label services.  
   Write the DDRINT record (first record) 
   Write the DDRDUB record (second record).  
   Close the DDR file.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "diskio.h"
#include "labserv.h"


int c_putddr
(
   char *hname,      /* I:  host image file name                         */
   struct DDR *ddr   /* I:  ddr data in record 1 and 2                   */
)

{
long   access;			 /* file access type		              */
long   action;		         /* file close action		              */
long   clen;		         /* length of char part of record             */
long   dlen;			 /* length of data part of record    	      */
long   fd;			 /* file descriptor 		              */
long   one = 1;			 /* nofatal error status		      */
/*
#ifdef UNIX_DISK
long nbytes;	
#endif
*/

unsigned char *dbuf;		 /* pointer to area where data is stuffed     */

char d_temp[DDSTCT][DDSYLN];	 /* temporary for squeezed strings	      */
char *temp;	 /* temporary for squeezed strings	      */
char hostddr[CMLEN];         	 /* host name of DDR file		      */
char *junk_temp;

/* Ensure that required parameters were specified.
--------------------------------------------------*/

if (ddr->nl < 1)
 {
  c_errmsg("Invalid number of lines specified","putddr-badnl",&one);
  return(E_FAIL);
 }

if (ddr->ns < 1)
 {
  c_errmsg("Invalid number of samples specified","putddr-badns",&one);
  return(E_FAIL);
 }

if ((ddr->nbands < 1) || (ddr->nbands > MAXBND))
 {
  c_errmsg("Invalid number of bands specified","putddr-badbnds",&one);
  return(E_FAIL);
 }

if ((ddr->dtype < 1) || (ddr->dtype > 4))
 {
  c_errmsg("Invalid data type specified","putddr-bdtype",&one);
  return(E_FAIL);
 }

if ((strcmp(ddr->system,"")) == 0)
 {
  c_errmsg("Bad system specified","putddr-badsys",&one);
  return(E_FAIL);
 }

#ifdef vms				  /* create associated DDR file name  */
c_lsmknm(hname,"ddr;1",hostddr);
#else
c_lsmknm(hname,"ddr",hostddr);
#endif
 
/*
#ifdef UNIX_DISK
 Calculate the number of bytes needed for the DDR file
---------------------------------------------------------
nbytes = HDRL + (DISIZE * 4) + (DDSTCT * DDSYLN);
nbytes = nbytes + HDRL + (DDSIZE * 8); 			
nbytes = nbytes + ((HDRL + DDTSLN + (DBSIZE * 8)) * ddr->nbands);
if (c_dkcre(0,hostddr,nbytes,0) != E_SUCC)
   {
   c_errmsg("Error returned from dkcre","putddr-call",&one);
   return(E_FAIL);
   }
#endif
*/

access = 1;				  /* open DDR file for write access   */
if (c_lsopen(&fd,hostddr,&access) != E_SUCC)	   
   {
   c_errmsg("Error returned from lsopen","putddr-call",&one);
   return(E_FAIL);
   }

/*  Place the string portion of the DDR into a temporary buffer
---------------------------------------------------------------*/
junk_temp = (char *)&ddr->spare;
strncpy(junk_temp,ddr->system,4);
temp = squeeze(ddr->system,DDSYLN);
strcpy(d_temp[0],temp);
free(temp);
temp = squeeze(ddr->proj_units,DDPULN);
strcpy(d_temp[1],temp);
free(temp);
temp = squeeze(ddr->last_used_date,DDLDLN);
strcpy(d_temp[2],temp);
free(temp);
temp = squeeze(ddr->last_used_time,DDLTLN);
strcpy(d_temp[3],temp);
free(temp);

clen = DDSTCT * DDSYLN - 1;		  /* set up and output record 1	      */
dlen = DISIZE * 4;
dbuf = (unsigned char *) ddr;
if (c_lswrit(&fd,"DDRINT",&clen,&dlen,d_temp[0],(char *)dbuf,"I4") != E_SUCC)
   {
   c_errmsg("Error returned from lswrit","putddr-call",&one);
   return(E_FAIL);
   }

/* Set up and output record 2
   There is no character part to this record 
--------------------------------------------*/
clen = 0;				  
dlen = DDSIZE * 8;		  
dbuf = (unsigned char *) &(ddr->proj_coef[0]);
if (c_lswrit(&fd,"DDRDUB",&clen,&dlen,d_temp[0],(char *)dbuf,"R8") != E_SUCC)
   {
   c_errmsg("Error returned from lswrit","putddr-call",&one);
   return (E_FAIL);
   }

action = 0;				  /* close associated DDR file	      */
if (c_lsclos(&fd,hostddr,&action) != E_SUCC)   
   {
   c_errmsg("Error returned from lsclos","putddr-call",&one);
   return(E_FAIL);
   }
/*
#ifdef UNIX_DISK
if (c_dkclos(hostddr) != E_SUCC)
   {
   c_errmsg("Error returned from dkclos","putddr-call",&one);
   return(E_FAIL);
   }
#endif
*/
return(E_SUCC);
}
