/*******************************************************************************
NAME			       C_SVMNMX

PURPOSE	     Output minimum and maximum intensity values and a
             corresponding validity flag for each band of an 
             output image to the associated DDR file.

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
K. Zanter	      Feb. 1988 	original development
B. Ailts	      Apr. 1988		Replaced newlas.h with las.h
					Replaced return status codes of the
					  ls___ calls with E_SUCC
					Replaced cerrmsg with c_errmsg
B.Ailts		      Jul. 1988		Fixed updating of validity flag
B.Ailts		      Sep. 1988		Fixed how the validity flag was updated
B.Ailts		      Dec. 1988		If the min/max values are outside of the
					min/max range of the output data type,
					set min/max values to the min and max
					values of the data type
B.Ailts		      Dec. 1990		Updated error messages

PROJECT		LAS

ALGORITHM:
   Create the associated DDR file name from the input host image name.
   Check for write access to the DDR file.  
   Open the DDR file for update access using label services.  
   Convert the flag string to INVAL(0) , VALID(1), or BOUND(2).
   For each output band 
       read in the specified record 
       change the min, max, and validity flag with the appropriate value
       replace the record
   Close the DDR file.

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "typlim.h"
#include "util.h"

int c_svmnmx
(
   char *hname,     /* I:  input host image name                        */
   long *totbnd,    /* I:  number of bands to be processed              */
   double minval[], /* I:  minimum intensity values                     */
   double maxval[], /* I:  maximum intensity values                     */
   char *m_flag     /* I:  min/max validity flag ("YES"/"NO"/"BOUND")   */
                       /*  from $MINMAX global                          */
)

{
struct BDDR bddr;		     /* band data for the DDR		      */

long access;			     /* file access type		      */
long action;			     /* file close action flag	              */
long clen;			     /* length of character part of record    */
long ctemp;		 	     /* temporary storage for clen	      */
long dlen;			     /* length of data part of record	      */
long dtemp;			     /* temporary storage for dlen	      */
long fd;			     /* file descriptor for opened file       */
long i;				     /* loop index			      */
long idtype;			     /* data type of input image	      */
long mode;			     /* lsstat mode			      */
long one = 1;			     /* errmsg status code		      */
long status;			     /* function return status code	      */
long v_flag;			     /* validity flag in char form	      */

unsigned char *dbuf;		     /* pointer to area where data is stuffed */

char dtype[3];			     /* type of data portion of record        */
char hostddr[CMLEN];		     /* host name of DDR file		      */
char key[16];			     /* record key			      */
char stemp[DDSRLN];		     /* temporary storage of source	      */
char tempchar[DDTSLN];		     /* temporary char. portion of record     */
char t_hname[1][CMLEN];              /* temporary array to meet param. reqmt. */

strcpy(t_hname[0],hname);
status = c_gettyp(t_hname,&one,&idtype);
if (status != E_SUCC)
   {
   c_errmsg("Error returned from gettyp","svmnmx-call",&one);
   return(E_FAIL);
   }
#ifdef vms
c_lsmknm(hname,"ddr;1",hostddr);	   /* make associated DDR file name   */
#else
c_lsmknm(hname,"ddr",hostddr);		   /* make associated DDR file name   */
#endif

mode = 2;				   /* check DDR file protection       */
status = c_lsstat(hostddr,&mode);
if (status != E_SUCC)
   return(E_PROT);

access = 3;				  /* open DDR file for read/write */
if (c_lsopen(&fd,hostddr,&access) != E_SUCC)
   {
   c_errmsg("Error returned from lsopen","svmnmx-call",&one);
   return(E_FAIL);
   }

/*  set output minmax validity flag */
/*  only upper case string is possible */
if (strcmp(m_flag,"YES") == 0)
   v_flag = VALID;
else if (strcmp(m_flag,"BOUNDED") == 0)
   v_flag = BOUND;
else
   v_flag = INVAL;

for (i = 0; i < *totbnd; i++)
    {
    /* read in current band dependent record */
    sprintf(key,"BAND%d",i+1);
    clen = DDTSLN;
    ctemp = clen - 1;
    dlen = dtemp = DBSIZE * 8;
    dbuf = (unsigned char *) &(bddr.minval);
    status = c_lsread(&fd,key,&clen,&dlen,tempchar,dbuf,dtype);
    if (status != E_SUCC)
       {
       c_errmsg("Error returned from lsread","svmnmx-call",&one);
       return(E_FAIL);
       }

    if ( (clen != ctemp) || (dlen != dtemp) )
       {
       c_errmsg("Error reading the DDR file","svmnmx-read",&one);
       return(E_FAIL);
       }

    /* update with new min, max, and validity flag */
    if (strcmp(m_flag,"NO") == 0)
       {
       bddr.minval = 0.0;
       bddr.maxval = 0.0;
       }
    else
       {
       /*  If the min/max values are outside of the range of the min/max values
	   of the image data type, set the min/max values of the DDR to the
           min/max values of the image data type
       -----------------------------------------------------------------------*/
       switch (idtype)
          {
	  case EBYTE: if (minval[i] < MINUTINY)
			 bddr.minval = MINUTINY;
		      else
			 bddr.minval = (double)(unsigned char)minval[i];

	              if (maxval[i] > MAXUTINY)
			 bddr.maxval = MAXUTINY;
		      else
			 bddr.maxval = (double)(unsigned char)maxval[i];
		      break;
	  case EWORD: if (minval[i] < MINWORD)
			 bddr.minval = MINWORD;
		      else
			 bddr.minval = (double)(short)minval[i];

	              if (maxval[i] > MAXWORD)
			 bddr.maxval = MAXWORD;
		      else
			 bddr.maxval = (double)(short)maxval[i];
		      break;
	  case ELONG: if (minval[i] < MINLONG)
			 bddr.minval = MINLONG;
		      else
			 bddr.minval = (double)(long)minval[i];

	              if (maxval[i] > MAXLONG)
			 bddr.maxval = MAXLONG;
		      else
			 bddr.maxval = (double)(long)maxval[i];
		      break;
	  case EREAL: if (minval[i] < MINREAL)
			 bddr.minval = MINREAL;
		      else
			 bddr.minval = (double)(float)minval[i];

	              if (maxval[i] > MAXREAL)
			 bddr.maxval = MAXREAL;
		      else
			 bddr.maxval = (double)(float)maxval[i];
		      break;
          }  /*  switch idtype  */
       }

    strcpy (stemp,(tempchar + DDBNLN + DDVLLN));
    sprintf((tempchar + DDBNLN),"%2d",v_flag);
    strcpy ((tempchar + DDBNLN + DDVLLN),stemp);

    /* replace record in associated DDR file */
    if (c_lsrepl(&fd,key,&clen,&dlen,tempchar,(char *)dbuf,dtype) != E_SUCC)
       {
       c_errmsg("Error returned from lsrepl","svmnmx-call",&one);
       return(E_FAIL);
       }

    }  /* end of for loop */

action = 0;				  /* close associated DDR file */
if (c_lsclos(&fd,hostddr,&action) != E_SUCC)   
   {
   c_errmsg("Error returned from lsclos","svmnmx-call",&one);
   return(E_FAIL);
   }

return(E_SUCC);
}
