/*******************************************************************************
NAME			      C_UPDDR

PURPOSE	     Updates records 1 and 2 of a DDR file

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
B. Ailts	      Jan. 1988 	original development
B. Ailts	      Apr. 1988		Replaced newlas.h with las.h
					Replaced cerrmsg with c_errmsg
					Changed calling sequences to the put
					and get ddr calls
					Added the rotated projection space
					subroutine
B. Ailts	      jul. 1988		COMB option can now have more output
					bands than input bands
B. Ailts	      Sept. 1988	Increased the buffer space for the 
					band records of the ddr when multiple
					bands are being combined
B.Ailts		      Jan. 1989		If COMB, all bands of one input image
					are not compared with the corresponding
					bands of another input image,  all the
					input bands of all the input images are
					compared with one another
B.Ailts		      Dec. 1989		Added code to free dynamic allocated 
					buffers
B.Ailts		      Dec. 1990		Updated error messages
J.Janssen             Jun. 1998         Added the capability to handle LAS
                                        images with DMS projection units

PROJECT       LAS

ALGORITHM 
   Consolidate all of the input image DDRs into one
   Write the output DDR to the DDR file
   Write the output BDDRs to the DDR file

    Return

ALGORITHM REFERENCES	none
*******************************************************************************/
#include <string.h>
#include "las.h"


int c_upddr
(
   struct PARBLK *parblk,   /* I:  TAE parameter block                  */
   char host_in[][CMLEN],   /* I:  array of input host names            */
   char *host_out,          /* I:  Output host name                     */
   long *nmrimg,            /* I:  Total number of input images         */
   long window[][4],        /* I:  Window specifications for each image */
   long bands[][MAXBND +1], /* I:  band numbers for each input image    */
   long nbands[],           /* I:  number of bands for each input image */
   double *line_inc,        /* I:  increment in the line direction      */
   double *samp_inc,        /* I:  increment in the sample direction    */
   long *out_nbands,        /* I:  number of output bands               */
   long *cflag              /* I:  combination of bands flag            */
)
{ 
    struct DDR out_ddr;        /* output ddr                            */
    struct BDDR *out_bddr;     /* output band dependent ddr record      */

    long i;                    /*  loop counter                         */
    long one = 1;              /*  nonfatal error status                */
    long status;               /*  status flag                          */

    /* Consolidate all of the image DDRs into one
       ------------------------------------------*/
    status = c_merge_ddrs( parblk, host_in, host_out, nmrimg, window, 
                           bands, nbands, line_inc, samp_inc, out_nbands, 
                           cflag, &out_bddr, &out_ddr, TRUE);
    if (status != E_SUCC)
    {
        c_errmsg("Error returned from merge_ddrs","upddr-call",&one);
        return(E_FAIL);
    }

    /* Write the band indepenent information to the output ddr
       -------------------------------------------------------*/
    status = c_putddr(host_out,&out_ddr);
    if (status != E_SUCC)
    {
        c_errmsg("Error returned from putddr","upddr-call",&one);
        return(E_FAIL);
    }

    for (i = 0; i < *out_nbands; i++)
    {
        /* Write each of the bands to the output ddr
           -----------------------------------------*/
        status = c_putbdr(host_out,&(out_bddr[i]));
        if (status != E_SUCC)
        {
            c_errmsg("Error returned from putbdr","upddr-call",&one);
            return(E_FAIL);
        }
    }
    free(out_bddr);
    return (E_SUCC);
} /*  c_upddr  */
