/******************************************************************************

NAME:   CLMNMX

FUNCTION: CLMNMX is function which is called to calculate the minimum and 
	  maximum pixel values of an image.  It is called from MINMAX subroutine

PROGRAM HISTORY:
	Version		Date	Author		Request
	-------		----	-----		-------
	  1.0		5/86	B.Ailts	    	initial development
	  2.0		9/86	K. Gacke	used host file names rather than
						TAE file names
	  3.0		7/88	B.Ailts		LAS 5.0 standards
	  4.0	       12/90	B.Ailts		Updated error messages

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:  
		Run under TAE

PROJECT:   LAS

ALGORITHM DESCRIPTION:
The space needed for errtxt is dynamically allocated
An message is output to show the user what is going on
The image is opened
For each line of the input image
   For each band specified
      The min max values are calculated  
The image is closed
The space allocated for thebuf is freed
The space allocated for errtxt is freed
These values are then passed back to the calling program

ALGORITHM REFERENCES:  none
*****************************************************************************/
#include "las.h"
#include "pixman.h"

int clmnmx
(
   char *hostname,  /* host name of input image                         */
   long *window,    /* window specified for the input image             */
   long *cl_bands,  /* bands that the min/max values are needed         */
   long *mmbands,   /* the correct place in the min/max arrays that
                                   the min/max values should be placed  */
   double *minval,  /* minimum value array                              */
   double *maxval   /* maximum value array                              */
)

{  /*  clmnmx  */

long acc = IREAD;		/*  read access	flag			*/
long band;			/*  current band			*/
long bufsize;			/*  size of the buffer			*/
long dtype = 0;			/*  data type 				*/
struct FDESC *fdesc[2];         /*  file descriptor array		*/
struct GDESC *gdesc;		/*  group descriptor 			*/
long line;			/*  current line			*/
long nonfatal = 1;		/*  nonfatal error code			*/
long offset[MAXBND];		/*  offsets into each band of the image */
long size;			/*  size of the data type in bytes	*/
long status;			/*  error status			*/
long zero = 0;			/*  value of zero			*/

char *errtxt;			/*  buffer of the error message		*/
char *thebuf;			/*  buffer of the input image data	*/

if (cl_bands[0] == 0)
   return(E_SUCC);

/*  Allocate the string buffers needed
--------------------------------------*/
errtxt = (char *)malloc(ERRLEN + 1);
if (errtxt == 0)
   {
   c_errmsg("Error allocating dynamic memory","minmax-alloc",&nonfatal);
   return(E_FAIL);
   }

/*  Output the informational message to the users
-------------------------------------------------*/
sprintf(errtxt,"Calculating the min and max values of %s",hostname);
c_errmsg(errtxt," ",&nonfatal);
c_errmsg("Please wait..."," ",&nonfatal);

/*  Open the input image
------------------------*/
status = c_eopens(&fdesc[0],hostname,cl_bands,&window[0],&window[1],&window[2],
	          &window[3],offset,&acc,&dtype,&zero,&zero);
if (status != E_SUCC)
   {
   sprintf(errtxt,"Error opening image %s",hostname);
   c_errmsg(errtxt,"minmax-iopen",&nonfatal);
   free(errtxt);
   return(E_FAIL);
   }
fdesc[1] = 0;

switch (dtype)
   {
   case (EBYTE): size = 1;  break;
   case (EWORD): size = 2;  break;
   case (ELONG): size = 4;  break;
   case (EREAL): size = 4;  break;
   }

thebuf = c_ealloc(fdesc,&bufsize);
if (thebuf == NULL)
   {
   c_errmsg("Error returned from ealloc","minmax-call",&nonfatal);
   return(E_FAIL);
   }

status = c_egroup(fdesc,&gdesc,(unsigned char *)thebuf,&bufsize);
if (status != E_SUCC)
   {
   c_errmsg("Error returned from egroup","minmax-call",&nonfatal);
   return(E_FAIL);
   }

/*  loop for every line of every band needed -- calculate the min/max values
----------------------------------------------------------------------------*/
for (line = 0; line < window[2]; line++)
   {
   status = c_estep(&gdesc);
   if (status != E_SUCC)
      {
      c_errmsg("Error returned from estep","minmax-call",&nonfatal);
      return(E_FAIL);
      }
   for (band = 0; cl_bands[band] != 0; band++)
      c_pxmnmx((long *)(thebuf + (offset[band] * size)),&window[3],
		&minval[mmbands[band]],&maxval[mmbands[band]],&dtype);
   }

/*  Close the image file
------------------------*/
status = c_egclse(&gdesc);
if (status != E_SUCC)
   {
   c_errmsg("Error returned from egclse","minmax-call",&nonfatal);
   return(E_FAIL);
   }

free(thebuf);
free(errtxt);

return(E_SUCC);
} /* clmnmx  */
