/*******************************************************************************
NAME			       UPMNMX

PURPOSE	     Output minimum and maximum intensity values and a
             corresponding validity flag for each band specified
	     of an image to the associated DDR file.

PROGRAM HISTORY
PROGRAMMER		DATE		REASON
----------		----		------
B. Ailts	      Jul. 1988 	original development
B. Ailts	      Dec. 1990 	Updated error messages
D. Etrheim            Oct. 1993		Corrected so that when the VALID flag
					is updated, tempchar does not get NULL
					terminated inadvertently by sprintf

PROJECT		LAS

ALGORITHM:
   Allocate space for the associated DDR file name
   Create the associated DDR file name from the input host image name.
   Check for write access to the DDR file.  
   Open the DDR file for update access using label services.  
   For each output band 
       read in the specified record 
       change the min, max, and validity flag with the appropriate value
       replace the record
   Close the DDR file.

ALGORITHM REFERENCES	Similar to the function C_SVMNMX
*******************************************************************************/
#include <string.h>
#include "las.h"
#include "labserv.h"

int upmnmx
(
   char *hname,      /* input host image name                              */
   long *cl_bands,   /* bands of image that were processed                 */
   long *mmbands,    /* bands of min/max arrays to be used                 */
   double minval[],  /* minimum intensity values                           */
   double maxval[]   /* maximum intensity values                           */
)

{
struct BDDR bddr;		     /* band data for the DDR		      */

long access;			     /* file access type		      */
long action;			     /* file close action flag	              */
long clen;			     /* length of character part of record    */
long ctemp;		 	     /* temporary storage for clen	      */
long dlen;			     /* length of data part of record	      */
long dtemp;			     /* temporary storage for dlen	      */
long fd;			     /* file descriptor for opened file       */
long i;				     /* loop index			      */
long mode;			     /* lsstat mode			      */
long nonfatal = 1;		     /* errmsg status code		      */
long status;			     /* function return status code	      */

unsigned char *dbuf;		     /* pointer to area where data is stuffed */

char dtype[3];			     /* type of data portion of record        */
char *hostddr;			     /*  host ddr name			      */
char key[16];			     /* record key			      */
char tempchar[DDTSLN];		     /* temporary char. portion of record     */
char tchar[3];                       /* temporary string                      */

/* if cl_bands[0] means that no bands need updating
---------------------------------------------------*/
if (cl_bands[0] == 0)
   return(E_SUCC);

/* Allocate the space needed for the host ddr name
--------------------------------------------------*/
hostddr = (char *)malloc(CMLEN);
if (hostddr == 0)
   {
   c_errmsg("Error allocating dynamic memory","minmax-alloc",&nonfatal);
   return(E_FAIL);
   }

#ifdef vms
c_lsmknm(hname,"ddr;1",hostddr);	   /* make associated DDR file name   */
#else
c_lsmknm(hname,"ddr",hostddr);		   /* make associated DDR file name   */
#endif

mode = 2;				   /* check DDR file protection       */
status = c_lsstat(hostddr,&mode);
if (status != E_SUCC)
   return(E_PROT);

access = 3;				  /* open DDR file for read/write */
if (c_lsopen(&fd,hostddr,&access) != E_SUCC)
   {
   c_errmsg("Error retruned from lsopen","minmax-call",&nonfatal);
   return(E_FAIL);
   }

for (i = 0; cl_bands[i] != 0; i++)
    {
    /* rewind to the beginning of the file each time in case the band order is
       not from lowest to highest band */
    if (c_lsrew(&fd) != E_SUCC)
       {
       c_errmsg("Error rewinding the DDR file","minmax-rew",&nonfatal);
       return(E_FAIL);
       }

    /* read in current band dependent record 
    ----------------------------------------*/
    sprintf(key,"BAND%d",cl_bands[i]);
    clen = DDTSLN;
    ctemp = clen - 1;
    dlen = dtemp = DBSIZE * 8;
    dbuf = (unsigned char *) &(bddr.minval);
    status = c_lsread(&fd,key,&clen,&dlen,tempchar,dbuf,dtype);
    if (status != E_SUCC)
       {
       c_errmsg("Error retruned from lsread","minmax-call",&nonfatal);
       return(E_FAIL);
       }

    if ( (clen != ctemp) || (dlen != dtemp) )
       {
       c_errmsg("Error reading the DDR file","minmax-read",&nonfatal);
       return(E_FAIL);
       }

    /* update with new min, max, and validity flag 
    ----------------------------------------------*/
    bddr.minval = minval[mmbands[i]];
    bddr.maxval = maxval[mmbands[i]];
    /*sprintf((tempchar + DDBNLN),"%2d",VALID);*/
    sprintf(tchar,"%2d",VALID);
    strncpy((tempchar + DDBNLN),tchar,2);

    /* replace record in associated DDR file 
    ----------------------------------------*/
    if (c_lsrepl(&fd,key,&clen,&dlen,tempchar,(char *)dbuf,dtype) != E_SUCC)
       {
       c_errmsg("Error retruned from lsrepl","minmax-call",&nonfatal);
       return(E_FAIL);
       }

    }  /* end of for loop */

action = 0;				  /* close associated DDR file */
if (c_lsclos(&fd,hostddr,&action) != E_SUCC)   
   {
   c_errmsg("Error retruned from lsclos","minmax-call",&nonfatal);
   return(E_FAIL);
   }

/*  Free the host DDR file name space
-------------------------------------*/
free(hostddr);

return(E_SUCC);
}
