/******************************************************************************
FUNCTION:       c_dkusage

PURPOSE:        For a given data path name, determine the number of
                times that it is currently being utilized within the
                disk reserve file.

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
  1.0      11/98    T. Baltzer          Original development, borrowed code
                                        from c_ckfree().

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
  Get the current process ID
  Initailze the symbolic link name
  Open and lock the disk reserved file
  Loop until the end of the file
        Read each record of the disk reserved file
            If the file name qualifies
                Increment file usage counter
  End loop until EOF

  Close and unlock the disk reserved file

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include "las.h"
#include "diskio.h"
#include "util.h"   /* c_sigblk/unblk prototype */
#include "diskdef.h"


int c_dkusage
(
    char *path,                 /* pathname to check usage thereof      */
    long *usage_counter         /* Utilization counter for path         */
)
{
        struct DISKRESERVE rsv_rec;     /* Disk reserve structure       */
        struct stat statp;      /* Status information structure for path*/

        int existflg;           /* does file exist status flag          */
        int i;                  /* Looping variable                     */
        int readstat;           /* Read status variable                 */
        int rfd;                /* File descriptor for the read         */
        int sig_mask;
        int status;             /* Status varilable                     */

        int fstat = E_SUCC;     /* function status                      */
        long nonfatal = 1;      /* Non-fatal error code                 */

        char linkname[255];     /* Link name                            */
        char msg[ERRLEN];       /* Error message string                 */
        char rsvdir[CMLEN];     /* Reserve directory name               */
        char rsvroot[CMLEN];    /* Reserve root name                    */
        char rsvext[CMLEN];     /* Reserve extension name               */


/* Initailize the usage counter                 */
*usage_counter = 0;

        c_sigblk(&sig_mask);

/* Initailize the link name                     */
        for (i=0; i<255; i++)
                linkname[i] = 0;

/* Get the symbolic link                        */
        status = readlink(path, linkname, 255);
        if (status <  0)
        {       
            /* it probably was not a link */
            strcpy(linkname, path); /* so use path*/
        }

/* determine whether the DISKRSVD file already exists */
        existflg = stat(DISKRSVD,&statp);

/* open and lock DISKRSVD file */
        if ( cmlock(DISKRSVD,&rfd,O_CREAT | O_RDWR,600) != E_SUCC)
           {
           sprintf(msg,"Error locking file %s",DISKRSVD);
           c_errmsg(msg,"dkusage-lock",&nonfatal);
           return(E_FAIL);
           }

        if ( existflg < 0 )             /* file was just created */
           {
           if ( chmod(DISKRSVD, 0666) == -1 )
              {
              sprintf(msg,"Error changing file permissions for %s",DISKRSVD);
              c_errmsg(msg,"dkusage-chmod",&nonfatal);
              return(E_FAIL);
              }
           }

/* while not eof do */
/* read rsv_rec */
        do
        {
            /* read the rsv_record      */
            readstat = read(rfd, &rsv_rec, sizeof(struct DISKRESERVE));
            if (readstat == -1)
               {
               c_errmsg("Error reading record","dkusage-read",&nonfatal);
               fstat = E_FAIL;
               }
            if (readstat > 0)
               {
               /* Consider this reserved file only if it is "active" */
               if ((rsv_rec.active == 'A') && (kill(rsv_rec.ppid,0) == 0))
                  {
                  /* Determine the path name part of the reserved file */
                  c_splitname(rsv_rec.filename,rsvdir,rsvroot,rsvext);

                  /* If the link name is not terminated by "/" but the reserve
                     file pathname is terminated by "/", get rid of it (don't
                     want the compare to fail just because of this one char).
                     We don't have to check for the reverse case because
                     c_splitname should always return a path ending in "/".
                  */
                  if ((linkname[strlen(linkname)-1] != '/') &&
                      (rsvdir[strlen(rsvdir)-1] == '/'))
                     rsvdir[strlen(rsvdir)-1] = '\0';

                  /* Check the path name                */
                  if (strcmp(linkname ,rsvdir) == 0)    
                     {
                     /* Increment usage counter */
                     (*usage_counter)++;
                     }
                  }
               }
        } while (readstat > 0);
/* end while not eof */

/* close and unlock /tmp/diskrsvd */
        if ( close(rfd) == -1 )
           {
           c_errmsg("Error closing file","dkusage-close",&nonfatal);
           return(E_FAIL);
           }

c_sigunblk(&sig_mask);
return(fstat);
}
