/*********************   Label Services Subsystem   *********************
FUNCTION:       lsread

PURPOSE:        Read from associated file

PROGRAM HISTORY:
Version    Date     Author              Change Request
-------    -----    -------------       --------------
           04/86    C. Greenhagen       initial development
           07/86    K. Gacke            newLAS development
           10/87    K. Zanter           bug fix (so doesn't reset cubf->length
                                        or check char buffer if length = 0)
           12/87    B. Ailts            place bridge routines in seperate file
                                        replace DESC arguments with char strings
                                        change include directory specifications
                                        use raw 'C' types
  5.0      04/88    D. Hollaren         change include spec to las
  5.1     11/88    B. Ailts             Replaced las.h with worgen.h so these
                                        routines can be placed in world
  5.2     07/90    D. VanderZee         Standardized error handling
  5.4     01/91    K. Gacke             Changed i/o to buffered i/o for sun --
                                        sun has a problem with direct i/o when
                                        the close takes to long to be executed
  5.5     05/90    B. Ailts             Added c_sigblk and c_sigunblk in order
                                        that the interupts produced by the 
                                        processing message does corrupt the I/O.
  5.6     09/92    T. Mittan            Typecast constant 0 to (char *) to 
                                        eliminate warnings on compile.
  5.7     11/92    D. Etrheim           Moved call to free so that allocated
                                        space for out_key is freed after each 
                                        loop. 
  5.8     11/93    T. Rockvam           Changed the sun version to unblock 
                                        signals when returning for any reason

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
        None.

PROJECT                 LAS

ALGORITHM 
Read each record in the file until EOF or key match is found
    Read header record
    look for a match to the specified key else skip the record
        if the specified character length is less than the actual character
        data length or the specified data length is less than the actual
        data length
            reset the character and/or data lengths to the actual
            backup to the beginning of the record
        read the character data
        read the other data
Verify that the lengths of the data requested is the same as the data
lengths to be transferred.
Free up any allocated space.

ALGORITHM REFERENCES    none
*******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include "tae.h"
#include "worgen.h"
#include "vll.h"
#include "util.h"
#include "lasfileio.h"
#include "labserv.h"
#include "local_protos.h"

static int lsread
(
    int fd,              /* I: file descriptor */
    char *in_key,        /* I/O: key of record to be read */
    int *clen,           /* I/O: number of char bytes to read */
    int *dlen,           /* I/O: number of data bytes to read */
    char *cbuf,          /* O: buffer to read char data into */
    void *dbuf,          /* O: data buffer to read data into */
    char *dtype          /* O: datatype of data record (B,I2,I4,R4,R8) */
)
{
    int   charlen, datalen;
    long  obyte;
    long  offset;
    short keymatch = FALSE;
    char  header[HDRL];
    char  *out_key;
    char  *ptr;

    while (!keymatch)
    {
        charlen = datalen = 0;
        if ((obyte = read (fd, header, HDRL)) == 0)/* return if EOF detected */
        {
            return(E_EOF);
        }
        if (obyte == -1)
        {
            c_lasmsg(FALSE,"lsread-read",
                     "Error reading header record from label services file");
            return(E_FAIL);
        }

        if (((ptr = strchr(header,'/')) != NULL) && (ptr - header < LENL))
            sscanf(header,"%d/%d%s",&charlen,&datalen,dtype);
        else
            sscanf(header,"%d%s",&datalen,dtype);

        out_key = squeeze(header+LENL+TYPL,KEYL -1);
        if ((strlen(in_key) == 0) || (strcmp(in_key," ") == 0) 
                                  || (strcmp(in_key,out_key) == 0))
            keymatch = TRUE;

        if (strcmp(out_key,"DELETED") == 0)
            keymatch = FALSE;

        if (keymatch) 
        {
            strcpy(in_key,out_key);
            if (((*clen <= charlen) && (charlen > 0)) || (*dlen < datalen)) 
                                      /* buffers are too small */
            {
                if (*clen <= charlen)/* all string to be null terminated   */
                    *clen = charlen + 1;

                if (*dlen < datalen)
                    *dlen = datalen;

                offset = -(HDRL);
                if (lseek(fd,offset,SEEK_CUR) == -1)
                {
                    c_lasmsg(FALSE,"lsread-seek",
                             "Error seeking in label services file");
                }
                free(out_key);
                return(E_SMAL);
            }

            if (charlen > 0) /* read character portion    */
            {
                obyte = read(fd, cbuf, charlen);
                if (obyte <= 0) /* Error if EOF encountered  */
                {
                    c_lasmsg(FALSE,"lsread-read","Error reading char record "
                             "from label services file");
                    free(out_key);
                    return(E_FAIL);
                }
                *clen = obyte;
                *(cbuf + *clen) = '\0';  
            }
            else
                *clen = 0;

            if (datalen > 0) /* read data portion         */
            {
                /* determine the number of bytes per data element */
                int bytes_per_element = labserv_datasize(dtype);
                
                if (bytes_per_element == 0)
                {
                    c_lasmsg(FALSE,"lswrit-dtype","%s is an unrecognized data "
                             "type", dtype);
                    free(out_key);
                    return(E_FAIL);
                }

                obyte = c_read(fd, dbuf, datalen, bytes_per_element);
                if (obyte <= 0) /* Error if EOF encountered  */
                {
                    c_lasmsg(FALSE,"lsread-read","Error reading data record "
                             "from label services file");
                    free(out_key);
                    return(E_FAIL);
                }
                *dlen = obyte;
            }
            else
                *dlen = 0;
        }
        else
        {
            offset = datalen + charlen;
            if (lseek(fd,offset,SEEK_CUR) == -1)
            {
                c_lasmsg(FALSE,"lsread-seek",
                         "Error seeking in label services file");
                free(out_key);
                return(E_FAIL);
            }
        }

        free(out_key);
    }

    if (*clen + *dlen != charlen + datalen)
    {
        c_lasmsg(FALSE,"lsread-read","Error reading from label services file");
        c_lasmsg(FALSE,"lsread-bytes","    %d bytes requested, %d bytes "
                 "transferred", charlen + datalen,*clen + *dlen);
        return(E_FAIL);
    }

    return(E_SUCC);
}

int c_labread
(
    int fd,             /* I: file descriptor */
    char *key,          /* I/O: key of record to be read, or key read if
                                any key requested */
    int *clen,          /* I/O: number of char bytes to be read */
    int *dlen,          /* I/O: number of data bytes to be read */
    char *cbuf,         /* O: character buffer to be read */
    void *dbuf,         /* O: data buffer to be read */
    char *dtype         /* O: datatype of data record (B,I2,I4,R4,R8) */
)
{
    int sig_mask;
    int status;
    char *in_key;

    in_key = squeeze(key,16);

    c_sigblk(&sig_mask);

    status = lsread(fd, in_key, clen, dlen, cbuf, dbuf, dtype);

    c_sigunblk(&sig_mask);

    strcpy(key,in_key);
    free(in_key);

    return (status);
}

#ifndef NO_OBSOLETE_INTERFACE

int c_lsread
(
    const long *fd,     /* I: file descriptor */
    char *key,          /* I/O: key of record to be read, or key read if
                                any key requested */
    long *clen,         /* I/O: number of char bytes to be written */
    long *dlen,         /* I/O: number of data bytes to be written */
    char *cbuf,         /* O: character buffer to be written */
    void *dbuf,         /* O: data buffer to be written */
    char *dtype         /* O: datatype of data record (B,I2,I4,R4,R8) */
)
{
    int sig_mask;
    int status;
    int local_clen = (int)*clen;
    int local_dlen = (int)*dlen; 
    char *in_key;

    in_key = squeeze(key,16);

    c_sigblk(&sig_mask);

    status = lsread(*fd, in_key, &local_clen, &local_dlen, cbuf, dbuf, dtype);

    c_sigunblk(&sig_mask);

    strcpy(key,in_key);
    free(in_key);

    *clen = local_clen;
    *dlen = local_dlen;
    return (status);
}

#endif /* Obsolete interfaces */
