/******************************************************************************
NAME			pxdiv

PURPOSE		Divide two input arrays into one output array.

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	D.GORDON	5/7/85	Original development for NEWLAS
	D.GORDON	11/20/85 Optimumized all routines.
	B.Ailts		   12/87	Placed bridge routines in seperate file
					Used raw 'C' types
					Changed include directory specifications
	B.Ailts		03/88	 Replaced newlas.h with worgen.h

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		Part of the LAS system.

PROJECT				LAS

ALGORITHM 

receive:
	-the input arrays and their type
	-the output array
	-the number of samples of the input arrays to divide.
	-the scaleing factors and the offset.
	-the value to be returned in the case of divide by zero.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	if the denominator is going to be a zero then
		set the status to DIVZERO
		set the output value to errval (specified by user)
	otherwise:

	divide the pixels from input arrays and store result in output array
	    (ie. output = inputarray1 / inputarray2 * scale + offset)
	if output is smaller than the lower limit of the data type then 
		output = the lower limit of data type and error is returned
	if output is larger than the higher limit of the data type then
		output = the higher limit of data type and error is returned.
return
*note:	Each operation is performed with variables one data type larger than the
	desired dta type. This makes checking for the operation exceding the
	bounds of the data type very easy while maintaining a fast algorithm.

*****************************************************************************/
#include "worgen.h"
#include "pixman.h"

#define DIVZERO 44



void bdiv
(
    int ns,
    unsigned char *in1,
    unsigned char *in2,
    unsigned char *out,
    float *scale,
    float *offset,
    unsigned char *errval,
    long *status
)

{
   int          tempin1,
		tempin2,
		tempout;

    if ((*scale == 1) && (*offset == 0))
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if(*in2 == 0)
			{
			*status = DIVZERO;
			tempout = *errval;
			}
		else if ((tempout = tempin1 / tempin2) > MAXUTINY)
			{
			tempout = MAXUTINY;
			*status = TOOBIG;
			}
		    else  if (tempout < MINUTINY)
				{
				tempout = MINUTINY;
				*status = TOOSMALL;
				}
		*out = tempout;
		}
    else
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if(*in2 == 0)
			{
			*status = DIVZERO;
 			tempout = *errval;
			}
		else tempout = (tempin1 / tempin2) * *scale + *offset;
		if (tempout > MAXUTINY)
			{
			tempout = MAXUTINY;
			*status = TOOBIG;
			}
		else	if (tempout < MINUTINY)
				{
				tempout = MINUTINY;
				*status = TOOSMALL;
				}
		*out = tempout;
		}
}



void wdiv
(
    int ns,
    short *in1,
    short *in2,
    short *out,
    float *scale,
    float *offset,
    short *errval,
    long *status
)

{
    long	tempin1,
		tempin2,
		tempout;

    if ((*scale == 1) && (*offset == 0))
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if(*in2 == 0)
			{
			*status = DIVZERO;
			tempout = *errval;
			}
		else if ((tempout = tempin1 / tempin2) > MAXWORD)
			{
			tempout = MAXWORD;
			*status = TOOBIG;
			}
		    else  if (tempout < MINWORD)
				{
				tempout = MINWORD;
				*status = TOOSMALL;
				}
		*out = (short)tempout;
		}
    else
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if(*in2 == 0)
			{
			*status = DIVZERO;
 			tempout = *errval;
			}
		else tempout = (long)((float)(tempin1 / tempin2) * *scale 
                               + *offset);
		if (tempout > MAXWORD)
			{
			tempout = MAXWORD;
			*status = TOOBIG;
			}
		else	if (tempout < MINWORD)
				{
				tempout = MINWORD;
				*status = TOOSMALL;
				}
		*out = (short)tempout;
		}
}



static void ldiv
(
    int ns,
    long *in1,
    long *in2,
    long *out,
    float *scale,
    float *offset,
    long *errval,
    long *status
)
	{
   double	tempin1,
		tempin2,
		tempout;

    if ((*scale == 1) && (*offset == 0))
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if(*in2 == 0)
			{
			*status = DIVZERO;
			tempout = *errval;
			}
		else if ((tempout = tempin1 / tempin2) > MAXLONG)
			{
			tempout = MAXLONG;
			*status = TOOBIG;
			}
		    else  if (tempout < MINLONG)
				{
				tempout = MINLONG;
				*status = TOOBIG;
				}
		*out = (long)tempout;
		}
    else
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if(*in2 == 0)
			{
			*status = DIVZERO;
 			tempout = *errval;
			}
		else tempout = (tempin1 / tempin2) * *scale + *offset;
		if (tempout > MAXLONG)
			{
			tempout = MAXLONG;
			*status = TOOBIG;
			}
		else	if (tempout < MINLONG)
				{
				tempout = MINLONG;
				*status = TOOBIG;
				}
		*out = (long)tempout;
		}
}



void rdiv
(
    int ns,
    float *in1,
    float *in2,
    float *out,
    float *scale,
    float *offset,
    float *errval,
    long *status
)
	{
   double	tempin1,
		tempin2,
		tempout;

    if ((*scale == 1) && (*offset == 0))
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if(*in2 == 0)
			{
			*status = DIVZERO;
			tempout = *errval;
			}
		else tempout = (tempin1 / tempin2) * *scale + *offset;
		if (tempout > MAXREAL)
			{
			tempout = MAXREAL;
			*status = TOOBIG;
			}
		else	if (tempout < MINREAL)
				{
				tempout = MINREAL;
				*status = TOOBIG;
				}
		*out = tempout;
		}
    else
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if(*in2 == 0)
			{
			*status = DIVZERO;
 			tempout = *errval;
			}
		else tempout = (tempin1 / tempin2) * *scale + *offset;
		if (tempout > MAXREAL)
			{
			tempout = MAXREAL;
			*status = TOOBIG;
			}
		else	if (tempout < MINREAL)
				{
				tempout = MINREAL;
				*status = TOOBIG;
				}
		*out = tempout;
		}
}




int c_pxdiv
(
   long *ns,      /* I: number of samples                                 */
   long *in1,     /* I: input array one                                   */
   long *in2,     /* I: input array two                                   */
   long *out,     /* O: output array                                      */
   float *scale,  /* I: scale applied to input arrays after mult.         */
   float *offset, /* I: offset added in after sum                         */
   long *dtype,   /* I: data type (EBYTE,EWORD,ELONG,EREAL)               */
   long *errval   /* I: value to place in array where div zero occurs     */
)

{
long	status = E_SUCC;	/* return status for pxmul */

switch(*dtype)
	{
   case (EBYTE): bdiv(*ns,(unsigned char *)in1,(unsigned char *)in2,
                      (unsigned char *)out,scale,offset,
                      (unsigned char *)errval,&status); 
                 break;
   case (EWORD): wdiv(*ns,(short *)in1,(short *)in2,(short *)out,scale,
                      offset,(short *)errval,&status); 
                 break;
   case (ELONG): ldiv(*ns,in1,in2,out,scale,offset,errval,&status); break;
   case (EREAL): rdiv(*ns,(float *)in1,(float *)in2,(float *)out,scale,
                      offset,(float *)errval,&status); 
                 break;
        default: status = TYPERR; break;
	} /* switch */

return(status);
} /* pxdiv */
