/* 
NAME			c_pxhiss

PURPOSE		Compute a histogram accumulation array given an input buffer
		with a scale factor.  Array of either char, short, long, 
		or real type.

PROGRAM HISTORY
	PROGRAMMER	DATE	 REASON
	----------	----	 ------
	D.GORDON	5/7/85	 Original development for NEWLAS
	B.AILTS		4/23/85  Add real data type capabilities.
	D.Hollaren	6/14/89  Modified histogramming (NS / SKIP) pixels
	K.Kunska	9/17/90  Added 1 to NPIX if NS MOD SKIP != 0 so that
                                 skipping is done correctly; added code to 
                                 count the number of pixels that were 
                                 histogrammed in BHIST.C and WHIST.C
        T.Baltzer       9/24/90  fix parameters to errmsg call


ALGORITHM:
receive:
	-the buffer and its type
	-a pointer to the histogram
	-and the number of samples.
depending on the type of buffer values
	call a function that uses that data type to perform the following

for char,short, or long data type
  for each pixel of the buffer
	incriment the histogram value of the value of that pixel
		(ie. histogram[bufferValue[pixel]]++)
  return

for real data type
  the histogram array has to be dimensioned to [16384][2].
  for each pixel of the buffer
	if value is not already in the histogram [*][1]
	then
  	   place the unique value in ascending order within histogram [*] [1]
	else 
	   increment histogram [*][2] that corresponds to this value by 1.
   return

ALGORITHM REFERENCES		none
 **********************************************************/
#include "tae.h"
#include "worgen.h"
#include "pixman.h"



static long bhist
(
    long ns,
    unsigned char *bufptr,
    long *histo,
    long *count,
    long skip
)
	{
	long npix;
/*		*count = 0; */
		npix = ns / skip;
                if (ns % skip)
                   npix++;
		for(; npix; npix--,bufptr+=skip)
		{
		(*(histo + *bufptr))++;
                *count = *count + 1;
		}
	return(E_SUCC);
	}



static long whist
(
    long ns,
    short *bufptr,
    long *histo,
    long *count,
    long skip
)
	{
	long npix;
/*   		*count = 0; */
 		npix = ns / skip;
                if (ns % skip)
                   npix++;
		for(; npix; npix--,bufptr+=skip)
		{
		(*(histo + *bufptr))++;
                *count = *count + 1;
		}
	return(E_SUCC);
	}



static long lhist
(
    long ns,
    long *bufptr,
    long *histo,
    long *count,
    long skip
)
	{
	long npix;
		npix = ns / skip;
                if (ns % skip)
                   npix++;
		for(; npix; npix--,bufptr+=skip)
		{
		(*(histo + *bufptr))++;
		if (*(histo + *bufptr) == 1)
   		*count = *count + 1;
		}
	return(E_SUCC);
	}



static long rhist
(
    long ns,
    long *bufprt,
    long *histo,
    long *mx,
    long skip
)
{
   float *realbuf;
   float *realhis;
   long low,
	  high,
	  mid,
	  npix,
	  i,
	  j;
long 	one;

realbuf = (float *)bufprt;
realhis = (float *)histo;
/*
 *mx=0;
	while (*(realhis + *mx) > zero)
		{
		 *mx = *mx + 1;
		}
*/
npix = ns / skip;
if (ns % skip)
   npix++;
for (i = 1; i < npix + 1; i++, realbuf+=skip)
	{
	if (*mx >= 16384)
	   {
	   c_errmsg ("Number of pixel values exceeded the maximum array size"
           ,"pxhist-maximum",&one);
	   return(E_FAIL);
	   }

	if ( *mx == 0)
		{ /* check for initail pass */
		 *mx =	*mx + 1;
		*(realhis + *mx-1) = *realbuf;
		*(realhis + *mx+16383) = 1;
		}
	else
		{
		low = 1;
		high = *mx;
		mid = (low+high)/2;
top:
		if (*realbuf < *(realhis +mid-1))
			{
			if (*realbuf < *(realhis +low-1))
				{
				 *mx = *mx + 1;
				for (j= *mx;j>low-1;--j)
				     {
				     *(realhis +j-1) = *(realhis+j-2);
				     *(realhis+j+16383) = *(realhis+j-1+16383);
				     }	
				*(realhis+low-1) = *realbuf;
				*(realhis+low+16383) = 1;
				}
			else
				{
				high = mid-1;
				mid = (low + high) / 2;
				goto top;
				}
			}
		else if (*realbuf > *(realhis+mid-1))
			{
			if (*realbuf > *(realhis+high-1))
				{
				*mx = *mx + 1; 
				for (j= *mx;j>high+1;j--)
				   {
				   *(realhis+j-1) = *(realhis+j-2);
				   *(realhis+j+16383) = *(realhis+j-1+16383);
				   }	
				*(realhis+high) = *realbuf;
				*(realhis+high+16384) = 1;
				}
			else
				{
				low = mid+1;
				mid = (low + high) /2;
				goto top;
				}
			}
		else
			{
			*(realhis+mid+16383) = *(realhis+mid+16383)+1;
			}  /*  where value fits into histogram */
		}  /* initial pass? */
	}  /* current sample  */
return(E_SUCC);
} /* end of function*/





int c_pxhiss
(
   long *ns,      /* I: pointer to number of samples in buffer             */
   long *bufptr,  /* I: 4 byte buffer pointer                              */
   long *datatyp, /* I: pointer to the type of buffer                      */
   long *histo,   /* O: pointer to a 4 byte histogram array                */
   long *count,   /* O: total number of values the histogramed             */
   long *skip     /* I: sampling step                                      */
)

	{
	long	status = E_SUCC; /* return status for the routine */

	switch (*datatyp)
		{
		case EBYTE: status=bhist(*ns,(unsigned char *)bufptr,histo,
                                         count,*skip); 
                            break;
		case EWORD: status=whist(*ns,(short *)bufptr,histo,count,
                                         *skip); 
                            break;
		case ELONG: status=lhist(*ns,bufptr,histo,count,*skip); break;
		case EREAL: status=rhist(*ns,bufptr,histo,count,*skip); break;
		default: status = TYPERR; break;
		}
	return(status);
	}
