/****************************************************************************** 
NAME			pxhist

PURPOSE		Compute a histogram accumulation array given an input buffer
		array of either char, short, long, or real type.

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	D.GORDON	5/7/85	Original development for NEWLAS
	B.AILTS		4/23/85 Add real data type capabilities.
	B.Ailts		  12/87	changed include file specifications
				moved bridge routines to a seperate file
				used raw 'C' types
	B.Ailts		03/88   Replaced newlas.h with worgen.h
	T.Baltzer	09/90   Fix errmsg key
        G.Schmidt       09/99   Changed rhist to accept float *s
                                Changed lhist to verify bufptr doesn't
                                exceed the histogram bounds.

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		Part of the LAS system.

PROJECT				NEWLAS

ALGORITHM 

receive:
	-the buffer and its type
	-a pointer to the histogram
	-and the number of samples.
depending on the type of buffer values
	call a function that uses that data type to perform the following

for char,short, or long data type
  for each pixel of the buffer
	incriment the histogram value of the value of that pixel
		(ie. histogram[bufferValue[pixel]]++)
  return

for real data type
  the histogram array has to be dimensioned to [16384][2].
  for each pixel of the buffer
	if value is not already in the histogram [*][1]
	then
  	   place the unique value in ascending order within histogram [*] [1]
	else 
	   increment histogram [*][2] that corresponds to this value by 1.
   return

*****************************************************************************/
#include "tae.h"
#include "worgen.h"
#include "pixman.h"



static long bhist
(
    long ns,
    unsigned char *bufptr,
    long *histo,
    long *count
)
{
	*count = 0;
	for(; ns; ns--,bufptr++)
	{
	(*(histo + *bufptr))++;
	}
return(E_SUCC);
}



static long whist
(
    long ns,
    short *bufptr,
    long *histo,
    long *count
)
{
   	*count = 0;
	for(; ns; ns--,bufptr++)
	{
	(*(histo + *bufptr))++;
	}
return(E_SUCC);
}




static long lhist
(
    long ns,
    long *bufptr,
    long *histo,
    long *count
)
{
long one = 1;

/*  The application now has to initialize the count to zero
    -------------------------------------------------------------------*/
	for(; ns; ns--,bufptr++)
	{
                if (*bufptr >= 16384)
                   {
                   c_errmsg (
              "Number of pixel values exceeded the maximum array size in lhist",
              "pxhist-maximum",&one);
                   return(E_FAIL);
                   }
		(*(histo + *bufptr))++;
		if (*(histo + *bufptr) == 1)
   			*count = *count + 1;
	}
return(E_SUCC);
}



static long rhist
(
    long ns,
    float *bufptr,
    float *histo,
    long *lmax
)
{
float *realbuf = bufptr;
float *realhis = histo;
long low,
     high,
     mid,
     i,
     j;
long one = 1;

/*  The application now has to initialize the count to zero because the
    data could contain negative numbers
    -------------------------------------------------------------------*/
/*
 *lmax=0;
	while (*(realhis + *lmax) > zero)
		{
		 *lmax = *lmax + 1;
		}
*/

for (i = 1;i<=ns;i++,realbuf++)
	{
	if (*lmax >= 16384)
	   {
  c_errmsg ("Number of pixel values exceeded the maximum array size in rhist"
  ,"pxhist-maximum",&one);
	   return(E_FAIL);
	   }
	if ( *lmax == 0)
		{ /* check for initial pass */
		 *lmax =	*lmax + 1;
		*(realhis + *lmax-1) = *realbuf;
		*(realhis + *lmax+16383) = 1.0;
		}
	else
		{
		low = 1;
		high = *lmax;
		mid = (low+high)/2;
top:
		if (*realbuf < *(realhis +mid-1))
			{
			if (*realbuf < *(realhis +low-1))
				{
				 *lmax = *lmax + 1;
				for (j= *lmax;j>low-1;--j)
				     {
				     *(realhis +j-1) = *(realhis+j-2);
				     *(realhis+j+16383) = *(realhis+j-1+16383);
				     }	
				*(realhis+low-1) = *realbuf;
				*(realhis+low+16383) = 1.0;
				}
			else
				{
				high = mid-1;
				mid = (low + high) / 2;
				goto top;
				}
			}
		else if (*realbuf > *(realhis+mid-1))
			{
			if (*realbuf > *(realhis+high-1))
				{
				*lmax = *lmax + 1; 
				for (j= *lmax;j>high+1;j--)
				   {
				   *(realhis+j-1) = *(realhis+j-2);
				   *(realhis+j+16383) = *(realhis+j-1+16383);
				   }	
				*(realhis+high) = *realbuf;
				*(realhis+high+16384) = 1.0;
				}
			else
				{
				low = mid+1;
				mid = (low + high) /2;
				goto top;
				}
			}
		else
			{
			*(realhis+mid+16383) = *(realhis+mid+16383)+1;
			}  /*  where value fits into histogram */
		}  /* initial pass? */
	}  /* current sample  */
return(E_SUCC);
} /* end of function*/




int c_pxhist
(
   long *ns,      /* I: pointer to number of samples in buffer             */
   long *bufptr,  /* I: 4 byte buffer pointer                              */
   long *datatyp, /* I: data type (EBYTE,EWORD,ELONG,EREAL)                */
   long *histo,   /* O: pointer to a 4 byte histogram array                */
   long *count    /* I/O: number of values the histogram array contains    */
)

{
long	status; /* return status for the routine */

status = E_SUCC;
switch (*datatyp)
	{
	case EBYTE: status=bhist(*ns,(unsigned char *)bufptr,histo,count); 
                    break;
	case EWORD: status=whist(*ns,(short *)bufptr,histo,count); 
                    break;
	case ELONG: status=lhist(*ns,bufptr,histo,count);
                    break;
	case EREAL: status=rhist(*ns,(float *)bufptr,(float *)histo,count);
                    break;
	default: status = TYPERR;
                 break;
	}
return(status);
}
