/*************************************************************
NAME			c_pxindr

PURPOSE			Searches an array from end to beginning
			to find the first occurence not equal to  a specified 
			value.

EXTERNAL ASSOCIATIONS		none

FILE REFERENCES			none

EXTERNAL VARIABLES REFERENCED   none

INTERNAL VARIABLES
NAME		TYPE		DESCRIPTION
----		----		-----------
*ns		long		Pointer to the number of samples of the input
				array
*dtype		long		Pointer to the type of data in the input array
*in		long		Pointer to the beginning of the input array.
*val		long		Value to be searched for
*index		long		Index of the first occurence different from  val
status		E_COUNT		Return status for routine (Check for E_SUCC)

DATA STRUCTURES	none

SUBPROGRAMS CALLED 
NAME		DESCRIPTION
----		-----------
bindx		performs pixel search on unsigned byte data
windx		performs pixel search on signed integer data
lindx		performs pixel search on signed long data 
rindx		performs pixel search on real data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS	none.

FATAL ERROR CONDITIONS		TYPERR 66 (choices: EBYTE,EWORD,ELONG,or EREAL)

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	M. Mazzella	7/87	original development for NEWLAS
	T. Baltzer	9/90	make E_SUCC the default return status
        D. Etrheim      4/96    Make zero relative and correct pointer
                                arithmetic

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
	none

PROJECT				NEWLAS

LANGUAGE			C

ALGORITHM (FLOW)

receive:
	-the input array and its type
	-the number of samples of the input array.
	-the number to be searched for
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	Assign the output variable 'index' the index of the input array of the 
	first occurence not equal to 'val', beginning at the end of the array
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"



static void bindr
(
    unsigned char *in,
    int val,
    long *index,
    long ns
)
	{
	for(*index = (ns - 1); *index && in[*index] == val; (*index)--)
		;
	}



static void windr
(
    short *in,
    int val,
    long *index,
    long ns
)
        {
        for(*index = (ns - 1); *index && in[*index] == val; (*index)--)
                ;
        }



static void lindr
(
    long *in,
    long val,
    long *index,
    long ns
)
	{
	for(*index = (ns - 1); *index && in[*index] == val; (*index)--)
		;
	}



static void rindr
(
    float *in,
    double val,
    long *index,
    long ns
)
	{
	for(*index = (ns - 1); *index && in[*index] == val; (*index)--)
		;
	}




int c_pxindr
(
   long *in,    /* I: input array                                          */
   long *val,   /* I: value to be searched for                             */
   long *ns,    /* I: number of samples                                    */
   long *index, /* O: index to be returned                                 */
   long *dtype  /* I: data type (EBYTE,EWORD,ELONG,EREAL)                  */
)

{
long status = E_SUCC;

switch(*dtype)
	{
   case (EBYTE): bindr((unsigned char *)in,*val,index,*ns); 
                 break;
   case (EWORD): windr((short *)in,*val,index,*ns); 
                 break;
   case (ELONG): lindr(in,*val,index,*ns); 
                 break;
   case (EREAL): rindr((float *)in,*val,index,*ns); 
                 break;
        default: status = TYPERR; break;
	} /* switch */
return(status);
} /* pxindr */
