/*************************************************************
NAME			c_pxl10

PURPOSE		Return the log base 10 of each element of the input array
		in an output array which is the result.
		The input array can be of type byte, signed 2 byte, or signed
		4 byte real.  The output array will be of type double

EXTERNAL ASSOCIATIONS		none

FILE REFERENCES			none

EXTERNAL VARIABLES REFERENCED   none

INTERNAL VARIABLES
NAME		TYPE		DESCRIPTION
----		----		-----------
*ns		long		Pointer to the number of samples of the input
				arrays to be used 
*dtype		long		Pointer to the type of data in the input arrays
*in		long		Pointer to the beginning of the input array.
*out		double		The address of beginning of the output array.
sample		E_COUNT		Integer used as a loop variable.
status		E_COUNT		Return status for routine (Check for E_SUCC)

DATA STRUCTURES	none

SUBPROGRAMS CALLED 
NAME		DESCRIPTION
----		-----------
blog		performs logarithm on unsigned byte data
wlog		performs logarithm on signed integer data
flogf		performs logarithm on signed floating point data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS	none.

FATAL ERROR CONDITIONS		TYPERR 66 (choices: EBYTE,EWORD)

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	M.MAZZELLA	6/87	Original development for NEWLAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		c_pxl10 must be linked with the imageio routines.

PROJECT				NEWLAS

LANGUAGE			C

ALGORITHM (FLOW)

receive:
	-the input array and its type
	-the output array
	-the number of samples of the input array.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
  Assign the output array pixel the base 10 log of the input array times a scale
  factor.
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include <math.h>
#include "worgen.h"
#include "pixman.h"



static void blog
(
    unsigned char *in,
    double *out,
    long ns
)
	{
	for(; ns; ns--,in++,out++)
		*out = 1000 * log10((double)*in);
	}



static void wlog
(
    short *in,
    double *out,
    long ns
)
	{
	for(; ns; ns--,in++,out++)
		*out = 1000 * log10((double)*in);
	}



static void flogf
(
    float *in,
    double *out,
    long ns
)
	{
	for(; ns; ns--,in++,out++)
		*out = log10((double)*in);
	}




int c_pxl10
(
   long *in,   /* I: input array                                          */
   long *out,  /* O: output array                                         */
   long *ns,   /* I: number of samples                                    */
   long *dtype /* I: data type (EBYTE,EWORD,ELONG,EREAL)                  */
)

	{
	short	status = E_SUCC;	/* return status for pxl10 */

switch(*dtype)
	{
   case (EBYTE): blog((unsigned char *)in,(double *)out,*ns);
                 break;
   case (EWORD): wlog((short *)in,(double *)out,*ns);
                 break;
   case (EREAL): flogf((float *)in,(double *)out,*ns);
                 break;
   case (ELONG):
        default: status = TYPERR; break;
	} /* switch */
return(status);
	} /* pxl10 */
