/*************************************************************
NAME			c_pxline

PURPOSE		Perform a linear transformation of each element of an input 
		array generating an output array which is the result.
		The input array can be of type byte, signed 2 byte, signed 4
		byte, or real. The output array will be of the same type as the
		input array.

EXTERNAL ASSOCIATIONS		none

FILE REFERENCES			none

EXTERNAL VARIABLES REFERENCED   none

INTERNAL VARIABLES
NAME		TYPE		DESCRIPTION
----		----		-----------
*ns		long		Pointer to the number of samples of the input
				arrays to be used
*dtype		long		Pointer to the type of data in the input arrays
*in		long		Pointer to the beginning of the first input
				array.
*out		long		The address of beginning of the output array.
*scale		long		The address of the scaling factor.
*offset		long		The address of the offset.

DATA STRUCTURES	none

SUBPROGRAMS CALLED 
NAME		DESCRIPTION
----		-----------
blin		performs linear transformation on unsigned byte data
wlin		performs linear transformation on signed integer data
llin		performs linear transformation on signed long data 
flin		performs linear transformation on real data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS	none.

FATAL ERROR CONDITIONS		TYPERR 66 (choices: EBYTE,EWORD,ELONG,or EREAL)

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	M. Mazzella	6/87	Original development for NEWLAS
	B. Ailts	12/88   SCALE + OFFSET arguments are now FLOAT

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		c_pxlin must be linked with the imageio routines.

PROJECT				NEWLAS

LANGUAGE			C

ALGORITHM (FLOW)

receive:
	-the input array and its type
	-the output array
	-the number of samples of the input array.
	-the sampling factor used
	-the offset used
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	Assign the output array pixel the product of the input array and
	the scale factor plus the offset.
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"



void blin
(
    unsigned char *in,
    unsigned char *out,
    long ns,
    double scale,
    double offset
)
	{
	for(; ns; ns--,in++,out++)
		*out = *in * scale + offset;
	}



void wlin
(
    short *in,
    short *out,
    long ns,
    double scale,
    double offset
)
	{
	for(; ns; ns--,in++,out++)
		*out = *in * scale + offset;
	}



void llin
(
    long *in,
    long *out,
    long ns,
    double scale,
    double offset
)
	{
	for(; ns; ns--,in++,out++)
		*out = (long)(*in * scale + offset);
	}



void flin
(
    float *in,
    float *out,
    long ns,
    double scale,
    double offset
)
	{
	for(; ns; ns--,in++,out++)
		*out = *in * scale + offset;
	}





int c_pxline
(
   long *in,     /* I: input array                                          */
   long *out,    /* O: output array                                         */
   long *ns,     /* I: number of samples                                    */
   long *dtype,  /* I: data type (EBYTE,EWORD,ELONG,EREAL)                  */
   float *scale, /* I: scale factor                                         */
   float *offset /* I: offset to be added                                   */
)

	{
	short	status = E_SUCC;	/* return status for c_pxline */

switch(*dtype)
	{
   case (EBYTE): blin((unsigned char *)in,(unsigned char *)out,*ns,*scale,
                      *offset); 
                 break;
   case (EWORD): wlin((short *)in,(short *)out,*ns,*scale,*offset);
                 break;
   case (ELONG): llin(in,out,*ns,*scale,*offset);
                 break;
   case (EREAL): flin((float *)in,(float *)out,*ns,*scale,*offset);
                 break;
        default: status = TYPERR; break;
	} /* switch */
return(status);
	} /* c_pxline */
