
/*************************************************************
NAME			c_pxlkup

PURPOSE		Perform a lut transformation of each element of an input 
		array generating an output array which is the result.
		The input array can be of type byte or signed 2 byte.
		The output array will be of the same type as the lut array.

EXTERNAL ASSOCIATIONS		none

FILE REFERENCES			none

EXTERNAL VARIABLES REFERENCED   none

INTERNAL VARIABLES
NAME		TYPE		DESCRIPTION
----		----		-----------
*ns		long		Pointer to the number of samples of the input
				array to be used
*dtype		long		Pointer to the type of data in the input array
*in		long		Pointer to the beginning of the input array.
*out		long		The address of beginning of the output array.
*lut		long		The address of beginning of the lookup table.
*luttype	long		The address of the type of the lookup table.

DATA STRUCTURES	none

SUBPROGRAMS CALLED 
NAME		DESCRIPTION
----		-----------
blutb		performs byte lut transformation on unsigned byte data
blutw		performs integer lut transformation on unsigned byte data
wlutw		performs integer lut transformation on integer data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS	none.

FATAL ERROR CONDITIONS		TYPERR 66 (choices: EBYTE or EWORD)

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	M. Mazzella	6/87	Original development for NEWLAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		c_pxlkup must be linked with the imageio routines.

PROJECT				NEWLAS

LANGUAGE			C

ALGORITHM (FLOW)

receive:
	-the input array and its type
	-the output array
	-the number of samples of the input array.
	-the lookup table.
	-the lookup table type.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	Assign the output array pixel the lut value of the input array 
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"



void blutb
(
    unsigned char *in,
    unsigned char *out,
    long ns,
    unsigned char *lut
)
		
	{
	for(; ns; ns--,in++,out++)
		*out = lut[*in];
	}



void blutw
(
    unsigned char *in,
    short *out,
    long ns,
    short *lut
)
		
	{
	for(; ns; ns--,in++,out++)
		*out = lut[*in];
	}



void wlutw
(
    short *in,
    short *out,
    long ns,
    short *lut
)
		
	{
	for(; ns; ns--,in++,out++)
		*out = lut[*in];
	}




int c_pxlkup
(
   long *in,     /* I: input array                                          */
   long *out,    /* O: output array                                         */
   long *ns,     /* I: number of samples                                    */
   long *dtype,  /* I: data type (EBYTE,EWORD,ELONG,EREAL)                  */
   long *lut,    /* I: lookup table array                                   */
   long *luttype /* I: data type of lookup table                            */
)

	{
short	status = E_SUCC;	/* return status for c_pxlkup */
switch(*dtype)
	{
   case (EBYTE):{
		if (*luttype == EBYTE)
                   {
		   blutb((unsigned char *)in,(unsigned char *)out,*ns,
                         (unsigned char *)lut);
                   break;
                   }
		else
		   if (*luttype == EWORD)
                       {
			blutw((unsigned char *)in,(short *)out,*ns,
                              (short *)lut);
                        break;
                        }
		   else
			{
			status = TYPERR;
			break;
			}
		}
   case (EWORD): wlutw((short *)in,(short *)out,*ns,(short *)lut); 
                 break;
   case (ELONG):
   case (EREAL):
        default: status = TYPERR; break;
	} /* switch */
return(status);
	} /* c_pxlkup */
