/*************************************************************
NAME			c_pxlor

PURPOSE		Perform a logical or of each element of two input arrays
		generating a third output array which is the "ored" result.
		The input array can be of type byte, signed 2 byte, signed 4
		byte, or real. The output array will be of the same type as the
		input array.

EXTERNAL ASSOCIATIONS		none

FILE REFERENCES			none

EXTERNAL VARIABLES REFERENCED   none

INTERNAL VARIABLES
NAME		TYPE		DESCRIPTION
----		----		-----------
*ns		long		Pointer to the number of samples of the input
				arrays to be used
*dtype		long		Pointer to the type of data in the input arrays
*in		long		Pointer to the beginning of the first input
				array.
*out		long		The address of beginning of the output array.
*map		long		The address of beginning of the mapping array.
sample		E_COUNT		Integer used as a loop variable.
status		E_COUNT		Return status for routine (Check for E_SUCC)

DATA STRUCTURES	none

SUBPROGRAMS CALLED 
NAME		DESCRIPTION
----		-----------
blgor		performs pixel logical or on unsigned byte data
wlgor		performs pixel logical or on signed integer data
llgor		performs pixel logical or on signed long data
flgor		performs pixel logical or on signed real data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS	none.

FATAL ERROR CONDITIONS		TYPERR 66 (choices: EBYTE,EWORD,ELONG,or EREAL)

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	M.MAZZELLA	6/87	Original development for NEWLAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		c_pxlor must be linked with the imageio routines.

PROJECT				NEWLAS

LANGUAGE			C

ALGORITHM (FLOW)

receive:
	-the input array and its type
	-the input mapping array
	-the output array
	-the number of samples of the input array.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	Assign the output array pixel the logical or of the input array and
	the mapping array.
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"



static void blgor
(
    unsigned char *in,
    unsigned char *map,
    unsigned char *out,
    long ns
)
	{
	for(; ns; ns--,in++,map++,out++)
		*out = *in || *map;
	}



static void wlgor
(
    short *in,
    short *map,
    short *out,
    long ns
)
	{
	for(; ns; ns--,in++,map++,out++)
		*out = *in || *map;
	}



static void llgor
(
    long *in,
    long *map,
    long *out,
    long ns
)
	{
	for(; ns; ns--,in++,map++,out++)
		*out = *in || *map;
	}



static void flgor
(
    float *in,
    float *map,
    float *out,
    long ns
)
	{
	for(; ns; ns--,in++,map++,out++)
		*out = *in || *map;
	}

 


int c_pxlor
(
   long *in,   /* I: input array                                          */
   long *map,  /* I: input mapping array                                  */
   long *out,  /* O: output "anded" array                                 */
   long *ns,   /* I: number of samples                                    */
   long *dtype /* I: data type (EBYTE,EWORD,ELONG,EREAL)                  */
)

	{
	short	status = E_SUCC;	/* return status for c_pxlor */

switch(*dtype)
	{
   case (EBYTE): blgor((unsigned char *)in,(unsigned char *)map,
                       (unsigned char *)out,*ns);
                 break;
   case (EWORD): wlgor((short *)in,(short *)map,(short *)out,*ns);
                 break;
   case (ELONG): llgor(in,map,out,*ns);
                 break;
   case (EREAL): flgor((float *)in,(float *)map,(float *)out,*ns);
                 break;
        default: status = TYPERR; break;
	} /* switch */
return(status);
	} /* c_pxlor */
