#define xor(a,b) ((a || b)  && (!(a && b)))
/*************************************************************
NAME			c_pxlgor

PURPOSE		Perform a logical excl. or of each element of two input arrays
		generating a third output array which is the "xored" result.
		The input array can be of type byte, signed 2 byte, signed 4
		byte, or real. The output array will be of the same type as the
		input array.

EXTERNAL ASSOCIATIONS		none

FILE REFERENCES			none

EXTERNAL VARIABLES REFERENCED   none

INTERNAL VARIABLES
NAME		TYPE		DESCRIPTION
----		----		-----------
*ns		long		Pointer to the number of samples of the input
				arrays to be used
*dtype		long		Pointer to the type of data in the input arrays
*in		long		Pointer to the beginning of the first input
				array.
*out		long		The address of beginning of the output array.
*map		long		The address of beginning of the mapping array.
sample		E_COUNT		Integer used as a loop variable.
status		E_COUNT		Return status for routine (Check for E_SUCC)

DATA STRUCTURES	none

SUBPROGRAMS CALLED 
NAME		DESCRIPTION
----		-----------
blxor		performs pixel logical exclusive or on unsigned byte data
wlxor		performs pixel logical exclusive or on signed integer data
llxor		performs pixel logical exclusive or on signed long data
flxor		performs pixel logical exclusive or on real data

INFORMATIONAL MESSAGES AND
NON-FATAL ERROR CONDITIONS	none.

FATAL ERROR CONDITIONS		TYPERR 66 (choices: EBYTE,EWORD,ELONG,or EREAL)

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	M.Mazzella	6/87	Original development for NEWLAS

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		c_pxlxor must be linked with the imageio routines.

PROJECT				NEWLAS

LANGUAGE			C

ALGORITHM (FLOW)

receive:
	-the input array and its type
	-the input mapping array
	-the output array
	-the number of samples of the input array.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	Assign the output array pixel the pixel logical exclusive or of 
        the input array and the mapping array.
return

ALGORITHM REFERENCES		none
**************************************************************************/
#include "worgen.h"
#include "pixman.h"



static void blxor
(
    unsigned char *in,
    unsigned char *map,
    unsigned char *out,
    long ns
)
	{
	for(; ns; ns--,in++,map++,out++)
		*out = xor(*in, *map);
	}



static void wlxor
(
    short *in,
    short *map,
    short *out,
    long ns
)
	{
	for(; ns; ns--,in++,map++,out++)
		*out = xor(*in, *map);
	}



static void llxor
(
    long *in,
    long *map,
    long *out,
    long ns
)
	{
	for(; ns; ns--,in++,map++,out++)
		*out = xor(*in, *map);
	}



static void flxor
(
    float *in,
    float *map,
    float *out,
    long ns
)
	{
	for(; ns; ns--,in++,map++,out++)
		*out = xor(*in, *map);
	}




int c_pxlxor
(
   long *in,   /* I: input array                                          */
   long *map,  /* I: input mapping array                                  */
   long *out,  /* O: output "xored" array                                 */
   long *ns,   /* I: number of samples                                    */
   long *dtype /* I: data type (EBYTE,EWORD,ELONG,EREAL)                  */
)

	{
	short	status = E_SUCC;	/* return status for c_pxlxor */

switch(*dtype)
	{
   case (EBYTE): blxor((unsigned char *)in,(unsigned char *)map,
                       (unsigned char *)out,*ns);
                 break;
   case (EWORD): wlxor((short *)in,(short *)map,(short *)out,*ns);
                 break;
   case (ELONG): llxor(in,map,out,*ns);
                 break;
   case (EREAL): flxor((float *)in,(float *)map,(float *)out,*ns);
                 break;
        default: status = TYPERR; break;
	} /* switch */
return(status);
	} /* c_pxlxor */
