/*****************************************************************************
NAME			pxmul

PURPOSE		Multiply two input arrays into one output array.

PROGRAM HISTORY
	PROGRAMMER	DATE	REASON
	----------	----	------
	D.GORDON	5/7/85	Original development for NEWLAS
	J.REED		11/12/85 Optimized byte subroutine.
	D.GORDON	11/20/85 Optimized all subroutines.
	D.GORDON	9/10/86 Corrected bug in byte optimization algorithm.
	B.Ailts		  12/87	Change the include directory specifications
				Use raw 'C' types
				Place bridge routines in a seperate file
	B.Ailts		03/88	Replaced newlas.h with worgen.h
	L.Huewe		04/90	Modified the routine bmul to leave the scale
				factor a float variable to maintain accuracy

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		Part of the LAS system.

PROJECT				NEWLAS

ALGORITHM

receive:
	-the input arrays and their type
	-the output array
	-the number of samples of the input arrays to multiply.
	-the scaling factors and the offset.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	multiply the pixels from input arrays and store result in output array
	    (ie. output = inputarray1 * inputarray2 * scale + offset)
	if output is smaller than the lower limit of the data type then 
		output = the lower limit of data type and error is returned
	if output is larger than the higher limit of the data type then
		output = the higher limit of data type and error is returned.
return

*note:	Each operation is performed with variables one data type larger than the
	desired dta type. This makes checking for the operation exceding the
	bounds of the data type very easy while maintaining a fast algorithm.

ALGORITHM REFERENCES		none
*******************************************************************************/
#include "worgen.h"
#include "pixman.h"



void bmul
(
    long ns,
    unsigned char *in1,
    unsigned char *in2,
    unsigned char *out,
    float *scale,
    float *offset,
    long *status
)
{
long	tempout;

    if((*scale == 1) && (*offset == 0))
	for(; ns; ns--,out++,in1++,in2++)
		{
		if ((tempout = *in1 * *in2) > MAXUTINY)
			{
			*out = MAXUTINY;
			*status = TOOBIG;
			}
		else	if (tempout < MINUTINY)
			{
			*out = MINUTINY;
			*status = TOOSMALL;
			}
		else	*out = (unsigned char)tempout;
		}
    else
	{
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempout = (long)((*in1 * *in2 * (*scale)) + (*offset));
		if (tempout > MAXUTINY)
			{
			*out = MAXUTINY;
			*status = TOOBIG;
			}
		else	if (tempout < MINUTINY)
			{
			*out = MINUTINY;
			*status = TOOSMALL;
			}
		else	*out = (unsigned char)tempout;
		}
	}
} /* bmul  */




void wmul
(
    long ns,
    short *in1,
    short *in2,
    short *out,
    float *scale,
    float *offset,
    long *status
)
{
long	tempin1,
		tempin2,
		tempout;

    if ((*scale == 1) && (*offset == 0))
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if ((tempout = tempin1 * tempin2) > MAXWORD)
			{
			tempout = MAXWORD;
			*status = TOOBIG;
			}
		else	if (tempout < MINWORD)
				{
				tempout = MINWORD;
				*status = TOOSMALL;
				}
		*out = (short)tempout;
		}
    else
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		tempout = (long)((float)(tempin1 * tempin2) * *scale + 
                                  *offset);
		if (tempout > MAXWORD)
			{
			tempout = MAXWORD;
			*status = TOOBIG;
			}
		else	if (tempout < MINWORD)
				{
				tempout = MINWORD;
				*status = TOOSMALL;
				}
		*out = (short)tempout;
		}
}


 

void lmul
(
    long ns,
    long *in1,
    long *in2,
    long *out,
    float *scale,
    float *offset,
    long *status
)
{
double	tempin1,
		tempin2,
		tempout;

    if ((*scale == 1) && (*offset == 0))
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if ((tempout = tempin1 * tempin2) > MAXLONG)
			{
			tempout = MAXLONG;
			*status = TOOBIG;
			}
		else	if (tempout < MINLONG)
				{
				tempout = MINLONG;
				*status = TOOBIG;
				}
		*out = (long)tempout;
		}
    else
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		tempout = (tempin1 * tempin2) * *scale + *offset;
		if (tempout > MAXLONG)
			{
			tempout = MAXLONG;
			*status = TOOBIG;
			}
		else	if (tempout < MINLONG)
				{
				tempout = MINLONG;
				*status = TOOBIG;
				}
		*out = (long)tempout;
		}
}




void rmul
(
    long ns,
    float *in1,
    float *in2,
    float *out,
    float *scale,
    float *offset,
    long *status
)
{
double	tempin1,
		tempin2,
		tempout;

    if ((*scale == 1) && (*offset == 0))
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if ((tempout = tempin1 * tempin2) > MAXREAL)
			{
			tempout = MAXREAL;
			*status = TOOBIG;
			}
		else	if (tempout < MINREAL)
				{
				tempout = MINREAL;
				*status = TOOBIG;
				}
		*out = tempout;
		}
    else
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		tempout = (tempin1 * tempin2) * *scale + *offset;
		if (tempout > MAXREAL)
			{
			tempout = MAXREAL;
			*status = TOOBIG;
			}
		else	if (tempout < MINREAL)
				{
				tempout = MINREAL;
				*status = TOOBIG;
				}
		*out = tempout;
		}
}




int c_pxmul
(
   long *ns,      /* I: number of samples                                  */
   long *in1,     /* I: input array one                                    */
   long *in2,     /* I: input array two                                    */
   long *out,     /* O: output array                                       */
   float *scale,  /* I: scale applied to input arrays after multiplication */
   float *offset, /* I: offset added in after sum                          */
   long *dtype    /* I: data type (EBYTE,EWORD,ELONG,EREAL)                */
)

{
long	status = E_SUCC;	/* return status for pxmul */

switch(*dtype)
	{
   case (EBYTE): bmul(*ns,(unsigned char *)in1,(unsigned char *)in2,
                      (unsigned char *)out,scale,offset,&status);
                 break;
   case (EWORD): wmul(*ns,(short *)in1,(short *)in2,(short *)out,scale,
                      offset,&status);
                 break;
   case (ELONG): lmul(*ns,in1,in2,out,scale,offset,&status);
                 break;
   case (EREAL): rmul(*ns,(float *)in1,(float *)in2,(float *)out,scale,
                      offset,&status);
                 break;
        default: status = TYPERR; break;
	} /* switch */
return(status);
} /* pxmul */
