/*****************************************************************************
NAME			pxsub

PURPOSE		Subtract two input arrays into one output array. Each input
		array can be scaled and an offset can be applied to the
		difference. The input arrays can be of type byte, signed 2 byte,
		signed 4 byte, or real. The output array will be of the same
		type as the input arrays.

PROGRAM HISTORY
	PROGRAMMER	DATE	 REASON
	----------	----	 ------
	D.GORDON	5/7/85	 Original development for NEWLAS
	J.REED		11/12/85 Optimized byte subroutine.
	D.GORDON	11/20/85 Optomized all subroutines.
	B.Ailts		   12/87 Change include directory specifications
				 Use raw 'C' data types
				 Place bridge routines in a seperate file
	B.Ailts		03/88	 Replaced newlas.h with worgen.h
	B.Ailts		07/88	 Fixed bug in R*4 and I*2 data
	L.Huewe		04/90	Modified the routine bsub to leave the scale
				factor a float variable to maintain accuracy

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS
		Part of the LAS system.

PROJECT				NEWLAS

ALGORITHM

receive:
	-the input arrays and their type
	-the output array
	-the number of samples of the input arrays to subtract.
	-the scaleing factors and the offset.
depending on the type of array values
	call a function for that particular type that performs the following:

for each pixel of the input arrays
	aubtract the pixels from each input array and store difference in output
	array (ie. output= inputarray1 * scale1 - inputarray2 * scale2 + offset)
	if the output values excede the boundaries of the data type then
		the output is reassigned that boundary that has been exceeded,
		and an error code is returned, either TOOBIG or TOOSMALL.
return
*note:	All operations are performed with a variable one data type larger than
	the specified data type. This is to allow for easy error checking of
	operation results exceding the bounds of the desired data type while
	maintaining a reasonably fast pixal manipulation routine.


ALGORITHM REFERENCES		none
******************************************************************************/
#include "worgen.h"
#include "pixman.h"



void bsub
(
    long ns,
    unsigned char *in1,
    unsigned char *in2,
    unsigned char *out,
    float *scale1,
    float *scale2,
    float *offset,
    long *status
)

{
long	tempout;

    if((*scale1 == 1) && (*scale2 == 1) && (*offset == 0))
	for(; ns; ns--,out++,in1++,in2++)
		{	
		if(MAXUTINY < (tempout = *in1 - *in2 ))
			{
			*out = MAXUTINY;
			*status = TOOBIG;
			}
		else	if (tempout < MINUTINY)
				{
				*out = MINUTINY;
				*status = TOOSMALL;
				}
			else *out = (unsigned char)tempout;
		}
    else 
	    	{
		for(; ns; ns--,out++,in1++,in2++)
			{
			tempout = (long)((*in1 * *scale1) - (*in2 * *scale2) + 
                                         *offset);
			if (tempout > MAXUTINY)
				{
				*out = MAXUTINY;
				*status = TOOBIG;
				}
			else	if (tempout < MINUTINY)
				{
				*out = MINUTINY;
				*status = TOOSMALL;
				}
			else *out = (unsigned char)tempout;
			}
		}
}


void wsub
(
    long ns,
    short *in1,
    short *in2,
    short *out,
    float *scale1,
    float *scale2,
    float *offset,
    long *status
)
{
long	tempin1,
		tempin2,
		tempout;

    if ((*scale1 == 1) && (*scale2 == 1) && (*offset == 0))
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if ((tempout = tempin1 - tempin2)  > MAXWORD)
			{
			tempout = MAXWORD;
			*status = TOOBIG;
			}
		else	if (tempout < MINWORD)
				{
				tempout = MINWORD;
				*status = TOOSMALL;
				}
		*out = (short)tempout;
		}
    else
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		tempout = (long)(((float)tempin1 * *scale1) - 
                                 ((float)tempin2 * *scale2) + *offset);
		if (tempout > MAXWORD)
			{
			tempout = MAXWORD;
			*status = TOOBIG;
			}
		else	if (tempout < MINWORD)
				{
				tempout = MINWORD;
				*status = TOOSMALL;
				}
		*out = (short)tempout;
		}
}	



void lsub
(
    long ns,
    long *in1,
    long *in2,
    long *out,
    float *scale1,
    float *scale2,
    float *offset,
    long *status
)
{	
double	tempin1,
		tempin2,
		tempout;

    if ((*scale1 == 1) && (*scale2 == 1) && (*offset == 0))
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if ((tempout = tempin1 - tempin2)  > MAXLONG)
			{
			tempout = MAXLONG;
			*status = TOOBIG;
			}
		else	if (tempout < MINLONG)
				{
				tempout = MINLONG;
				*status = TOOSMALL;
				}
		*out = (long)tempout;
		}
    else
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		tempout = (tempin1 * *scale1) - (tempin2 * *scale2) + *offset;
		if (tempout > MAXLONG)
			{
			tempout = MAXLONG;
			*status = TOOBIG;
			}
		else	if (tempout < MINLONG)
				{
				tempout = MINLONG;
				*status = TOOSMALL;
				}
		*out = (long)tempout;
		}
}



void rsub
(
    long ns,
    float *in1,
    float *in2,
    float *out,
    float *scale1,
    float *scale2,
    float *offset,
    long *status
)
{	
double	tempin1,
		tempin2,
		tempout;

    if ((*scale1 == 1) && (*scale2 == 1) && (*offset == 0))
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		if ((tempout = tempin1 - tempin2) > MAXREAL)
		if (tempout > MAXREAL)
			{
			tempout = MAXREAL;
			*status = TOOBIG;
			}
		else	if (tempout < MINREAL)
				{
				tempout = MINREAL;
				*status = TOOSMALL;
				}
		*out = tempout;
		}
    else
	for(; ns; ns--,out++,in1++,in2++)
		{
		tempin1 = *in1;
		tempin2 = *in2;
		tempout = (tempin1 * *scale1) - (tempin2 * *scale2) + *offset;
		if (tempout > MAXREAL)
			{
			tempout = MAXREAL;
			*status = TOOBIG;
			}
		else	if (tempout < MINREAL)
				{
				tempout = MINREAL;
				*status = TOOSMALL;
				}
		*out = tempout;
		}
}




int c_pxsub
(
   long *ns,      /* I: number of samples                                   */
   long *in1,     /* I: input array one                                     */
   long *in2,     /* I: input array two                                     */
   long *out,     /* O: output array                                        */
   float *scale1, /* I: scale applied to input array one before subtraction */
   float *scale2, /* I: scale applied to input array two before subtraction */
   float *offset, /* I: offset added in after difference                    */
   long *dtype    /* I: data type (EBYTE,EWORD,ELONG,EREAL)                 */
)

{
long	status = E_SUCC;	/* return status for pxadd */

switch(*dtype)
	{
   case (EBYTE): bsub(*ns,(unsigned char *)in1,(unsigned char *)in2,
                      (unsigned char *)out,scale1,scale2,offset,&status);
                 break;
   case (EWORD): wsub(*ns,(short *)in1,(short *)in2,(short *)out,scale1,
                      scale2,offset,&status);
                 break;
   case (ELONG): lsub(*ns,in1,in2,out,scale1,scale2,offset,&status);
                 break;
   case (EREAL): rsub(*ns,(float *)in1,(float *)in2,(float *)out,scale1,
                      scale2,offset,&status);
                 break;
        default: status = TYPERR; break;
	} /* switch */
return(status);
} /* pxsub */
