/******************************************************************************

NAME:   tperror

FUNCTION:       To print out the correct tapeio error message

   Program History:
   version	Date	Contrib		Change
   -------	----	-------		------
    1.0                                 Original development
    2.0         12/87   F. Irani/SAR    Add qiow call to sense tape mode on
                                        unsuccessful read.  If status = EOT 
					return (TAPE_EOT)
    2.1         12/87   B. Ailts        Change include directory specifications
                                        Placed bridge routines in a seperate 
					file. Added 'c_' to function name
    2.2         6/89    K.Gacke         Port to SUN 4
    2.3         7/90    K.Gacke         output system error code on error condition
    3.0		1/91	K.Gacke		tape config file
    3.1		3/91	K.Gacke		Add Silicon Graphics
    3.2         8/94    D. Etrheim      Change 777 to 666 on file open
    3.3         10/98   D. Hames        Add code to comply
                                        with localtime & y2k



COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:

PROJECT:        LAS

ALGORITHM (FLOW):
   Based upon the code recieved print out the correct error message

******************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>

#include "tae.h"
#include "worgen.h"
#include "tapeio.h"
#include "local_protos.h"

static int logit
(
    char *txt
);

static void size_check
(
    char *txt,
    char *fname,
    FILE *fp,
    struct tm *tm_ptr,
    long *year
);

void tp_log
(
    struct TAPEDESC *tp
);


int tperror
(
    struct TAPEDESC *tp,
    long code,
    long mode
)
{
#ifdef unix
#ifndef linux
extern char *sys_errlist[];
#endif
#endif

long nonfatal = 1;
char error_key[30];
char buf[ERRLEN];
char msg_buf[ERRLEN];

if ((tp != NULL) && (tp->err != 0))
    {
    tp_log(tp);
#ifdef unix
    sprintf(buf,"%s\n",sys_errlist[tp->err]);
    strcpy(error_key,"tapeio-system");
    c_errmsg(buf,error_key,&nonfatal);
    tpmsg(buf);
#endif
    tp->err = 0;
    }
if (mode == NO_HANDLE)
   return(code);
strcpy(error_key," ");
switch (code)
    {
    case TAPE_SUCC:
	*buf = '\0';
        break;
    case TAPE_FAIL:
        sprintf(buf,"%s","Fatal tape I/O error encountered");
        strcpy(error_key,"tapeio-fatal");
        break;
    case TPOPEN_OPEN:
        sprintf(buf,"Error opening tape on drive %s",tp->name);
        strcpy(error_key,"tapeio-open");
        break;
    case TPOPEN_OFFLINE:
        sprintf(buf,"Error accessing tape on drive %s.  %s",
                tp->name,"Ensure the tape drive is online.");
        strcpy(error_key,"tapeio-offline");
        break;
    case TPOPEN_WLOCK:
        sprintf(buf,"Error accessing tape on drive %s.  %s",tp->name,
                "Check tape for write ring and ensure tape drive is online.");
        strcpy(error_key,"tapeio-access");
        break;
    case TPOPEN_NOAVAIL:
        sprintf(buf,"No tape drive is available at this time");
        strcpy(error_key,"tapeio-avail");
        break;
    case TPOPEN_REW:
        sprintf(buf,"Error rewinding tape on drive %s.",tp->name);
        strcpy(error_key,"tapeio-rewind");
        break;
    case TPOPEN_SKIP:
        sprintf(buf,"Error skipping tape files on drive %s.",tp->name);
        strcpy(error_key,"tapeio-skip");
        break;
    case TPOPEN_MNT:
        sprintf(buf,"Error mounting tape on drive %s.",tp->name);
        strcpy(error_key,"tapeio-mount");
        break;
    case TPOPEN_MNTSZ:
        sprintf(buf,"%s","Too many mounted devices");
        strcpy(error_key,"tapeio-mntsize");
        break;
    case TPOPEN_ALL:
        sprintf(buf,"Error allocating tape %s",tp->name);
        strcpy(error_key,"tapeio-alloc");
        break;
    case TPOPEN_OPS:
        sprintf(buf,"%s",
           "Error opening communication line to operator's console");
        strcpy(error_key,"tapeio-console");
        break;
    case TPOPEN_OPSWR:
        sprintf(buf,"%s","Error sending mount message to operator");
        strcpy(error_key,"tapeio-mntmsg");
        break;
    case TPOPEN_OPSRD:
        sprintf(buf,"%s","Error receiving tape drive from the operator");
        strcpy(error_key,"tapeio-receive");
        break;
    case TPOPEN_SOCKET:
        sprintf(buf,"%s",
           "Error creating socket to send mount message to the operator");
        strcpy(error_key,"tapeio-socket");
        break;
    case TPOPEN_ASN:
        sprintf(buf,"Error assigning tape on drive %s.",tp->name);
        strcpy(error_key,"tapeio-assign");
        break;
    case TPOPEN_DENS:
        sprintf(buf,"Invalid density for tape drive %s.",tp->name);
        strcpy(error_key,"tapeio-density");
        break;
    case TPOPEN_DRNUM:
        sprintf(buf,"%s","Invalid drive number from the operator");
        strcpy(error_key,"tapeio-drive");
        break;
    case TP_SKIP:
        sprintf(buf,"%s","Number of records/files must be a positive number");
        strcpy(error_key,"tapeio-positive");
        break;
    case TPBSR_BSR:
        sprintf(buf,"Error back spacing records on tape drive %s",tp->name);
        strcpy(error_key,"tapeio-backrec");
        break;
    case TPBSF_BSF:
        sprintf(buf,"Error back spacing files on tape drive %s",tp->name);
        strcpy(error_key,"tapeio-backfile");
        break;
    case TPCLOS_EOF:
        sprintf(buf,"%s","Invalid number of EOF marks on tape close");
        strcpy(error_key,"tapeio-marks");
        break;
    case TPWEOF_WEOF:
        sprintf(buf,"Error writing tape EOF on tape drive %s",tp->name);
        strcpy(error_key,"tapeio-writeeof");
        break;
    case TPCLOS_CLOSE:
        sprintf(buf,"Error closing tape drive %s.",tp->name);
        strcpy(error_key,"tapeio-close");
        break;
    case TPFSF_FSF:
        sprintf(buf,"Error forward spacing files on tape drive %s",tp->name);
        strcpy(error_key,"tapeio-forwfile");
        break;
    case TPFSR_FSR:
        sprintf(buf,"Error forward spacing records on tape drive %s",tp->name);
        strcpy(error_key,"tapeio-forwrec");
        break;
    case TPREAD_READ:
        sprintf(buf,"Error reading record on tape drive %s",tp->name);
        strcpy(error_key,"tapeio-read");
        break;
    case TPREW_CLOSE:
        sprintf(buf,"Error closing tape drive %s for rewind operation",
                tp->name);
        strcpy(error_key,"tapeio-rewclose");
        break;
    case TPREW_OPEN:
        sprintf(buf,"Error opening tape drive %s for rewind operation",
                tp->name);
        strcpy(error_key,"tapeio-rewopen");
        break;
    case TPSK_CLOSE:
        sprintf(buf,"Error closing tape drive %s for forward skip operation",
                tp->name);
        strcpy(error_key,"tapeio-fsclose");
        break;
    case TPSK_OPEN:
        sprintf(buf,"Error opening tape drive %s for forward skip operation",
                tp->name);
        strcpy(error_key,"tapeio-fsopen");
        break;
    case TPREW_OFFL:
        sprintf(buf,"Error rewinding tape off line on tape drive %s",
                tp->name);
        strcpy(error_key,"tapeio-rewoff");
        break;
    case TPREW_REW:
        sprintf(buf,"Error rewinding tape on tape drive %s",
                tp->name);
        strcpy(error_key,"tapeio-rewind");
        break;
    case TPWRIT_SIZE:
        sprintf(buf,
                "%s %s.  Record size must be greater than %d and less than %d.",
                "Error writing record on tape drive",tp->name,
		tp->minbyte,tp->maxbyte);
        strcpy(error_key,"tapeio-recsize");
        break;
    case TPWRIT_WRITE:
        sprintf(buf,"Error writing record on tape drive %s",tp->name);
        strcpy(error_key,"tapeio-write");
        break;
    case TPBSF_NO:
        sprintf(buf,"Tape drive %s does not support BSF operation",tp->name);
        strcpy(error_key,"tapeio-nobsf");
        break;
    case TPBSR_NO:
        sprintf(buf,"Tape drive %s does not support BSR operation",tp->name);
        strcpy(error_key,"tapeio-nobsr");
        break;
    case TPFSF_NO:
        sprintf(buf,"Tape drive %s does not support FSF operation",tp->name);
        strcpy(error_key,"tapeio-nofsf");
        break;
    case TPFSR_NO:
        sprintf(buf,"Tape drive %s does not support FSR operation",tp->name);
        strcpy(error_key,"tapeio-nobsr");
        break;
    case TPAPP_NO:
        sprintf(buf,"Tape drive %s cannot append data",tp->name);
        strcpy(error_key,"tapeio-noapp");
        break;
    case TPEOT_NO:
        sprintf(buf,"Tape drive %s does not detect EOT properly",tp->name);
        strcpy(error_key,"tapeio-noeot");
        break;
    case TP_NONET:
        sprintf(buf,"%s %s","Application program is not built for",
		"accessing remote tape drives");
        strcpy(error_key,"tapeio-nonet");
        break;
   default:
        sprintf(buf,"Undefined error code in tperror -- code = %d",code);
        strcpy(error_key,"tapeio-undefined");
        break;
   }
if (strlen(buf))
    {
    sprintf(msg_buf,"%s\n",buf);
    c_errmsg(buf,error_key,&nonfatal);
    tpmsg(msg_buf);
    }
return(code);
}

void tp_log
(
    struct TAPEDESC *tp
)
{
long nonfatal = 1;
char buf[ERRLEN];
char msg_buf[ERRLEN];
char prcnam[ERRLEN];

getprc(prcnam);
sprintf(buf,"Program %s: drive %s system error code = %d",
	prcnam,tp->name,tp->err);
sprintf(msg_buf,"%s\n",buf);
c_errmsg(buf," ",&nonfatal);
logit(buf);
tpmsg(msg_buf);
return;
}

#define MAXFSIZ 1024 * 500      /* new file after 500 K bytes */
#define FNAME   "tape_err.log"

static int logit
(
        char *txt
)
{
        long            my_time;
        struct tm       *tm_ptr;
        char username[100];
        char fname[256];
        char env_val[256];
        char *ptr;
        FILE *fp;
        long year; 
/*
  Translate the APPLOG logical name or environment variable into a
  text string and append the "applog.log" file name.
*/
#ifdef vms
        if (get_logical("APPLOG",env_val) != E_SUCC)
                return(E_FAIL);
        sprintf(fname,"%s%s",env_val,FNAME);
#endif
#ifdef unix
        if ((ptr = getenv("APPLOG")) == NULL)
                return(E_FAIL);
        strcpy(env_val,ptr);
        sprintf(fname,"%s/%s",env_val,FNAME);
#endif

        my_time=time(0);
        tm_ptr=localtime(&my_time);
/*
  Open the log file and check the current size.  If the file size is greater
  than the constant MAXFSIZ, the log file is renamed with the date appended
  and function logit is called recursively.  Otherwise the program is logged.
*/
        fp = fopen(fname, "a");
        if (fp == NULL)
            {
            printf("error open tapeio error log file\n");
            return(E_FAIL);
            }
        chmod(fname,0666);

/*
  The log file entry consists of the date, time, user code, and program name.
*/

        year = tm_ptr->tm_year + 1900;
        year = year%100;              /* used to retrieve 2 year digits for
                                         dates after the year 2000 */

        size_check(txt,fname,fp,tm_ptr,&year);
        getusern(username);

        fprintf(fp,"%2d/%02d/%02d %02d:%02d:%02d    "
        ,tm_ptr->tm_mon+1,tm_ptr->tm_mday,year
        ,tm_ptr->tm_hour,tm_ptr->tm_min,tm_ptr->tm_sec);
        fprintf(fp,"%-12s %s\n",username, txt);
        fclose(fp);
        return(E_SUCC);
}

static void size_check
(
    char *txt,
    char *fname,
    FILE *fp,
    struct tm *tm_ptr,
    long *year
)
{
char newnam[256];

fseek(fp,0,SEEK_END);
if (ftell(fp) > MAXFSIZ)
        {
        fclose(fp);
        sprintf(newnam,"%s_%2d%02d%02d",fname,year,
                tm_ptr->tm_mon+1,tm_ptr->tm_mday);
        rename(fname,newnam);
        logit(txt);
        }
return;
}
