/******************************************************************************
NAME:   C_GETOUT

FUNCTION:
        C_GETOUT is a routine which parses the output image specifications.
        It appends the image extension to the names if the extension was
        not specified.  A bad status is returned if the image file already
        exists on disk.

PROGRAM HISTORY:
  Version       Date   Author       Request
  -------       ----   ------       -------
    1.0         8/86   K. Gacke     initial development
    1.1        10/86   K. Gacke     changed NAMLEN references to CMLEN
    1.2        12/86   K. Gacke     allow user to specify images with
                                    extensions other the "img"
    2.0        12/87   K. Zanter    Change 1) type defs to raw C types
                                    and 2) string descriptors to strings
                                    Change include directory specifications
                                    Place bridge routines in a seperate file
    5.0        04/88   D. Hollaren  changed include spec to las
                                    changed cerrmsg to c_errmsg and
                                    passed error key by address
    5.1        09/88   B. Ailts     If no output image name is specified
                                    taeout and hosout return as a blank
    5.2        09/88   B. Ailts     If no output image name is specified
                                    taeout and hosout now return a null value
    5.3        07/90   D. VanderZee Standardized error handling 
    5.4        11/93   T. Mittan    Modified compression option to be 
                                    set by a LAS global
    7.0        08/98   J. Janssen   Modified to allow multiple output 
                                    image specifications 

PROJECT:        LAS

ALGORITHM:
        Retrieve the output image from the TAE parameter block.
        If the count of the output image name equals zero
           Copy a blank into the host file name argument
           Copy a blank into the las file name argument
           Return
        For each image specification
           Substitute any aliases in the TAE parameter
           If it does not have the image extension,
              the image extension is appended.
           The full host file name is retrieved and 
              a check is made to see if the host file exists.
           If it exists, 
              a bad status code is returned.
        Retrieve the compres_opt

NOTE:
        If nothing is found for the TAE parameter, then "\0" is returned in
        the 0th element of the return string.

******************************************************************************/

#include <stdio.h>
#include <string.h>

#include "las.h"
#include "util.h"
#include "alias.h"
#include "labserv.h"

long compres_opt = 0;

int c_get_multiple_outs
(
   struct PARBLK *vblock, /* I:  TAE parameter block                   */
   int max_num_of_outs,   /* I:  Maximum number of output files that 
                                 can be retrieved                      */ 
   char *param,           /* I:  TAE parameter name                    */
   char hosout[][CMLEN],  /* O:  Host image file name(s)               */
   int *number_of_outs    /* O:  Total count of output files           */
)

{
struct VARIABLE *val;
long  i;
long  mode = 0;
long  nonfatal = 1;
char *ptr;
char msg[ERRLEN+1];
char taeout[CMLEN];
char newtae[CMLEN];
char inparm[CMLEN];
char compres_flag[CMLEN];

val = p_find(vblock,param);
*number_of_outs = val->v_count;

if (*number_of_outs == 0)
   {
   strcpy(hosout[0],"\0");
   return(E_SUCC);
   }

/* Issue a fatal error if the number of outs exceeds the size of the 
   output array
   ---------------------------------------------------------------------*/
if (*number_of_outs > max_num_of_outs)
   {
   c_lasmsg(FALSE, "get_mult_out-call", "The number of output files %ld "
           "is greater than the size of the output array %ld", 
            *number_of_outs, max_num_of_outs);
   return (E_FAIL);
   }

for (i = 0; i < *number_of_outs; i++)
   {
   strcpy(inparm,SVAL(*val,i));
   if (c_getalias(inparm,taeout) != E_SUCC)
      {
      c_errmsg("Error returned from getalias","getout-call",&nonfatal);
      return (E_FAIL);
      }

   /* Determine whether the filename is specified in TAE (VMS) format
      or Unix format.
      ---------------------------------------------------------------*/
   if (strchr(taeout, TAE_DIR_BEG) || strchr(taeout, TAE_EXT))
      {
      /* If no extension, append ";img". */
      if ((ptr = strrchr(taeout, TAE_EXT)) == 0)
         sprintf(newtae, "%s%c%s", taeout, TAE_EXT, "img");
      else 
         {
         if (strchr(taeout, TAE_EXT) != ptr)
            {
            c_errmsg("Error: Invalid LAS image name: Multiple extensions "
                     "not allowed", "getout-name", &nonfatal);
            return(E_FAIL);
            }
         strcpy(newtae, taeout);
         }
      }
   else    /* Unix format */
      {
      /* If no ".img" extension, add. */
      if (strstr(taeout, ".img") == NULL)
         sprintf(newtae, "%s%c%s", taeout, HOST_EXT, "img");
      else 
         strcpy(newtae, taeout);
      }

   if (c_cmhost(newtae,hosout[i]) != E_SUCC)
      {
      c_errmsg("Error returned from cmhost","getout-call",&nonfatal);
      sprintf(msg,"Error retrieving host file name for %s",newtae);
      c_errmsg(msg,"getout-cmhost",&nonfatal);
      return(E_FAIL);
      }

   if (c_lsstat(hosout[i],&mode) == E_SUCC)
      {
      sprintf(msg,"Error: %s already exists",newtae);
      c_errmsg(msg,"getout-exists",&nonfatal);
      return(E_FAIL);
      }
   }

val = p_find(vblock,"$COMPFLG");
if (val->v_count != 0)
   {
   strcpy(compres_flag,SVAL(*val,0));
   if (strcmp(compres_flag,"ON") == 0)
      compres_opt = COMPRES;
   else
   if (strcmp(compres_flag,"on") == 0)
      compres_opt = COMPRES;
   else
   if (strcmp(compres_flag,"OFF") == 0)
      compres_opt = NO_COMPRES;
   else
   if (strcmp(compres_flag,"off") == 0)
      compres_opt = NO_COMPRES;
   else
      compres_opt = 0;
   }

return(E_SUCC);
}

/*****************************************************************************
     Function:  c_getout
     Purpose:
                This routine calls c_get_multiple_outs and returns
                the TAE and host output image file names.

*****************************************************************************/
#include "las.h"

int c_getout
(
   struct PARBLK *vblock, /* I:  TAE parameter block                        */
   char *param,           /* I:  TAE parameter name                         */
   char taeout[][CMLEN],  /* O:  TAE image name(s)                          */
   char hostout[][CMLEN]  /* O:  Host image file name(s)                    */
)

{
    int index;              /* Variable used for looping       */
    int number_of_outs;     /* Total number of output files    */

    /* Call c_get_multiple_outs to retrieve the host and TAE names
    --------------------------------------------------------------*/
    if (c_get_multiple_outs(vblock, MAXBND, param, hostout,
        &number_of_outs) != E_SUCC)
    {
        c_lasmsg(FALSE, "getout-call", 
                 "Error returned from c_get_multiple_outs");
        return (E_FAIL);
    }

    /* Copy the the host output image file names to taeout
    -----------------------------------------------------*/
    for (index = 0; index < number_of_outs; index++)
        strcpy(taeout[index], hostout[index]);

    return (E_SUCC);
}

