/******************************************************************************
NAME:	HSTOUT

FUNCTION:
	HSTOUT is an application callable function which is called to 
	update the history records of a specified image.  It is callable 
	from both FORTRAN and C programs.

	HSTOUT creates the history file for the specified image.  The history
	includes the date and time the module was executed, system on which 
	the module was executed (VAX or UNIX), user login name, module name, 
	and the TAE parameter values used.  Optionally, all history records 
	of the input image(s) may be copied to the output image's history file.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    1.0         8/85       K. Gacke     initial development
    2.0         4/86       K. Gacke     LASD  (VMS LAS)
    3.0         8/86       K. Gacke     use the new label service routines  
    4.0         9/86       K. Gacke     used the host file names rather than
				      	TAE file name.
    5.0        12/87       B. Ailts	Change include directory specifications
					Replaced DESC arguments with char. str.
					Use raw 'C' types
					Placed bridge routines in seperate file
    5.0		04/88	   D. Hollaren  changed include spec to las
					changed cerrmsg to c_errmsg and
					passed status code by address
					removed call to squeeze routine
					return E_SUCC/E_FAIL
    5.2		07/88	  B. Ailts	Removed the garbage characters that
					appeared in the VMS  history file
    5.3		10/88	  B. Ailts	Removed the flag argument -- this 
					routine will retrieve the $HISTRY
					global itself
    5.4		01/90	  B. Davis	Made the function cphist non-static,
					so that it may be called by the world
					outside of c_hstout.c.
    5.5        07/90      D. VanderZee	Standardized error handling 
    5.6        08/92      T. Rockvam	Change the size of malloc in formhist.

PROJECT:	LAS

ALGORITHM:
	If the TAE global $HISTRY = "YES" then the input image(s) history 
	records are copied to the output image's history file.

	The TAE parameter block is read and inserted into the history record
	and written to the output image's history file.

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <time.h>
#include "las.h"
#include "util.h"
#include "labserv.h"

static int RECSIZ = 1024;

/*
   Function hstout is the main routine.  It opens (or creates) the history
   file for write access.  If the input image(s) history records are to be
   copied, it calls cphist to do so.  It then calls formhist to format and
   output the history record for the current proc.
*/
int c_hstout
(
   struct PARBLK *vblock, /* I:  pointer to TAE parameter block            */
   char *hosout,          /* I:  image whose history is to be updated      */
   char (*hosin)[CMLEN],  /* I:  image names whose history is to be copied */
   long *numin,           /* I:  number of images specified in "hosin"     */
   char *histext          /* I:  text to be included in history record     */
)

{
struct VARIABLE *var;		/* the value of the parameter		      */
long	access = 2;		/* history file is opened for write access    */
long	action = 0;		/* type of action to take place on close      */
long	nonfatal = 1;		/* Non-fatal code for c_errmsg		      */
long	fd;			/* file descriptor to history file	      */
char    outhist[CMLEN];		/* output host file name buffer               */
char    flag[4];		/* history flag				      */

/* First call routine to copy all associated files 
--------------------------------------------------*/
c_cpasso(vblock,hosout,hosin,numin);


c_lsmknm(hosout,"his",outhist); 
if (c_lsopen(&fd,outhist,&access) != E_SUCC)
   {
   c_errmsg ("Error returned from lsopen","hstout-call",&nonfatal);
   return(E_FAIL);
   }

var = p_find(vblock,"$HISTRY");
strcpy (flag,SVAL(*var,0));
/*
*   If the input image(s) history records are to be copied, then call cphist
*/
if (strncmp(flag,"YES",3) == 0)
   {
   if (cphist(fd,hosin,numin) != E_SUCC)
      c_errmsg("Error copying input image(s) history file","hstout-copy",
		&nonfatal);
   }

/*
*   Call formhist to format and output the history record for the current proc.
*/
if (formhist(fd,vblock,histext) != E_SUCC)
   {
   c_errmsg("Error writing record to history file","hstout-write",&nonfatal);
   return(E_FAIL);
   }

/*
*   Close the history file and return to calling program
*/
if (c_lsclos(&fd,outhist,&action) != E_SUCC)
   {
   c_errmsg ("Error returned from lsclos","hstout-call",&nonfatal);
   return(E_FAIL);
   }

return(E_SUCC);
}




/*
*   Function cphist copies the history records of the input image(s) into the
*   history file of the output image.  
*/
int cphist
(
   long fd,               /* I:  file descriptor to opened history file    */
   char (*hosin)[CMLEN],  /* I:  input image names                         */
   long *numin            /* I:  number of input images                    */
)

{
long	access = 0;		/* type of access to input image history file*/
long	action = 0;		/* normal close of the history file          */
long	nonfatal = 1;		/* Non-fatal code for c_errmsg		     */
long	clen;			/* char length of label service record       */
long	dlen = 0;	        /* data length of label service record       */
long	image;			/* loop counter for number of input images   */
long	infd;			/* file descriptor of input images's history */
long	return_stat = E_SUCC;   /* return status code                        */
long	status;			/* return status code                        */
char	*dbuf;			/* data buffer pointer                       */
char	dtype[4];		/* data type buffer                          */
char	*histrec;	        /* history record                            */
char    inhist[CMLEN];          /* buffer for input history host file name   */
char	key[16];		/* key buffer                                */

if ((histrec = (char *)malloc(RECSIZ)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","hstout-alloc",&nonfatal);
   return(E_FAIL);
   }

/*
*   For each input image: open the history file, read the history records, write
*   the history records to the history file of the output image.  
*/
for (image = 0; image < *numin; image++)
   {
   c_lsmknm(hosin[image],"his",inhist);

   if (c_lsopen(&infd,inhist,&access) != E_SUCC)
      {
      continue;
      }

   for (;;)
      {
      strcpy(key," ");
      clen = RECSIZ;
      status = c_lsread(&infd,key,&clen,&dlen,histrec,(unsigned char *)dbuf,
                        dtype);
      if (status == E_SMAL)
	{
	free (histrec);

        if ((histrec = (char *)malloc (clen)) == NULL)
	  {
	  c_errmsg ("Error allocating dynamic memory","hstout-alloc",&nonfatal);
	  return (E_FAIL);
          }

	status = c_lsread(&infd,key,&clen,&dlen,histrec,(unsigned char *)dbuf,
                      dtype);
	if (status == E_FAIL)
	  {
 	  c_errmsg ("Error returned from lsread","hstout-call",&nonfatal);
	  return (E_FAIL);
	  }
	}
      else if (status != E_SUCC)
         {
	 if (status == E_FAIL)
	    {
 	    c_errmsg ("Error returned from lsread","hstout-call",&nonfatal);
	    return_stat = E_FAIL;
	    }
	 break;
	 }

      if (c_lswrit(&fd,key,&clen,&dlen,histrec,dbuf,dtype) != E_SUCC)
	 {
 	 c_errmsg ("Error returned from lswrit","hstout-call",&nonfatal);
	 return_stat = E_FAIL;
	 break;
  	 }
      }

   if (c_lsclos(&infd,inhist,&action) != E_SUCC)
      {
      c_errmsg ("Error returned from lsclos","hstout-call",&nonfatal);
      return_stat = E_FAIL;
      }
      
   }

free(histrec);

return(return_stat);
}





/*
*   Function formhist is a routine which formats and outputs the logical history
*   record for the current proc being executed.  It first finds the proc name
*   and the subcommand (if it exists).  The history record consists of: 
*	1)  Date
*       2)  System that the module is being run on (VMS or UNIX)
*	3)  User's login name executing the module
*	4)  Proc-Subcommand name
*	5)  History text supplied by the calling program
*	6)  Parameter names and their values
*   The vblock is read to obtain parameter names and their values in a 
*   displayable form (excluding the global and local variables).  The history 
*   record consists of parameter names and their values of the proc.  To help 
*   uncompact the record, each parameter name and its value are separated by 
*   a new line character.  
*/
int formhist
(
   long fd,               /* I:  file descriptor to opened history file    */
   struct PARBLK *vblock, /* I:  pointer to TAE parameter block            */
   char *text             /* I:  history text supplied by calling program  */
)

{
#ifdef vms			/* define SYSTEM to be the system the module */
#define SYSTEM "VMS"		/*   is being executed on                    */
#else
#define SYSTEM "UNIX"
#endif

#define MAXLEN  1024		/* maximum line length of v-block text       */

struct VARIABLE	*val;		/* pointer to variable with v-block          */
long	clen;			/* length of history record                  */
long	dlen = 0;		/* length of data buffer                     */
long	nonfatal = 1;		/* Non-fatal code for c_errmsg		     */
long	length;			/* length of text line obtained from v-block */
long	status;			/* return status code                        */
long	sec;			/* time in elapsed seconds                   */
char	*dbuf = NULL;           /* data buffer                               */
char	dtype[4];		/* data type of associated history file      */
char    *cptr;			/* pointer to time character string          */
char    *histrec;		/* history record                            */
char    *tmp_ptr;		/* temporary pointer */
char    key[16];		/* key of history record to be written       */
char	line[MAXLEN];		/* line of text obtained from v-block        */
char	proc[10];		/* proc name				     */
char	subcmd[10];		/* subcommand name			     */
char	user[20];		/* user name                                 */

/* Malloc RECSIZ+1 to allow for null charater that strcat appends.  Otherwise
   problems occur when the sum of the strlen's equal RECSIZ */
if ((histrec = (char *)malloc(RECSIZ+1)) == NULL)
   {
   c_errmsg("Error allocating dynamic memory","hstout-alloc",&nonfatal);
   return(E_FAIL);
   }

time(&sec);					/* Get the time and computer  */
cptr = ctime(&sec);				/*   system the program is    */
sprintf(histrec,"\n%s%s\n",cptr,SYSTEM);	/*   being executed on        */

getusern(user);					/* Get the user login name    */
strcat(histrec,user);
cptr = histrec + strlen(histrec);

val = p_find(vblock,"_PROC");			/* Find the proc-subcommand   */
strcpy(proc,SVAL(*val,0));			/*   name within the v-block  */

val = p_find(vblock,"_SUBCMD");
strcpy(subcmd,SVAL(*val,0));

if (strlen(subcmd))
   sprintf(cptr,"\n%s-%s\n%s   ",proc,subcmd,text);
else
   sprintf(cptr,"\n%s\n%s   ",proc,text);

/*
*   Obtain the parameter names and their values in a displayable form from the
*   v-block by using the TAE function "p_forp".  Note that this function also
*   obtains the global and local variables which are not wanted.  Lines of text
*   are read from the v-block and formatted into the history record.  
*/
p_inifor(vblock);
for (status=E_SUCC,length=1; status!=FP_EOP;)
   {
   if ((status = p_forp(vblock,line,length)) == FP_NULL)
      {
      ++length;
      continue;
      }

   length = 1;
   if ((*line == '_') || (*line == '$'))                   /* exclude globals */
      {
      *(histrec + strlen(histrec) - 3) = '\0';   /* erase "+"       */
      break;
      }

   *(histrec + strlen(histrec) - 1) = '\0';      /* erase "+"       */
   if ((*line != '"') && (*line != '(') && (strchr(line,'=')))
      {
      *(histrec + strlen(histrec) - 2) = '\0';   /* erase ", "      */
      strcat(histrec,"\n");
      }

   if (strlen(histrec) + strlen(line) > RECSIZ)
      {
      tmp_ptr = (char *)malloc (strlen(histrec) + strlen(line) + 1);
      if (tmp_ptr == NULL)
	{
        c_errmsg("Error allocating dynamic memory","hstout-alloc",&nonfatal);
        return(E_FAIL);
	}
      strcpy (tmp_ptr,histrec);
      free (histrec);
      histrec = tmp_ptr;
      }

   strcat(histrec,line);
   }

clen = (long)strlen(histrec);
strcpy(key," ");
strcpy(dtype,"B");

if (c_lswrit(&fd,key,&clen,&dlen,histrec,dbuf,dtype) != E_SUCC)
   {
   c_errmsg ("Error returned from lswrit","hstout-call",&nonfatal);
   return(E_FAIL);
   }

free(histrec);

return(E_SUCC);
}
