/******************************************************************************
PROGRAM:	MKNAM

PURPOSE:	Retrieves a file name specification. It appends the specified
		extension to the name if the extension was not specified.  If
		the file is to be opened for output, a bad status is returned
		if the file already exists on disk.

PROGRAM HISTORY:
  Version	Date       Author       Request
  -------	----	   ------       -------
    5.0        04/88	   D. Hollaren  Initail development taken from GETNAM
    5.0        	07/90      D. VanderZee Standardized error handling 
				

PROJECT:	 LAS

ALGORITHM:
	Solve any aliases within the input name
	If it does not have the specified extension,
	    the extension is appended.
	The full host file name is retrieved.
	If the file is to be opened for output and the file already exists,
	    a bad status code is returned.
	If the file is to be opened for input and the file does not exist,
	    a bad status code is returned.

******************************************************************************/

#include <stdio.h>
#include <string.h>
#include "las.h"
#include "util.h"
#include "alias.h"
#include "labserv.h"

int c_mknam
(
   char *filename,      /* I:  Input file name                         */
   char *outfile,       /* O:  Host file name to be returned           */
   char *ext,           /* I:  default extension                       */
   long *access         /* I:  Type of I/O access                      */
)

{
long  mode = 0;
long  nonfatal = 1;
long stat;
char msg[ERRLEN+1];
char tfull[CMLEN];
char tname[CMLEN];
char *ptr;              /* temporary character pointer                 */


stat = c_getalias(filename,tname);
if (stat != E_SUCC)
   {
   c_errmsg ("Error returned from getalias","mknam-call",&nonfatal);
   return (E_FAIL);
   }

/* Determine whether name is in VMS or Unix syntax.  Check for appropriate
   extension, and add if missing.
   -----------------------------------------------------------------------*/
if (strchr(tname, TAE_DIR_BEG) || strchr(tname, TAE_EXT))  /* VMS */
   {
   if (strrchr(tname, TAE_EXT) == NULL)	  /* if no extension add default*/
      sprintf(tfull, "%s%c%s", tname, TAE_EXT, ext);
   else
      strcpy(tfull, tname);
   }
else /* Unix */
   {
   /* Since dots can reside in the path as well as the filename,
      we need to ensure that we are checking for dots (as extensions)
      after any path that may be included.
      ---------------------------------------------------------------*/
   if ((ptr = strrchr(tname, HOST_DIR_SEP)) == NULL)
      ptr = tname;
   if (strrchr(ptr, HOST_EXT) == NULL)  /* if no extension add default*/
      sprintf(tfull, "%s%c%s", tname, HOST_EXT, ext);
   else
      strcpy(tfull, tname);
   }

if (c_cmhost(tfull,outfile) != E_SUCC)
   {
   c_errmsg ("Error returned from cmhost","mknam-call",&nonfatal);
   sprintf(msg,"Error retrieving host file name for %s",tfull);
   c_errmsg(msg,"mknam-cmhost",&nonfatal);
   return(E_FAIL);
   }

if (*access == IREAD)			/* if file is opened for read access */
   {					/*   the file must exist             */
   if (c_lsstat(outfile,&mode) != E_SUCC)
      {
      sprintf(msg,"Error: %s does not exist",tfull);
      c_errmsg(msg,"mknam-exist",&nonfatal);
      return(E_FAIL);
      }
   }

if (*access == IWRITE)			/* if file is opened for write access */
   {					/*   the file cannot exist            */
   if (c_lsstat(outfile,&mode) == E_SUCC)
      {
      sprintf(msg,"Error: %s already exists",tfull);
      c_errmsg(msg,"mknam-exists",&nonfatal);
      return(E_FAIL);
      }
   }

return(E_SUCC);
}
