/*****************************************************************************

NAME	inittm , startm, stoptm, cinittm

PURPOSE	
    inittm
	Initializes the application program for display messages.  The input
	parameters specify the format of the display message, and how often
	the message is to be outputted to the terminal.

    startm
	Starts the display message utility.

    stoptm
	Stop display message utility.  The utility may be restarted with 
	startm.

    cinittm
	C callable which builds string descriptors for its input arguments
  	and calls "inittm" to initialize the display message utility.

    NOTE:
	The catalog manager routines do NOT like to be interrupted (it brings
	the catalog manager down).  Therefore no catalog manager routines 
	should be called between the inittm and posttm calls.

DATA STRUCTURES	
  struct static_val --> a structure containing all static fields of the display
			message.  This includes the subject, verb, entity,
			category, text, current entity, maximum entity, current
			category, maximum category, number of seconds between 
			display messages, and a flag indicating the format of 
			the message.

PROGRAM HISTORY
	PROGRAMMER	  DATE		REASON
	----------	  ----		------
	K. Gacke	  11/85		Initial development of unix-based
					raster processing system (newlas).
	K. Gacke	   2/86	        Modified so that inittm does not 
					output the first display message.
	K. Gacke	   5/86         changed t_write call to xtwrit to 
					solve batch log problem with fortran
					programs.
	K. Gacke	   6/86		made the display message utility more
					flexible by adding additional parameters
					to INITTM.  These parameters are:
					CAT:  Type of category (e.g. 'bands')
					TEXT: Addition text appended to the msg.
					CUR_ENTITY:  Current entity 
					CUR_CAT:  Current category 
        K. Gacke           6/86         Removed CUR_ENTITY and CUR_CAT from 
					parameter list and made them globals.
					This was done to solve the problem on
					VMS of putting the variables in a
					register.
	K. Gacke	   8/86		Made internal functions to be "static"
        K. Gacke          11/86         Added function "cinittm" which is a
					C callable to build the string 
					descriptors and calls "inittm".  Also
					initialize global variables curent and
					curcat to 1.
	K. Gacke			Created a new function called setmsg
					and placed curent and curcat as
					arguments because ALTOS would not
					support them as globals
	B. Ailts	  12/87		Changed include directory specifications
					Replaced DESC arguments with char. str.
					Place bridge routines in a seperate file
					Use raw 'C' types
	B. Ailts	  06/88		Placed curent and curcat as globals 
					again.  ALTOS is not going to be 
					supported
	B. Ailts	  10/88		Fixed bug with linking to Fortran
	B. Ailts	  01/89		Eliminated a processing message if
					seconds is equal to zero -- this will
					assure that the batch log are not 
					corrupted by the processing message --
					msgtim is set to zero in batch
	K. Gacke	  01/90		Fixed bug in which parameters to the
					t_init call were wrong.  (PRF 5343)
	D. VanderZee	07/90		Standardized error handling
	D. Etrheim	09/92		Change reset_time from long to void


COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	none.

PROJECT:	LAS

ALGORITHM:
    INITTM
	INITTM is called by all application programs before image processing
	begins.  The input parameters specify the format of the display
	message which are assigned to a global structure.  Whenever it is time
	to output a display message, the structure is referenced.  The input 
	parameter 'flag' specifies whether all bands are processed at once, 
	or if one band is processed at a time.  

    STARTM
	STARTM is called by application programs to enable the processing
	message utility.  STARTM sets the alarm for the number the specified
	number of seconds.  When the alarm interrupt is encountered, the 
	function SETTIM is called to output the processing message.

    STOPTM
	STOPTM is called by application programs to disable the processing
	message utility.  It turns off the alarm interrupt and outputs the
	final processing message.

    NOTE:  It was required to develop a separate version for the VAX VMS
	computer and the unix computer.  In the unix environment, fortran
	and C modules can call each other without causing any side effects.
	In the VMS environment, a fortran module cannot always call a C
	function.  The reason for this is that a C program initiates a
 	process name C$MAIN which sets up the program environment such as
	interrupt signals.  A fortran program does not do this, and therefore
	interrupt signals are not caught.  For this reason, the VMS version
	is required to call system service routines directly to set the system
	timer and to call the display message routine each time the timer goes
	off.

ALGORITHM REFERENCES
	Basic idea was suggested by Ken Johnson.

******************************************************************************/

#ifdef sgi
#ifdef _CFE
#ifndef _BSD_COMPAT
#define _BSD_COMPAT
#endif
#endif
#endif

#include <stdio.h>		/* C standard I/O library                    */
#include <signal.h>		/* interrupt include file                    */

#ifdef vms
#include <ssdef.h>		/* System include file                       */
#endif

#include "las.h"		/* LAS include file                          */
#include "desc.h"
#include "util.h"


struct static_val		/* structure defining the static protions of */
	{			/*   the display message                     */
	char *subject;
	char *verb;
	char *entity;
	char *cat;
	char *text;
	long  max_entity;
	long  max_cat;

#ifdef vms
	double	msgtim;
#else
	long	msgtim;
#endif
	long	flag;
	};

long curcat;
long curent;
static long _TOTLIN;			/* Number of lines on terminal       */
static long _TOTCOL;			/* Number of columns on terminal     */
static struct static_val msg_values;	/* global display line structure     */
static void output_line(void);
static void reset_time(void);



/*
  INITTM is called by all aplication programs before image processing begins.  
  The input parameters specify the format of the display message which are 
  assigned to a global structure.  Whenever it is time to output a display 
  message, the structure is referenced.  The input parameter 'flag' specifies 
  the type of processing message to be displayed.  
*/
void c_inittm
(
   char  *subject,    /* I: subject of message (e.g. process name)     */
   char  *verb,       /* I: verb of message (e.g. 'Processing')        */
   char  *entity,     /* I: entity of display message (e.g. 'line')    */
   char  *cat,        /* I: category of message (e.g. 'band')          */
   char  *text,       /* I: appended text string of processing msg.    */
   long  *max_entity, /* I: maximum number of 'entity' processed       */
   long  *max_cat,    /* I: maximum number of bands processed          */
   long  *seconds,    /* I: number of seconds between messages         */
   long  *flag        /* I: specifies format of display message.
                            = ALLCAT -- processing all bands at once
                            = ONECAT -- processing one band at a time
                            = GEN1   -- general display message one
                            = GEN2   -- general display message two    */
)
{
char    outmsg[80];		      /* message to be outputted to terminal */
long	cc = T_STDCC;		      /* type of carriage control            */
long	status;			      /* function return status code         */
long	term_type;		      /* terminal type                       */
long    nonfatal = 1;

#ifdef vms
   {
   struct DESC  atim;		      /* character time string               */
   char		atim_buf[20];	      /* buffer storage for "atim"           */

/*
*   Convert "seconds" into an ascii string that VMS takes as an offset to
*   the current time.  Call the system sevice routine "sys$bintim" to convert
*   the ascii time into binary format, and assign this value to the global
*   structure "msg_values.msgtim".
*/
   sprintf(atim_buf,"0 :%02d:%02d",*seconds / 60, *seconds % 60);
   atim.addr = atim_buf;
   atim.length = strlen(atim_buf);
   status = sys$bintim(&atim,&msg_values.msgtim);
   if (status != SS$_NORMAL)
      status = m_msg("Error converting to binary time","INITTM");
   }
#else
   msg_values.msgtim = *seconds;
#endif
   status = t_init(&_TOTLIN,&_TOTCOL,&term_type);
   if (status != SUCCESS)
       c_errmsg("Error initializing terminal","inittm-init",&nonfatal);
   if (term_type != T_CRT)
       {
       _TOTLIN = 1;
       _TOTCOL = 80;
       }

/*
   Assign the input parameters to the global structure of msg_values.  These
   values are used for all display messages.  Ensure the null character is 
   appended to each input string parameter.        
*/

msg_values.subject = subject;
msg_values.verb = verb;	
msg_values.entity = entity;
msg_values.cat = cat;
msg_values.text = text;
msg_values.max_entity = *max_entity;
msg_values.max_cat = *max_cat;
msg_values.flag = *flag;

strcpy(outmsg," ");
t_write(outmsg,cc);
}



/*
*   Reset the system timer to start displaying processing messages.  
*/
void c_startm(void)
{

if (msg_values.msgtim != 0)
   output_line();
#ifdef vms
   {
   long status;			      /* function return status code         */
   if (msg_values.msgtim != 0)		/* if message time is greater than   */
      {					/*   zero then reset the timer.      */
      status = sys$setimr(0,&msg_values.msgtim,(long)reset_time,5); 
      if (status != SS$_NORMAL)
         status = m_msg("Error setting timer","INITTM");
      }
   }
#else
   signal(SIGALRM,reset_time);     /* call reset_time on next interrupt*/
   alarm(msg_values.msgtim);	     /* reset the alarm to appropriate number*/
#endif
}



/*
*   STOPTM is called to temporarly disable the interrupt signal.  It outputs the
*   current line processed.  It is assumed "startm" will be called to restart
*   the display message utility.
*/
void c_stoptm(void)
{
char outmsg[80];		      /* message to be written to terminal   */
long   cc = T_STDCC;		      /* terminal carriage control           */

#ifdef vms
   {
   long   status;		      /* function return status code         */
   if (msg_values.msgtim != 0)	      /* if message time is greater than     */
      status = sys$cantim(5,0);	      /*   zero then disable the timer       */
   }
#else
   alarm(0);			      /* disable the alarm                   */
#endif
if (msg_values.msgtim != 0)
   {
   if (curent - 1 == msg_values.max_entity)
      --curent;
   if (curcat - 1 == msg_values.max_cat)
      --curcat;
   output_line();

#ifdef unix
   strcpy(outmsg,"");
   t_write(outmsg,cc);
#endif

   } /*  if msg_values.msgtim != 0  */
}


static void output_line(void)
{
char	line[200];
char	*ptr;
long	len;

if (msg_values.flag == ALLCAT)	      /* output appropriate display message  */
   sprintf( line, "%-8s %s %s %d of %d for all %d %s %s",msg_values.subject, 
            msg_values.verb,msg_values.entity,curent,
            msg_values.max_entity,msg_values.max_cat,msg_values.cat,
            msg_values.text);
else if (msg_values.flag == ONECAT)
   sprintf( line, "%-8s %s %s %d of %d for %s %d of %d %s",msg_values.subject, 
            msg_values.verb,msg_values.entity,curent,
	    msg_values.max_entity,msg_values.cat,curcat,
            msg_values.max_cat,msg_values.text);
else if (msg_values.flag == GEN1)
   sprintf( line, "%-8s %s %s %d %s",msg_values.subject,msg_values.verb, 
 	    msg_values.entity,curent,msg_values.text);
else if (msg_values.flag == GEN2)
   sprintf( line, "%-8s %s %s %d of %d %s",msg_values.subject,msg_values.verb, 
 	    msg_values.entity,curent,msg_values.max_entity,
            msg_values.text);
len = (long)strlen(line);
if (len < _TOTCOL)
   {
   ptr = line + len;
   sprintf(ptr,"%*s",_TOTCOL-len-1," ");
   }
t_pos(_TOTLIN,1);
t_write(line,T_NULL);
}


/*
   reset_time is the function that is called when the alarm interrupt occurs.  
   It outputs the appropriate display message, resets the alarm so that on the
   next alarm interrupt function 'reset_time' is called again.
*/
static void reset_time(void)
{

if (curent - 1 == msg_values.max_entity)
   --curent;
if (curcat - 1 == msg_values.max_cat)
   --curcat;
output_line();
#ifdef vms
   {
   long	status;			     /* function status code                 */
   if (msg_values.msgtim != 0)		/* if message time is greater than   */
      {					/*   zero then reset the timer.      */
      status = sys$setimr(0,&msg_values.msgtim,(long)reset_time,5); 
      if (status != SS$_NORMAL)
         status = m_msg("Error setting timer","INITTM");
      }
   }
#else
   signal(SIGALRM,reset_time);	/* call reset_time on next interrupt */
   alarm(msg_values.msgtim);	     /* reset the alarm to appropriate number*/
#endif
}
