/*******************************************************************************
NAME: c_fwrite

PURPOSE:  Provides an implementation of the C library fwrite function that will
          convert data written to the export endian.

PROGRAM HISTORY:
        PROGRAMMER      DATE    REASON
        ----------      ----    ------
        T. Beckmann     11/00   Original development

ALGORITHM DESCRIPTION:
   Convert the data to the export endian
   Write the data using fwrite

ALGORITHM REFERENCES:
   none
*******************************************************************************/
#include <stdio.h>      /* fwrite prototype */
#include <string.h>     /* memcpy prototype */
#include "las.h"        /* standard LAS defines */
#include "lasendian.h"  /* byte swapping routines */
#include "lasfileio.h"  /* prototype for this function */

size_t c_fwrite
(
   const void *buf,         /* I: pointer to the buffer of data to write */
   size_t size,             /* I: size of each element written */
   size_t count,            /* I: number of elements to write */
   FILE *stream             /* I/0: stream to write to */
)
{
    /* make sure the size of each member is a power of two up to the number 
       of bytes in a 64-bit word */
    if (((size > 1) && (size & 0x01)) || (size > 8) || (size == 6))
    {
        c_lasmsg(FALSE,"fwrite-size","Element size of %d bytes is not a legal "
                 "word size", size);
        return E_FAIL;
    }

    if (count < 1)
    {
        c_lasmsg(FALSE,"fwrite-count","Element count of %d is not legal",
                 count);
        return E_FAIL;
    }

#ifdef LAS_STANDARD_ENDIAN
    /* use the normal fwrite function if standard endian machine */
    return (fwrite(buf, size, count, stream));
#else
#define BUFSIZE 4096
    if (size == 1)
    {
        /* single byte data, so no byte swapping needed */
        return (fwrite(buf, size, count, stream));
    }
    else
    {
        unsigned char swapbuf[BUFSIZE];
        size_t index;
        size_t elements = BUFSIZE/size;
        size_t elements_written = 0;
        
        /* convert the data read to the export byte order */
        for (index = 0; index < count; index += elements)
        {
            /* calculate the elements in the last sub-buffer */
            if ((index + elements) > count)
                elements = count - index;
                
            /* copy the buffer since the original buffer of data shouldn't be 
               modified, then swap the bytes */
            memcpy(swapbuf, &((char *)buf)[index * size], elements * size);
            exportendian_buf(swapbuf, size, elements);

            /* write the data */
            elements_written = fwrite(swapbuf, size, elements, stream);
            if (elements_written != elements)
            {
                return index + elements_written;
            }
        }
        
        return count;
    }
#endif

}
