/*******************************************************************************
NAME:    C_GETPARM

PURPOSE: Extract TAE parameters from a project parameter file.
	 This routine will extract the nth occurrence of a parameter and send
	 all of its values back to the calling routine.  If successful
	 c_getparm will return the number of values found for the named
	 parameter, if it is unable to find the nth occurrence of the parameter
	 it will return 0, and E_FAIL for all errors.  To find the last
	 occurrence of a parameter use a position of 0.


PROGRAM HISTORY:

VERSION  PROGRAMMER       DATE   CODE  REASON
-------  ---------------  -----  ----  -----------------------------------------
  1.0    J.Fenno          01/92  EDC   Development
  1.1    D.Etrheim        11/94  EDC   Use c_lsmknm to get file name

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
   C_GETPARM must run under TAE

PROJECT: LAS

ALGORITHM DESCRIPTION: Initial code copied from the LAS function GETPARM and
		       then modified to retrieve the nth occurrence of a
		       parameter in a ppf file.
*******************************************************************************/
#include <stdio.h>
#include <string.h>
#include "las.h"
#include "labserv.h"
#include "util.h"


int c_getparm
(
   char *filename,             /* Name of the project parameter file       */
   char *parm_name,            /* Name of the variable to be retrieved     */
   char string_value[][CMLEN], /* String values of variable being returned */
   char *type,                 /* Type of the parameter being returned     */
   double double_value[],      /* Double values of the variable returned   */
   long long_value[],          /* variable being returned                  */
   long position               /* The nth occurrence to retrieve           */
)

{
char   tmp_filename[CMLEN];  /* filename with the .ppf extension              */
char   tmp_name[CMLEN];      /* Temporary parameter name                      */
char   tmp_string[CMLEN];    /* Temporary string variable                     */
char   tmp_type[2];          /* Temporary type variable                       */

double tmp_double;           /* Temporary double value of a variable          */

long   dimension;            /* Dimension of the requested parameter          */
long   i;                    /* Loop control                                  */
long   j;                    /* Loop control                                  */
long   nonfatal = 1;         /* Nonfatal error flag                           */
long   tmp_dimension;        /* Temporary dimension of the current parameter  */
long   tmp_long;             /* Temporary long value of a variable            */

FILE   *parm_file;           /* Project parameter file                        */


/* Check for a valid position value
-----------------------------------*/
if (position < 0)
   {
   c_errmsg("Position must be greater than or equal to zero",
	    "c_getparm-pos", &nonfatal);
   return(E_FAIL);
   }

/* Add the .ppf extension onto filename
---------------------------------------*/
c_lsmknm(filename,"ppf",tmp_filename);

/* Open the project parameter file for reading
----------------------------------------------*/
if ((parm_file = fopen(tmp_filename, "r")) == NULL)
   {
   c_errmsg("Error opening file", "c_getparm-open", &nonfatal);
   return(E_FAIL);
   }

/* Convert the requested parameter name to all uppercase
--------------------------------------------------------*/
j = (long)strlen(parm_name);
c_low2up(parm_name, &j);

i = 0;
dimension = 0;
while (TRUE)
   {
   /* Read a parameter name, its type, and its number of values
   ------------------------------------------------------------*/
   j = fscanf(parm_file, "%s %s %d", tmp_name, tmp_type, &tmp_dimension);
   if ((j == EOF) && (position == 0))
      {
      fclose(parm_file);
      return(dimension);
      }
   else if (j == EOF)
      {
      fclose(parm_file);
      return(0);
      }
   else if (j != 3)
      {
      fclose(parm_file);
      c_errmsg("Invalid file format", "c_getparm-format", &nonfatal);
      return(E_FAIL);
      }

   /* Convert the parameter name and type to all uppercase
   -------------------------------------------------------*/
   j = (long)strlen(tmp_name);
   c_low2up(tmp_name,&j);
   j = (long)strlen(tmp_type);
   c_low2up(tmp_type,&j);

   /* If this is the requested parameter
   -------------------------------------*/
   if (strcmp(parm_name, tmp_name) == 0)
      {
      i++;
      strcpy(type, tmp_type);
      dimension = tmp_dimension;

      /* Read the indicated number of values for the current parameter
      ----------------------------------------------------------------*/
      if (strcmp(type, "I") == 0)
	 {
	 for (j = 0; j < dimension; j++)
	    if (fscanf(parm_file, "%d", &long_value[j]) != 1)
	       {
	       fclose(parm_file);
	       c_errmsg("Invalid file format", "c_getparm-format", &nonfatal);
	       return(E_FAIL);
	       }
	 }
      else if (strcmp(type, "S") == 0)
	 {
	 for (j = 0; j < dimension; j++)
	    if (fscanf(parm_file, "%s", string_value[j]) != 1)
	       {
	       fclose(parm_file);
	       c_errmsg("Invalid file format", "c_getparm-format", &nonfatal);
	       return(E_FAIL);
	       }
	 }
      else if (strcmp(type, "R") == 0)
	 {
	 for (j = 0; j < dimension; j++)
	    if (fscanf(parm_file, "%lf", &double_value[j]) != 1)
	       {
	       fclose(parm_file);
	       c_errmsg("Invalid file format", "c_getparm-format", &nonfatal);
	       return(E_FAIL);
	       }
	 }
      else
	 {
	 fclose(parm_file);
	 sprintf(tmp_string, "Invalid parameter type %s", type);
	 c_errmsg(tmp_string, "c_getparm-type", &nonfatal);
	 return(E_FAIL);
	 }

      /* If this is the requested occurrence the return the dimension
      ---------------------------------------------------------------*/
      if (i == position)
	 {
	 fclose(parm_file);
	 return(dimension);
	 }
      }
   else
      {
      /* Skip past the indicated number of values for the current parameter
      ---------------------------------------------------------------------*/
      if (strcmp(tmp_type, "I") == 0)
	 {
	 for (j = 0; j < tmp_dimension; j++)
	    if (fscanf(parm_file, "%d", &tmp_long) != 1)
	       {
	       fclose(parm_file);
	       c_errmsg("Invalid file format", "c_getparm-format", &nonfatal);
	       return(E_FAIL);
	       }
	 }
      else if (strcmp(tmp_type, "S") == 0)
	 {
	 for (j = 0; j < tmp_dimension; j++)
	    if (fscanf(parm_file, "%s", tmp_string) != 1)
	       {
	       fclose(parm_file);
	       c_errmsg("Invalid file format", "c_getparm-format", &nonfatal);
	       return(E_FAIL);
	       }
	 }
      else if (strcmp(tmp_type, "R") == 0)
	 {
	 for (j = 0; j < tmp_dimension; j++)
	    if (fscanf(parm_file, "%lf", &tmp_double) != 1)
	       {
	       fclose(parm_file);
	       c_errmsg("Invalid file format", "c_getparm-format", &nonfatal);
	       return(E_FAIL);
	       }
	 }
      else
	 {
	 fclose(parm_file);
	 sprintf(tmp_string, "Invalid parameter type %s", tmp_type);
	 c_errmsg(tmp_string, "c_getparm-type", &nonfatal);
	 return(E_FAIL);
	 }
      }
   }
}
