#include <stdio.h>
#include <string.h>

#include "las.h"
#include "sysdef.h"
#include "util.h"

/******************************************************************************
FUNCTION:	SYSSET

PURPOSE:	Set a system string or system numerical code.  Given the system
		string return the system numerical code or given the system
		numerical code return the system string.

PROGRAM HISTORY:
Version    Date     Author       	Change Request
-------    -----    -------------	--------------
  1.0		    K. Gacke		Original development
  5.0      02/89    D. Hollaren		Conversion to 5.0 standards
  5.1	   07/90     D. VanderZee	Standardized error handling
  5.2	   04/91    T. Baltzer		Add Silicon Graphics, Data General

COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
	None.

PROJECT			LAS

ALGORITHM 
  Check system flag for conversion from 
      system string to system numerical code
	 Convert the input system string to lower case
	 Find the input system string in the list of valid system strings
	 If the input system string was not found
	     display an error message and
	     return bad status

      OR system numerical code to system string.
	 Give the system numerical code
	     Copy the appropriate valid system string
	     to the output system string

ALGORITHM REFERENCES	none
*******************************************************************************/
int c_sysset
(
   char *csys,       /* I/O: character coded system (e.g. "gould-utx")  */
   long *nsys,       /* I/O: system numerical code (e.g. SYS_GOULD_UTX) */
   long *sysflg      /* I:  System conversion flag:                     */
                          /*  SETN = 0 - convert system string to value */
                          /*  SETS = 1 - convert system value to string */
)
{
long len;			/* Length of input system string	      */
long index;			/* Looping variable			      */
long status = E_SUCC;		/* Return status			      */
long nonfatal = 1;		/* Non-fatal error code			      */

char *system;			/* Pointer to the input system string	      */
static char *sys_name[] = 	/* Array of valid system strings	      */
		{
		UTX, BSD, ULT, VMS, IEEE, HP, PRIMOS, AIX, MVS, UNICOS,
		IRX, DG_UX, MSC, ""
		};


/*
  Check system flag for conversion from system string to system numerical code
  or system numerical code to system string.
*/
switch(*sysflg)
    {
    case SETN:
       /*
         Remove any blanks and NULL terminate the the input system string
       */
       system = squeeze(csys,CMLEN);
	/*
	  Convert the input system string to lower case
	*/
	len = (long)strlen(system);
	c_up2low(system,&len);

	/*
	  Find the input system string in the list of valid system strings
	*/
	for (index = 0; *sys_name[index] != '\0'; index++)
	    {
	    if (strcmp(system,sys_name[index]) == 0)
		{
		*nsys = index;
		break;
		}
	    }

	/*
	  If the input system string was not found display an error message
	  and return bad status
	*/
	if (*sys_name[index] == '\0')
	    {
	    c_errmsg("Error: Invalid system string code","sysset-invsys",
			&nonfatal);
	    status = E_FAIL;
	    }
        free(system);
	break;

    case SETS:
	/*
	  Copy the appropriate valid system string to the output system string
	  give the system numerical code
	*/
	switch(*nsys)
	    {
	    case SYS_GOULD_UTX:
		strcpy(csys,UTX);
		break;

	    case SYS_SUN_BSD:
		strcpy(csys,BSD);
		break;

	    case SYS_VAX_ULT:
		strcpy(csys,ULT);
		break;

	    case SYS_VAX_VMS:
		strcpy(csys,VMS);
		break;

	    case SYS_IEEE_STD:
		strcpy(csys,IEEE);
		break;

	    case SYS_HP_MPE:
		strcpy(csys,HP);
		break;

	    case SYS_PRIME_PRIMOS:
		strcpy(csys,PRIMOS);
		break;

	    case SYS_IBM_AIX:
		strcpy(csys,AIX);
		break;

	    case SYS_IBM_MVS:
		strcpy(csys,MVS);
		break;

	    case SYS_CRAY_UNICOS:
		strcpy(csys,UNICOS);
		break;

	    case SYS_SGI_IRX:
		strcpy(csys,IRX);
		break;

	    case SYS_DG_UX:
		strcpy(csys,DG_UX);
		break;

	    case SYS_OTHER_MSC:
		strcpy(csys,MSC);
		break;

	    default:
		c_errmsg("Error: Invalid system numerical code","sysset-invsys",
		          &nonfatal);
		status = E_FAIL;
		break;
	    }
	break;

    default:
	c_errmsg("Error: Invalid system conversion option","sysset-invopt",
			&nonfatal);
	status = E_FAIL;
	break;
    }

return(status);
}
