/****************************************************************************
   NAME:                          GETPAR

   FUNCTION:
      GETPAR retrieves the parameters.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2     02/00    G. Schmidt        CSB       Original Development
****************************************************************************/

#include <string.h>
#include "tae.h"      /* p_find proto */
#include "picmatch.h"

void getpar (
  struct PARBLK *vblock,  /* O: TAE parameter block                        */
  float rmag[],           /* O: Magnification factor                       */
  int  *search,           /* O: Initial search area size                   */
  int  *msrc,             /* O: Minimum search area size                   */
  float *rmcor,           /* O: Minimum correlation value                  */
  int  *getw,             /* O: Kernal size                                */
  int  *getr,             /* O: Threshold value                            */
  int  *nohpf,            /* O: High-pass filter flag                      */
  int  *subpix,           /* O: Subpixel correlation flag                  */
  int  *doedge,           /* O: Edge enhancement flag                      */
  int  *ptinc,            /* O: Point increment or step size               */
  int  *retry,            /* O: Retry flag                                 */
  int  *nretry,           /* O: Number of retries (up to 4)                */
  float *tretry,          /* O: Correlation threshold                      */
  float *rretry,          /* O: Distance to move in the line/sample value
                                direction from the current location        */
  char hofile[][CMLEN],   /* O: First array value is output IBIS graphics
                                file name                                  */
  char tae2[],            /* O: TAE input filename (IN1)                   */
  char host2[],           /* O: Host input filename (IN1)                  */
  long window2[4],        /* O: Window of the input file (IN1)             */
  long *bands2,           /* O: Band in input filename (IN1)               */
  char tae3[],            /* O: TAE input filename (IN2)                   */
  char host3[],           /* O: Host input filename (IN2)                  */
  long window3[4],        /* O: Window of the input file (IN2)             */
  long *bands3,           /* O: Band in input filename (IN2)               */
  char host4[],           /* O: Host cutline filename                      */
  int  *contur,           /* O: Flag to specify if a cutline has been
                                specified                                  */
  float gridsp[4],        /* O: Grid display specifications (xl,xs,yl,ys)  */
  int  *nah,              /* O: Number of columns in the grid              */
  int  *nav,              /* O: Number of rows in the grid                 */
  float ritie[],          /* O: Tie points for first input image (maximum
                                of 10 tiepoint pairs)                      */
  float rotie[],          /* O: Tie points for second input image (maximum
                                of 10 tiepoint pairs)                      */
  int  *count)            /* O: Number of tie point pairs                  */
{
  long access;               /* READ or WRITE access to the file           */
  long fatal = -1;           /* fatal error flag                           */
  long one = 1;              /* constant one                               */
  long qflag[2];             /* window and/or band flag                    */
  long same;                 /* 1 if all windows are the same, 0 if not    */
  long status;               /* return status                              */
  long std_out = 0;          /* standard output                            */
  long zero = 0;             /* constant zero                              */
  char temp[4];              /* temp char string for YES and NO            */
  long nmrimg;               /* number of images read                      */
  long nbands;               /* number of bands per image                  */
  long totbnd;               /* total number of bands specified            */
  int  citie;                /* number of tie point values in TVAL1        */
  int  cotie;                /* number of tie point values in TVAL2        */
  int  i;                    /* looping variable                           */

  struct VARIABLE *var;      /* pointer to TAE par block var               */

  c_inlas (vblock, &std_out);

  /* Get magnification factor.
   */
  var = p_find (vblock, "SCALFACT");
  rmag[0] = RVAL (*var,0);
  if (var->v_count > 1)
    rmag[1] = RVAL (*var, 1);
  else
    rmag[1] = rmag[0];
  
  /* Get initial search area size.  If greater than 128, then set to 128.
   */
  var = p_find (vblock, "STRSSIZ");
  *search = IVAL (*var, 0);
  if (*search > 128)
    *search = 128;
  
  /* Get the minimum search area size.
   */
  var = p_find (vblock, "MINSSIZ");
  *msrc = IVAL (*var, 0);

  /* Get the minimum correlation value.
   */
  var = p_find (vblock, "MINCORR");
  *rmcor = RVAL (*var,0);

  /* Get the kernal size and threshold value.
   */
  var = p_find (vblock, "KERNDIM");
  *getw = IVAL (*var, 0);
  var = p_find (vblock, "THRVAL");
  *getr = IVAL (*var, 0);

  /* Get high-pass filter flag.
   */
  var = p_find (vblock, "HPFFLG");
  strcpy (temp, SVAL (*var, 0));
  if (strcmp (temp, "YES"))
    *nohpf = TRUE;
  else
    *nohpf = FALSE;

  /* Get subpixel correlation flag.
   */
  var = p_find (vblock, "SPFLG");
  strcpy (temp, SVAL (*var, 0));
  if (strcmp (temp, "YES"))
    *subpix = FALSE;
  else
    *subpix = TRUE;

  /* Get the edge enhancement flag.
   */
  var = p_find (vblock, "EDGEFLG");
  strcpy (temp, SVAL (*var, 0));
  if (strcmp (temp, "YES"))
    *doedge = FALSE;
  else
    *doedge = TRUE;

  /* Get the retry option parameters.
   */
  var = p_find (vblock, "RETSPEC");
  if (var->v_count == 3)
  {
    *retry = TRUE;
    *nretry = IVAL (*var, 0);
    if (*nretry > 4)
      *nretry = 4;
    *tretry = (float) IVAL (*var, 1);
    *rretry = (float) IVAL (*var, 2);
  }

  /* Get output IBIS graphics file name.
   */
  access = IWRITE;

  status = c_getnam (vblock, "OUTFILE", &zero, hofile[0], "ibis_graph",
                     &access);
  if (status != E_SUCC || !strcmp (hofile[0], ""))
  {
    c_errmsg("Error retrieving the OUTFILE name", "picmatch-infile", &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  /* Get first input file (IN1) specification.
   */
  status = c_getin (vblock, "IN1", &zero, tae2, host2, &nmrimg, window2,
                    bands2, &nbands, &totbnd, &one, &same, qflag);
  if (status == E_FAIL || !strcmp (tae2, ""))
  {
    c_errmsg("Error getting the IN1 parameter", "picmatch-getin", &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }
  if (totbnd > 1)
  {
    c_errmsg("Too many bands specified in the input image", "picmatch-getin",
             &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  /* Get second input file (IN2) specification.
   */
  status = c_getin (vblock, "IN2", &zero, tae3, host3, &nmrimg, window3,
                    bands3, &nbands, &totbnd, &one, &same, qflag);
  if (status == E_FAIL || !strcmp (tae3, ""))
  {
    c_errmsg("Error getting the IN2 parameter", "picmatch-getin", &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }
  if (totbnd > 1)
  {
    c_errmsg("Too many bands specified in the input image", "picmatch-getin",
             &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  /* Get cutline file name.
   */
  access = IREAD;
  if (c_getnam (vblock, "INFILE", &zero, host4, "ibis_graph", &access)
      != E_SUCC)
  {
    c_errmsg("Error retrieving the INFILE name", "picmatch-infile", &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }
  if (!strcmp (host4, ""))
    *contur = FALSE;
  else
    *contur = TRUE;

  /* If necessary, get grid specifications.
   */
  if (! (*contur))
  {
    var = p_find (vblock, "GRIDSPEC");
    if (var->v_count != 4)
    {
      c_errmsg("Missing grid specification.  Either the INFILE parameter or "
        "four GRIDSPEC values need specified.", "picmatch-grdspec", &one);
      c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
    }
    for (i = 0; i < 4; i++)
      gridsp[i] = RVAL (*var, i);

    var = p_find (vblock, "NCOLS");
    if (var->v_count == 0)
    {
      c_errmsg("Missing grid number of columns", "picmatch-ncols", &one);
      c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
    }
    *nah = IVAL (*var, 0);

    var = p_find (vblock, "NROWS");
    if (var->v_count == 0)
    {
      c_errmsg("Missing grid number of rows", "picmatch-nrows", &one);
      c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
    }
    *nav = IVAL (*var, 0);
  }
  else
  { /* Get the point increment or step size for the cutline.
     */
    var = p_find (vblock, "PNTINC");
    *ptinc = IVAL (*var, 0);
  }

  /* Get the tie points in TVAL1.
   */
  var = p_find (vblock, "TVAL1");
  citie = var->v_count;

  if (citie > 20)
  {
    c_errmsg
      ("The number of values in TVAL1 exceeds the valid 10 tiepoint pairs",
       "picmatch-toolarge", &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  if (citie < 6)
  {
    c_errmsg
      ("A minimum of 3 tiepoint pairs is required for TVAL1",
       "picmatch-toofew", &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  if (citie % 2 != 0)
  {
    c_errmsg ("The number of values in TVAL1 must be a multiple of 2",
      "picmatch-multiple", &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  for (i = 0; i < citie; i++)
    ritie[i] = RVAL (*var, i);

  /* Get the tie points in TVAL2.
   */
  var = p_find (vblock, "TVAL2");
  cotie = var->v_count;

  if (cotie > 20)
  {
    c_errmsg
      ("The number of values in TVAL2 exceeds the valid 10 tiepoint pairs",
       "picmatch-toolarge", &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  if (cotie < 6)
  {
    c_errmsg
      ("A minimum of 3 tiepoint pairs is required for TVAL2",
       "picmatch-toofew", &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  if (cotie % 2 != 0)
  {
    c_errmsg ("The number of values in TVAL2 must be a multiple of 2",
      "picmatch-multiple", &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  for (i = 0; i < cotie; i++)
    rotie[i] = RVAL (*var, i);

  if (citie != cotie)
  {
    c_errmsg ("The number of values in TVAL1 and TVAL2 are not equal",
      "picmatch-unequal", &one);
    c_errmsg("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  *count = citie / 2;   /* actual number of tiepoint pairs */
}
