/****************************************************************************
   NAME:                          GETZVL

   FUNCTION:
      GETZVL gets the Z values from the input 2D array, which is a gridded
      subset of the original image.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2      1/00    G. Schmidt        CSB       FORTRAN to C

   NOTES:
      When a point is rejected, the value of 25 is written to the output
      graphics file in place of the average radiometric value (Z value).
****************************************************************************/
#include "picmatch.h"

void getzvl (
  float *a,         /* I: 2D array of size N x N                             */
  int n,            /* I: Size of the array a                                */
  float fl,         /* I: Fractional line value of the grid's center point   */
  float fs,         /* I: Fractional sample value of the grid's center point */
  int nw,           /* I: Kernal size.  The size of the square kernel
                          grid used to calculate the average radiometric
                          value (Z value) for the matching coordinates.      */
  int nr,           /* I: Threshold value.  The number of pixel values in
                          the kernel equal to zero that are accepted before
                          the point is rejected.                             */
  float *z)         /* O: Z value (average radiometric value)                */
{
  const float ni = 25.0; /* default Z value                */
  int ill;               /* lower line value               */
  int jsl;               /* lower sample value             */
  int ilu;               /* upper line value               */
  int jsu;               /* upper sample value             */
  int ire;               /* count of a values that are 0.0 */
  int i, j;              /* looping variables              */
  int i_x_n;             /* value of i * n                 */
  long one = 1;          /* constant one                   */
  float fll;             /* fractional lower line value    */
  float fsl;             /* fractional lower sample value  */
  float flu;             /* fractional upper line value    */
  float fsu;             /* fractional upper sample value  */
  float fdata;           /* value of current grid element  */
  float sum;             /* sum of all the grid elements   */

  /* Calculate the lowest line and sample values for this grid (zero-based).
   */
  ill = (int) fl - nw/2 - 2;
  jsl = (int) fs - nw/2 - 2;

  /* Calculate the fraction that is left over from the fractional center
     point value.
   */
  flu = fl - (int) fl;
  fsu = fs - (int) fs;
  fll = 1.0 - flu;
  fsl = 1.0 - fsu;

  /* Calculate the highest line and sample values for this grid (zero-based).
   */
  ilu = ill + nw - 1;
  jsu = jsl + nw - 1;

  /* Check for validity of the lowest and highest line and sample values.
   */
  *z = ni;
  if (ill < 0.0 || ilu > (float) (n-1))
  {
    c_errmsg ("The calculated line value for the grid is either too large or too small.  Defaulting the Z value to 25.0.", "picmatch-zvalue", &one);
    return;
  }

  if (jsl < 0.0 || jsu > (float) (n-1))
  {
    c_errmsg ("The calculated sample value for the grid is either too large or too small.  Defaulting the Z value to 25.0.", "picmatch-zvalue", &one);
    return;
  }

  /* Sum up the array values in the grid.
   */
  ire = 0;
  sum = 0.0;
  for (i = ill; i <= ilu; i++)
  {
    i_x_n = i * n;   /* index to the ith row in a */
    for (j = jsl; j <= jsu; j++)
    {
      fdata = a[i_x_n + j];

      if (fdata == 0.0) ire++;
      if (i == ill) fdata *= fll;
      if (i == ilu) fdata *= flu;
      if (j == jsl) fdata *= fsl;
      if (j == jsu) fdata *= fsu;

      sum += fdata;
    }
  }

  /* Calculate the Z value from the sum of the grid values, only if the
     number of elements with a value of 0.0 is within the specified
     threshold.
   */
  if (ire <= nr)
    *z = sum / (float) ((nw-1) * (nw-1));
}
