/****************************************************************************
   NAME:                          MYGRID

   FUNCTION:
      MYGRID obtains the DN values in the points of the linearly
      transcalfactormed grid.  DN values are obtained by the bilinear
      interpolation between four pixels adjacent to the grid point.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
                 10/80   Boris Gokhman               Original Development
        7.2      01/00   G. Schmidt        CSB       FORTRAN to C

   NOTES:
     This basically is the original LAMS "getgrd" subroutine.
     However, changes were made so that it passes back an R*4 grid
     instead of a BYTE one.  This allows PICMATCH to process images of
     all data types (BYTE, I*2, I*4, and R*4).  In addition, for the
     PICMATCH program only, the computed grid values needed to be
     rounded to the nearest integer.  K. Zanter   (May l987)

****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include "picmatch.h"

void mygrid (
  struct FDESC *ifcb,
                  /* I: File descriptor of the input grid                */
  float *grid,    /* O: 2D output array of interpolated DN values        */
  int ndim,       /* I: Dimension of the grid (max of 128), must be even */
  int n,          /* I: Defines the portion of the grid to be processed,
                        must be even                                     */
  int line,       /* I: Number of lines in the image                     */
  int samp,       /* I: Number of samples in the image                   */
  float centr[2], /* I: Grid center, [0] = y, [1] = x (one-based)        */
  float *trans,   /* I: Array of the grid transformation coefficients:
                        ynew = trans[0]*yold + trans[1]*xold + trans[2]
                        xnew = trans[3]*yold + trans[4]*xold + trans[5]  */
  float shift[],  /* O: Vector to add to the transcalfactormed grid to
                        bring its center to "centr", [0] = y, [1] = x    */
  float magnif[], /* I: Array containing magnification of X and Y
                        distances in the grid, [0] = y, [1] = x          */
  int *chop,      /* O: Number of grid points outside the image.
                        Corresponding DN values in the "grid" are set
                        to zero.                                         */
  long band,      /* I: Band number to be processed                      */
  int *status)    /* O: Return status                                    */
{
  float *bufin = NULL;  /* 2D array of buffers */
  float basey,
        basex,
        dn1 = 0.0,
        dn12,
        dn2 = 0.0,
        dn3 = 0.0,
        dn34,
        dn4 = 0.0,
        factx,
        gapy,
        gapx,
        idn = 0.0,
        t1gapy,
        t2gapx,
        t4gapy,
        t5gapx,
        x[4],
        xinc2[128],
        xinc5[128],
        xmax,            /* maximum x value */
        xmin,            /* minimum x value */
        x0,
        y[4],
        yinc1[128],
        yinc4[128],
        ymax,            /* maximum y value */
        ymin,            /* minimum y value */
        y0;
  int  i,
       iabbr,            /* leftmost corner of the grid */
       icount,           /* size of the grid */
       ihi,
       ihi_x_samp,       /* value of ihi * samp */
       ilow,
       ilow_x_samp,      /* value of ilow * samp */
       imin,             /* uppermost corner of the transcalfactormed quad */
       iright,           /* rightmost corner of the grid */
       isgnes[4] = {1, -1, -1, 1},
       isig,             /* i sign           */
       ist,
       istart[128],
       iy,
       ix,
       i0,
       i1,
       itmp,             /* temp value for the labs value */
       j,
       jfin,
       jsgnes[4] = {1, 1, -1, -1},
       jsig,             /* j sign           */
       jx,
       jxplus,
       j0,
       j1,
       loopbreak = FALSE,/* specifies to break the outer loop */
       noffb,
       mytmp,
       endj[128];
  long one = 1,
       fatal = -1,
       irec;
  char errmsg[CMLEN];

  /* Verify the grid size is not larger than 128.
   */
  if (ndim > 128)
  {
    sprintf (errmsg,
      "Grid size can only be a maximum of 128.  Current grid size is %d",
      ndim);
    c_errmsg (errmsg, "mygrid-gridsize", &one);
    c_errmsg ("Fatal error occurred", "mygrid-fatal", &fatal);
  }

  /* Validate the grid center point.
   */
  if (centr[0] < 1.0 || centr[0] > (float) line)
  {
    *status = 4;
    return;
  }

  if (centr[1] < 1.0 || centr[1] > (float) samp)
  {
    *status = 4;
    return;
  }

  /* Allocate space for bufin[2][#samps].
   */
  bufin = (float *) malloc (2 * samp * sizeof (float));
  if (bufin == NULL)
  {
    c_errmsg ("Error allocating memory", "mygrid-alloc", &one);
    c_errmsg ("Fatal error occurred", "mygrid-fatal", &fatal);
  }

  /* Preliminary calculations.
   */
  noffb = (ndim - n) * (ndim + 1) / 2;
  ihi = 0;
  ilow = 1;
  *chop = 0;
  gapx = magnif[1];
  gapy = magnif[0];
  shift[0] = centr[0] - (n+1) / 2.0 * (trans[0] + trans[1]) - trans[2];
  shift[1] = centr[1] - (n+1) / 2.0 * (trans[3] + trans[4]) - trans[5];
  xmin = (n+1) / 2.0 - gapx * (n-1) / 2.0;
  ymin = (n+1) / 2.0 - gapy * (n-1) / 2.0;
  xmax = (n+1) / 2.0 + gapx * (n-1) / 2.0;
  ymax = (n+1) / 2.0 + gapy * (n-1) / 2.0;

  /* Find the uppermost corner of the transcalfactormed quad and the
     sign of the array indexes change.
   */
  y[0] = trans[0] * ymin + trans[1] * xmin + trans[2] + shift[0];
  y[1] = trans[0] * ymax + trans[1] * xmin + trans[2] + shift[0];
  y[2] = trans[0] * ymax + trans[1] * xmax + trans[2] + shift[0];
  y[3] = trans[0] * ymin + trans[1] * xmax + trans[2] + shift[0];
  imin = 0;
  for (i = 1; i < 4; i++)
  {
    if (y[i] < y[imin])
      imin = i;
  }
  isig = isgnes[imin];
  jsig = jsgnes[imin];

  /* Find the sample coordinates of the leftmost and rightmost corners
     of the grid.
   */
  x[0] = trans[3] * ymin + trans[4] * xmin + trans[5] + shift[1];
  x[1] = trans[3] * ymax + trans[4] * xmin + trans[5] + shift[1];
  x[2] = trans[3] * ymax + trans[4] * xmax + trans[5] + shift[1];
  x[3] = trans[3] * ymin + trans[4] * xmax + trans[5] + shift[1];
  iabbr = x[0];
  iright = x[0];
  for (i = 1; i < 4; i++)
  {
    if (x[i] < iabbr)
      iabbr = x[i];
    if (x[i] > iright)
      iright = x[i];
  }
  iabbr = (iabbr-10) > 0 ? (iabbr-10) : 0;
  icount = iright - iabbr + 10;

  /* Set up parameters for the loop.
   */
  t1gapy = trans[0] * gapy;
  t2gapx = trans[1] * gapx;
  t4gapy = trans[3] * gapy;
  t5gapx = trans[4] * gapx;
  basey = trans[0]*(ymin-gapy) + trans[1]*(xmin-gapx) + trans[2] + shift[0];
  basex = trans[3]*(ymin-gapy) + trans[4]*(xmin-gapx) + trans[5] + shift[1];
  for (i = 0; i < n; i++)
  {
    yinc1[i] = t1gapy * (i+1);
    xinc2[i] = t2gapx * (i+1);
    yinc4[i] = t4gapy * (i+1);
    xinc5[i] = t5gapx * (i+1);
    istart[i] = 0;
  }

  i0 = (n+1) * (1-isig) / 2 - 1;
  j0 = (n+1) * (1-jsig) / 2 - 1;

  /* Find isign (n, jsig) which is the magnitude of the first argument
   * with the sign of the second argument.
   */
  itmp = labs (n);
  if (jsig < 0)
    itmp = -itmp;
  jfin = j0 + itmp;
  irec = (long) y[imin];
  if (irec > 0 && irec <= line)
  {
    myread (bufin, ifcb, &irec, iabbr, icount, samp, 0, band, status);
    if (*status == E_FAIL)
    {
      free (bufin);
      bufin = NULL;
      return;
    }
  }

  irec++;
  if (irec > 0 && irec <= line)
  {
    myread (bufin, ifcb, &irec, iabbr, icount, samp, 1, band, status);
    if (*status == E_FAIL)
    {
      free (bufin);
      bufin = NULL;
      return;
    }
  }

  /* Begin the loop.
   */
  while (TRUE)
  {
    ihi_x_samp = ihi * samp;
    ilow_x_samp = ilow * samp;

    for (j1 = 0; j1 < n; j1++)
    {
      /* Find isign (j1+1, jsig) which is the magnitude of the first
       * argument with the sign of the second argument.
       */
      itmp = labs (j1+1);
      if (jsig < 0)
        itmp = -itmp;
      j = j0 + itmp;
      if (!endj[j] && (istart[j] == n-1))
        continue;
      ist = istart[j];
      endj[j] = FALSE;

      for (i1 = ist; i1 < n; i1++)
      {
        /* Find isign (i1+1, isig) which is the magnitude of the first
         * argument with the sign of the second argument.
         */
        itmp = labs (i1+1);
        if (isig < 0)
          itmp = -itmp;
        i = i0 + itmp;

        /* Calculate the y and x location of the kernal grid point.
         */
        y0 = basey + yinc1[i] + xinc2[j];
        x0 = basex + yinc4[i] + xinc5[j];
        if (y0 >= (float) irec && i1 == 0)
          loopbreak = TRUE;
        else
          loopbreak = FALSE;

        /* Check if the end of column #j in the strip is reached.
         */
        istart[j] = i1;
        if (y0 >= (float) irec)
        {
          endj[j] = TRUE;
          break;
        }

        if (x0 >= 1.0 && x0 <= (float) samp && y0 >= 1.0 &&
            y0 <= (float) line)
        {
          ix = x0;
          jx = ix - iabbr;
          jxplus = jx + 1;
          iy = y0;

          /* Interpolation of the four points around the kernal grid point.
           */
          dn1 = bufin[ihi_x_samp + jx - 1];
          dn2 = bufin[ihi_x_samp + jxplus - 1];
          dn3 = bufin[ilow_x_samp + jx - 1];
          dn4 = bufin[ilow_x_samp + jxplus - 1];
          factx = x0 - ix;
          dn12 = dn1 + (dn2 - dn1) * factx;
          dn34 = dn3 + (dn4 - dn3) * factx;
          idn = dn12 + (dn34 - dn12) * (y0 - iy);
          mytmp = (int) (idn + 0.5);
        }
        else
        {
          idn = 0;
          mytmp = 0;
          (*chop)++;
          if (*chop > 5000)
          {
            c_errmsg ("Too many grid points lie outside of image bounds",
                      "getgrd-point", &one);
            *status = E_FAIL;
            free (bufin);
            bufin = NULL;
            return;
          }
        }

        grid[ndim * i + j + noffb] = (float) mytmp;
      }  /* for i1 */

      if (loopbreak)
        break;
    }  /* for j1 */

    /* Read the new line and repeat the loop or exit.
     */
    if (!endj[jfin] && istart[jfin] == n-1)
    {
      *status = E_SUCC;
      free (bufin);
      bufin = NULL;
      return;
    }

    /* Read the new line.
     */
    i = ihi;
    ihi = ilow;
    ilow = i;
    irec++;
    if (irec > 0 && irec <= line)
    {
      myread (bufin, ifcb, &irec, iabbr, icount, samp, ilow, band, status);
      if (*status == E_FAIL)
      {
        free (bufin);
        bufin = NULL;
        return;
      }
    }
  }  /* while (TRUE) */
}
