/****************************************************************************
   NAME:                          MYREAD

   FUNCTION:
      This routine mimics the VICAR read routine by providing similar
      capability.  The calling sequence is NOT the same though.  It is
      assumed that the user has previously opened the file using a
      TAE OPEN.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
                 2/82    D. Nichols
                 4/87    B. Ailts                    Put in new LAS I/O
        7.2      1/00    G. Schmidt        CSB       FORTRAN to C
****************************************************************************/
#include "picmatch.h"

void myread (
  float *bufin,      /* O: 2D image line return buffer                  */
  struct FDESC *fdesc,
                     /* I: Image file control block                     */
  long *record,      /* I/O: Record number to read, if 0 then read the
                             next record (one-based)                    */
  int skpbyt,        /* I: Number of bytes to skip before reading       */
  int nbytes,        /* I: Total number of bytes to read                */
  int ns,            /* I: Number of samples in the line                */
  int ilow,          /* I: Row to be read (zero-based)                  */
  long band,         /* I: Band to be read                              */
  int *status)       /* O: Return status
                           0 = success
                           4 = end of file
                           8 = read error                               */
{
  float *locbuf = NULL;
  int bufcnt,        /* count of the values read from the local buffer */
      ilow_x_ns,     /* value of ilow * ns                             */
      ix,
      numbyt;
  long one = 1,      /* constant one value                             */
       fatal = -1;   /* constant fatal error value                     */

  *status = 0;

  /* If sequential reading requested - set record number.
   */
  if (*record <= 0)
    *record = 1;

  /* Read one line.
   */
  ilow_x_ns = ilow * ns;    /* index into the 2D array for ilow-th row */
  if (skpbyt == 0 && nbytes >= ns)
  {
    /* Read directly into the bufin buffer.
     */
    *status = c_eread (&fdesc, &band, record,
                      (unsigned char *) &bufin[ilow_x_ns], &one);
    if (*status < E_SUCC)
    {
      c_errmsg ("Error reading input file", "myread-read", &one);
      *status = E_FAIL;
      return;
    }
  }
  else
  {
    /* Allocate space for the buffer.
     */
    locbuf = (float *) malloc (ns * EREALSIZE);
    if (locbuf == NULL)
    {
      c_errmsg ("Error allocating memory", "myread-alloc", &one);
      c_errmsg ("Fatal error occurred", "myread-fatal", &fatal);
    }

    /* Read into the local buffer so that the skipped bytes can be
       retrieved.
     */
    *status = c_eread (&fdesc, &band, record, (unsigned char *) locbuf, &one);
    if (*status < E_SUCC)
    {
      c_errmsg ("Error reading input file", "myread-read", &one);
      *status = E_FAIL;
      return;
    }

    /* Load buffer from local buffer if needed.
     */
    numbyt = ns < nbytes ? ns : nbytes;
    bufcnt = 0;
    for (ix = 0; ix < numbyt + skpbyt - 1; ix++)
    {
      if (ix >= skpbyt)
      {
        bufin[ilow_x_ns + bufcnt] = locbuf[ix];
        bufcnt++;
      }
    }

    /* Free the local buffer.
     */
    free (locbuf);
  }

  *status = E_SUCC;
  return;
}
