/****************************************************************************
   NAME:                          PICMATCH

   FUNCTION:
      The primary purpose of PICMATCH is the matching of two input images
      along a cutline (polygonal contour).

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
                         A. Zobrist        JPL       Original Development
                 5/87    K. Zanter                   LAMS conversion
                 2/89    B. Ailts                    LAS 5.0 conversion
                 10/91   D. Etrheim                  Add include file to rfit
        7.2      1/00    G. Schmidt        CSB       FORTRAN to C

   COMPUTER HARDWARE AND/OR SOFTWARE LIMITATIONS:
      PICMATCH must be run under TAE.

   PROJECT: LAMS

   ALGORITHM DESCRIPTION:
      The program begins with the standard initialization and 
   retrieval of user-specified parameters.  The input images, the 
   IBIS graphics file containing the coordinates of the cutline, and 
   the output IBIS graphics file are opened. 

      A geometric model is set up relating the two images.  The 
   geometric model is used to estimate a search location and to specify 
   a resampling of the first image to match the raster gridding of the 
   second image.  Initially, the model is a linear least squares fit to 
   the input tie points.  

      The second step is to set up the correlation point set in the 
   second image by one of two methods.  If an input cutline (INFILE) 
   and point increment (PNTINC) are specified, the correlation point 
   set is spaced along the cutline.  Otherwise, if no input cutline is 
   given, a grid of points is used as defined by the GRIDSPEC, NROWS, 
   and NCOLS parameters.

      The iteration over the correlation point set then proceeds.  At 
   the requested point in the second input image, a 32 x 32 sub-image is
   extracted.  If the point is a fractional pixel location, the center of
   extraction is moved to align with pixel spacing to avoid interpolation.
   Interpolation will occur only if a non-integral value is specified in
   the SCALFACT parameter.  A search area in the first input image is then 
   calculated using the geometric model.  If this is a fractional pixel 
   location, it is also aligned with pixel spacing to minimize inter-
   polation near the center.  A sub-image of the determined size is then 
   extracted from the first image.  The size will be somewhere between the 
   values of STRSSIZ and MINSSIZ.  This sub-image is extracted by bilinear 
   interpolation at a rotation and pixel size to match the second input 
   geometry as specified by the geometric model.  If the user specified 
   that edge enhancement should be performed, a simple 3 x 3 edge 
   enhancement is applied to both sub-images.

      Correlation is performed on 32 x 32 windows of the sub-images.  The
   32 x 32 pieces of the two inputs are each subjected to a complex valued
   Discrete Fourier Transform (DFT) via the Fast Fourier Transform (FFT)
   algorithm.  One of the DFT arrays has its low frequency terms set to 
   zero by zeroing the first row and the first column unless the user 
   specified that the high-pass filter should not be applied.  The DFT's 
   are multiplied element by element (one DFT is conjugated) and also each 
   product is divided by the absolute value of its multiplicands to give 
   phase correlation.  An inverse FFT is applied to this result and the 
   peak and its eight neighbors are saved.  The window is moved over the 
   search area with an increment of at most 12 pixels.  When the entire 
   search area is covered, the largest peak is taken to be the correlation 
   point.  A final FFT correlation is performed at the correlation peak to 
   get a refined correlation.  Then, if specified by the user, a subpixel 
   match is obtained by fitting a quadratic to the peak and its eight 
   neighbors.  The match point is calculated back to the original image 
   location via the inverse of the geometric model and incorporating 
   alignment shifts performed in both images.

      If the user sepcified the RETSPEC parameter, the maximum correlation 
   value for the current location is checked against the specified 
   correlation threshold and more tries near the location are made if the 
   threshold is not achieved.  The retry pattern is to move to the corners 
   of a square at the specified distance from the location.

      After the matching locations have been determined, the average 
   radiometric value of the kernel around the point is also determined for 
   each image.  The results are written to the output IBIS graphics file.  
   Also, while the program is running, relevant information such as search
   size and the correlation maximum value are printed out.  If correlation
   fails, nothing is written to the output IBIS graphics file, but a failure
   number is given in the output printout under the FNR column heading.  
   The failure numbers are:
      1. over 100 grid points outside image
      2. correlation peak unreliable
      3. subpixel correlation failure
      4. failure of retry option

      Points determined to be completely outside of either image are ignored.
   Finally, the input and output files are closed and a completion message
   is output.

   ALGORITHM REFERENCES:
      None
****************************************************************************/
#include <math.h>
#include <stdio.h>
#include "lams.h"
#include "picmatch.h"

extern void llsq_ (float *a, float *b, int *m, int *n, int *l,
                   float *x, int ipiv[], float *eps, int *ier,
                   float aux[]);

void main ()
{

  double dsum,
         latlon[3];         /* latitude/longitude/height of grid center   */

  float a[3][60],
        aa[180],            /* N x M coefficient matrix for llsq call     */
        ag1d[30],           /* 10 x 3 coefficient matrix for llsq call    */
        asrch[128][128],    /* array of interpolated DN values from mygrid
                               for the search window                      */
        aux[6],             /* auxiliary storage array of dimension
                               max (2*N, L).  On return from llsq, the
                               first L locations of aux contain the
                               resulting least squares                    */
        b[2][60],
        bb[120],            /* L x M right hand side matrix for llsq call */
        bg1d[20],           /* 10 x 2 coefficient matrix for llsq call    */
        centr[2],
        chip1[32][32],      /* array of interpolated DN values from mygrid*/
        corr[3][3],         /* correlation array for search window (asrch)*/
        eps,                /* epsilon value                              */
        gcntr[2],           /* grid center, [0] = y, [1] = x (one-based)  */
        gridsp[4],          /* grid display specifications (xl,xs,yl,ys)  */
        outbuf[8],          /* buffer for the output IBIS file            */
        pred[2],            /* predicted grid center, [0] = y, [1] = x
                               (one-based)                                */
        rcskip,             /* float of ptinc value                       */
        res,
        resl,
        ress,
        rf1,
        rf2,
        ritie[20],          /* tie points for first input image           */
        rmag[2],            /* magnification factor [0] = y  [1] = x      */
        rmcor,              /* minimum correlation value                  */
        rotie[20],          /* tie points for second input image          */
        rretry = 100.0,     /* distance to move in the line/sample value
                               direction from the current location        */
        rtb,
        rvec[10] = {0.0, 1.0, -1.0, 1.0, -1.0, 0.0, 1.0, -1.0, -1.0, 1.0},
        rx,
        ry,
        shift[2],           /* vector to add to the transcalfactormed
                               grid to bring its center to "centr"
                               [0] = y, [1] = x                           */
        tbuf[MMXPPR],       /* buffer for the input IBIS file             */
        tretry,             /* correlation threshold                      */
        vloff,              /* line offset                                */
        vmax,               /* maximum value encountered in search window */
        vsoff,              /* sample offset                              */
        wl,
        wld,
        wlmax,              /* line value of wmax                         */
        wmax,               /* maximum value in the window                */
        ws,
        wsd,
        wsmax,              /* sample value of wmax                       */
        x[3][3],
        xg[2][3],
                            /* L x N solution matrix for llsq call        */
        xid[6] = {1.0, 0.0, 0.0, 0.0, 1.0, 0.0},
                            /* array of grid transformation coefficients  */
        xl,                 /* grid display UL line value                 */
        xld,                /* distance between grid line points          */
        xlt,
        xs,                 /* grid display UL sample value               */
        xsd,                /* distance between grid sample points        */
        xst,
        yl,                 /* grid display LR line value                 */
        ys,                 /* grid display LR sample value               */
        z1,                 /* Z value of asrch                           */
        z2;                 /* Z value of chip1                           */

  int chop,                 /* number of grid points outside the image    */
      count,                /* number of tie points divided by 2          */
      ctb,
      eight = 8,            /* constant eight value                       */
      getr,                 /* threshold value                            */
      getw,                 /* kernal size                                */
      i,
      ibig,                 /* current point for correlation              */
      ic,
      ier,                  /* error status                               */
      ifail,
      ih,
      ihor,
      ilin,                 /* input start line value in the search window*/
      inns,
      ipiv[20],             /* integer output vector of dimension N which
                               contains information on column interchanges
                               in matrix ag                               */
      iver,
      j,
      jbig,
      jsmp,                 /* input start sample value in search window  */
      jv,
      lcount = 10000,       /* number of grid points (max of 10000)       */
      loopbreak = FALSE,    /* specifies if outer loop should be exited   */
      mm,                   /* defines the portion of the grid to be
                               processed, must be even for mygrid call    */
      msrc,                 /* minimum search area size                   */
      msrchw,
      nah,                  /* number of columns in the grid              */
      nahv,
      nav,                  /* number of rows in the grid                 */
      neq,
      neqp,                 /* previous neq                               */
      nrecs,               /* number of lines in cutline file (INFILE)    */
      nretry = 1,          /* number of retries (up to 4)                 */
      num,
      nwid,
      onrecs,              /* number of lines in output IBIS graphics
                              file (OUTFILE)                              */
      outptr,              /* pointer in the output buffer                */
      psrch,
      ptb1,
      ptb2,
      ptbx,
      ptinc,                /* point increment or step size               */
      rec1,
      recsiz,               /* number of samples in INFILE                */
      search,               /* initial search area size                   */
      srchw,
      status,               /* error status                               */
      tbptr,
      three = 3,            /* constant three value                       */
      two = 2,              /* constant two value                         */
      tmpmax;               /* temporary maximum value                    */

  int contur,               /* flag to specify if a cutline has been
                               specified                                  */
      doedge,               /* edge enhancement flag                      */
      nohpf,                /* high-pass filter flag                      */
      oldfft,
      retry,                /* retry flag                                 */
      subpix;               /* subpixel correlation flag                  */

  long fatal = -1,          /* constant fatal value                       */
       one = 1,             /* constant one value                         */
       ltwo = 2,            /* constant two value                         */
       zero = 0,            /* constant zero value                        */
       acc,                 /* file access type (IREAD, IWRITE)           */
       bands2[2],           /* band in input filename (IN1)               */
       bands3[2],           /* band in input filename (IN2)               */
       dtype,               /* file data type (EREAL)                     */
       fyle,                /* file output flag                           */
       lp,                  /* line printer output flag                   */
       nl,                  /* number of lines in the image (IN1)         */
       nl3,                 /* number of lines in the image (IN2)         */
       ns,                  /* number of samples in the image (IN1)       */
       ns3,                 /* number of samples in the image (IN2)       */
       numout,
       sl,                  /* starting line in the image                 */
       ss,                  /* starting sample in the image               */
       term,                /* terminal output flag                       */
       window2[4],          /* window of the input file (IN1)             */
       window3[4];          /* window of the input file (IN2)             */

  char host2[CMLEN],        /* host input filename (IN1)                  */
       host3[CMLEN],        /* host input filename (IN2)                  */
       host4[CMLEN],        /* host cutline filename (INFILE)             */
       hofile[2][CMLEN],    /* first array value is output IBIS graphics
                               filename, second value is the print
                               filename                                   */
       tae2[CMLEN],         /* TAE input filename (IN1)                   */
       tae3[CMLEN],         /* TAE input filename (IN2)                   */
       tofile[2][CMLEN],    /* second value is the output filename        */
       prbuf[70];           /* print buffer                               */

  FILE *prtfile = NULL;     /* output print file pointer                  */
 
  struct FDESC *fdesc2 = NULL,  /* file descriptor for input file IN1       */
               *fdesc3 = NULL;  /* file descriptor for input file IN2       */
  struct MFDESC *fdesc1 = NULL, /* file descriptor of output IBIS graphics
                                   (OUTFILE)                                */
                *fdesc4 = NULL; /* file descriptor of cutline file (INFILE) */
  struct PARBLK vblock;     /* TAE parameter block                          */

  /* Get TAE parameters.
   */
  getpar (&vblock, rmag, &search, &msrc, &rmcor, &getw, &getr, &nohpf,
          &subpix, &doedge, &ptinc, &retry, &nretry, &tretry, &rretry,
          hofile, tae2, host2, window2, bands2, tae3, host3, window3,
          bands3, host4, &contur, gridsp, &nah, &nav, ritie, rotie,
          &count);
  rcskip = (float) ptinc;

  /* Get the print option.
   */
  strcpy (hofile[1], "picmatch");
  status = c_setprt (&vblock, &term, &fyle, &lp, hofile[1], tofile[1]);
  if (status != E_SUCC)
  {
    c_errmsg ("Error setting the print file", "picmatch-setprt", &one);
    c_errmsg ("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  /* Open IN1 image.
   */
  sl = 1;
  ss = 1;
  nl = 0;
  ns = 0;
  acc = IREAD;
  dtype = EREAL;
  status = c_eopenr (&fdesc2, host2, &one, &sl, &ss, &nl, &ns, &acc,
           &dtype, &zero, &one);
  if (status != E_SUCC)
  {
    c_errmsg ("Cannot open IN1 image", "picmatch-open", &one);
    c_errmsg ("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  if ((nl != window2[2]) || (ns != window2[3]))
  {
    c_errmsg ("Windowing is not allowed", "picmatch-window", &one);
    c_errmsg ("Full image size for IN1 will be used", "picmatch-window", &one);
    c_errmsg ("Nonfatal error encountered", "picmatch-warn", &one);
  }

  /* Open IN2 image.
   */
  nl3 = 0;
  ns3 = 0;
  acc = IREAD;
  dtype = EREAL;
  status = c_eopenr (&fdesc3, host3, &one, &sl, &ss, &nl3, &ns3, &acc,
           &dtype, &zero, &one);
  if (status != E_SUCC)
  {
    c_errmsg ("Cannot open IN2 image", "picmatch-open", &one);
    c_errmsg ("Fatal error encountered", "picmatch-fatal", &fatal);
  }

  if ((nl3 != window3[2]) || (ns3 != window3[3]))
  {
    c_errmsg ("Windowing is not allowed", "picmatch-window", &one);
    c_errmsg ("Full image size for IN2 will be used", "picmatch-window", &one);
    c_errmsg ("Nonfatal error encountered", "picmatch-warn", &one);
  }

  /* If necessary, open the cutline file for reading.
   */
  if (contur)
  {
    status = c_gropen (&fdesc4, host4, &recsiz, &nrecs, &zero);
    if (status != E_SUCC)
    {
      c_errmsg("Error opening cutline file", "picmatch-open", &one);
      c_errmsg("Fatal error encountered","picmatch-fatal",&fatal);
    }
  }

  /* Open the output IBIS graphics file for writing.
   */
  onrecs = MEMXPT / 8;
  status = c_gropen (&fdesc1, hofile[0], &eight, &onrecs, &one);
  if (status != E_SUCC)
  {
    c_errmsg("Error opening output file", "picmatch-open", &one);
    status = c_grclos (&fdesc1, hofile[0]);
    c_clean (&vblock, "picmatch", hofile, &one);
  }

  /* If necessary, get grid specifications.
   */
  if (!contur)
  {
    /* Assign the UL and LR line/sample values.
     */
    xl = gridsp[0];
    xs = gridsp[1];
    yl = gridsp[2];
    ys = gridsp[3];

    /* Determine the distance between two grid points in both the line
       and sample direction.
     */
    xld = (yl - xl) / nav;
    xsd = (ys - xs) / nah;

    /* Calculate the number of grid points.
     */
    lcount = (nah + 1) * (nav + 1);
  }

  /* Process tie points to set up the affine transform.  a[2][i] should
   * be a constant 1.0 value.
   */
  for (i = 0; i < 60; i++)
    a[2][i] = 1.0;

  for (i = 0; i < count; i++)
  {
    a[0][i] = rotie[i*2];
    a[1][i] = rotie[i*2 + 1];
    b[0][i] = ritie[i*2];
    b[1][i] = ritie[i*2 + 1];
  }

  neq = (count < 10) ? count : 10;
  neqp = neq;

  for (i = 0; i < 8; i++)
    outbuf[i] = 0.0;

  /* Plug the a and b values into a 1D array of the appropriate size
   * for the llsq call.
   */
  for (j = 0; j < 3; j++)
  {
    for (i = 0; i < count; i++)
    {
      ag1d[i + j * count] = a[j][i];
      if (j != 2)
        bg1d[i + j * count] = b[j][i];
    }
  }

  /* Perform initial least squares fit on the tie points.
     NOTE: llsq is a FORTRAN routine.
   */
  eps = 1.0001e-7;
  llsq_ (ag1d, bg1d, &count, &three, &two, xg, ipiv, &eps, &ier, aux);
  if (ier != 0)
  {
    c_errmsg ("Error performing least squares fit", "picmatch-open", &one);
    status = c_grclos (&fdesc1, hofile[0]);
    c_clean (&vblock, "picmatch", hofile, &one);
  }

  /* More initializations.
   */
  rec1 = 0;
  outptr = 0;
  srchw = search - 32;
  msrchw = msrc - 32;

  /* Read the cutline from the graphics file.
   */
  if (contur)
  {
    if (recsiz * nrecs > MMXPPR * 4)
    {
      c_errmsg ("Input graphics file is too large", "picmatch-IBIS", &one);
      status = c_grclos (&fdesc1, hofile[0]);
      c_clean (&vblock, "picmatch", hofile, &one);
    }

    tbptr = 0;
    for (i = 1; i <= nrecs; i++)
    {
      status = c_grread (&fdesc4, &i, &tbuf[tbptr]);
      if (status != E_SUCC)
      {
        c_errmsg ("Error reading cutline file", "picmatch-read", &one);
        status = c_grclos (&fdesc1, hofile[0]);
        c_clean (&vblock, "picmatch", hofile, &one);
      }

      tbptr += recsiz;
    }

    inns = recsiz;
    num = inns;
    for (i = 0; i < inns; i++)
    {
      if (tbuf[i] == 0.0)
      {
        num = i;
        break;
      }
    }

    ptb1 = 1;
    ptb2 = 0;
    ptbx = num - 1;
  }

  /* Open the print file.
   */
  if (lp || fyle)
  {
    prtfile = fopen (hofile[1], "w");
    if (prtfile == (FILE *) NULL)
    {
      c_errmsg ("Error opening print file", "picmatch-open", &one);
      status = c_grclos (&fdesc1, hofile[0]);
      c_clean (&vblock, "picmatch", hofile, &one);
    }
  }

  /* Output the heading to the terminal, if appropriate.
   */
  if (term)
    printf (" SEQ. NUMBER     SEARCH SIZE     CORRELATION MAX     FNR\n");

  /* Main processing loop through all grid points.
   */
  for (ibig = 1; ibig <= lcount; ibig++)
  {
    for (jbig = 1; jbig <= nretry; jbig++)
    {
      sprintf (prbuf, " %5d", ibig);

      if (!contur)
      { /* Using the grid specs */
        ihor = (ibig - 1) % (nah + 1);
        iver = (ibig - 1) / (nah + 1);
        xlt = xl + (float) iver * xld + rretry * rvec[jbig-1];
        xst = xs + (float) ihor * xsd + rretry * rvec[jbig+4];
      }
      else if (ptb1 >= ptbx)
      {
        loopbreak = TRUE;
        break;
      }
      else
      { /* Using the cutline */
        rx = tbuf[ptb1+1] - tbuf[ptb1-1];
        ry = tbuf[ptb1+2] - tbuf[ptb1];
        rtb = (float) sqrt (rx * rx + ry * ry);
        ctb = (int) (rtb / rcskip) + 1;
        ic = ibig - ptb2;
        rf1 = ((float) (ic - 1) * rx) / (float) ctb;
        rf2 = ((float) (ic - 1) * ry) / (float) ctb;
        xlt = tbuf[ptb1-1] + rf1 + rretry * rvec[jbig-1];
        xst = tbuf[ptb1] + rf2 + rretry * rvec[jbig+4];
        if (ic >= ctb)
        {
          ptb2 = ibig;
          ptb1 += 2;
        }
      }

      centr[0] = 16.5;
      centr[1] = 16.5;
      gcntr[0] = (float) ((int) xlt) + 0.5;
      gcntr[1] = (float) ((int) xst) + 0.5;
      ifail = 1;

      /* Extract grid from the second image.
       */
      mygrid (fdesc3, chip1, 32, 32, nl3, ns3, gcntr, xid, shift, rmag,
              &chop, bands3[0], &status);
      if (status != E_SUCC)
        continue;

      if (chop < 100)
      {
        latlon[0] = (double) gcntr[0];
        latlon[1] = (double) gcntr[1];
        latlon[2] = (double) 1.0;

        for (j = 0; j < 3; j++)
        {
          for (i = 0; i < neq; i++)
          {
            aa[i + j * neq] = a[j][i];
            if (j != 2)
              bb[i + j * neq] = b[j][i];
          }
        }

        /* Perform least squares fit.
           NOTE: llsq is a FORTRAN routine.
         */
        eps = 1.0001e-7;
        llsq_ (aa, bb, &neq, &three, &two, x, ipiv, &eps, &ier, aux);
        if (ier != 0)
        {
          c_errmsg ("Error performing least squares fit", "picmatch-open",
                    &one);
          if (lp || fyle)
            fclose (prtfile);
          status = c_grclos (&fdesc1, hofile[0]);
          c_clean (&vblock, "picmatch", hofile, &ltwo);
        }

        for (i = 0; i < 2; i++)
        {
          dsum = (double) 0.0;
          for (j = 0; j < 3; j++)
          {
            dsum += latlon[j] * x[i][j];
          }
          pred[i] = (int) dsum + 0.5;
        }

        /* Put search size into output buffer.
         */
        sprintf (prbuf, "%s           %3d", prbuf, srchw + 32);

        /* Extract grids in the search window of the first image.
         */
        mm = (128 < srchw + 40) ? 128 : srchw + 40;
        mm = (mm / 2) * 2;
        mygrid (fdesc2, asrch, 128, mm, nl, ns, pred, xg, shift,
                rmag, &chop, bands2[0], &status);
        if (status != E_SUCC)
          continue;

        /* If necessary, do edge enhancement on the grids.
         */
        if (doedge)
        {
          edge (chip1, 32);
          edge (asrch, 128);
        }

        /* Iterate over window for best match.
         */
        wmax = -1.0e20;
        psrch = 49 - srchw / 2;
        nahv = (srchw + 23) / 12;
        nwid = 1;
        if (nahv >= 2)
          nwid = srchw / (nahv - 1);
        oldfft = FALSE;

        for (ih = 0; ih < nahv; ih++)
        {
          ilin = psrch + ih * nwid;
          for (jv = 0; jv < nahv; jv++)
          {
            jsmp = psrch + jv * nwid;
            rfit (ilin, jsmp, &vmax, &vloff, &vsoff, corr, nohpf, oldfft,
                  chip1, asrch, &status);
            oldfft = TRUE;
            if (vmax > wmax)
            {
              wmax = vmax;
              wlmax = vloff + (float) ilin;
              wsmax = vsoff + (float) jsmp;
            }
          }
        }

        /* Refine at max.
         */
        ilin = (int) wlmax;
        jsmp = (int) wsmax;
        ifail = 2;

        if (ilin > 0 && ilin < 98 && jsmp > 0 && jsmp < 98)
        {
          rfit (ilin, jsmp, &vmax, &vloff, &vsoff, corr, nohpf, oldfft,
                chip1, asrch, &status);
          if (status != E_SUCC)
          {
            c_errmsg ("Error performing least squares fit", "picmatch-open",
                      &one);
            if (lp || fyle)
              fclose (prtfile);
            status = c_grclos (&fdesc1, hofile[0]);
            c_clean (&vblock, "picmatch", hofile, &ltwo);
          }

          ifail = 3;
          if (vmax < rmcor)
            break;

          if (subpix)
          {
            status = refine (corr, &vloff, &vsoff);
          }
          else
          {
            /* Initialize status to FALSE so the next block of code will
             * be executed.
             */
            status = FALSE;
          }

          if (!status)
          {
            wld = (float) ilin + vloff + centr[0] - 65.5;
            wsd = (float) jsmp + vsoff + centr[1] - 65.5;
            wl = pred[0] + (xg[0][0] * wld + xg[0][1] * wsd) * rmag[0];
            ws = pred[1] + (xg[1][0] * wld + xg[1][1] * wsd) * rmag[1];
            sprintf (prbuf, "%s             %6.2f", prbuf, vmax);
            ifail = 4;

            if (!retry || (vmax >= tretry))
            {
              /* Get the Z values.
               */
              getzvl (chip1[0], 32, centr[0], centr[1], getw, getr, &z2);
              getzvl (asrch[0], 128, 64.5 + wld, 64.5 + wsd, getw, getr, &z1);

              /* Move computed values to output buffer.
               */
              outbuf[outptr] = wl;
              outbuf[outptr+1] = ws;
              outbuf[outptr+2] = gcntr[0];
              outbuf[outptr+3] = gcntr[1];
              outbuf[outptr+4] = z1;
              outbuf[outptr+5] = z2;
              outbuf[outptr+6] = vmax;
              outbuf[outptr+7] = (float) ibig;
              outptr = outptr + 8;

              /* Write to output file when buffer is filled.
               */
              if (outptr >= 8)
              {
                rec1++;
                status = c_grwrit (&fdesc1, &rec1, outbuf);
                if (status != E_SUCC)
                {
                  c_errmsg ("Error writing output file", "picmatch-write",
                    &one);
                  if (lp || fyle)
                    fclose (prtfile);
                  status = c_grclos (&fdesc1, hofile[0]);
                  c_clean (&vblock, "picmatch", hofile, &ltwo);
                }

                /* Reinitialize buffer and buffer pointer.
                 */
                outptr = 0;
                for (i = 0; i < 8; i++)
                  outbuf[i] = 0;
              }

              resl = fabs (pred[0] - wl);
              ress = fabs (pred[1] - ws);
              res = (float) sqrt (resl * resl + ress * ress);

              /* Output the print buffer to the terminal or file.
               */
              if (term)
                printf ("%s\n", prbuf);
              if (lp || fyle)
                fprintf (prtfile, "%s\n", prbuf);

              neq = neq+1 < 60 ? neq+1 : 60;
              neqp = (neqp % 60) + 1;
              a[0][neqp-1] = (float) latlon[0];
              a[1][neqp-1] = (float) latlon[1];
              b[0][neqp-1] = wl;
              b[1][neqp-1] = ws;
              tmpmax = msrchw > (4 * srchw + 2 * (int) res) / 5 ? msrchw :
                       (4 * srchw + 2 * (int) res) / 5;
              srchw = tmpmax < srchw ? tmpmax : srchw;
              break;
            }
          }
        }
      }

      sprintf (prbuf, "%s              %2d\n", prbuf, ifail);
      if (term)
        printf ("%s\n", prbuf);
      if (lp || fyle)
        fprintf (prtfile, "%s\n", prbuf);
    }

    if (loopbreak)
      break;
  }

  /* Final write to output file.
   */
  if (outptr > 0)
  {
    rec1++;
    status = c_grwrit (&fdesc1, &rec1, outbuf);
    if (status != E_SUCC)
    {
      c_errmsg ("Error writing output file", "picmatch-write",
                &one);
      if (lp || fyle)
        fclose (prtfile);
      status = c_grclos (&fdesc1, hofile[0]);
      c_clean (&vblock, "picmatch", hofile, &ltwo);
    }
  }

  /* Close output file.
   */
  status = c_grclos (&fdesc1, hofile[0]);
  if (status != E_SUCC)
  {
    c_errmsg ("Error closing output file", "picmatch-close", &one);
    c_errmsg ("Nonfatal error encountered", "picmatch-warn", &one);
  }

  /* Close IN1 image.
   */
  status = c_eclose (&fdesc2);
  if (status != E_SUCC)
  {
    c_errmsg ("Error closing IN1 image", "picmatch-close", &one);
    c_errmsg ("Nonfatal error encountered", "picmatch-warn", &one);
  }

  /* Close IN2 image.
   */
  status = c_eclose (&fdesc3);
  if (status != E_SUCC)
  {
    c_errmsg ("Error closing IN2 image", "picmatch-close", &one);
    c_errmsg ("Nonfatal error encountered", "picmatch-warn", &one);
  }

  /* Close input cutline file.
   */
  if (contur)
  {
    status = c_grclos (&fdesc4, host4);
    if (status != E_SUCC)
    {
      c_errmsg ("Error closing cutline file", "picmatch-close", &one);
      c_errmsg ("Nonfatal error encountered", "picmatch-warn", &one);
    }
  }

  /* Close print file.
   */
  if (lp || fyle)
    fclose (prtfile);

  /* Output completion message.
   */
  if (fyle)
    numout = 2;
  else
    numout = 1;

  /* Call complt to end the routine.
   */
  c_complt ("PICMATCH", hofile, &numout);
} /* end */
