/****************************************************************************
   NAME:                          REFINE

   FUNCTION:
      REFINE calculates/increments the line and sample offset values for
      the input matrix.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2      1/00    G. Schmidt        CSB       FORTRAN to C

   NOTES: If FALSE is returned then the next statement in the calling
      routine will be implemented.  If TRUE is returned then the branch
      will be taken.
****************************************************************************/
#include "picmatch.h"

extern void llsq_ (float *a, float *b, int *m, int *n, int *l,
                   float *x, int ipiv[], float *eps, int *ier,
                   float aux[]);

int refine (
  float corr[3][3],    /* I:   2D input matrix  */
  float *vloff,        /* I/O: Line offset      */
  float *vsoff)        /* I/O: Sample offset    */
{
  float a[6][9],          /* N x M coefficient matrix for llsq call     */
        b[9],             /* L x M right hand side matrix for llsq call */
        s[6],             /* L x N solution matrix for llsq call        */
        aux[12],          /* auxiliary storage array of dimension
                             max (2*N, L).  On return from llsq, the
                             first L locations of aux contain the
                             resulting least squares                    */
        x,                /* floating point value of j */
        x0,
        y,                /* floating point value of i */
        y0,
        eps = 1.0e-7,
        b2a;
  int ipiv[6],            /* integer output vector of dimension N which
                             contains information on column interchanges
                             in matrix A                                */
      i, j,               /* looping variables         */
      ier,                /* error returned from llsq  */
      iq,                 /* array index               */
      nine = 9,           /* constant defined 9        */
      one = 1,            /* constant defined 1        */
      six = 6;            /* constant defined 6        */

  for (i = 0; i < 3; i++)
  {
    y = (float) (i + 1);

    for (j = 0; j < 3; j++)
    {
      x = (float) (j + 1);
      iq = i*3 + j;
      a[0][iq] = x * x;
      a[1][iq] = x * y;
      a[2][iq] = y * y;
      a[3][iq] = x;
      a[4][iq] = y;
      a[5][iq] = 1.0;
      b[iq] = corr[i][j];
    }
  }

  /* Solve the linear least squares problem (i.e. minimize the Euclidean
     norm of B - A*X), where A is an M x N matrix with M not less than N.
     Matrix a and array b are both destroyed in this subroutine.
     NOTE: llsq is a FORTRAN routine.
   */
  llsq_ (a, b, &nine, &six, &one, s, ipiv, &eps, &ier, aux);
  if (ier != 0)
    return TRUE;
  if (s[0] == 0.0)
    return TRUE;

  b2a = s[1] / (s[0] * 2.0);
  y0 = (b2a * s[3] - s[4]) / (2.0 * s[2] - b2a * s[1]) - 2.0;
  x0 = -b2a * (y0 + 2.0) - s[3] / (s[0] * 2.0) - 2.0;

  if (x0*x0 + y0*y0 >= 4.0)
    return FALSE;

  *vloff += y0;
  *vsoff += x0;

  return FALSE;
}
