/****************************************************************************
   NAME:                          RFIT

   FUNCTION:
      RFIT will try to fit the input chip to some point in the input search
      window.

   PROGRAM HISTORY:
      Version    Date    Author          Code/contr  Request
      -------    ----    ------          ----------  -------
        7.2      1/00    G. Schmidt        CSB       FORTRAN to C
****************************************************************************/
#include <math.h>
#include "picmatch.h"

extern void rft2_c_ (float a[][32], int *m, int *n, int *isn, int *istat);

void rfit (
  int il,                /* I: Input start line value in search window    */
  int js,                /* I: Input start sample value in search window  */
  float *vmax,           /* O: Maximum value encountered in search window */
  float *vloff,          /* O: Line offset                                */
  float *vsoff,          /* O: Sample offset                              */
  float corr[3][3],      /* O: 2D correlation array for asrch             */
  int nohpf,             /* I: Flag for no hpf                            */
  int oldfft,            /* I: Flag to specify the old FFT or not         */
  float chip1[32][32],   /* I: 32x32 chip to fit                          */
  float asrch[128][128], /* I: 128x128 search window                      */
  int *status)           /* O: Status flag                                */
{
  int i, j,              /* looping variables                             */
      ixmax,             /* i index of vmax                               */
      jxmax,             /* j index of vmax                               */
      thirty_two = 32,   /* constant thirty two                           */
      neg_one = -1,      /* constant negative one                         */
      one = 1,           /* constant one                                  */
      ret_val;           /* status return value                           */
  float bij,
        bij1,
        tmp,             /* temporary value for (bij * bij + bij1 * bij1) */
        tmp_sqrt,        /* temporary sqrt value                          */
        v,
        rv,
        a[34][32],       /* storage for the chip1 data                    */
        b[34][32];       /* storage for the asrch data                    */
  long fatal = -1,       /* fatal error flag                              */
       lone = 1;         /* constant one                                  */

  *status = E_SUCC;

  for (i = 0; i < 32; i++)
  {
    for (j = 0; j < 32; j++)
    {
      if (! oldfft)
        a[i][j] = chip1[i][j];
      b[i][j] = asrch[il+i-1][js+j-1];
    }
  }

  if (! oldfft)
  {
    /* NOTE: the call to rft2_c is a FORTRAN call.
     */
    rft2_c_ (a, &thirty_two, &thirty_two, &one, &ret_val);
    if (ret_val != 0)
    {
      c_errmsg ("Error occurred in RFT2 subroutine", "picmatch-rft2", &lone);
      *status = E_FAIL;
      return;
    }
  }

  /* NOTE: the call to rft2_c is a FORTRAN call.
   */
  rft2_c_ (b, &thirty_two, &thirty_two, &one, &ret_val);
  if (ret_val != 0)
  {
    c_errmsg ("Error occurred in RFT2 subroutine", "picmatch-rft2", &lone);
    *status = E_FAIL;
    return;
  }

  if (! oldfft)
  {
    a[0][0] = 0.0;

    if (! nohpf)
    {
      /* Make the top row all 0.0s */
      for (i = 0; i < 32; i++)
        a[0][i] = 0.0;

      /* Make the first column all 0.0s */
      for (j = 0; j < 34; j++)
        a[j][0] = 0.0;
    }
  }

  for (i = 0; i < 32; i++)
  {
    for (j = 0; j < 33; j += 2)
    {
      bij = a[j][i] * b[j][i] + a[j+1][i] * b[j+1][i];
      bij1 = a[j][i] * b[j+1][i] - a[j+1][i] * b[j][i];
      tmp = bij * bij + bij1 * bij1;
      if (tmp < 0.0)
      {
        c_errmsg ("Value is < 0.0 prior to sqrt call", "picmatch-sqrt", &lone);
        c_errmsg ("Fatal error encountered", "picmatch-fatal", &fatal);
      }
      tmp_sqrt = (float) sqrt (tmp);    /* floating point square root */
      v = tmp_sqrt > 1.0e-6 ? tmp_sqrt : 1.0e-6;
      b[j+1][i] = bij1 / v;
      b[j][i] = bij / v;
    }
  }

  /* NOTE: the call to rft2_c is a FORTRAN call.
   */
  rft2_c_ (b, &thirty_two, &thirty_two, &neg_one, &ret_val);
  if (ret_val != 0)
  {
    c_errmsg ("Error occurred in RFT2 subroutine", "picmatch-rft2", &lone);
    *status = E_FAIL;
    return;
  }

  for (i = 0; i < 16; i++)
  {
    for (j = 0; j < 16; j++)
    {
      /* Swap b[j][i] with b[j+16][i+16]
       */
      tmp = b[j][i];
      b[j][i] = b[j+16][i+16];
      b[j+16][i+16] = tmp;

      /* Swap b[j][i+16] with b[j+16][i]
       */
      tmp = b[j][i+16];
      b[j][i+16] = b[j+16][i];
      b[j+16][i] = tmp;
    }
  }

  *vmax = -1.0e20;
  for (i = 7; i < 25; i++)
  {
    for (j = 7; j < 25; j++)
    {
      rv = b[i][j];
      if (*vmax < rv)
      {
        *vmax = rv;
        ixmax = i+1;
        jxmax = j+1;
      }
    }
  }

  *vloff = (float) (ixmax - 17);
  *vsoff = (float) (jxmax - 17);

  for (i = 0; i < 3; i++)
    for (j = 0; j < 3; j++)
      corr[i][j] = b[ixmax+i-2][jxmax+j-2];

  return;
}
